(* ::Package:: *)

(* ::Subtitle:: *)
(*Numeric`*)


(*BeginPackage["Numeric`",{"List`","Boole`"}]*)
BeginPackage["Numeric`",{"List`"}]


(* ::Subtitle:: *)
(*names*)


NTolerance;


NRound;
NChop;


NSign;
NUnitize;


NIntersectionIndices;
NIntersection;
NComplement;
NDeleteDuplicates;
NContainsAll;

NEqual;
NEqualTo;
NLess;
NLessThan;
NLessEqual;
NLessEqualThan;
NGreater;
NGreaterEqual;
NGreaterEqualThan;
NGreaterThan;


NPositive;
NNonPositive;
NNegative;
NNonNegative;


PositiveBoole;
NonPositiveBoole;
NegativeBoole;
NonNegativeBoole;


NPositiveBoole;
NNonPositiveBoole;
NNegativeBoole;
NNonNegativeBoole;


ListNEqualTo;
ListNUnequalTo;
ListNLessThan;
ListNLessEqualThan;
ListNGreaterThan;


ListNEqualToBoole;
ListNUnequalToBoole;
ListNLessThanBoole;
ListNLessEqualThanBoole;
ListNGreaterThanBoole;
ListNGreaterEqualThanBoole;


(* ::Subtitle:: *)
(*usage*)


NDeleteDuplicates::usage = "NDeleteDuplicates[list] removes duplicate elements in list (up to numerical tolerance)."
NRound::usage = "NRound[x] approximates x numerically with a precision set by the Tolerance option for NRound.";
NUnitize::usage = "NUnitize[x] returns 0 if |x|\[LessEqual]\[CurlyEpsilon] and 1 otherwise (up to numeric tolerance).";
NSign::usage = "NSign[x] returns the sign of x (as approximated with an accuracy set by the Tolerance option of NRound).";
NChop::usage = "NChop[values] retuns a copy of values where values within NTolerance of zero are set to 0.";
NEqual::usage = "NEqual[x, y, \[Ellipsis]] returns True if x \[Equal] y (up to numeric tolerance).";
NEqualTo::usage = "NEqual[y] is an operator that yields x \[Equal] y when applied to an expression x (up to numerical tolerance).";
NLess::usage = "NLess[x, y, \[Ellipsis]] yields True if x < y (up to numeric tolerance).";
NLessEqual::usage = "NLessEqual[x, y, \[Ellipsis]] yields True if x < y (up to numeric tolerance).";
NLessEqualThan::usage = "NLessEqualThan[y] is an operator that yields x \[LessEqual] y when applied to an expression x.";
NLessThan::usage = "NLessThan[y] is an operator that yields x < y when applied to an expression x (up to numerical tolerance).";
NGreater::usage = "NGreater[x, y, \[Ellipsis]] yields True if x > y (up to numeric tolerance).";
NGreaterEqual::usage = "NGreaterEqual[x, y, \[Ellipsis]] yields True if x \[GreaterEqual] y (up to numeric tolerance).";
NGreaterEqualThan::usage = "NGreaterEqualThan[y] is an operator that yields x \[GreaterEqual] y when applied to an expression x (up to numerical tolerance).";
NGreaterThan::usage = "NGreaterThan[y] is an operator that yields x > y when applied to an expression x (up to numerical tolerance).";
ListNEqualTo::usage = "ListEqualTo[list, value] returns the boolean list of element-wise comparrisons {element == value} for each element in list.";
ListNLessThan::usage = "ListNLessThan[list, value] returns the boolean list of element-wise comparrisons {element < value} for each element in list.";
ListNLessEqualThan::usage = "ListNLessEqualThan[list, value] returns the boolean list of element-wise comparrisons {element \[LessEqual] value} for each element in list.";
ListNLessEqualThan::usage = "ListNLessEqualThan[list, value] returns the boolean list of element-wise comparrisons {element > value} for each element in list.";
ListNGreaterEqualThan::usage = "ListEqualTo[list, value] returns the boolean list of element-wise comparrisons {element \[GreaterEqual] value} for each element in list.";
NIntersection::usage = "NIntersection[A, B, \[Ellipsis]] returns the elements of A that are in the intersection A \[Intersection] B \[Intersection] \[CenterEllipsis] (up to numeric tolerance).";
NIntersectionIndices::usage = "NIntersectionIndices[A,B] returns the indices of list A for which A\[LeftDoubleBracket]i\[RightDoubleBracket]\[Element]B (up to numerical tolerance).";
NContainsAll::usage = "NContainsAll[a,b] := ContainsAll[NRound[a],NRound[b]]";
NContainsAny::usage = "NContainsAny[a,b] := ContainsAny[NRound[a],NRound[b]]";
NContainsExactly::usage = "NContainsExactly[a,b] := ContainsExactly[NRound[a],NRound[b]]";
NContainsNone::usage = "NContainsNone[a,b] := ContainsNone[NRound[a],NRound[b]]";
NContainsOnly::usage = "NContainsOnly[a,b] := ContainsOnly[NRound[a],NRound[b]]";



(* ::Subtitle:: *)
(*definitions*)


Begin["`Private`"]


(* ::Subsection:: *)
(*rounding*)


(* ::Subsubsection::Closed:: *)
(*NTolerance*)


NTolerance = 10.^-10;


(* ::Subsubsection::Closed:: *)
(*NRound*)


NRound[x_] := Round[x, NTolerance];


(* ::Subsubsection::Closed:: *)
(*NChop*)


NChop[x_] := Times[x,NUnitize[x]]


(* ::Subsection:: *)
(*sign value*)


(* ::Subsubsection:: *)
(*NUnitize*)


NUnitize[x_] := UnitStep[Subtract[Abs[x],NTolerance]]


(* ::Subsubsection:: *)
(*NSign*)


NSign[x_] := Times[Sign[x],NUnitize[x]]


(* ::Subsection::Closed:: *)
(*sign test*)


NNonNegative[x_] := Positive[Plus[x,NTolerance]]
NNegative[x_] := NonPositive[Plus[x,NTolerance]]
NPositive[x_] := NonNegative[Subtract[x,NTolerance]]
NNonPositive[x_] := Negative[Subtract[x,NTolerance]]


(* ::Subsubsection::Closed:: *)
(*sign test | boole*)


NonNegativeBoole[x_] := UnitStep[x]
NonPositiveBoole[x_] := UnitStep[Minus[x]]
PositiveBoole[x_] := Ramp[Sign[x]]
NegativeBoole[x_] := Subtract[1,UnitStep[x]]


(* ::Subsubsection::Closed:: *)
(*sign test | boole | tolerance*)


NNonNegativeBoole[x_] := UnitStep[Subtract[x,Minus[NTolerance]]]
NNegativeBoole[x_] := UnitStep[Subtract[Minus[NTolerance],x]]
NPositiveBoole[x_] := UnitStep[Subtract[x,NTolerance]]
NNonPositiveBoole[x_] := UnitStep[Subtract[NTolerance,x]]


(* ::Subsection::Closed:: *)
(*value compare*)


NEqual[x_,y__] := Apply[Equal,NRound[List[x,y]]]
NEqualTo[y_][x_] := NEqual[x,y]


NLess[x_,y__] := Apply[Less,NRound[List[x,y]]]
NLessEqual[x_,y__] := Apply[LessEqual,NRound[List[x,y]]]
NLessThan[y_][x_] := NLess[x,y]
NLessEqualThan[y_][x_] := NLessEqual[x,y]


NGreater[x_,y__] := Apply[Greater,NRound[List[x,y]]]
NGreaterEqual[x_,y__] := Apply[GreaterEqual,NRound[List[x,y]]]
NGreaterEqualThan[y_][x_] := NGreaterEqual[x,y]
NGreaterThan[y_][x_] := NGreater[x,y]


(* ::Subsection:: *)
(*list compare | tolerance*)


ListNEqualTo[list_, value_] := NonPositive[Subtract[Abs[Subtract[list,value]],NTolerance]]
ListNUnequalTo[list_, value_] := Positive[Subtract[Abs[Subtract[list,value]],NTolerance]]


ListNLessThan[list_, value_] := Negative[Plus[list,Plus[Minus[value],NTolerance]]]
ListNLessEqualThan[list_, value_] := NonPositive[Plus[list,Plus[Minus[value],Minus[NTolerance]]]]


ListNGreaterThan[list_, value_] := Positive[Plus[list,Plus[Minus[value],Minus[NTolerance]]]]
ListNGreaterEqualThan[list_, value_] := NonNegative[Plus[list,Plus[Minus[value],NTolerance]]]


(* ::Subsection:: *)
(*set-wise operations*)


(* ::Subsubsection:: *)
(*NIntersectionIndices*)


NIntersectionIndices[x_,y_] := 
	Module[{xN,yN,zN,zxN,zxI,xI},
		xN = NRound[x];
		yN = NRound[y];
		zN = Intersection[xN,yN];
		zxI = Take[Values[PositionIndex[Join[zN,xN]]],Length[zN]];
		xI = Subtract[Part[zxI,All,2],Length[zN]];
		Return[xI];
	]


(* ::Subsubsection:: *)
(*NIntersection*)


NIntersection[a_,b_] := Part[a,NIntersectionIndices[a,b]]


(* ::Subsubsection:: *)
(*NComplement*)


NComplement[a_,b_] := Part[a,Complement[Range[Length[a]],NIntersectionIndices[a,b]]]


(* ::Subsubsection:: *)
(*NDeleteDuplicates*)


NDeleteDuplicates[x_] := 
	Module[{xN,zN,zxN,zxI,xI},
		xN = NRound[x];
		zN = DeleteDuplicates[xN];
		zxI = Take[Values[PositionIndex[Join[zN,xN]]],Length[zN]];
		xI = Subtract[Part[zxI,All,2],Length[zN]];
		Return[Part[x,xI]];
	]


(* ::Subsubsection:: *)
(*NContains~*)


NContainsAll[a_,b_] := ContainsAll[NRound[a],NRound[b]]
NContainsAny[a_,b_] := ContainsAny[NRound[a],NRound[b]]
NContainsExactly[a_,b_] := ContainsExactly[NRound[a],NRound[b]]
NContainsNone[a_,b_] := ContainsNone[NRound[a],NRound[b]]
NContainsOnly[a_,b_] := ContainsOnly[NRound[a],NRound[b]]


(* ::Subtitle:: *)
(*end*)


End[]


EndPackage[]
