(* ::Package:: *)

(* ::Title:: *)
(*Vector*)


(* ::Text:: *)
(*vector spaces | affine spaces | linear transformations *)


(* ::Subtitle::Closed:: *)
(*Vector`	*)


BeginPackage["Vector`",{"Numeric`","List`"}]


(* ::Subtitle:: *)
(*Names*)

VectorPlanes;
CoordinateDiagonalVector;
DiagonalVector;
EmbeddedNullSpace;
OriginVector;
PolygonArea;
PolygonEdges;
PolygonVertexOrdering;
PolygonVertexSort;
RandomOrthogonalMatrix;
RandomReflectionMatrix;
RandomSpecialOrthogonalMatrix;
SimplexMeasure;
SimplexNormal;
SimplexOrientedFacets;
SimplexOrientedQ;
StereographicProjection;
StereographicProjectionInverse;
UnitVectorQ;
VectorAffineDiamaterPointIndices;
VectorAffineDimension;
VectorAffineNullSpace;
VectorSpaceRegionMemberQ;
VectorAffineSpace;
VectorAffineSpaceData;
VectorAffineSpaceInit;
VectorAffineSpaceIntersection;
VectorAffineSpaceMethodHandler;
VectorAffineSpacePrint;
VectorAffineSpan;
VectorAffineSpanDimension;
VectorAffineSpanningSetIndices;
VectorAffineTransform;
VectorArg;
VectorBoundedBoole;
VectorBoundedQ;
VectorCases;
VectorCentroid;
VectorChiralQ;
VectorChopPoints;
VectorCompoundHelix;
VectorCoordinateSum;
VectorDilate;
VectorDimension;
VectorDistance;
VectorDistanceToHelix;
VectorDistanceToLine;
VectorEmbed;
VectorDimension;
VectorHelix;
VectorMatchQ;
VectorMatrixPlot;
VectorMaximalByNormal;
VectorNorm;
VectorNormalize;
VectorNullSpace;
VectorNullSpaceBasis;
VectorOrthogonalQ;
VectorPathParallelTransport;
VectorProject;
VectorRecenter;
VectorRegularSimplexQ;
VectorRotate;
VectorShells;
VectorSlicePoints;
VectorSort;
VectorSpace;
VectorSpaceData;
VectorSpaceIntersection;
VectorSpaceMethodHandler;
VectorSpacePrincipalAngles;
VectorSpacePrint;
VectorSpan;
VectorSpanDimension;
VectorSpanningSet;
VectorSpanningSetIndices;
VectorSquaredNorm;
VectorSquaredNorm;
VectorTransform;
VectorTranslate;


(* ::Subtitle::Closed:: *)
(*`Private`*)


Begin["`Private`"];

VectorPlanes::usage = "VectorPlanes[points, normal] gathers a set of points into planar subsets perpendicular to the"
VectorPlanes[points_, normalVector_] := Map[Function[Part[points,#]], Values[PositionIndex[NRound[Dot[points, normalVector]]]]]


(* ::Subtitle:: *)
(*vectors*)


(* ::Subsection::Closed:: *)
(*construct*)


(* ::Subsubsection::Closed:: *)
(*OriginVector*)


OriginVector::usage = 
"OriginVector[dimension] gives the origin vector {0,\[Ellipsis],0} in the specified dimension.";


OriginVector[dimension_] := ConstantArray[0,dimension]


(* ::Subsubsection::Closed:: *)
(*DiagonalVector*)


DiagonalVector::usage = 
"DiagonalVector[dimension] returns the diagonal vector {1,\[Ellipsis],1} in the specified dimension.";


DiagonalVector[dimension_] := ConstantArray[1,dimension]


(* ::Subsection:: *)
(*attributes*)


(* ::Subsubsection::Closed:: *)
(*VectorDimension*)


VectorDimension::usage = 
	"VectorDimension[points] returns the coordinate dimension of points.";


VectorDimension[vectors_?ArrayQ] := Last[Dimensions[vectors]]
VectorDimension[vectorLists_List] := VectorDimension[First[vectorLists]]


(* ::Subsubsection::Closed:: *)
(*UnitVectorQ*)


UnitVectorQ::usage = 
	"UnitVectorQ[vector] returns True if vector has unit norm (up to numerical Tolerance).";


UnitVectorQ[vector_?VectorQ] := NEqual[1,Norm[N[vector]]]


(* ::Subsubsection::Closed:: *)
(*VectorArg*)


VectorArg::usage = 
	"VectorArg[points] returns the argument of points in the plane.";


VectorArg[points_?ArrayQ] := Arg[Dot[points,IdentityMatrix[{VectorDimension[points],2}],{1,I}]]
VectorArg[points_] := Map[VectorArg,points];


(* ::Subsubsection::Closed:: *)
(*VectorOrthogonalQ*)


VectorOrthogonalQ[vector_, basis_] := NonPositive[Chop[VectorNorm[VectorProject[vector,basis]]]]


(* ::Subtitle::Closed:: *)
(*orthogonal matrices*)


(* ::Subsubsection::Closed:: *)
(*EmbeddedNullSpace*)


EmbeddedNullSpace::uasge = 
	"VectorEmbeddedNullSpace[n, basis] returns an the NullSpace of basis within an n-dimensional embedding space."<>"\n"<>
	"VectorEmbedded[n, basis] returns the n\[ThinSpace]\[Times]\[ThinSpace]n identity matrix if basis has rank zero.";


EmbeddedNullSpace[embeddingDimension_, {}] := IdentityMatrix[embeddingDimension]
EmbeddedNullSpace[embeddingDimension_, basis_] := NullSpace[basis]
EmbeddedNullSpace[embeddingDimension_][basis_] := EmbeddedNullSpace[embeddingDimension, basis]


(* ::Subsubsection::Closed:: *)
(*RandomOrthogonalMatrix*)


RandomOrthogonalMatrix::usage = 
"RandomOrthogonalMatrix[n] returns a random element of O(n).";


RandomOrthogonalMatrix[dimension_Integer] := Orthogonalize[RandomReal[{-1,1},{dimension,dimension}]]
RandomOrthogonalMatrix[dimensions_List] := Orthogonalize[RandomReal[{-1,1},dimensions]]


(* ::Subsubsection::Closed:: *)
(*RandomSpecialOrthogonalMatrix*)


RandomSpecialOrthogonalMatrix::usage = 
"RandomRotationMatrix[n] returns a random element of SO(n).";


RandomSpecialOrthogonalMatrix[dimension_] := 
	Module[{randomRotationMatrix},
		randomRotationMatrix = Orthogonalize[RandomReal[{-1,1},{dimension,dimension}]];
		If[Negative[Det[randomRotationMatrix]],
			randomRotationMatrix[[1]] *= -1;];
		Return[randomRotationMatrix];
	]


(* ::Subsubsection::Closed:: *)
(*RandomReflectionMatrix*)


RandomReflectionMatrix::usage = 
"RandomReflectionMatrix[n] returns the reflection matrix for a random vector in Rn.";


RandomReflectionMatrix[dimension_] := ReflectionMatrix[RandomReal[{-1,1},{dimension}]]


(* ::Subtitle:: *)
(*VectorSpace*)


(* ::Subsubsection::Closed:: *)
(*VectorSpace*)


VectorSpace::usage = 
	"VectorSpace is a symbolic warpper for the VectorSpace data type."<>"\n"<>
	"VectorSpace[data] represents a linear space embedded within some ambient Euclidean space."<>"\n"<>
	"VectorSpace[n] returns a full Euclidean n-space."<>"\n"<>
	"VectorSpace[k,n] returns the space of the first k dimension in \[DoubleStruckCapitalE]n."<>"\n"<>
	"VectorSpace[vectors] returns the span of vectors inside their embedding space.";


VectorSpace[data_Association][key_, args___] := VectorSpaceMethodHandler[VectorSpace[data], key, args]


VectorSpace[vectors_List?MatrixQ] := VectorSpan[vectors]


VectorSpace /: NullSpace[vectorSpace_VectorSpace] := VectorNullSpace[vectorSpace]


VectorSpace /: Print[vectorSpace_VectorSpace] := VectorSpacePrint[vectorSpace]


(* ::Subsubsection::Closed:: *)
(*VectorSpacePrint*)


VectorSpacePrint[VectorSpace[data_]] := Print[Grid[Apply[List,Normal[Map[MatrixForm,data]],1],Dividers->All]]


VectorSpacePrint[VectorSpace[data_]] := Print[Column[{Item["VectorSpace",Background->GrayLevel[0.8],Alignment->Center],Grid[Apply[List,Normal[Map[MatrixForm,data]],1],Dividers->All]}]]


(* ::Subsubsection::Closed:: *)
(*VectorSpaceData*)


VectorSpaceData[] := 
	Module[{vectorSpaceData},
		vectorSpaceData = Association[];
		vectorSpaceData["Dimension"] = 0;
		vectorSpaceData["EmbeddingDimension"] = 0;
		vectorSpaceData["NullSpaceDimension"] = 0;
		vectorSpaceData["Basis"] = {};
		vectorSpaceData["NullSpaceBasis"] = {};
		Return[vectorSpaceData];
	];


(* ::Subsubsection::Closed:: *)
(*VectorSpaceMethodHandler*)


VectorSpaceMethodHandler[VectorSpace[vectorSpaceData_], keyInput_, args___] :=
	Module[{key,keyMemberQ},
		key = keyInput;
		keyMemberQ = Composition[MemberQ[ToLowerCase[key]],ToLowerCase];
		
		If[keyMemberQ[{"Dimension","dim","d"}],
			Return[vectorSpaceData["Dimension"]];];
		
		If[keyMemberQ[{"NullSpaceDimension","Codimension","NullDimension","NullDim","CoDim","nd"}],
			Return[Lookup[vectorSpaceData,"NullSpaceDimension"]];];
		
		If[keyMemberQ[{"Basis","B"}],
			Return[Lookup[vectorSpaceData,"Basis"]];];
			
		If[keyMemberQ[{"NullSpaceBasis","NullSpace","NullBasis","NB"}],
			Return[Lookup[vectorSpaceData,"NullSpaceBasis"]];];
		
		If[keyMemberQ[{"BasePoint","Origin","AffineOrigin","Point"}],
			Return[OriginVector[Lookup[vectorSpaceData,"EmbeddingDimension"]]];];

		Return[];
	]


(* ::Subsubsection::Closed:: *)
(*VectorSpan*)


VectorSpan[vectorsInput_?MatrixQ] :=
	Module[{
			vectors,
			embeddingDimension,
			dimension,
			basisVectorIndices,
			basis,
			nullSpaceBasis,
			nullSpaceDimension,
			vectorSpaceData
		},
		(* input *)
			vectors = N[vectorsInput];
		(* embedding dimension *)
			embeddingDimension = VectorDimension[vectors];
		(* vector space basis & dimension *)
			basisVectorIndices = VectorSpanningSetIndices[vectors];
			basis = Part[vectors,basisVectorIndices];
			basis = NChop[Orthogonalize[basis]];
			basis = Select[basis, Composition[Positive,Norm]];
		(* dimension *)
			dimension = Length[basis];
			If[Equal[dimension,embeddingDimension],
				basis = IdentityMatrix[embeddingDimension];];
		(* null space basis & dimension *)
			nullSpaceBasis = If[Unequal[basis,{}], NChop[Orthogonalize[NullSpace[basis]]], IdentityMatrix[embeddingDimension]];
			nullSpaceBasis = Select[nullSpaceBasis, Composition[Positive,Norm]];
			nullSpaceDimension = Length[nullSpaceBasis];
		(* output data *)
			vectorSpaceData = VectorSpaceData[];
			vectorSpaceData["EmbeddingDimension"] = embeddingDimension;
			vectorSpaceData["Basis"] = basis;
			vectorSpaceData["Dimension"] = dimension;
			vectorSpaceData["NullSpaceBasis"] = nullSpaceBasis;
			vectorSpaceData["NullSpaceDimension"] = nullSpaceDimension;
		(* return *)
			Return[VectorSpace[vectorSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorSpanDimension*)


VectorSpanDimension[points_?MatrixQ] := If[OrderedQ[Dimensions[points,2]], MatrixRank[N[points]], MatrixRank[Transpose[N[points]]]]


(* ::Subsubsection::Closed:: *)
(*VectorSpanningSetIndices*)


VectorSpanningSetIndices::usage = 
	"VectorSpaceSpanningSetIndices[vectors] returns the indices of a minimal spanning set of vectors.";


VectorSpanningSetIndices[vectorsInput_?MatrixQ, maximumDimension_:Infinity] := 
	Module[{
			vectors,
			vectorSpaceDimension,
			embeddingSpaceDimension,
			vectorIndex,
			vector,
			spanningVertexIndices
		},
		vectors = N[vectorsInput];
		embeddingSpaceDimension = VectorDimension[vectors];
		spanningVertexIndices = {};
		Do[
			vectorIndex = FirstMaximumIndex[NRound[VectorNorm[vectors]]];
			vector = Part[vectors,vectorIndex];
			If[Equal[0,Chop[Norm[vector]]],Break[];];
			spanningVertexIndices = Append[spanningVertexIndices,vectorIndex];
			If[Equal[Length[spanningVertexIndices],embeddingSpaceDimension],Break[];];
			vectors = VectorProject[vectors,NChop@Orthogonalize[NullSpace[{vector}]]];
		,{vectorSpaceDimension,Min[embeddingSpaceDimension,maximumDimension]}];
		Return[spanningVertexIndices];
	]


(* ::Subsubsection::Closed:: *)
(*VectorSpanningSet*)


VectorSpanningSet[vectors_List?MatrixQ, maximumDimension_:Infinity] := Part[vectors,VectorSpanningSetIndices[vectors,maximumDimension]]


(* ::Subsection::Closed:: *)
(*operators*)


(* ::Subsubsection::Closed:: *)
(*VectorNullSpace*)


VectorNullSpace[VectorSpace[vectorSpaceData_]] := 
	Module[{nullSpaceData},
		nullSpaceData = VectorSpaceData[];
		nullSpaceData["EmbeddingDimension"] = vectorSpaceData["EmbeddingDimension"];
		nullSpaceData["Dimension"] = vectorSpaceData["NullSpaceDimension"];
		nullSpaceData["NullSpaceDimension"] = vectorSpaceData["Dimension"];
		nullSpaceData["Basis"] = vectorSpaceData["NullSpaceBasis"];
		nullSpaceData["NullSpaceBasis"] = vectorSpaceData["Basis"];
		Return[VectorSpace[nullSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorSpaceIntersection | vector spaces*)


VectorSpaceIntersection[VectorSpace[vectorSpace1_], VectorSpace[vectorSpace2_]] := 
	Module[
		{
			embeddingDimension,
			nullSpaceBasis1,
			nullSpaceBasis2,
			vectorSpaceBasis,
			nullSpaceBasis,
			vectorSpaceData
		},
		(* check embedding dimensions *)
			If[Unequal[vectorSpace1["EmbeddingDimension"],vectorSpace2["EmbeddingDimension"]],Abort[];];
		(* input *)
			embeddingDimension = vectorSpace1["EmbeddingDimension"];
			nullSpaceBasis1 = vectorSpace1["NullSpaceBasis"];
			nullSpaceBasis2 = vectorSpace2["NullSpaceBasis"];
		(* null space of intersection *)
			nullSpaceBasis = Join[vectorSpace1["NullSpaceBasis"],vectorSpace2["NullSpaceBasis"]];
		(*  *)
			If[Equal[nullSpaceBasis,{}],
				vectorSpaceBasis = IdentityMatrix[embeddingDimension];
				,				
				vectorSpaceBasis = NullSpace[nullSpaceBasis]//Orthogonalize//NChop;
			];
		(* output *)
			vectorSpaceData = VectorSpaceData[];
			vectorSpaceData["EmbeddingDimension"] = embeddingDimension;
			vectorSpaceData["Dimension"] = Length[vectorSpaceBasis];
			vectorSpaceData["NullSpaceDimension"] = Length[nullSpaceBasis];
			vectorSpaceData["Basis"] = vectorSpaceBasis;
			vectorSpaceData["NullSpaceBasis"] = nullSpaceBasis;
			
		(* return *)
			Return[VectorSpace[vectorSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorSpaceIntersection | basis matrices*)


VectorSpaceIntersection[{},{}] := {}
VectorSpaceIntersection[a_?MatrixQ,{}] := {}
VectorSpaceIntersection[{},b_?MatrixQ] := {}
VectorSpaceIntersection[a_?MatrixQ,b_?MatrixQ] := NChop[Orthogonalize[EmbeddedNullSpace[VectorDimension[a],Join[NullSpace[a],NullSpace[b]]]]]


(* ::Subsubsection::Closed:: *)
(*VectorSpacePrincipalAngles*)


VectorSpacePrincipalAngles[basis1_?MatrixQ,basis2_?MatrixQ] := ArcCos[SingularValueList[Dot[basis1,Transpose[basis2]]]]


VectorSpacePrincipalAngles[VectorSpace[vectorSpaceData1_], VectorSpace[vectorSpaceData2_]] := 
	VectorSpacePrincipalAngles[vectorSpaceData1["Basis"],vectorSpaceData2["Basis"]]


(* ::Subsubsection::Closed:: *)
(*VectorAffineDimension*)


VectorAffineDimension[points_] := VectorAffineSpanDimension[points]


(* ::Subsubsection::Closed:: *)
(*VectorSpaceRegionMemberQ*)


VectorSpaceRegionMemberQ[ vectorSpace_, points_ ] := 
	Module[{basePoint, hyperPlaneNormals, hyperPlaneBounds, pointValues, pointBs, pointQs},
		basePoint = vectorSpace["BasePoint"];
		hyperPlaneNormals = vectorSpace["NullSpaceBasis"];
		hyperPlaneBounds = NRound[Dot[hyperPlaneNormals,basePoint]];
		pointValues = Round[VectorNorm[Transpose[Subtract[Dot[hyperPlaneNormals,Transpose[points]],hyperPlaneBounds]]],10.^-6];
		pointBs = Subtract[1,Sign[pointValues]];
		pointQs = Positive[pointBs];
		Return[pointQs];
	]


(* ::Subtitle:: *)
(*VectorAffineSpace*)


(* ::Subsection::Closed:: *)
(*init*)


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpace*)


VectorAffineSpace::usage = 
	"VectorAffineSpace[points] returns a VectorAffineSpace data object representing the affine span of points.";


VectorAffineSpace[] := VectorAffineSpace[VectorAffineSpaceData[]]


VectorAffineSpace[points_?MatrixQ] := VectorAffineSpan[points]


VectorAffineSpace[basePoint_, basis_] := VectorAffineSpaceInit[basePoint,basis]


VectorAffineSpace[affineSpaceData_Association][key_, args___] := VectorAffineSpaceMethodHandler[VectorAffineSpace[affineSpaceData],key,args]


VectorAffineSpace /: Print[vectorAffineSpace_VectorAffineSpace] := VectorAffineSpacePrint[vectorAffineSpace]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpacePrint*)


VectorAffineSpacePrint[VectorAffineSpace[data_]] := 
	Print[
		Column[{
			Item[
				"VectorAffineSpace", 
				Background -> GrayLevel[0.8],
				Alignment -> Center,
				Frame -> 1
			],
			Item[
				Grid[Apply[List,Normal[Map[MatrixForm,data]],1], Dividers -> All, Alignment -> {{Right,Left}}],
				Frame -> 1
			]
		}]
		]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpaceData*)


VectorAffineSpaceData[] := 
	Module[{affineSpaceData},
		affineSpaceData = Association[];
		affineSpaceData["Dimension"] = -1;
		affineSpaceData["EmbeddingDimension"] = -1;
		affineSpaceData["NullSpaceDimension"] = {};
		affineSpaceData["Basis"] = {};
		affineSpaceData["NullSpaceBasis"] = {};
		affineSpaceData["BasePoint"] = {};
		Return[affineSpaceData];
	];


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpaceMethodHandler*)


VectorAffineSpaceMethodHandler[VectorAffineSpace[affineSpaceData_], keyInput_, args___] :=
	Module[{key,keyMemberQ},
		key = keyInput;
		keyMemberQ = Composition[MemberQ[ToLowerCase[key]],ToLowerCase];
		
		If[keyMemberQ[{"Dimension","dim"}],
			Return[Lookup[affineSpaceData,"Dimension"]];];
			
		If[keyMemberQ[{"EmbeddingDimension","EmbeddingSpaceDimension","AmbientSpaceDimension","AmbientDimension","EmbDim","EDim"}],
			Return[Lookup[affineSpaceData,"EmbeddingDimension"]];];
			
		If[keyMemberQ[{"Basis"}],
			Return[Lookup[affineSpaceData,"Basis"]];];
		
		If[keyMemberQ[{"NullSpaceDimension","Codimension","NullDimension","NullDim","CoDim"}],
			Return[Lookup[affineSpaceData,"NullSpaceDimension"]];];
			
		If[keyMemberQ[{"NullSpaceBasis","NullSpace","NullBasis","NullDim"}],
			Return[Lookup[affineSpaceData,"NullSpaceBasis"]];];
		
		If[keyMemberQ[{"BasePoint","Origin","AffineOrigin","InteriorPoint","Point"}],
			Return[Lookup[affineSpaceData,"BasePoint"]];];

		Return[];
	]


(* ::Subsection::Closed:: *)
(*construct*)


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpaceInit*)


VectorAffineSpaceInit[basePoint_?VectorQ, {}] := 
	VectorAffineSpaceInit[
		basePoint,
		{OriginVector[VectorDimension[basePoint]]}
	]


VectorAffineSpaceInit[basePoint_?VectorQ, basis_?MatrixQ] := 
	Module[{affineSpace},
		affineSpace = VectorAffineSpaceData[];
		affineSpace["BasePoint"] = basePoint;
		affineSpace["EmbeddingDimension"] = VectorDimension[basePoint];
		affineSpace["Basis"] = Select[NChop[Orthogonalize[basis]],Positive@*Norm];
		affineSpace["Dimension"] = Length[affineSpace["Basis"]];
		If[Equal[affineSpace["Dimension"],affineSpace["EmbeddingDimension"]],
			affineSpace["Basis"] = IdentityMatrix[affineSpace["EmbeddingDimension"]];];
		affineSpace["NullSpaceBasis"] = EmbeddedNullSpace[affineSpace["EmbeddingDimension"],affineSpace["Basis"]];
		affineSpace["NullSpaceDimension"] = Length[affineSpace["NullSpaceBasis"]];
		Return[VectorAffineSpace[affineSpace]];
	]


(* ::Subsubsection:: *)
(*VectorAffineSpan*)


VectorAffineSpan[] := VectorAffineSpace[]


VectorAffineSpan[{}] := VectorAffineSpace[]


VectorAffineSpan[{{}}] := VectorAffineSpace[]


VectorAffineSpan[{point_?VectorQ}] := 
	Module[{affineSpaceData},
		affineSpaceData = VectorAffineSpaceData[];
		affineSpaceData["EmbeddingDimension"] = VectorDimension[point];
		affineSpaceData["Basis"] = {};
		affineSpaceData["Dimension"] = 0;
		affineSpaceData["NullSpaceBasis"] = IdentityMatrix[VectorDimension[point]];
		affineSpaceData["NullSpaceDimension"] = VectorDimension[point];
		affineSpaceData["BasePoint"] = point;
		Return[VectorAffineSpace[affineSpaceData]];
	];


VectorAffineSpan[pointsInput_?MatrixQ] := 
	Module[{
			points,
			embeddingDimension,
			affineSpanningPointIndices,
			affineSpanningPoints,
			affineSpaceData,
			affineSpaceBasis,
			affineSpaceDimension,
			affineNullSpaceBasis,
			affineNullSpaceDimension,
			affineBasePoint
		},
		(* points *)
			points = N[pointsInput];
			points = NChop[pointsInput];
			embeddingDimension = VectorDimension[points];
		(* affine space *)
			affineBasePoint = Mean[points];
			affineSpanningPointIndices = VectorAffineSpanningSetIndices[points];
			affineSpanningPoints = Part[points,affineSpanningPointIndices];
			affineSpaceBasis = NChop[VectorTranslate[Rest[affineSpanningPoints],Minus[First[affineSpanningPoints]]]];
			affineSpaceBasis = NChop[Orthogonalize[affineSpaceBasis]];
			affineSpaceDimension = Length[affineSpaceBasis];
			affineNullSpaceBasis = NChop[EmbeddedNullSpace[embeddingDimension,affineSpaceBasis]];
			affineNullSpaceBasis = NChop[Orthogonalize[affineNullSpaceBasis]];
			affineNullSpaceDimension = Length[affineNullSpaceBasis];
		(* output *)
			affineSpaceData = VectorAffineSpaceData[];
			affineSpaceData["EmbeddingDimension"] = embeddingDimension;
			affineSpaceData["Basis"] = affineSpaceBasis;
			affineSpaceData["Dimension"] = affineSpaceDimension;
			affineSpaceData["NullSpaceBasis"] = affineNullSpaceBasis;
			affineSpaceData["NullSpaceDimension"] = affineNullSpaceDimension;
			affineSpaceData["BasePoint"] = affineBasePoint;
		(* return *)
			Return[VectorAffineSpace[affineSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpanningSetIndices*)


VectorAffineSpanningSetIndices[pointsInput_?MatrixQ] := 
	Module[{
			points,
			vectors,
			affineSpanningSetIndices
		},
		points = N[pointsInput];
		vectors = NChop[VectorTranslate[Rest[points],Minus[First[points]]]];
		affineSpanningSetIndices = Plus[1,VectorSpanningSetIndices[vectors]];
		affineSpanningSetIndices = Prepend[affineSpanningSetIndices,1];
		Return[affineSpanningSetIndices];
	]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpanDimension*)


VectorAffineSpanDimension::usage = 
	"VectorAffineSpanDimension[points] returns the dimension of the affine space spanned by points.";


VectorAffineSpanDimension[{}] := -1
VectorAffineSpanDimension[point_?VectorQ] := 0
VectorAffineSpanDimension[{point_?VectorQ}] := 0
VectorAffineSpanDimension[points_?MatrixQ] := VectorSpanDimension[NChop[VectorTranslate[Rest[points],Minus[First[points]]]]]


(* ::Subsection:: *)
(*operators*)


(* ::Subsubsection::Closed:: *)
(*VectorAffineNullSpace*)


VectorAffineNullSpace::usage = 
	"VectorAffineNullSpace[points] returns a VectorAffineSpace data object representing the affine null space of points.";


VectorAffineNullSpace[] := VectorAffineSpace[]
VectorAffineNullSpace[{}] := VectorAffineSpace[]


(* ::Subsubsection::Closed:: *)
(*VectorAffineNullSpace[ VectorAffineSpace ]*)


VectorAffineNullSpace[VectorAffineSpace[affineSpaceData_]] := 
	Module[{affineNullSpaceData},
		affineNullSpaceData = VectorAffineSpaceData[];
		affineNullSpaceData["EmbeddingDimension"] = affineSpaceData["EmbeddingDimension"];
		affineNullSpaceData["Dimension"] = affineSpaceData["NullSpaceDimension"];
		affineNullSpaceData["NullSpaceDimension"] = affineSpaceData["Dimension"];
		affineNullSpaceData["Basis"] = affineSpaceData["NullSpaceBasis"];
		affineNullSpaceData["NullSpaceBasis"] = affineSpaceData["Basis"];
		affineNullSpaceData["BasePoint"] = affineSpaceData["BasePoint"];
		Return[VectorAffineSpace[affineNullSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorAffineNullSpace[ VectorSpace ]*)


VectorAffineNullSpace[VectorSpace[vectorSpaceData_]] := 
	Module[{affineNullSpaceData},
		affineNullSpaceData = VectorAffineSpaceData[];
		affineNullSpaceData["EmbeddingDimension"] = vectorSpaceData["EmbeddingDimension"];
		affineNullSpaceData["Dimension"] = vectorSpaceData["NullSpaceDimension"];
		affineNullSpaceData["NullSpaceDimension"] = vectorSpaceData["Dimension"];
		affineNullSpaceData["Basis"] = vectorSpaceData["NullSpaceBasis"];
		affineNullSpaceData["NullSpaceBasis"] = vectorSpaceData["Basis"];
		affineNullSpaceData["BasePoint"] = OriginVector[vectorSpaceData["EmbeddingDimension"]];
		Return[VectorAffineSpace[affineNullSpaceData]];
	]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpaceIntersection*)


VectorAffineSpaceIntersection::EmbeddingDimensionMismatch = "Affine subspaces are not embedded in the same dimension.";


VectorAffineSpaceIntersection::NoIntersection = "Affine subspaces do not intersect.";


VectorAffineSpaceIntersection[VectorAffineSpace[aff1_], VectorAffineSpace[aff2_]] := 
	Module[{
			aff,
			dim,
			liftedBasis1,
			liftedBasis2,
			liftedIntersectionBasis,
			liftedBasePoint,
			basePoint,
			basis
		},
		aff = VectorAffineSpace[];
		If[Unequal[aff1["EmbeddingDimension"],aff2["EmbeddingDimension"]], 
			Message[VectorAffineSpaceIntersection::EmbeddingDimensionMismatch];
			Return[VectorAffineSpace[]];
		];
		dim = aff1["EmbeddingDimension"];
		
		If[Equal[aff2["Dimension"],dim], Return[VectorAffineSpace[aff1]]];
		If[Equal[aff1["Dimension"],dim], Return[VectorAffineSpace[aff2]]];
		
		basis = VectorSpaceIntersection[aff1["Basis"],aff2["Basis"]];

		liftedBasis1 = ArrayFlatten[{{aff1["Basis"],0},{{aff1["BasePoint"]},1}}];
		liftedBasis2 = ArrayFlatten[{{aff2["Basis"],0},{{aff2["BasePoint"]},1}}];
		
		liftedIntersectionBasis = VectorSpaceIntersection[liftedBasis1,liftedBasis2];

		If[NoneTrue[Part[liftedIntersectionBasis,All,-1],UnequalTo[0]],
			Message[VectorAffineSpaceIntersection::NoIntersection];
			Return[VectorAffineSpace[]];
		];
		
		liftedBasePoint = SelectFirst[liftedIntersectionBasis,Last /* Chop /* UnequalTo[0]];
		liftedBasePoint = Divide[liftedBasePoint,Last[liftedBasePoint]];
		basePoint = Drop[liftedBasePoint,-1];
		aff = VectorAffineSpace[basePoint,basis];
		Return[aff];
	]


(* ::Subsection::Closed:: *)
(*convert (up/down values)*)


(* ::Subsubsection::Closed:: *)
(*VectorSpace \[Function] VectorAffineSpace*)


VectorAffineSpace /: VectorAffineSpace[VectorSpace[vectorSpaceData_]] := 
	Module[{affineSpaceData, basePoint},
		(* init *)
			affineSpaceData = VectorAffineSpaceData[];
		(* copy *)
			affineSpaceData["EmbeddingDimension"] = vectorSpaceData["EmbeddingDimension"];
			affineSpaceData["Dimension"] = vectorSpaceData["Dimension"];
			affineSpaceData["NullSpaceDimension"] = vectorSpaceData["NullSpaceDimension"];
			affineSpaceData["Basis"] = vectorSpaceData["Basis"];
			affineSpaceData["NullSpaceBasis"] = vectorSpaceData["NullSpaceBasis"];
		(* base point *)
			basePoint = OriginVector[vectorSpaceData["EmbeddingDimension"]];
			affineSpaceData["BasePoint"] = basePoint;
		(* return *)
			Return[affineSpaceData];
	]


(* ::Subsubsection::Closed:: *)
(*VectorAffineSpace \[Function] VectorSpace*)


VectorAffineSpace /: VectorSpace[VectorAffineSpace[vectorAffineSpaceData_]] := 
	Module[{vectorSpace},
		(* init *)
			vectorSpace = VectorSpaceData[];
		(* copy attributes (except "BasePoints") *)
			vectorSpace["EmbeddingDimension"] = vectorAffineSpaceData["EmbeddingDimension"];
			vectorSpace["Basis"] = vectorAffineSpaceData["Basis"];
			vectorSpace["Dimension"] = vectorAffineSpaceData["Dimension"];
			vectorSpace["NullSpaceBasis"] = vectorAffineSpaceData["NullSpaceBasis"];
			vectorSpace["NullSpaceDimension"] = vectorAffineSpaceData["NullSpaceDimension"];
		(* return *)
			Return[VectorSpace[vectorSpace]];
	]


(* ::Subsection::Closed:: *)
(*compositions*)


(* ::Subsubsection::Closed:: *)
(*VectorNullSpace: VectorAffineSpace \[Function] VectorSpace \[Function] VectorSpace*)


VectorAffineSpace /: VectorNullSpace[VectorAffineSpace[vectorAffineSpaceData_]] := VectorNullSpace[VectorSpace[VectorAffineSpace[vectorAffineSpaceData]]]


(* ::Subsubsection::Closed:: *)
(*VectorAffineNullSpace: VectorSpace \[Function] VectorAffineSpace \[Function] VectorAffineSpace*)


VectorAffineNullSpace[VectorSpace[vectorAffineSpaceData_]] := VectorAffineNullSpace[VectorAffineSpace[VectorSpace[vectorAffineSpaceData]]]


(* ::Subtitle:: *)
(*geometry*)


(* ::Subsection:: *)
(*helix*)


(* ::Subsubsection::Closed:: *)
(*VectorHelix*)


VectorHelix::usage = 
	"VectorHelix[theta] returns the coordinates of a helix for the given value(s) of theta.Options: \"BasePoint\", \"Axis\", \"Radius\", \"Wavelength\", and \"Phase\".";


Options[VectorHelix] = {
	Rule["BasePoint",{0,0,0}],
	Rule["Axis",{0,0,1}],
	Rule["Radius",1],
	Rule["Wavelength",1],
	Rule["Phase",0]
	};

VectorHelix[theta_,OptionsPattern[]] := 
	Module[{base,axis,radius,Wavelength,phase,helix},
		{base,axis,radius,Wavelength,phase} = OptionValue[{"BasePoint","Axis","Radius","Wavelength","Phase"}];
		helix = {radius*Cos[theta+phase],radius*Sin[theta+phase],Wavelength*theta/(2Pi)};
		If[MatrixQ[helix],helix = Transpose[helix];];
		helix = VectorProject[helix,RotationMatrix[{{0,0,1},N[axis]}]];
		helix = VectorTranslate[helix,base];
		Return[helix];
	]


(* ::Subsubsection::Closed:: *)
(*VectorDistanceToHelix*)


VectorDistanceToHelix::usage = 
	"VectorDistanceToHelix[base,axis,radius,wavelength,phase,start][points] returns the distance between points and the helix as prescribed by the parameters.";


VectorDistanceToHelix[base_,axis_,radius_,wavelength_,phase_,phaseZeroVertex_:Null][points_?ArrayQ] := 
	Module[{r,\[Theta],startphase,h,startHeight,axisPerp,pointsPerp,pointsCylindriacal,axisCylindrical,baseCylindrical,distCylindrical},
		axisPerp = Orthogonalize[NullSpace[{axis}]];
		pointsPerp = VectorTranslate[points,-base].Transpose[axisPerp];
		startphase = If[phaseZeroVertex == Null,0,Apply[ArcTan,(phaseZeroVertex-base).Transpose[axisPerp]]];
		startHeight = If[phaseZeroVertex == Null,0,(phaseZeroVertex-base).axis];
		r = VectorNorm[pointsPerp];
		\[Theta] = Arg[pointsPerp.{1,I}]-startphase;
		h = VectorTranslate[points,-base].axis-startHeight;
		h = h+phase/(2*Pi)*wavelength;
		h = Mod[h,wavelength,-wavelength/2];
		pointsCylindriacal = TransposeArray[{r,\[Theta],h}];
		baseCylindrical = {radius,0,0};
		axisCylindrical = {0.,2\[Pi],wavelength}//Normalize;
		distCylindrical = ArrayMin[
			VectorDistanceToLine[baseCylindrical,axisCylindrical][pointsCylindriacal],
			VectorDistanceToLine[baseCylindrical+{0,2\[Pi],0},axisCylindrical][pointsCylindriacal],
			VectorDistanceToLine[baseCylindrical-{0,2\[Pi],0},axisCylindrical][pointsCylindriacal]
		];
		Return[distCylindrical];
	];
VectorDistanceToHelix[args__][points_] := Map[VectorDistanceToHelix[args],points]


(* ::Subsubsection::Closed:: *)
(*VectorCompoundHelix*)


VectorCompoundHelix::usage = 
	"VectorCompoundHelix[theta] returns the point coordinates of a compounded helix for the given value(s) of theta.Options: \"BasePoint\", \"Axis\", \"Radius\", \"Wavelength\",\"Phase\",\"Radius2\",\"Phase2\",\"Wavelength2\".";


Options[VectorCompoundHelix] = 
	{
		Rule["BasePoint",{0,0,0}],
		Rule["Axis",{0,0,1}],
		Rule["Radius",1],
		Rule["Wavelength",1],
		Rule["Phase",0],
		Rule["Radius2",1/5],
		Rule["Wavelength2",2],
		Rule["Phase2",\[Pi]]
	};
	
VectorCompoundHelix[thetaInput_,OptionsPattern[]] := 
	Module[{theta,base,axis,radius,wavelength,phase,radius2,wavelength2,phase2,theta2,helix,unitTangentVector,curvatureVector,unitNormalVector,unitBiNormalVector,compoundHelix},
		theta = thetaInput;
		If[Not[ListQ[theta]],theta = List[theta];];
		{base,axis,radius,wavelength,phase,radius2,wavelength2,phase2} = OptionValue[{"BasePoint","Axis","Radius","Wavelength","Phase","Radius2","Wavelength2","Phase2"}];
		
		helix = {radius*Cos[theta+phase],radius*Sin[theta+phase],wavelength*theta/(2Pi)};
		unitTangentVector = Divide[{-2 \[Pi] radius Sin[phase+theta],2 \[Pi] radius Cos[phase+theta],wavelength},Sqrt[4*Pi^2*radius^2+wavelength^2]];
		curvatureVector = Divide[{-2 \[Pi] radius Cos[phase+theta],-2 \[Pi] radius Sin[phase+theta],0},Sqrt[4*Pi^2*radius^2+wavelength^2]];
		unitNormalVector = {-((radius*Cos[phase + theta])/(Sqrt[radius^2/(4*Pi^2*radius^2 + wavelength^2)]*Sqrt[4*Pi^2*radius^2 + wavelength^2])), -((radius*Sin[phase + theta])/(Sqrt[radius^2/(4*Pi^2*radius^2 + wavelength^2)]*Sqrt[4*Pi^2*radius^2 + wavelength^2])), 0*theta};
		unitBiNormalVector = {(Sqrt[radius^2]*wavelength*Sqrt[(4*Pi^2*radius^2 + wavelength^2)^(-1)]*Sin[phase + theta])/radius, -((Sqrt[radius^2]*wavelength*Sqrt[(4*Pi^2*radius^2 + wavelength^2)^(-1)]*Cos[phase + theta])/radius),0*theta+2*Pi*Sqrt[radius^2]*Sqrt[(4*Pi^2*radius^2 + wavelength^2)^(-1)]};
		theta2 = Divide[theta*Sqrt[4*Pi^2*radius^2 + wavelength^2],wavelength2]*2*Pi;

		compoundHelix = Transpose[helix] + Transpose[unitBiNormalVector]*radius2*Cos[theta2+phase2] + Transpose[unitNormalVector]*radius2*Sin[theta2+phase2];
		compoundHelix = VectorProject[compoundHelix,RotationMatrix[{{0,0,1},N[axis]}]];
		compoundHelix = VectorTranslate[compoundHelix,base];
		Return[compoundHelix];
	]


(* ::Subsection::Closed:: *)
(*path*)


(* ::Subsubsection::Closed:: *)
(*VectorPathEdges*)


PathEdges[path_] := Partition[path,2,1]


(* ::Subsubsection::Closed:: *)
(*VectorPathParallelTransport*)


VectorPathParallelTransport::usage = 
"VectorPathParallelTransport[path] returns a list of orthonormal, positively oriented coordinates frames at each vertex of path, corresponding to the discretized parrallel transport ";


VectorPathParallelTransport[path_] := 
	Module[{dim,rotations,frames,tangent,tangentNew,frame,rotation},
		dim = VectorDimension[path];
		frame = IdentityMatrix[dim];
		rotations = {};
		frame = IdentityMatrix[dim];
		frames = {};
		tangent = First[frame];
		Do[
			tangentNew = VectorNormalize[path[[i+1]]-path[[i]]];
			If[MatrixRank[{tangent,tangentNew}] == 2,
				rotation = RotationMatrix[{tangent,tangentNew}];
				,
				If[Negative[Dot[tangent,tangentNew]],
					rotation = ReflectionMatrix[tangentNew];
					,
					rotation = IdentityMatrix[dim];
				];
			];
			tangent = tangentNew;
			frame = Dot[frame,Transpose[rotation]];
			AppendTo[frames,frame];
			AppendTo[rotations,rotation];
		,{i,Length[path]-1}
		];
		AppendTo[rotations,Last[rotations]];
		AppendTo[frames,Last[frames]];
		Return[frames];
	]


(* ::Subsection:: *)
(*simplex*)


(* ::Subsubsection::Closed:: *)
(*SimplexNormal*)


SimplexNormal::usage = 
"SimplexNormal[vertices] returns a unit vector normal to simplex.";


SimplexNormal[{{x_}}] := {1}
SimplexNormal[simplex_] := Apply[Cross,VectorTranslate[Rest[simplex],Minus[First[simplex]]]]


(* ::Subsubsection::Closed:: *)
(*SimplexMeasure*)


SimplexMeasure::usage = 
"SimplexMeasure[simplex] returns the measure of the simplex (d-dimensional volume of d-dimensional simplex).";


SimplexMeasure /: SimplexMeasure[Simplex[pointCoordinates_]] := Abs[SimplexMeasure[pointCoordinates]]


SimplexMeasure[simplexInput_?MatrixQ] := 
	Module[{
			simplexVertices,
			embeddingDimension,
			affineDimension,
			simplexVectors,
			measure
		},
		simplexVertices = simplexInput;
		embeddingDimension = VectorDimension[simplexVertices];
		affineDimension = Subtract[Length[simplexVertices],1];
		If[Greater[affineDimension,embeddingDimension],
			Return[0];];
		simplexVectors = VectorTranslate[Rest[simplexVertices],Minus[First[simplexVertices]]];
		If[Less[affineDimension,embeddingDimension],
			simplexVectors = VectorProject[simplexVectors,Orthogonalize[simplexVectors]];];
		measure = Divide[Det[simplexVectors],Factorial[affineDimension]];
		measure = Abs[measure];
		Return[measure];
	]


(* ::Subsubsection::Closed:: *)
(*SimplexOrientedFacets*)


SimplexOrientedFacets::usage = 
"SimplexOrientedFacets[simplex] returns the facets of simplex, each facet is ordered to have outward facing normals.";


SimplexOrientedFacets[simplexVertices_] := 
	If[LessEqual[Length[simplexVertices],2],
		Subsets[simplexVertices,{Length[simplexVertices]-1}]
		,
		Module[{simplexVertexIndices,simplexVertexCount,simplexFacetCount,orientedFacetVertexIndices,orientedFacetVertices},
			simplexVertexIndices = ListIndices[simplexVertices];
			simplexVertexCount = Length[simplexVertices];
			simplexFacetCount = simplexVertexCount;
			orientedFacetVertexIndices = ConstantArray[Most[simplexVertexIndices],simplexFacetCount];
			orientedFacetVertexIndices[[1,{1,2}]] = orientedFacetVertexIndices[[1,{2,1}]];
			Do[
				orientedFacetVertexIndices[[index+1,index]] = simplexVertexCount;
			,{index,1,simplexVertexCount-1}];
			orientedFacetVertices = Parts[simplexVertices,orientedFacetVertexIndices];
			Return[orientedFacetVertices];
		]
	]


(* ::Subsubsection::Closed:: *)
(*SimplexOrientedQ*)


SimplexOrientedQ::usage = 
"SimplexOrientedQ[simplexVertices] returns True if sipmlex is positively oriented with respect to its embedding space.
It is assumed that the simplex spans its embedding space.";


SimplexOrientedQ[simplex_?SquareMatrixQ] := Positive[Det[VectorTranslate[Rest[simplex],N[Minus[First[simplex]]]]]]


(* ::Subsection:: *)
(*polygon*)


(* ::Subsubsection::Closed:: *)
(*PolygonVertexOrdering*)


PolygonVertexOrdering[points_, planeBasisSpec_:Automatic] := 
	Module[{vectors,planeBasis,ordering},
		If[LessEqual[Length[points],3], Return[ListIndices[points]];];
		vectors = VectorRecenter[N[points]];
		planeBasis = 
			If[SameQ[planeBasisSpec,Automatic],
				If[Equal[VectorDimension[vectors],2],
					IdentityMatrix[2],
					Orthogonalize[VectorSpanningSet[vectors,2]]
				]
				,
				Orthogonalize[N[planeBasisInput]]
			];
		ordering = Ordering[VectorArg[VectorProject[vectors,planeBasis]]];
		Return[ordering];
	];


(* ::Subsubsection::Closed:: *)
(*PolygonVertexSort*)


PolygonVertexSort::usage = 	
"PolygonVertexSort[polygon, planeBasis] returns a copy of polygon with vertices ordered by the argument as porjected to the specified plane.
PolygonVertexSort[polygon] returns a copy of polygon with vertices ordered by the argument as porjected to a plane found in the affine span of points."


PolygonVertexSort[polygon_, planeBasisSpec_:Automatic] := Part[polygon,PolygonVertexOrdering[polygon,planeBasisSpec]]


(* ::Subsubsection::Closed:: *)
(*PolygonEdges*)


PolygonEdges::usage = 
"PolygonEdges[polygon] returns the list of edges of polygon.";


PolygonEdges[polygon_] := 
	Transpose[{polygon,RotateLeft[polygon]}]


(* ::Subsubsection::Closed:: *)
(*PolygonArea*)


PolygonArea::usage = 
"PolygonArea[polygon] returns the area of the polygon.";


PolygonArea[polygonInput_] := 
	Module[
		{
			polygon,
			basis
		},
		polygon = N[polygonInput];
		polygon = VectorTranslate[polygon,Minus[First[polygon]]];
		basis = VectorSpanningSetIndices[polygon,2];
	]


(* ::Subtitle:: *)
(*vectors*)


(* ::Subsection::Closed:: *)
(*operations*)


(* ::Subsubsection::Closed:: *)
(*VectorSquaredNorm*)


VectorSquaredNorm::usage = 
	"VectorSquaredNorm[vectors] returns the Euclidean squared norms of vectors."<>"\n"<>
	"The input can be a nested list of vectors.";


VectorSquaredNorm[{}] := {}
VectorSquaredNorm[points_?ArrayQ] := Dot[Times[points,points],ConstantArray[1,{Last[Dimensions[points]]}]]
VectorSquaredNorm[points_] := Map[VectorSquaredNorm,points]


(* ::Subsubsection::Closed:: *)
(*VectorCoordinateSum*)


VectorCoordinateSum[vectors_?ArrayQ] := Dot[vectors,CoordinateDiagonalVector[VectorDimension[vectors]]]
VectorCoordinateSum[vectorLists_] := Map[VectorCoordinateSum,vectorLists]


(* ::Subsubsection::Closed:: *)
(*CoordinateDiagonalVector*)


CoordinateDiagonalVector[n_] := ConstantArray[1,n]


(* ::Subsubsection::Closed:: *)
(*VectorNorm*)


VectorNorm::usage = 
"VectorNorm[points] returns the Euclidean norm of points.";


VectorNorm[points_] := Sqrt[VectorSquaredNorm[points]]


(* ::Subsubsection::Closed:: *)
(*VectorNormalize*)


VectorNormalize::usage = 
"VectorNormalize[vectors] returns the normalized list of vectors.";


VectorNormalize[{}] := {}
VectorNormalize[vectors_] := 
	Module[{norms,signs,scalars,result},
		norms = NChop[VectorNorm[vectors]];
		signs = Sign[norms];
		scalars = Times[Divide[1,Plus[norms,Subtract[1,signs]]],signs];
		result = Times[vectors,scalars];
		Return[result];
	];	


(* ::Subsection:: *)
(*points*)


(* ::Subsubsection::Closed:: *)
(*VectorCentroid*)


VectorCentroid::usage = 
	"VectorCentroid[points] returns the centroid of a points." <>"\n"<>
	"VectorCentroid[pointSets] returns the centroid of each set of points in pointSets."; 


VectorCentroid[points_?MatrixQ] := Mean[points]
VectorCentroid[pointSets_] := Map[Mean,pointSets,{-3}]


(* ::Subsubsection::Closed:: *)
(*VectorDilate*)


VectorDilate::usage = 
	"VectorDilate[points,scale,pivot] returns points as dialated about the point pivot by a scalar sacle.";


VectorDilate[points_?MatrixQ,scale_,pivot_?VectorQ] := VectorTranslate[Times[scale,VectorTranslate[points,Minus[pivot]]],Plus[pivot]]
VectorDilate[points_?MatrixQ,scale_] := VectorDilate[points,scale,VectorCentroid[points]]


VectorDilate[scale_][points_] := VectorDilate[points,scale]
VectorDilate[scale_,pivot_?VectorQ][points_List?MatrixQ] := VectorDilate[points,scale,pivot]


(* ::Subsubsection::Closed:: *)
(*VectorBoundedQ*)


VectorBoundedQ::usage = 
	"VectorBoundedQ[points, normals, bounds] returns True if {point.normals \[LessEqual] bounds}, False otherwise.";


VectorBoundedQ[p_?ArrayQ, n_?MatrixQ, b_] := NonPositive[Dot[ConstantArray[1.,Length[n]],NPositiveBoole[Subtract[Dot[n,TransposeArray[p]],b]]]]
VectorBoundedQ[p_?ArrayQ, n_?VectorQ, b_] := NonPositive[NPositiveBoole[Subtract[Dot[n,TransposeArray[p]],b]]]
VectorBoundedQ[p_?VectorQ, n_?MatrixQ, b_] := NonPositive[Dot[ConstantArray[1.,Length[n]],NPositiveBoole[Subtract[Dot[n,p],b]]]]
VectorBoundedQ[p_?VectorQ, n_?VectorQ, b_] := NNonPositive[Subtract[Dot[p,n],b]]


(*
VectorBoundedQ[points_?MatrixQ, normals_?MatrixQ, bounds_?VectorQ] := Table[VectorLessEqual[{NChop[normals.point-bounds],0}],{point,points}]
VectorBoundedQ[point_?VectorQ, normals_?MatrixQ, bounds_?VectorQ] := VectorLessEqual[{NChop[normals.point-bounds],0}]
VectorBoundedQ[points_?MatrixQ, normal_?VectorQ, bound_?NumericQ] := Table[VectorLessEqual[{NChop[normal.point-bound],0}],{point,points}]
VectorBoundedQ[point_?VectorQ, normal_?VectorQ, bound_?NumericQ] := VectorLessEqual[{NChop[normal.point-bound],0}]
VectorBoundedQ[pointSets_List, normals_, bounds_] := Map[Function[pointSet, VectorBoundedQ[pointSet, normals, bounds]], pointSets]
*)


(* ::Subsubsection::Closed:: *)
(*VectorBoundedBoole*)


VectorBoundedBoole[p_?ArrayQ, n_?MatrixQ, b_] := NonPositiveBoole[Dot[ConstantArray[1.,Length[n]],NPositiveBoole[Subtract[Dot[n,TransposeArray[p]],b]]]]
VectorBoundedBoole[p_?ArrayQ, n_?VectorQ, b_] := NonPositiveBoole[NPositiveBoole[Subtract[Dot[n,TransposeArray[p]],b]]]
VectorBoundedBoole[p_?VectorQ, n_?MatrixQ, b_] := NonPositiveBoole[Dot[ConstantArray[1.,Length[n]],NPositiveBoole[Subtract[Dot[n,p],b]]]]
VectorBoundedBoole[p_?VectorQ, n_?VectorQ, b_] := NNonPositiveBoole[Subtract[Dot[p,n],b]]


(* ::Subsubsection::Closed:: *)
(*VectorSort*)


VectorSort::usage = 
"VectorSort[vectors] sorts the (nested) list(s) of vectors.";


VectorSort[vector_?VectorQ] := vector
VectorSort[vectors_?MatrixQ] := Sort[vectors]
VectorSort[vectors_?ArrayQ] := Map[Sort,vectors,Subtract[Most[Range[ArrayDepth[vectors]]],1]]
VectorSort[vectors_] := Map[VectorSort,vectors]


(* ::Subsubsection::Closed:: *)
(*VectorMatchQ*)


VectorMatchQ::usage = 
"VectorMatchQ[pointsA,pointsB] returns True if the points or point-sets A and B are concurrent (up to numerical tolerance).";


VectorMatchQ[pointA_?VectorQ,pointB_?VectorQ] := Equal[NRound[pointA],NRound[pointB]]
VectorMatchQ[pointsA_?MatrixQ,pointsB_?MatrixQ] := Equal[Sort[NRound[pointsA]],Sort[NRound[pointsB]]]
VectorMatchQ[pointSetsA_List,pointSetsB_List] := Equal[Sort[Map[Sort,NRound[pointSetsA]]],Sort[Map[Sort,NRound[pointSetsB]]]]


(* ::Subsubsection::Closed:: *)
(*VectorCases*)


VectorCases::usage = 
"VectorCases[points,pattern] returns the subset of points, p, for which (p+pattern) is a subset of points.";


VectorCases[pointsInput_,patternInput_] := 
	Module[{points,pointsN,pointsN\[CurlyEpsilon],pointsT,pattern,matchesN\[CurlyEpsilon],matches,vectorIndex,vector},
		points = pointsInput;
		pattern = patternInput // N;
		pointsN = N[points];
		pointsN\[CurlyEpsilon] = NRound[pointsN];
		matchesN\[CurlyEpsilon] = pointsN\[CurlyEpsilon];
		Monitor[
			Do[
				vector = pattern[[vectorIndex]];
				matchesN\[CurlyEpsilon] = Intersection[matchesN\[CurlyEpsilon],NRound[VectorTranslate[pointsN,Minus[vector]]]];
			,{vectorIndex,Length[pattern]}];
		,Column[{
			ProgressIndicator[vectorIndex,{1,Length[pattern]}],
			Length[matchesN\[CurlyEpsilon]]}]
			,1];
		matches = NIntersection[pointsInput,matchesN\[CurlyEpsilon]];
		Return[matches];
	]


(* ::Subsubsection::Closed:: *)
(*VectorShells*)


VectorShells[points_] := SortBy[Parts[points,GatherIndices[NRound[VectorNorm[N[points]]]]],Composition[Norm,N,First]]


(* ::Subsubsection::Closed:: *)
(*VectorSlicePoints*)


VectorSlicePoints::usage = 
"VectorSlicePoints[points,normal,bound] returns those points on the hyperplane defined by x.normal == bound.";


VectorSlicePoints[{},normal_,value_] := {}
VectorSlicePoints[points_?ArrayQ,normal_,value_] := Pick[points,NSign[Subtract[Dot[points,normal],value]],0]
VectorSlicePoints[points_,normal_,value_] := Map[VectorSlicePoints[#,normal,value]&,points]
VectorSlicePoints[normal_,value_][points_] := VectorSlicePoints[points,normal,value]


(* ::Subsubsection::Closed:: *)
(*VectorChopPoints*)


VectorChopPoints::usage = 
"VectorChopPoints[points,normal,bound] returns those points on the hyperplane defined by x.normal <= bound.";


VectorChopPoints[{},normal_,value_] := {}
VectorChopPoints[points_?ArrayQ,normal_,value_] := Pick[points,NonPositive[Sign[Chop[Subtract[Dot[points,normal],value]]]]]
VectorChopPoints[points_,normal_,value_] := Map[VectorChopPoints[#,normal,value]&,points]
VectorChopPoints[normal_,value_][points_] := VectorChopPoints[points,normal,value]


(* ::Subsubsection::Closed:: *)
(*VectorMaximalByNormal*)


VectorMaximalByNormal::usage = 
"VectorMaximalByNormal[points,normal] returns all the points of points which maximize the dot product with normal.";


VectorMaximalByNormal[points_,normal_] := 
	Module[{dots,bound,pointsMaximal},
		dots = Dot[points,N[normal]];
		dots = NRound[dots];
		bound = Max[dots];
		pointsMaximal = Pick[points,dots,bound];
		Return[pointsMaximal];
	]


(* ::Subsubsection::Closed:: *)
(*VectorDistance*)


VectorDistance::usage = 
"VectorDistance[a,b] returns the Euclidean distrqnace betwen points a and b.";


VectorDistance[a_,b_] := VectorNorm[VectorTranslate[a,Minus[b]]]


(* ::Subsubsection::Closed:: *)
(*VectorDistanceToLine*)


VectorDistanceToLine::usage = 
"VectorDistanceToLine[base,axis][points] returns the distance between POINT and the infinite line at base extending in the direction axis.";


VectorDistanceToLine[base_,axis_][points_] := 
	VectorNorm[
		VectorProject[
			VectorTranslate[points,Minus[base]],
			Orthogonalize[NullSpace[{axis}]]
		]
	];


(* ::Subsubsection::Closed:: *)
(*VectorChiralQ*)


Options[VectorChiralQ] = {"Verbosity" -> 0};
VectorChiralQ[vertexSetsInput_,vertexSetsChiralInput_,rotationsInput_,OptionsPattern[]] := 
	Module[{verbosity,vertexSets,vertexSetsChiral,rotations,chiralQ,indexA,indexB,matchQ,rotationA,rotationB,rotation,intersectionCounts,rotationIndex,vertexSetsChiralRotated},
		vertexSets = vertexSetsInput // NRound;
		verbosity = OptionValue["Verbosity"];
		vertexSetsChiral = vertexSetsChiralInput // N // Chop;
		rotations = rotationsInput // N // Chop;
		rotationIndex = 1;
		intersectionCounts = {};
		Monitor[
		chiralQ = True;
		Do[
			rotation = rotations[[rotationIndex]];
			vertexSetsChiralRotated = VectorRotate[vertexSetsChiral,rotation];
			intersectionCounts = Outer[Length@*Intersection,vertexSets,NRound@vertexSetsChiralRotated,1];
			matchQ = Equal[Map[Max,intersectionCounts],Map[Length,vertexSets]];
			If[matchQ,
				chiralQ = False;
				If[Greater[verbosity,0],
					Print["not chiral"];
					Print["rotation index: \n\t",rotationIndex];
					Print["rotation: \n\t" MatrixForm[rotation]];
				];
				Break[];
			];
		,{rotationIndex,Length[rotations]}
		];
		,Column[{
			Row[{ProgressIndicator[rotationIndex,{1,Length[rotations]}]," ",rotationIndex," : ",Length[rotations]}],
			intersectionCounts // MatrixPlot
			}]
		];
		Return[chiralQ];
	];


(* ::Subsubsection::Closed:: *)
(*VectorRegularSimplexQ*)


VectorRegularSimplexQ[ verticesInput_?MatrixQ ] :=
	Module[{
			vertexCount,
			edgeCount,
			vertices,
			regularSimplexQ,
			edgeLengthFunction,
			edgeLengthTarget,
			edgeIndex,
			edgeVertices,
			edgeLength
		},
		vertexCount = Length[verticesInput];
		If[Equal[vertexCount,1], Return[True];];
		vertices = N[verticesInput];
		edgeCount = Binomial[vertexCount,2];
		edgeLengthFunction = RightComposition[Differences,First,Norm];
		edgeLengthTarget = First[Subsets[vertices,{2},{1}]] // edgeLengthFunction;
		regularSimplexQ = True;
		Do[
			edgeVertices = First[Subsets[vertices,{2},{edgeIndex}]];
			edgeLength = edgeVertices // edgeLengthFunction;
			If[Not[NEqual[edgeLength,edgeLengthTarget]],
				regularSimplexQ = False;
				Break[];
			];
		,{edgeIndex,edgeCount}];
		Return[regularSimplexQ];
	]


(* ::Subsection:: *)
(*transform*)


(* ::Subsubsection::Closed:: *)
(*VectorTranslate*)


VectorTranslate::usage = 
"VectorTranslate[points,vector] returns points as translated by vector. VectorTranslate threads over lists of points as well as vectors.";


VectorTranslate[{},vector_] := {}
VectorTranslate[points_,{}] := {}
VectorTranslate[point_?VectorQ,vector_?VectorQ] := Plus[point,vector]
VectorTranslate[points_?MatrixQ,vector_?VectorQ] := Transpose[Plus[Transpose[points],vector]]
VectorTranslate[points_?ArrayQ,vector_?VectorQ] := TransposeArray[Plus[TransposeArray[points],vector]]
VectorTranslate[points_,vector_?VectorQ] := Map[VectorTranslate[#,vector]&,points]
VectorTranslate[points_,vectors_] := Map[VectorTranslate[points,#]&,vectors]
VectorTranslate[vectors_][points_] := VectorTranslate[points,vectors]


(* ::Subsubsection::Closed:: *)
(*VectorRecenter*)


VectorRecenter[points_] := VectorTranslate[points,Minus[VectorCentroid[points]]]


(* ::Subsubsection::Closed:: *)
(*VectorTransform*)


VectorTransform::usage = 
"VectorTransform[points,matrix] returns the points as transformed by matrix.";


VectorTransform[{},{}_] := {}
VectorTransform[{},matrix_] := {}
VectorTransform[points_,matrix_] := Dot[points,Transpose[matrix]]
VectorTransform[matrix_][points_] := Dot[points,Transpose[matrix]]


(* ::Subsubsection::Closed:: *)
(*VectorRotate*)


VectorRotate = VectorProject;


(* ::Subsubsection::Closed:: *)
(*VectorProject*)


VectorProject::usage = 
"VectorProject[points, basis] projects points to the subpsace defined by basis.";


VectorProject[{},space_] := {}
VectorProject[points_,{}] := Map[Function[{}],points,{-2}]
VectorProject[points_?ArrayQ,vector_?VectorQ] := Dot[points,vector];
VectorProject[points_?ArrayQ,subspace_?MatrixQ] := Dot[points,Transpose[subspace]]
VectorProject[points_,subspace_?MatrixQ] := Map[VectorProject[#,subspace]&,points]

VectorProject[points_,0] := VectorProject[points,{}]

VectorProject[subspace_][points_] := VectorProject[points,subspace]


(* ::Subsubsection::Closed:: *)
(*VectorEmbed*)


VectorEmbed::usage = 
"VectorEmbed[points,subspace] embeds points into subspace.";


VectorEmbed[points_?ArrayQ,vector_?VectorQ] := Dot[points,{vector}];
VectorEmbed[points_?ArrayQ,subspace_?MatrixQ] := Dot[points,subspace]
VectorEmbed[points_,subspace_?MatrixQ] := Map[VectorEmbed[#,subspace]&,points]
VectorEmbed[subspace_][points_] := VectorEmbed[points,subspace]


(* ::Subsubsection::Closed:: *)
(*VectorAffineTransform*)


VectorAffineTransform::usage = 
"VectorAffineTransform[points,xform,base] applies the linear transformation xform about the base point to points.";


VectorAffineTransform[points_,xform_,base_] := VectorTranslate[VectorTransform[VectorTranslate[points,Minus[base]],xform],base]


(* ::Subsection::Closed:: *)
(*plot*)


(* ::Subsubsection::Closed:: *)
(*VectorMatrixPlot*)


VectorMatrixPlot::usage = 
"VectorMatrixPlot[vectors] returns a formatted image of the vector coordinates.";


VectorMatrixPlot[vectors_?MatrixQ] := 
	MatrixPlot[
		Transpose[vectors],
		Rule[Frame,None],
		Rule[Mesh,All],
		Rule[MeshStyle,{{AbsoluteThickness[2],Hue[0,0,0,.1]}}],		
		Rule[PixelConstrained,{8,8}]
	]


(* ::Subsection::Closed:: *)
(*stereographic projection*)


(* ::Subsubsection::Closed:: *)
(*StereographicProjection*)


StereographicProjection::usage = 
"StereographicProjection[points] maps the points via stereographic projection from Sn to Rn.";


StereographicProjectionS1[{x1_,x2_}] := {x1/(1 - x2)}
StereographicProjectionS2[{x1_,x2_,x3_}] := {x1/(1 - x3), x2/(1 - x3)}
StereographicProjectionS3[{x1_,x2_,x3_,x4_}] := {x1/(1 - x4), x2/(1 - x4), x3/(1 - x4)}
StereographicProjectionS4[{x1_,x2_,x3_,x4_,x5_}] := {x1/(1 - x5), x2/(1 - x5), x3/(1 - x5), x4/(1 - x5)}
StereographicProjectionS5[{x1_,x2_,x3_,x4_,x5_,x6_}] := {x1/(1 - x6), x2/(1 - x6), x3/(1 - x6), x4/(1 - x6), x5/(1 - x6)}
StereographicProjectionS6[{x1_,x2_,x3_,x4_,x5_,x6_,x7_}] := {x1/(1 - x7), x2/(1 - x7), x3/(1 - x7), x4/(1 - x7), x5/(1 - x7), x6/(1 - x7)}
StereographicProjectionS7[{x1_,x2_,x3_,x4_,x5_,x6_,x7_,x8_}] := {x1/(1 - x8), x2/(1 - x8), x3/(1 - x8), x4/(1 - x8), x5/(1 - x8), x6/(1 - x8), x7/(1 - x8)}


StereographicProjection[point_?VectorQ] := 
	Switch[VectorDimension[point],
		2, StereographicProjectionS1[point],
		3, StereographicProjectionS2[point],
		4, StereographicProjectionS3[point],
		5, StereographicProjectionS4[point],
		6, StereographicProjectionS5[point],
		7, StereographicProjectionS6[point],
		8, StereographicProjectionS7[point]
	]


StereographicProjection[points_?ArrayQ] := 
	Switch[
		VectorDimension[points],
		2, TransposeArray[StereographicProjectionS1[TransposeArray[points]]],
		3, TransposeArray[StereographicProjectionS2[TransposeArray[points]]],
		4, TransposeArray[StereographicProjectionS3[TransposeArray[points]]],
		5, TransposeArray[StereographicProjectionS4[TransposeArray[points]]],
		6, TransposeArray[StereographicProjectionS5[TransposeArray[points]]],
		7, TransposeArray[StereographicProjectionS6[TransposeArray[points]]],
		8, TransposeArray[StereographicProjectionS7[TransposeArray[points]]]
	]


StereographicProjection[points_] := Map[StereographicProjection,points];


(* ::Subsubsection::Closed:: *)
(*StereographicProjectionInverse*)


StereographicProjectionInverse::usage = 
"StereographicProjectionInverse[points] maps the points via inverse stereographic projection from Rn to Sn.";


StereographicProjectionInverse[point_?VectorQ] := 
	Switch[VectorDimension[point],
		1, StereographicProjectionInverseS1[point],
		2, StereographicProjectionInverseS2[point],
		3, StereographicProjectionInverseS3[point],
		4, StereographicProjectionInverseS4[point],
		5, StereographicProjectionInverseS5[point],
		6, StereographicProjectionInverseS6[point],
		7, StereographicProjectionInverseS7[point]
	]

StereographicProjectionInverse[points_?ArrayQ] := 
	Switch[VectorDimension[points],
		1, TransposeArray[StereographicProjectionInverseS1[TransposeArray[points]]],
		2, TransposeArray[StereographicProjectionInverseS2[TransposeArray[points]]],
		3, TransposeArray[StereographicProjectionInverseS3[TransposeArray[points]]],
		4, TransposeArray[StereographicProjectionInverseS4[TransposeArray[points]]],
		5, TransposeArray[StereographicProjectionInverseS5[TransposeArray[points]]],
		6, TransposeArray[StereographicProjectionInverseS6[TransposeArray[points]]],
		7, TransposeArray[StereographicProjectionInverseS7[TransposeArray[points]]]
	]

StereographicProjectionInverse[points_] := Map[StereographicProjectionInverse,points]


StereographicProjectionInverseS1[{x1_}] := 
	{
		(2*x1)/(1+x1^2),
		1-2/(1+x1^2)
	}


StereographicProjectionInverseS2[{x1_,x2_}] := 
	{
		(2*x1)/(1+x1^2+x2^2),
		(2*x2)/(1+x1^2+x2^2),
		1-2/(1+x1^2+x2^2)
	};


StereographicProjectionInverseS3[{x1_,x2_,x3_}] := 
	{
		(2*x1)/(1+x1^2+x2^2+x3^2),
		(2*x2)/(1+x1^2+x2^2+x3^2),
		(2*x3)/(1+x1^2+x2^2+x3^2),
		1-2/(1+x1^2+x2^2+x3^2)
	}


StereographicProjectionInverseS4[{x1_,x2_,x3_,x4_}] := 
	{
		(2*x1)/(1+x1^2+x2^2+x3^2+x4^2),
		(2*x2)/(1+x1^2+x2^2+x3^2+x4^2),
		(2*x3)/(1+x1^2+x2^2+x3^2+x4^2),
		(2*x4)/(1+x1^2+x2^2+x3^2+x4^2),
		1-2/(1+x1^2+x2^2+x3^2+x4^2)
	};


StereographicProjectionInverseS5[{x1_,x2_,x3_,x4_,x5_}] := 
	{
		(2*x1)/(1+x1^2+x2^2+x3^2+x4^2+x5^2),
		(2*x2)/(1+x1^2+x2^2+x3^2+x4^2+x5^2),
		(2*x3)/(1+x1^2+x2^2+x3^2+x4^2+x5^2),
		(2*x4)/(1+x1^2+x2^2+x3^2+x4^2+x5^2),
		(2*x5)/(1+x1^2+x2^2+x3^2+x4^2+x5^2),
		1-2/(1+x1^2+x2^2+x3^2+x4^2+x5^2)
	};


StereographicProjectionInverseS6[{x1_,x2_,x3_,x4_,x5_,x6_}] := 
	{
		(2*x1)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		(2*x2)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		(2*x3)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		(2*x4)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		(2*x5)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		(2*x6)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2),
		1-2/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2)
	};


StereographicProjectionInverseS7[{x1_,x2_,x3_,x4_,x5_,x6_,x7_}] := 
	{
		(2*x1)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x2)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x3)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x4)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x5)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x6)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		(2*x7)/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2),
		1-2/(1+x1^2+x2^2+x3^2+x4^2+x5^2+x6^2+x7^2)
	};


(* ::Subtitle::Closed:: *)
(*end*)


End[]


EndPackage[]
