(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 13.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    424443,       8267]
NotebookOptionsPosition[    410701,       8024]
NotebookOutlinePosition[    411192,       8043]
CellTagsIndexPosition[    411149,       8040]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Gosset ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["4", "21"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "c286401b-e2c7-4dd2-87eb-9e7b55c7400f"],
 " polytope, 600-cells, compound of five 24-cells, \nand compounding of \
tetrahedral 4-Groups into a 20-Group"
}], "Title",
 CellChangeTimes->{{3.695411426288307*^9, 3.6954114764427643`*^9}, 
   3.78266308925074*^9, 3.7941581634366913`*^9, {3.794162532034059*^9, 
   3.794162534593748*^9}, {3.805036521914987*^9, 3.805036525052906*^9}, {
   3.892831788645644*^9, 3.892831811258358*^9}, {3.892832445798884*^9, 
   3.8928324485770683`*^9}, 3.892836247839758*^9, 3.8944615349834085`*^9, {
   3.9157586395145807`*^9, 3.915758701502232*^9}, {3.915831046313656*^9, 
   3.9158311315894814`*^9}, {3.9158312221871586`*^9, 
   3.9158312765635986`*^9}, {3.9162665029196625`*^9, 
   3.916266547102614*^9}},ExpressionUUID->"88b4d7db-17fc-49b5-b855-\
b70064b435c4"],

Cell[TextData[{
 "Richard Clawson\nQuantum Gravity Research\nLos Angeles, CA, USA\n\
richard@quantumgravityresearch.org\nVersion date 20220208\nDependencies : ",
 StyleBox["ConvexPolytope`",
  FontSlant->"Italic"],
 ", ",
 StyleBox["Vector`",
  FontSlant->"Italic"],
 ", ",
 StyleBox["List`",
  FontSlant->"Italic"],
 ", ",
 StyleBox["Numeric`",
  FontSlant->"Italic"],
 " (attached packages)"
}], "Text",
 CellChangeTimes->{{3.916269496165553*^9, 3.916269594366593*^9}, {
   3.916269670388831*^9, 3.9162696847787957`*^9}, {3.9165247629537954`*^9, 
   3.9165247641191435`*^9}, {3.9165259717662745`*^9, 3.916525996623336*^9}, {
   3.9165280411675797`*^9, 3.916528123985813*^9}, 
   3.9165281678668365`*^9},ExpressionUUID->"dfd051c7-62e2-429d-a929-\
1491a1b035c2"],

Cell[TextData[{
 StyleBox[" Abstract:",
  FontWeight->"Bold",
  FontSlant->"Italic"],
 StyleBox["  This notebook serves as a companion to the paper \
\[OpenCurlyDoubleQuote]From the Fibonacci Icosagrid to ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "8"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "09256b35-eefb-457b-81e6-65a45eae7b30"],
 StyleBox[" (Part II): The Composite Mapping of the Cores.\
\[CloseCurlyDoubleQuote]   In this work we begin with \
Gosset\[CloseCurlyQuote]s ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["4", "21"], TraditionalForm]],
  FontSlant->"Italic",ExpressionUUID->
  "8c99b15d-15d3-4ff9-99f6-2683643f4cdb"],
 StyleBox[" polytope, which is the root polytope of ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["E", "8"], "."}], TraditionalForm]],ExpressionUUID->
  "f2e96e6f-8111-48cc-99e0-457cc9f65699"],
 StyleBox[" We decompose this by a Hopf fibration into 10 24-cells; map those \
24-cells by the Elser-Sloane projection to two 600-cells in 4D, each 600-cell \
being a compound of 5 of the projected 24-cells; and map the five 24-cells of \
one 600-cell to the five cuboctahedra in 3D of the icosahedrally symmetric \
compound of five cuboctahedra (C5C). The cuboctahedra come from the 3D \
equators of the 24-cells, and it is shown that the 12 cuboctahedra of a \
single 24-cell all map naturally to the same member of the C5C, thus creating \
a natural correspondence between the 24-cells in the 600-cell and the \
cuboctahedra in the C5C. Each cuboctahedron has the vertices of a tetrahedral \
4-Group, and the compound of five has the vertices of a tetrahedral 20-Group, \
which is the core polytope of the Fibonacci icosagrid.\n\n",
  FontSlant->"Italic"],
 "* Clawson, R.; Fang, F.; Irwin, K. From the Fibonacci Icosagrid to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "8"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "cd87a164-cf57-4cf5-967a-d9e358c6e5ac"],
 " (Part II): The Composite Mapping of the Cores. ",
 StyleBox["Crystals",
  FontSlant->"Italic"],
 " 2024, 1, 0."
}], "Text",
 CellFrame->{{0, 0}, {0, 0.5}},
 InitializationCell->True,
 CellFrameColor->GrayLevel[0.75],
 CellChangeTimes->{{3.9165248728189163`*^9, 3.9165249020884237`*^9}, {
   3.916524945724057*^9, 3.9165254005295887`*^9}, 3.9165254590139256`*^9, {
   3.9165255058900876`*^9, 3.916525823592605*^9}, 3.91652591836176*^9, {
   3.9165279433323097`*^9, 3.916527944530461*^9}, {3.9165280007387075`*^9, 
   3.9165280011381235`*^9}, {3.916533673268057*^9, 3.9165337136881104`*^9}, {
   3.9169609992838607`*^9, 3.9169610028922963`*^9}, {3.9169610689190383`*^9, 
   3.91696130148118*^9}},ExpressionUUID->"f06b0642-2092-4b5c-a375-\
a290bef69f1d"],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 CellChangeTimes->{{3.91652473791576*^9, 
  3.916524739355754*^9}},ExpressionUUID->"c212aa7d-65ac-4002-9761-\
9922237546c3"],

Cell[TextData[{
 "The Gosset ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["4", "21"], TraditionalForm]],ExpressionUUID->
  "2e7e53fb-2cf2-4176-ad73-01197b52707b"],
 " polytope in 8D projects by the Elser-Sloane projection to a pair of \
concentric 600-cells in 4D. The vertices of a 600-cell can be seen as the \
vertices of a compound of 5 24-cells. Indeed, one can generate all the \
vertices of the 600-cell by starting with one 24-cell, making 4 more copies \
of it, and applying  a 2",
 StyleBox["k",
  FontSlant->"Italic"],
 "\[Pi]/5 isoclinic rotation to the ",
 StyleBox["k",
  FontSlant->"Italic"],
 "-th one. These isoclinic rotations can be either left or right.\n\nEach \
24-cell has 12 distinct 3D cuboctahedral equators. We arbitrarily choose a \
single one of these cuboctahedra from a single 24-cell and call it the \
primary. Its 3D space we call ",
 StyleBox["V",
  FontSlant->"Italic"],
 ". Within ",
 StyleBox["V",
  FontSlant->"Italic"],
 ", we can apply to this cuboctahedron the standard procedure for making the \
icosahedrally symmetric compound of 5 cuboctahedra. It turns out that there \
is a natural unique pairwise correspondence between the 5 24-cells and the 5 \
cuboctahedra. For if one takes one of the cuboctahedra in 24-cell ",
 StyleBox["k",
  FontSlant->"Italic"],
 " and isoclinically rotates it to ",
 StyleBox["V",
  FontSlant->"Italic"],
 ", using the opposite-handed rotation as was used to construct the 24-cell \
5-compound, then it coincides with one specific member of the cuboctahedron \
5-compound -- and this is independent of the choice of cuboctahedron one \
makes in 24-cell ",
 StyleBox["k",
  FontSlant->"Italic"],
 "!\n\nIf one rotates any of the cuboctahedra in ",
 StyleBox["k",
  FontSlant->"Italic"],
 " to ",
 StyleBox["V",
  FontSlant->"Italic"],
 " using the same handed rotation as was used to make the 24-cell 5-compound, \
then it rotates it back to the primary cuboctahedron, regardless of which \
cuboctahedron one started with. If one uses, instead, a simple direct \
rotation, from the 3D space of the chosen cuboctahedron to ",
 StyleBox["V",
  FontSlant->"Italic"],
 ", then at lands in ",
 StyleBox["V",
  FontSlant->"Italic"],
 " with an incoherent, non-symmetric relationship with the primary \
cuboctahedron; this orientation varies widely based on which cuboctahedron \
was chosen in ",
 StyleBox["k",
  FontSlant->"Italic"],
 ". But if one uses the opposite isoclinic rotation, then it lands exactly in \
the position of one member of the cuboctahedron 5-compound. \n\nThe \
cuboctahedron can be made into a 4-Group (4G) of tetrahedra by selecting a \
disjoint set of triangular faces and and connecting their vertices to the \
centroid, thus forming regular tetrahedra. When these are compounded in the \
same icosahedrally symmetric manner, they form the 20-Group, having chiral \
icosahedral symmetry. On each cuboctahedron, there are two ways to choose a \
disjoint set of triangular faces, and a certain care must be taken in this \
choice to make a consistent selection that yields the chirally symmetric \
object. \n\nThis notebook generates the Gosset polytope and the Elser-Sloane \
projection to two 600-cells. It then decomposes one of them into a compound \
of 5 25-cells, identifies all their cuboctahedral equators, and constructs \
their rotations to the space ",
 StyleBox["V",
  FontSlant->"Italic"],
 " of a chosen primary cuboctahedron. The graphic at the end shows the \
results of these rotated into ",
 StyleBox["V",
  FontSlant->"Italic"],
 ", one cuboctahedron from each 24-cell. You can choose which of the 12 \
cuboctahedra to view from each 24-cell, and which of the 5 to view. You can \
slide the rotations from left- to right-isoclinic to see how at one extreme \
all cuboctahedra coincide, at the other extreme they all combine in the \
icosahedrally symmetric compound, and in the middle (simple direct rotation) \
they form an incoherent jumble. Yu can also select whether to view the \
cuboctahedra or the 4-Groups. \n"
}], "Text",
 CellChangeTimes->{{3.9162664934197445`*^9, 3.9162664972963076`*^9}, {
   3.9162665644079833`*^9, 3.916266766649367*^9}, {3.9162668031123085`*^9, 
   3.916266949499398*^9}, {3.916267004273612*^9, 3.916267621912304*^9}, {
   3.916267671322302*^9, 3.9162685799033184`*^9}, {3.91626865284723*^9, 
   3.916268885113546*^9}, {3.9162689176640635`*^9, 3.9162689356749725`*^9}, {
   3.9162697270510373`*^9, 3.916269729041236*^9}, 3.9165246325932655`*^9, {
   3.916524677587309*^9, 
   3.916524679054041*^9}},ExpressionUUID->"d0ffc935-ae7b-48b0-8757-\
0c2d8dca3b1b"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Init", "Chapter",
 CellChangeTimes->{{3.8878927236734114`*^9, 3.8878927239744453`*^9}, {
  3.88806761727197*^9, 3.888067624859198*^9}, {3.89040466447641*^9, 
  3.890404664876559*^9}, {3.890493209936246*^9, 3.890493210655165*^9}, {
  3.8937680297500114`*^9, 3.8937680300706034`*^9}, {3.898261655362233*^9, 
  3.898261661634091*^9}},ExpressionUUID->"e373e991-a9a6-4a1b-8868-\
f838cb9c4c44"],

Cell[CellGroupData[{

Cell["Packages and constants", "Section",
 CellChangeTimes->{{3.88833808737787*^9, 3.888338092404277*^9}, 
   3.899662132545719*^9},ExpressionUUID->"f52d77d8-d75d-40fd-9812-\
eb54a92c9a08"],

Cell[BoxData[{
 RowBox[{"Needs", "[", "\"\<ConvexPolytope`\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Needs", "[", "\"\<Vector`\>\"", "]"}]}], "Input",
 CellChangeTimes->{{3.887822025988402*^9, 3.887822036890785*^9}, {
   3.887893746307029*^9, 3.887893751921861*^9}, {3.8879727822785816`*^9, 
   3.88797283153327*^9}, {3.8879761282906733`*^9, 3.8879761465477023`*^9}, {
   3.8880664012432737`*^9, 3.888066407628313*^9}, {3.8903257594983573`*^9, 
   3.890325771291005*^9}, {3.8903258588595743`*^9, 3.8903258623584347`*^9}, {
   3.890325938859583*^9, 3.890325944934308*^9}, {3.8934177892388754`*^9, 
   3.893417801655426*^9}, 3.8994668556618385`*^9, 3.8994806121648717`*^9, 
   3.8995551037365446`*^9, 3.8996620691358995`*^9},
 CellLabel->"In[2]:=",ExpressionUUID->"e390329a-fd8c-4ec5-b350-ef4f3c2821c6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ctr3", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ctr4", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CurlyPhi]", "=", "GoldenRatio"}], ";"}]}], "Input",
 CellChangeTimes->{{3.887844932875083*^9, 3.8878449484251776`*^9}, {
   3.887902856668981*^9, 3.887902863283017*^9}, {3.8880771108299384`*^9, 
   3.8880771111505766`*^9}, {3.8906148846804457`*^9, 3.890614885280836*^9}, {
   3.8934178810354967`*^9, 3.8934179101516757`*^9}, {3.9157473358456326`*^9, 
   3.915747388244343*^9}, 3.9157474195202227`*^9, 3.9157475165042*^9},
 CellLabel->"In[4]:=",ExpressionUUID->"9e241e0b-ac14-4a1f-9f00-450f79a71112"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions", "Section",
 CellChangeTimes->{{3.889108765810895*^9, 3.8891087681786947`*^9}, {
   3.899204456473049*^9, 3.899204464505418*^9}, 
   3.8992962567509704`*^9},ExpressionUUID->"bc872a82-b50d-49a7-98ae-\
61c7ce0538ab"],

Cell[CellGroupData[{

Cell["\<\
Find the equatorial point set on a polytope with respect to some pole.\
\>", "Subsubsection",
 CellChangeTimes->{{3.8892191971678677`*^9, 3.8892192134317956`*^9}, {
   3.8892192449867706`*^9, 3.8892192594143248`*^9}, {3.8892728201116886`*^9, 
   3.8892728345271773`*^9}, {3.8892728663516455`*^9, 
   3.8892728980792556`*^9}, {3.889272952942688*^9, 3.88927295473518*^9}, {
   3.8902243333906856`*^9, 3.890224412593422*^9}, {3.8913388891808553`*^9, 
   3.8913389539958057`*^9}, {3.9151981734594145`*^9, 
   3.9151981862450757`*^9}, {3.915198323629943*^9, 3.9151983643574696`*^9}, {
   3.915198421542796*^9, 3.9151985281402717`*^9}, {3.9151988705432367`*^9, 
   3.9151989463450766`*^9}, {3.915664955980112*^9, 3.915665017823809*^9}, {
   3.915754423487322*^9, 3.9157545906666255`*^9}, {3.9157546235404963`*^9, 
   3.915754710589784*^9}, {3.915754754051405*^9, 3.91575488682337*^9}, {
   3.915755020291705*^9, 3.915755053060788*^9}, 
   3.915755088769548*^9},ExpressionUUID->"7979290d-83a9-4fed-871a-\
7055b0b6c3a3"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "equatorFxn", "]"}], ";"}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"equatorFxn", "::", "Usage"}], " ", "=", " ", 
    "\"\<\!\(\*StyleBox[\"Find\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"vertices\",FontColor->GrayLevel[\
0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"of\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"a\",FontColor->GrayLevel[0]]\)\!\
\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"polytope\",FontColor->GrayLevel[\
0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"that\",FontColor->GrayLevel[0]]\
\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"lie\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"on\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"the\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"equator\",FontColor->GrayLevel[\
0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"of\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"a\",FontColor->GrayLevel[0]]\)\!\
\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"given\",FontColor->GrayLevel[0]]\
\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"pole\",FontColor->GrayLevel[0]]\
\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"(\",FontColor->GrayLevel[0]]\)\!\
\(\*StyleBox[\"hyperplane\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"through\",FontColor->GrayLevel[\
0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"origin\",FontColor->GrayLevel[0]\
]\)\!\(\*StyleBox[\",\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"normal\",FontColor->GrayLevel[0]\
]\)\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"to\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"the\",FontColor->GrayLevel[0]]\)\
\!\(\*StyleBox[\" \
\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\"pole\",FontColor->GrayLevel[0]]\
\)\!\(\*StyleBox[\")\",FontColor->GrayLevel[0]]\)\!\(\*StyleBox[\".\",\
FontColor->GrayLevel[0]]\)\>\""}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"--", " ", 
     RowBox[{"Input", ":", " ", 
      RowBox[{
      "A", " ", "set", " ", "of", " ", "polytope", " ", "vertices", " ", 
       "and", " ", "a", " ", "vector", " ", 
       RowBox[{"(", 
        RowBox[{"the", " ", "pole"}], ")"}]}]}]}], ",", " ", 
    RowBox[{"in", " ", "any", " ", "number", " ", "of", " ", 
     RowBox[{"dimensions", ".", " ", 
      RowBox[{"--", 
       RowBox[{"--", 
        RowBox[{"--", "-"}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"--", " ", 
    RowBox[{"Output", ":", " ", 
     RowBox[{
     "The", " ", "set", " ", "of", " ", "vertices", " ", "from", " ", "the", 
      " ", "given", " ", "polytope", " ", "that", " ", "are", " ", "in", " ", 
      "the", " ", "hyperplane", " ", "orthogonal", " ", "to", " ", "the", " ",
       "given", " ", 
      RowBox[{"pole", ".", " ", 
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", "-"}]}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"equatorFxn", "[", 
    RowBox[{"verts_", ",", "pole_"}], "]"}], " ", ":=", " ", 
   RowBox[{"Select", "[", 
    RowBox[{"verts", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{"#", ".", "pole"}], "]"}], "<", 
       RowBox[{"10", "^", 
        RowBox[{"-", "6"}]}]}], "&"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.889219988235572*^9, 3.889219993707673*^9}, {
   3.8902543598502226`*^9, 3.890254360167053*^9}, 3.8902543980890675`*^9, {
   3.8907461039687195`*^9, 3.890746301540927*^9}, 3.890746581902849*^9, {
   3.890747250367752*^9, 3.8907472538238106`*^9}, {3.8907604640206537`*^9, 
   3.8907604713742743`*^9}, {3.8913588315608377`*^9, 
   3.8913588499977736`*^9}, {3.8934177678605456`*^9, 3.893417770153564*^9}},
 CellLabel->"In[7]:=",ExpressionUUID->"6d9e5cca-721c-47f4-8283-c628706f3580"]
}, Closed]],

Cell[CellGroupData[{

Cell["Find the dual of a simple rotation R in 4D", "Subsubsection",
 CellChangeTimes->{{3.8892191971678677`*^9, 3.8892192134317956`*^9}, {
   3.8892192449867706`*^9, 3.8892192594143248`*^9}, {3.8892728201116886`*^9, 
   3.8892728345271773`*^9}, {3.8892728663516455`*^9, 
   3.8892728980792556`*^9}, {3.889272952942688*^9, 3.88927295473518*^9}, {
   3.8902243333906856`*^9, 3.890224412593422*^9}, {3.8913388891808553`*^9, 
   3.8913389539958057`*^9}, {3.9151981734594145`*^9, 
   3.9151981862450757`*^9}, {3.915198323629943*^9, 3.9151983643574696`*^9}, {
   3.915198421542796*^9, 3.9151985281402717`*^9}, {3.9151988705432367`*^9, 
   3.9151989463450766`*^9}, {3.915664955980112*^9, 3.915665017823809*^9}, {
   3.915754423487322*^9, 3.9157545906666255`*^9}, {3.9157546235404963`*^9, 
   3.915754710589784*^9}, {3.915754754051405*^9, 3.91575488682337*^9}, {
   3.9157551419285545`*^9, 3.9157551498471217`*^9}, 3.9157552419103312`*^9, {
   3.9162697108810406`*^9, 
   3.916269712822464*^9}},ExpressionUUID->"fe30af3d-0573-4a1b-ba3e-\
3ad4c1686918"],

Cell[TextData[{
 "We find the eigenplane of R, then take its dual plane, and construct a \
rotation R\[CloseCurlyQuote] by the same angle in that dual plane. \nFor the \
direction of R\[CloseCurlyQuote], let v1 be in the initial eigneplane, and v2 \
in the dual plane. Then choose R\[CloseCurlyQuote] so that {v1, R.v1, v2, R\
\[CloseCurlyQuote].v2} is a matrix with Det = +1.\nIf the rotation is \
compound, we find both its eigenplanes, and exchange their rotation angles.\n",
 StyleBox["Left",
  FontWeight->"Bold"],
 " isoclinic:\nThe compound rotation RR\[CloseCurlyQuote] is equivalent to \
quaternion rotation by ",
 "left",
 " multiplication, if quaternions {1,",
 StyleBox["i",
  FontWeight->"Bold"],
 ",",
 StyleBox["j",
  FontWeight->"Bold"],
 ",",
 StyleBox["k",
  FontWeight->"Bold"],
 "] correspond to the 4x4 identity matrix.\n",
 StyleBox["Right",
  FontWeight->"Bold"],
 " isoclinic:\nTo get the opposite dual rotation, set the optional a",
 "ngle_Fraction",
 " input to  -1."
}], "Text",
 CellChangeTimes->{{3.8892191971678677`*^9, 3.8892192134317956`*^9}, {
  3.8892192449867706`*^9, 3.8892192594143248`*^9}, {3.8892728201116886`*^9, 
  3.8892728345271773`*^9}, {3.8892728663516455`*^9, 3.8892728980792556`*^9}, {
  3.889272952942688*^9, 3.88927295473518*^9}, {3.8902243333906856`*^9, 
  3.890224412593422*^9}, {3.8913388891808553`*^9, 3.8913389539958057`*^9}, {
  3.9151981734594145`*^9, 3.9151981862450757`*^9}, {3.915198323629943*^9, 
  3.9151983643574696`*^9}, {3.915198421542796*^9, 3.9151985281402717`*^9}, {
  3.9151988705432367`*^9, 3.9151989463450766`*^9}, {3.915664955980112*^9, 
  3.915665017823809*^9}, {3.915754423487322*^9, 3.9157545906666255`*^9}, {
  3.9157546235404963`*^9, 3.915754710589784*^9}, {3.915754754051405*^9, 
  3.91575488682337*^9}, {3.9157551419285545`*^9, 
  3.9157551498471217`*^9}},ExpressionUUID->"49460a08-eaac-410c-bc26-\
0326715b00f8"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ClearAll", "[", "rotMatrixToDual4D", "]"}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"--", " ", 
    RowBox[{"Input", ":", " ", 
     RowBox[{"A", " ", "4", "D", " ", "rotation", " ", "matrix", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"simple", " ", "or", " ", "compound", " ", "rotation"}], 
        ")"}], ".", "\[IndentingNewLine]", "Option"}]}], ":", " ", 
     RowBox[{"Angle", " ", 
      RowBox[{"fraction", ".", " ", 
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", "-"}]}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"--", " ", 
     RowBox[{"Output", ":", " ", 
      RowBox[{
      "4", "D", " ", "rotation", " ", "matrix", " ", "of", " ", "the", " ", 
       "dual", " ", 
       RowBox[{"rotation", ".", "\[IndentingNewLine]", "If"}], " ", "the", 
       " ", "given", " ", "rotation", " ", "is", " ", "simple"}]}]}], ",", 
    " ", 
    RowBox[{
    "the", " ", "dual", " ", "is", " ", "a", " ", "rotation", " ", "by", " ", 
     "the", " ", "same", " ", "angle", " ", "in", " ", "the", " ", "dual", 
     " ", 
     RowBox[{"plane", ".", "\[IndentingNewLine]", "If"}], " ", "the", " ", 
     "given", " ", "rotation", " ", "is", " ", "compound"}], ",", " ", 
    RowBox[{
    "the", " ", "dual", " ", "is", " ", "a", " ", "rotation", " ", "in", " ", 
     "the", " ", "same", " ", "two", " ", "planes"}], ",", " ", 
    RowBox[{"but", " ", "with", " ", "the", " ", "angles", " ", 
     RowBox[{"switched", ".", " ", "\[IndentingNewLine]", 
      RowBox[{"Option", ":", " ", 
       RowBox[{"The", " ", "rotation", " ", "angle", 
        RowBox[{"(", "s", ")"}], " ", "is", " ", "scaled", " ", "by", " ", 
        "the", " ", "given", " ", "angle", " ", 
        RowBox[{"fraction", ".", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", "-"}]}]}]}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rotMatrixToDual4D", "::", "NotRotation"}], " ", "=", " ", 
   "\"\<Input matrix is not a 4D rotation.\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rotMatrixToDual4D", "[", 
   RowBox[{"rotationMatrix_", ",", 
    RowBox[{"angleFraction_", ":", "1"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "evals", ",", "evecs", ",", "firstCmplxIndex", ",", "angles", ",", 
      "planeVecs", ",", "dualMatrix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Dimensions", "[", "rotationMatrix", "]"}], "!=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "4"}], "}"}]}], " ", "||", " ", 
        RowBox[{"!", 
         RowBox[{"OrthogonalMatrixQ", "[", "rotationMatrix", "]"}]}], " ", "||",
         " ", 
        RowBox[{
         RowBox[{"Sign", "@", 
          RowBox[{"Det", "[", "rotationMatrix", "]"}]}], "<", "0"}]}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"Message", "[", 
         RowBox[{"rotMatrixToDual4D", "::", "NotRotation"}], "]"}], "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"evals", ",", "evecs"}], "}"}], " ", "=", "  ", 
      RowBox[{
       RowBox[{"Eigensystem", "[", "rotationMatrix", "]"}], "//", "Chop"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"firstCmplxIndex", " ", "=", " ", 
      RowBox[{
       RowBox[{"Position", "[", 
        RowBox[{"evals", " ", ",", "  ", 
         RowBox[{"_", "?", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"Im", "[", "#", "]"}], "!=", "0"}], "&"}], " ", 
           ")"}]}]}], "]"}], "//", "Flatten"}]}], " ", ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "If", " ", "the", " ", "given", " ", "matrix", " ", "is", " ", 
        "orthogonal", " ", "with", " ", "positive", " ", "Det", " ", 
        RowBox[{"(", 
         RowBox[{"checked", " ", "above"}], ")"}], " ", "but", " ", "no", " ",
         "complex", " ", "eigenvalues", " ", 
        RowBox[{"(", 
         RowBox[{"checking", " ", "next"}], ")"}]}], ",", " ", 
       "\[IndentingNewLine]", 
       RowBox[{"it", " ", "must", " ", "be", " ", "the", " ", "identity"}], 
       ",", " ", 
       RowBox[{
        RowBox[{
        "so", " ", "it", " ", "is", " ", "its", " ", "own", " ", "dual"}], 
        ";", " ", 
        RowBox[{"Return", " ", "the", " ", "original", " ", 
         RowBox[{"matrix", ".", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
            "Or", " ", "it", " ", "could", " ", "be", " ", "a", " ", "\[Pi]", 
             " ", "rotation"}], ";", " ", 
            RowBox[{
             RowBox[{"haven", "'"}], "t", " ", "accounted", " ", "for", " ", 
             "that", " ", 
             RowBox[{"yet", "."}]}]}], ")"}]}], "\[IndentingNewLine]", "If", 
         " ", "it", " ", "does", " ", "have", " ", "complex", " ", 
         "eigenvalues"}]}], ",", " ", 
       RowBox[{
       "identify", " ", "the", " ", "first", " ", "complex", " ", 
        "eigenvector"}], ",", " ", 
       RowBox[{
       "and", " ", "shift", " ", "the", " ", "lists", " ", "so", " ", "that", 
        " ", "appears", " ", 
        RowBox[{"first", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"firstCmplxIndex", "==", 
        RowBox[{"{", "}"}]}], ",", " ", 
       RowBox[{"Return", "[", "rotationMatrix", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"firstCmplxIndex", " ", "=", " ", 
        RowBox[{"First", "@", "firstCmplxIndex"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"evals", ",", "evecs"}], "}"}], " ", "=", " ", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"RotateLeft", "[", 
          RowBox[{"#", ",", 
           RowBox[{"firstCmplxIndex", "-", "1"}]}], "]"}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{"evals", ",", "evecs"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"angles", " ", "=", " ", 
      RowBox[{"Arg", "/@", "evals"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"If", " ", "it", " ", "is", " ", "a", " ", "rotation"}], ",", 
       " ", 
       RowBox[{
       "the", " ", "eigenvector", " ", "will", " ", "be", " ", "a", " ", 
        "null", " ", "complex", " ", "vector"}], ",", " ", 
       RowBox[{
       "and", " ", "the", " ", "next", " ", "vector", " ", "should", " ", 
        "be", " ", "its", " ", 
        RowBox[{"conjugate", ".", "\[IndentingNewLine]", "Then"}], " ", "the",
         " ", "real", " ", "and", " ", "imaginary", " ", "parts", " ", "are", 
        " ", "actually", " ", "two", " ", "vectors", " ", "in", " ", "the", 
        " ", 
        RowBox[{"eigenplane", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Chop", "[", " ", 
          RowBox[{
           RowBox[{"evecs", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ".", 
           RowBox[{"evecs", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], " ", "]"}], "==", "0"}], " ", "&&",
         " ", 
        RowBox[{
         RowBox[{"Round", "[", 
          RowBox[{
           RowBox[{"evecs", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"10.", "^", 
            RowBox[{"-", "10"}]}]}], "]"}], " ", "==", " ", 
         RowBox[{"Conjugate", "@", 
          RowBox[{"Round", "[", 
           RowBox[{
            RowBox[{"evecs", "[", 
             RowBox[{"[", "2", "]"}], "]"}], ",", 
            RowBox[{"10.", "^", 
             RowBox[{"-", "10"}]}]}], "]"}]}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"planeVecs", " ", "=", " ", 
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "*", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Im", "[", 
            RowBox[{"evecs", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", 
           RowBox[{"Re", "[", 
            RowBox[{"evecs", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{
        "Print", "[", 
         "\"\<Hmm...First complex EVec is not null, or not followed by its \
conjugate.\>\"", "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Chop", "[", " ", 
         RowBox[{
          RowBox[{"evecs", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ".", 
          RowBox[{"evecs", "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], " ", "]"}], "==", "0"}], " ", 
       ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "If", " ", "rotation", " ", "occurs", " ", "in", " ", "the", " ", 
          "2", "nd", " ", "plane", " ", "as", " ", "well"}], ",", " ", 
         RowBox[{
         "create", " ", "2", "nd", " ", "plane", " ", "basis", " ", "from", 
          " ", "complex", " ", "Evecs"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"planeVecs", " ", "=", " ", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"planeVecs", " ", ",", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "*", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Im", "[", 
               RowBox[{"evecs", "[", 
                RowBox[{"[", "3", "]"}], "]"}], "]"}], ",", 
              RowBox[{"Re", "[", 
               RowBox[{"evecs", "[", 
                RowBox[{"[", "3", "]"}], "]"}], "]"}]}], "}"}]}]}], " ", 
          "]"}]}], ";", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
          "If", " ", "the", " ", "rotation", " ", "is", " ", "isoclinic"}], 
          ",", " ", 
          RowBox[{"eigenplanes", " ", "are", " ", "not", " ", "unique"}], ",",
           " ", 
          RowBox[{
          "hence", " ", "eigenvectors", " ", "found", " ", "by", " ", 
           "Mathematica", " ", "may", " ", "not", " ", "be", " ", 
           "orthogonal"}], ",", " ", 
          RowBox[{"so", " ", "we", " ", 
           RowBox[{"orthogonalize", "."}]}]}], " ", "*)"}], 
        "\[IndentingNewLine]", "\t", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{
             RowBox[{"angles", "[", 
              RowBox[{"[", "1", "]"}], "]"}], " ", "-", 
             RowBox[{"angles", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "]"}], "<", 
           RowBox[{"10", "^", 
            RowBox[{"-", "6"}]}]}], ",", 
          RowBox[{"planeVecs", " ", "=", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Orthogonalize", "[", "planeVecs", "]"}], ")"}], "//", 
            "Chop"}]}]}], " ", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "If", " ", "the", " ", "rotation", " ", "is", " ", "simple"}], ",", 
         " ", 
         RowBox[{
          RowBox[{"Eigensystem", "[", "]"}], " ", "nonetheless", " ", 
          "sometimes", " ", "returns", " ", "cmplx", " ", "evecs", " ", "for",
           " ", "the", " ", "dual", " ", "plane"}], ",", " ", 
         RowBox[{
          RowBox[{
           RowBox[{
           "so", " ", "assure", " ", "they", " ", "are", " ", "real"}], " ", 
           "&"}], " ", 
          RowBox[{"orthogonal", ".", "\[IndentingNewLine]", "Also"}]}], ",", 
         " ", 
         RowBox[{
         "the", " ", "dual", " ", "plane", " ", "has", " ", "a", " ", "sign", 
          " ", 
          RowBox[{"ambiguity", ".", " ", "Resolve"}], " ", "this", " ", "by", 
          " ", "requiring", " ", "positive", " ", "Det", " ", "for", " ", 
          "the", " ", "set", " ", "of", " ", "4", " ", "basis", " ", 
          "vectors", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"the", " ", "two", " ", "planes"}], ")"}], "."}]}]}], 
        "  ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"Element", "[", 
            RowBox[{
             RowBox[{"evecs", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], ",", "Reals"}], 
            "]"}]}], ",", 
          RowBox[{
           RowBox[{"evecs", "[", 
            RowBox[{"[", 
             RowBox[{"{", 
              RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], " ", "=", " ", 
           RowBox[{
            RowBox[{"Re", "[", 
             RowBox[{"evecs", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], "]"}], " ", "+",
             " ", 
            RowBox[{"Im", "[", 
             RowBox[{"evecs", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], "]"}]}]}]}], 
         " ", "]"}], ";", "\[IndentingNewLine]", "\t", 
        RowBox[{"planeVecs", " ", "=", " ", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"planeVecs", " ", ",", " ", 
           RowBox[{
            RowBox[{"Orthogonalize", "[", 
             RowBox[{"evecs", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], "]"}], " ", "//",
             "Chop"}]}], "]"}]}], ";", "\[IndentingNewLine]", "\t", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Det", "[", "planeVecs", "]"}], "<", "0"}], ",", 
          RowBox[{
           RowBox[{"planeVecs", "[", 
            RowBox[{"[", 
             RowBox[{"{", 
              RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], "=", 
           RowBox[{"planeVecs", "[", 
            RowBox[{"[", 
             RowBox[{"{", 
              RowBox[{"4", ",", "3"}], "}"}], "]"}], "]"}]}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"dualMatrix", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"RotationMatrix", "[", 
         RowBox[{
          RowBox[{"angleFraction", "*", 
           RowBox[{"angles", "[", 
            RowBox[{"[", "3", "]"}], "]"}]}], ",", 
          RowBox[{"planeVecs", "[", 
           RowBox[{"[", 
            RowBox[{"{", 
             RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}]}], "]"}], ".", 
        RowBox[{"RotationMatrix", "[", 
         RowBox[{
          RowBox[{"angleFraction", "*", 
           RowBox[{"angles", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ",", 
          RowBox[{"planeVecs", "[", 
           RowBox[{"[", 
            RowBox[{"{", 
             RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}]}], "]"}]}], " ", "//",
        " ", "Chop"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "dualMatrix", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.889216995179845*^9, 3.8892170080150833`*^9}, {
   3.8892170565696397`*^9, 3.8892171312659225`*^9}, {3.8892173452401724`*^9, 
   3.889217351673221*^9}, {3.889222372480855*^9, 3.889222381602784*^9}, 
   3.8892224457516727`*^9, {3.8892239739190397`*^9, 3.8892240577462263`*^9}, {
   3.889224501104953*^9, 3.8892245124416666`*^9}, {3.889224689516943*^9, 
   3.8892247783158464`*^9}, {3.889273001679553*^9, 3.88927301920842*^9}, {
   3.8892949182861776`*^9, 3.8892950665920224`*^9}, {3.8893038869731627`*^9, 
   3.8893039017105837`*^9}, {3.8893043898147078`*^9, 3.889304424642844*^9}, 
   3.8893045335922594`*^9, {3.889304580446627*^9, 3.889304657848653*^9}, {
   3.889397031645237*^9, 3.889397039294314*^9}, 3.8893970770352955`*^9, {
   3.889546026731017*^9, 3.889546076204792*^9}, {3.8895461066129627`*^9, 
   3.8895462121929197`*^9}, {3.889546372014063*^9, 3.8895463846854095`*^9}, {
   3.889546705085156*^9, 3.889546714913913*^9}, 3.8898872077340155`*^9, 
   3.8898872654946375`*^9, {3.8899520578034353`*^9, 3.889952106986191*^9}, 
   3.8899521488281655`*^9, {3.8899524448836317`*^9, 3.8899524691449966`*^9}, 
   3.8901499694775887`*^9, {3.8901500605800447`*^9, 3.8901501028685827`*^9}, {
   3.890150138499775*^9, 3.890150146011624*^9}, {3.8901502012989016`*^9, 
   3.89015020198771*^9}, {3.8901502336671767`*^9, 3.8901502846442623`*^9}, {
   3.890150535762725*^9, 3.8901505383546*^9}, 3.8901570980496807`*^9, 
   3.8901582862501802`*^9, {3.890158339674535*^9, 3.8901584004089613`*^9}, 
   3.890159320695029*^9, {3.8901597336200104`*^9, 3.8901597752992*^9}, {
   3.8901598432191205`*^9, 3.8901598644187694`*^9}, 3.8901604659666653`*^9, {
   3.8901629588538694`*^9, 3.8901629690136533`*^9}, {3.8902243091446424`*^9, 
   3.8902243098088074`*^9}, {3.8902244607948885`*^9, 3.890224533153144*^9}, {
   3.890224581302994*^9, 3.890224613521625*^9}, {3.890225578894167*^9, 
   3.89022564862352*^9}, {3.890225685345379*^9, 3.8902257157896075`*^9}, {
   3.890225756157399*^9, 3.890225867344361*^9}, {3.890226055720599*^9, 
   3.890226068830533*^9}, {3.8902543487975917`*^9, 3.8902543637699203`*^9}, 
   3.890254405386156*^9, {3.8903261684597764`*^9, 3.890326216987403*^9}, {
   3.8907463087250166`*^9, 3.890746371873395*^9}, {3.890746432884392*^9, 
   3.890746555313586*^9}, 3.890747256702915*^9, {3.8907600052852187`*^9, 
   3.8907600559871807`*^9}, {3.8907601181620045`*^9, 
   3.8907601266932526`*^9}, {3.8907622074678955`*^9, 3.890762213198827*^9}, {
   3.890958894787957*^9, 3.8909588962204485`*^9}, {3.8909589290961533`*^9, 
   3.890958958371143*^9}, {3.8933583139154773`*^9, 3.8933583544076443`*^9}, {
   3.8933583849706945`*^9, 3.8933584291980505`*^9}, {3.893358462861721*^9, 
   3.8933585018236074`*^9}, {3.8934177008722467`*^9, 
   3.8934177046450014`*^9}, {3.895332399668356*^9, 3.8953324027717066`*^9}, 
   3.8987944706600757`*^9, {3.898794515973215*^9, 3.8987945223136435`*^9}, {
   3.915665066811531*^9, 3.915665067323341*^9}, {3.915665127258222*^9, 
   3.9156652251182237`*^9}, {3.91566531525238*^9, 3.9156653242334685`*^9}},
 CellLabel->"In[10]:=",ExpressionUUID->"a84d1679-450b-468a-a8d1-f0a8245c4bd3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Making a 4G from a cuboctahedron.", "Subsubsection",
 CellChangeTimes->{{3.899295784777296*^9, 3.899295994734908*^9}, {
   3.899296031312354*^9, 3.8992960371332164`*^9}, {3.915475129215588*^9, 
   3.915475138627266*^9}, {3.9154752509693356`*^9, 3.9154753129208307`*^9}, {
   3.915475354086731*^9, 3.9154755052856717`*^9}, {3.9154841381161156`*^9, 
   3.915484166771947*^9}, {3.9156655749962044`*^9, 3.9156655762739687`*^9}, 
   3.9156656070445366`*^9, {3.915668992412733*^9, 3.915669014212542*^9}, 
   3.915755273105947*^9},ExpressionUUID->"067ea792-2558-4a78-ab09-\
cf262c60d79d"],

Cell["\<\
Given a cuboctahedron in 4D, its 3-space normal, and a compounding axis, \
choose the 4G that is left-handed with respect to that compounding. 
(If no axis is specified, it just picks a 4G.)\
\>", "Text",
 CellChangeTimes->{{3.899295784777296*^9, 3.899295994734908*^9}, {
   3.899296031312354*^9, 3.8992960371332164`*^9}, {3.915475129215588*^9, 
   3.915475138627266*^9}, {3.9154752509693356`*^9, 3.9154753129208307`*^9}, {
   3.915475354086731*^9, 3.9154755052856717`*^9}, {3.9154841381161156`*^9, 
   3.915484166771947*^9}, {3.9156655749962044`*^9, 3.9156655762739687`*^9}, 
   3.9156656070445366`*^9, {3.915668992412733*^9, 
   3.915669014212542*^9}},ExpressionUUID->"474ac226-6772-48d1-9791-\
093c25206665"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
     "Function", " ", "to", " ", "make", " ", "a", " ", "tetrahedron", " ", 
      "4"}], "-", 
     RowBox[{
     "Group", " ", "from", " ", "a", " ", "cuboctahedron", " ", "in"}]}], ",",
     " ", 
    RowBox[{
     RowBox[{"4", 
      RowBox[{"D", ".", " ", "\[IndentingNewLine]", "The"}], " ", "default"}],
      " ", "+", 
     RowBox[{
      RowBox[{"/", 
       RowBox[{"-", " ", "assignment"}]}], " ", "is", " ", "arbitrary"}]}], 
    ",", " ", 
    RowBox[{
    "but", " ", "will", " ", "be", " ", "based", " ", "on", " ", "a", " ", 
     "compounding", " ", "axis", " ", "if", " ", 
     RowBox[{"given", ".", " ", "Since"}], " ", "it", " ", "is", " ", "in", 
     " ", "4", "D"}], ",", " ", 
    RowBox[{
     RowBox[{"this", " ", "requires", " ", "also", " ", "a", " ", "3"}], "-", 
     
     RowBox[{"space", " ", "normal", " ", "to", " ", "be", " ", 
      RowBox[{"given", ".", " ", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"--", " ", 
          RowBox[{"Input", ":", " ", 
           RowBox[{"Cuboctahedron", " ", "vertices", " ", "in", " ", "4", 
            RowBox[{"D", ".", " ", "\[IndentingNewLine]", "Option"}]}], ":", 
           " ", 
           RowBox[{
            RowBox[{
            "A", " ", "compoounding", " ", "axis", " ", "within", " ", "the", 
             " ", 
             RowBox[{"cuboctahedron", "'"}], "s", " ", "3"}], "-", 
            "space"}]}]}], ";", " ", 
         RowBox[{
          RowBox[{"its", " ", "3"}], "-", 
          RowBox[{"space", " ", "normal"}]}], ";", " ", 
         RowBox[{
          RowBox[{"sign", " ", "for"}], " ", "+", 
          RowBox[{
           RowBox[{"/", 
            RowBox[{"-", " ", "4"}]}], "G", " ", 
           RowBox[{"choice", ".", " ", 
            RowBox[{"--", 
             RowBox[{"--", 
              RowBox[{"--", "-"}]}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
       
       RowBox[{"(*", 
        RowBox[{"--", " ", 
         RowBox[{"Output", ":", " ", 
          RowBox[{
          "A", " ", "set", " ", "of", " ", "4", " ", "disjoint", " ", 
           "triangular", " ", "faces", " ", "of", " ", "the", " ", 
           "cuboctahedron", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
             "as", " ", "4", " ", "lists", " ", "fo", " ", "3", " ", 
              "vertices"}], ")"}], ".", " ", "Appending"}], " ", "the", " ", 
           RowBox[{"cuboctahedron", "'"}], "s", " ", "centroid", " ", "to", 
           " ", "each", " ", "face", " ", "will", " ", "give", " ", "the", 
           " ", "4", " ", "tetrahedra", " ", "of", " ", "a", " ", "4", 
           RowBox[{"G", ".", "  ", 
            RowBox[{"--", 
             RowBox[{"--", 
              RowBox[{"--", "-"}]}]}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
       
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", "A"}], " ", 
      "cuboctahedron", " ", "here", " ", "has", " ", "two", " ", "sets", " ", 
      "of", " ", "4", " ", "disjoint", " ", "triangular", " ", 
      RowBox[{"faces", ".", " ", "Relative"}], " ", "to", " ", "a", " ", 
      "given", " ", "compounding", " ", "axis"}]}], ",", " ", 
    RowBox[{
     RowBox[{"these", " ", "can", " ", "be", " ", "considered", " ", "left"}],
      "-", " ", 
     RowBox[{"and", " ", "right"}], "-", 
     RowBox[{"handed", " ", "within", " ", "the", " ", "3"}], "-", 
     RowBox[{
      RowBox[{"space", ".", " ", "So"}], " ", "given", " ", "said", " ", 
      "axis", " ", "and", " ", "the", " ", "3"}], "-", 
     RowBox[{"space", " ", "normal"}]}], ",", " ", 
    RowBox[{"this", " ", "function", " ", "chooses", " ", "the", " ", 
     RowBox[{
      StyleBox["left",
       FontSize->16,
       FontWeight->"Bold"], 
      StyleBox["?",
       FontWeight->"Bold"], 
      RowBox[{"-", "handed"}]}], " ", "one", " ", "by", " ", "default"}], ",",
     " ", 
    RowBox[{
     RowBox[{
     "or", " ", "the", " ", "right", " ", "handed", " ", "one", " ", "if", 
      " ", "the", " ", "chirality", " ", "input", " ", "is", " ", "given", 
      " ", "as"}], " ", "-", 
     RowBox[{
      RowBox[{
       RowBox[{"1.", " ", "--"}], "--"}], "--"}], "-"}]}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"ClearAll", "[", "cbctTo4G4D", "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"cbctTo4G4D", "::", "DependentVectors"}], " ", "=", " ", 
     "\"\<Warning: Chirality is being ignored, because the function arguments \
for 5-fold axis and 3-space normal are linearly dependent on the nearest \
cuboctahedron vertices (within 10^-10).\>\""}], ";"}], "\[IndentingNewLine]", 
   
   RowBox[{
    RowBox[{
     RowBox[{"cbctTo4G4D", "::", "MissingVector"}], " ", "=", " ", 
     "\"\<Warning: Chirality is being ignored, because either the axis or the \
3-space normal is missing, or one has zero norm.\>\""}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"cbctTo4G4D", "[", 
      RowBox[{"cbctVerts_", ",", 
       RowBox[{"compoundingAxis_", ":", "Null"}], ",", 
       RowBox[{"normal4D_", ":", "Null"}], ",", 
       RowBox[{"sgn_", ":", "1"}]}], "]"}], " ", ":=", " ", 
     RowBox[{"Module", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "triangleVertIndices", ",", "triVerts", ",", "first2TriVerts", ",", 
         "firstTriTestVerts", ",", "vertAxNormDet", ",", "tri1Inds", ",", 
         "nearAxVert", ",", "altTriVertInds", ",", "altTetVerts"}], "}"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"triangleVertIndices", " ", "=", " ", 
         RowBox[{"Select", "[", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"ConvexPolytope", "[", "cbctVerts", "]"}], "[", 
            RowBox[{"\"\<FaceVertexIndices\>\"", ",", "2"}], "]"}], " ", ",", 
           " ", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "#", "]"}], "==", "3"}], "&"}]}], "]"}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"If", " ", "a", " ", "5"}], "-", 
           RowBox[{
           "fold", " ", "compounding", " ", "axis", " ", "is", " ", "given", 
            " ", 
            RowBox[{"(", 
             RowBox[{
             "along", " ", "with", " ", "a", " ", "normal", " ", "to", " ", 
              "the", " ", "3", "D", " ", "space"}], ")"}], " ", 
            "\[IndentingNewLine]", "then", " ", "relative", " ", "to", " ", 
            "that", " ", "there", " ", "is", " ", "a", " ", "unique", " ", 
            "left", " ", "and", " ", "a", " ", "right", " ", "4", 
            RowBox[{"G", ".", " ", "\[IndentingNewLine]", "This"}], " ", 
            RowBox[{"If", "[", "]"}], " ", "statement", " ", "is", " ", "to", 
            " ", "select", " ", "a", " ", "triangle", " ", "face", " ", "of", 
            " ", "the", " ", 
            RowBox[{
             StyleBox["right",
              FontSize->16,
              FontWeight->"Bold"], 
             StyleBox["?",
              FontWeight->"Bold"], " ", "4"}], 
            RowBox[{"G", ".", "\[IndentingNewLine]", "Or"}]}]}], ",", " ", 
          RowBox[{"if", " ", "no", " ", "axis", " ", "is", " ", "given"}], 
          ",", " ", 
          RowBox[{
          "then", " ", "it", " ", "just", " ", "selects", " ", "the", " ", 
           "first", " ", "triangle", " ", "face", " ", "in", " ", "the", " ", 
           
           RowBox[{"list", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"VectorQ", "[", "compoundingAxis", "]"}], " ", "&&", " ", 
           RowBox[{
            RowBox[{"Norm", "[", "compoundingAxis", "]"}], ">", 
            RowBox[{"10.", "^", 
             RowBox[{"-", "4"}]}]}], " ", "&&", " ", 
           RowBox[{"VectorQ", "[", "normal4D", "]"}], " ", "&&", "  ", 
           RowBox[{
            RowBox[{"Norm", "[", "normal4D", "]"}], ">", 
            RowBox[{"10.", "^", 
             RowBox[{"-", "4"}]}]}]}], "\[IndentingNewLine]", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", 
           RowBox[{
           "All", " ", "the", " ", "triangular", " ", "faces", " ", "of", " ",
             "the", " ", "cuboctahedron"}], "*)"}], "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"triVerts", " ", "=", " ", 
             RowBox[{"Map", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"cbctVerts", "[", 
                 RowBox[{"[", "#", "]"}], "]"}], " ", "&"}], ",", 
               "triangleVertIndices"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
             "Select", " ", "the", " ", "vertex", " ", "closest", " ", "to", 
              " ", "the", " ", "compounding", " ", "axis", " ", "vector"}], 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"nearAxVert", " ", "=", " ", 
             RowBox[{"First", "@", 
              RowBox[{"MinimalBy", "[", 
               RowBox[{"cbctVerts", ",", 
                RowBox[{
                 RowBox[{"Norm", "[", 
                  RowBox[{"#", "-", "compoundingAxis"}], "]"}], "&"}]}], 
               "]"}]}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
             "Select", " ", "the", " ", "two", " ", "triangels", " ", "that", 
              " ", "share", " ", "the", " ", "nearAxVert"}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"first2TriVerts", " ", "=", " ", 
             RowBox[{"Select", "[", 
              RowBox[{"triVerts", ",", 
               RowBox[{"ContainsAll", "[", 
                RowBox[{"{", "nearAxVert", "}"}], "]"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{
              "Pick", " ", "one", " ", "of", " ", "the", " ", "two", " ", 
               "triangles"}], ",", " ", 
              RowBox[{
              "and", " ", "select", " ", "the", " ", "nearAxVert", " ", 
               "plus", " ", "one", " ", "of", " ", "the", " ", "other", " ", 
               "two", " ", "vertices"}], ",", " ", 
              RowBox[{
              "making", " ", "some", " ", "contortion", " ", "to", " ", 
               "assure", " ", "that", " ", "the", " ", "nearAxVert", " ", 
               "is", " ", "the", " ", "second", " ", "in", " ", "the", " ", 
               RowBox[{"list", ".", "\[IndentingNewLine]", "It"}], " ", 
               RowBox[{"doesn", "'"}], "t", " ", "matter", " ", "which", " ", 
               "of", " ", "the", " ", "other", " ", "two", " ", "is", " ", 
               "the", " ", "first", " ", "one", " ", "in", " ", "the", " ", 
               "list"}], ",", " ", 
              RowBox[{
              "since", " ", "either", " ", "way", " ", "they", " ", "will", 
               " ", "generate", " ", "the", " ", "same", " ", "handedness", 
               " ", "when", " ", "joined", " ", "with", " ", "the", " ", 
               "axis", " ", "and", " ", "the", " ", "space", " ", 
               RowBox[{"normal", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"firstTriTestVerts", " ", "=", " ", 
             RowBox[{"ReplacePart", " ", "[", 
              RowBox[{
               RowBox[{"DeleteCases", "[", 
                RowBox[{
                 RowBox[{"first2TriVerts", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "nearAxVert"}], "]"}], 
               ",", 
               RowBox[{"2", "->", "nearAxVert"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"Check", " ", "if", " ", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"the", " ", "two", " ", "vertices"}], ",", " ", 
                 RowBox[{"the", " ", "compunding", " ", "axis"}], ",", " ", 
                 RowBox[{
                  RowBox[{"and", " ", "the", " ", "3"}], "-", 
                  RowBox[{"space", " ", "normal"}]}]}], "}"}], " ", "form", 
               " ", "a", " ", "left", " ", "handed", " ", "system", " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"positive", " ", "4", "D", " ", "determinant"}], 
                 ")"}], ".", "\[IndentingNewLine]", "\[IndentingNewLine]", 
                "If"}], " ", "Det", " ", "is", " ", "positive"}], ",", " ", 
              RowBox[{"keep", " ", "that", " ", 
               RowBox[{"triangle", ".", "\[IndentingNewLine]", "If"}], " ", 
               "Det", " ", "is", " ", "negative"}], ",", " ", 
              RowBox[{"switch", " ", "to", " ", "the", " ", "other", " ", 
               RowBox[{"triangle", "."}]}]}], " ", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"vertAxNormDet", " ", "=", " ", 
             RowBox[{"Quiet", "[", 
              RowBox[{
               RowBox[{"Det", "@", 
                RowBox[{"Join", "[", 
                 RowBox[{"firstTriTestVerts", ",", 
                  RowBox[{"{", 
                   RowBox[{"compoundingAxis", ",", "normal4D"}], "}"}]}], 
                 "]"}]}], ",", 
               RowBox[{"Det", "::", "luc"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"If", " ", "the", " ", "Det", " ", "vanishes"}], ",", 
              " ", 
              RowBox[{
              "the", " ", "compounding", " ", "axis", " ", "is", " ", 
               "linearly", " ", "dependent", " ", "on", " ", "the", " ", 
               RowBox[{"triangle", "'"}], "s", " ", "two", " ", "vertices"}], 
              ",", " ", 
              RowBox[{
              "so", " ", "replace", " ", "the", " ", "one", " ", "that", " ", 
               "is", " ", "not", " ", "nearAxVert", " ", "with", " ", "the", 
               " ", "third", " ", 
               RowBox[{"vertex", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
            " ", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Abs", "@", "vertAxNormDet"}], "<", 
               RowBox[{"10.", "^", 
                RowBox[{"-", "10"}]}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"firstTriTestVerts", " ", "=", " ", 
                 RowBox[{"RotateLeft", "@", 
                  RowBox[{"ReplacePart", " ", "[", 
                   RowBox[{
                    RowBox[{"DeleteCases", "[", 
                    RowBox[{
                    RowBox[{"first2TriVerts", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", "nearAxVert"}], 
                    "]"}], ",", 
                    RowBox[{"1", "->", "nearAxVert"}]}], "]"}]}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"vertAxNormDet", " ", "=", " ", 
                 RowBox[{"Det", "@", 
                  RowBox[{"Join", "[", 
                   RowBox[{"firstTriTestVerts", ",", 
                    RowBox[{"{", 
                    RowBox[{"compoundingAxis", ",", "normal4D"}], "}"}]}], 
                   "]"}]}]}]}], "}"}]}], "\[IndentingNewLine]", "]"}], ";", 
            " ", "\[IndentingNewLine]", 
            RowBox[{"Which", "[", "\[IndentingNewLine]", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{"If", " ", "Det", " ", "still", " ", "vanishes"}], ",",
                " ", 
               RowBox[{
                RowBox[{
                 RowBox[{"something", "'"}], "s", " ", "wrong"}], ";", " ", 
                RowBox[{
                "ignore", " ", "chirality", " ", "input", " ", "and", " ", 
                 "use", " ", 
                 RowBox[{"default", "."}]}]}]}], "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{
               RowBox[{"Abs", "@", "vertAxNormDet"}], "<", 
               RowBox[{"10.", "^", 
                RowBox[{"-", "10"}]}]}], "\[IndentingNewLine]", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"tri1Inds", " ", "=", " ", 
                 RowBox[{"triangleVertIndices", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], ";", 
                RowBox[{"Message", "[", 
                 RowBox[{"cbctTo4G4D", "::", "DependentVectors"}], "]"}]}], 
               "}"}], "\[IndentingNewLine]", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"If", " ", "Det", " ", "is", " ", "positive"}], ",", 
                " ", 
                RowBox[{"keep", " ", "that", " ", 
                 RowBox[{"triangle", "."}]}]}], "*)"}], "\[IndentingNewLine]",
               ",", 
              RowBox[{"vertAxNormDet", ">", "0"}], "\[IndentingNewLine]", ",", 
              RowBox[{"tri1Inds", " ", "=", " ", 
               RowBox[{"Flatten", "@", 
                RowBox[{"triangleVertIndices", "[", 
                 RowBox[{"[", "  ", 
                  RowBox[{"First", "@", 
                   RowBox[{"Position", "[", 
                    RowBox[{"triVerts", ",", 
                    RowBox[{"first2TriVerts", "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], " ", "]"}], 
                 "]"}]}]}], "\[IndentingNewLine]", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"If", " ", "Det", " ", "is", " ", "negative"}], ",", 
                " ", 
                RowBox[{"switch", " ", "to", " ", "the", " ", "other", " ", 
                 RowBox[{"triangle", "."}]}]}], " ", "*)"}], 
              "\[IndentingNewLine]", ",", 
              RowBox[{"vertAxNormDet", "<", "0"}], "\[IndentingNewLine]", ",", 
              RowBox[{"tri1Inds", " ", "=", " ", 
               RowBox[{"Flatten", "@", 
                RowBox[{"triangleVertIndices", "[", 
                 RowBox[{"[", " ", 
                  RowBox[{"First", "@", 
                   RowBox[{"Position", "[", 
                    RowBox[{"triVerts", ",", 
                    RowBox[{"first2TriVerts", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], " ", "]"}], 
                 "]"}]}]}]}], "\[IndentingNewLine]", "]"}]}], 
           "\[IndentingNewLine]", "}"}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"tri1Inds", " ", "=", " ", 
             RowBox[{"triangleVertIndices", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{
               RowBox[{
               "If", " ", "either", " ", "a", " ", "good", " ", "axis", " ", 
                "or", " ", "a", " ", "good", " ", "3"}], "-", 
               RowBox[{"space", " ", "normal", " ", "was", " ", "input"}]}], 
              ",", " ", 
              RowBox[{"but", " ", "not", " ", "both"}], ",", " ", 
              RowBox[{
              "give", " ", "warning", " ", "and", " ", "ignore", " ", 
               RowBox[{"chirality", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
            
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"VectorQ", "[", "compoundingAxis", "]"}], " ", "&&", 
                 " ", 
                 RowBox[{
                  RowBox[{"Norm", "[", "compoundingAxis", "]"}], ">", 
                  RowBox[{"10.", "^", 
                   RowBox[{"-", "4"}]}]}]}], " ", ")"}], "||", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"VectorQ", "[", "normal4D", "]"}], " ", "&&", "  ", 
                 RowBox[{
                  RowBox[{"Norm", "[", "normal4D", "]"}], ">", 
                  RowBox[{"10.", "^", 
                   RowBox[{"-", "4"}]}]}]}], ")"}]}], ",", 
              RowBox[{"Message", "[", 
               RowBox[{"cbctTo4G4D", "::", "MissingVector"}], "]"}]}], 
             "]"}]}], "\[IndentingNewLine]", "}"}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
          "With", " ", "one", " ", "of", " ", "the", " ", "triangle", " ", 
           "faces", " ", "now", " ", "fixed"}], ",", " ", 
          RowBox[{
          "find", " ", "the", " ", "other", " ", "3", " ", "that", " ", "go", 
           " ", "with", " ", "it", " ", "as", " ", "to", " ", "form", " ", 
           "4", " ", "disjoint", " ", "sets", " ", "of", " ", 
           RowBox[{"vertices", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
        
        RowBox[{"altTriVertInds", " ", "=", " ", 
         RowBox[{
          RowBox[{"Select", "[", 
           RowBox[{"triangleVertIndices", ",", 
            RowBox[{
             RowBox[{"ContainsAny", "[", 
              RowBox[{"#", ",", "tri1Inds"}], "]"}], "&"}]}], "]"}], " ", "//",
           " ", 
          RowBox[{
           RowBox[{"DeleteCases", "[", 
            RowBox[{"#", ",", "tri1Inds"}], "]"}], "&"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"altTriVertInds", " ", "=", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"altTriVertInds", ",", 
           RowBox[{"Select", "[", 
            RowBox[{"triangleVertIndices", ",", 
             RowBox[{
              RowBox[{"ContainsNone", "[", 
               RowBox[{"#", ",", 
                RowBox[{"Flatten", "@", "altTriVertInds"}]}], "]"}], "&"}]}], 
            "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
          "If", " ", "the", " ", "negative", " ", "chirality", " ", "choice", 
           " ", "was", " ", "input"}], ",", " ", 
          RowBox[{
          "switch", " ", "to", " ", "the", " ", "opposite", " ", "set", " ", 
           "of", " ", "triangle", " ", "faces"}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"sgn", "==", 
           RowBox[{"-", "1"}]}], ",", 
          RowBox[{"altTriVertInds", " ", "=", " ", 
           RowBox[{"Complement", "[", 
            RowBox[{"triangleVertIndices", ",", "altTriVertInds"}], "]"}]}]}],
          "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"altTetVerts", " ", "=", " ", 
         RowBox[{"Chop", "[", 
          RowBox[{"Map", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"cbctVerts", "[", 
                RowBox[{"[", "#", "]"}], "]"}], ",", "ctr4"}], "]"}], " ", 
             "&"}], ",", "altTriVertInds"}], "]"}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.910794284662715*^9, 3.910794339577205*^9}, {
   3.910794803303755*^9, 3.910794807734883*^9}, {3.910794910042824*^9, 
   3.910794913818918*^9}, {3.9107949632762485`*^9, 3.9107949735006704`*^9}, {
   3.9107986191479945`*^9, 3.9107986649415407`*^9}, {3.9107987015550184`*^9, 
   3.910798719918098*^9}, {3.9107987839998827`*^9, 3.9107988932346725`*^9}, 
   3.9107989428996477`*^9, {3.9107989968019724`*^9, 3.9107990121173997`*^9}, {
   3.910799189800376*^9, 3.9107992527315197`*^9}, {3.910799447425314*^9, 
   3.9107994952475247`*^9}, {3.9107995651687484`*^9, 
   3.9107995819050446`*^9}, {3.910799629058571*^9, 3.910799629922988*^9}, {
   3.9107999993863125`*^9, 3.910800102012506*^9}, {3.9108011723377733`*^9, 
   3.910801268803418*^9}, {3.910813932488002*^9, 3.910814109854495*^9}, {
   3.910814190496168*^9, 3.910814223343311*^9}, 3.9108142636005116`*^9, 
   3.910814857410947*^9, {3.9108149074750504`*^9, 3.9108149385040693`*^9}, {
   3.9108150116855392`*^9, 3.910815172506268*^9}, {3.9108152050988665`*^9, 
   3.9108152272600074`*^9}, {3.910815259146358*^9, 3.9108152726508403`*^9}, {
   3.910815305387883*^9, 3.910815330652371*^9}, {3.910815386220747*^9, 
   3.910815396205389*^9}, {3.910815468669281*^9, 3.91081547089276*^9}, {
   3.9108155166221676`*^9, 3.910815524334256*^9}, {3.910815572958712*^9, 
   3.910815574334627*^9}, {3.9108158523385997`*^9, 3.910815853298483*^9}, {
   3.910815893971158*^9, 3.910815900258664*^9}, {3.9108159626170473`*^9, 
   3.9108159767396517`*^9}, 3.910816008579994*^9, {3.9108160540190306`*^9, 
   3.910816156373213*^9}, {3.9108161917986193`*^9, 3.9108161971738453`*^9}, {
   3.910816273846168*^9, 3.9108162816549053`*^9}, {3.9108163433202534`*^9, 
   3.9108163925200405`*^9}, {3.91083088408604*^9, 3.9108310148443613`*^9}, {
   3.9108310686305304`*^9, 3.910831180990587*^9}, {3.910831247892824*^9, 
   3.910831333872391*^9}, {3.9108316409126415`*^9, 3.9108316426280975`*^9}, {
   3.910885267267748*^9, 3.9108853361918545`*^9}, {3.9108853678973417`*^9, 
   3.9108853922902827`*^9}, {3.9108854417128525`*^9, 
   3.9108855326774545`*^9}, {3.9108855870932646`*^9, 3.910885713578554*^9}, {
   3.9108857849245296`*^9, 3.9108858132756453`*^9}, {3.910885848391324*^9, 
   3.9108859435975714`*^9}, 3.9108860503472586`*^9, {3.9108861199308243`*^9, 
   3.910886291424881*^9}, {3.91088642310756*^9, 3.9108864527846165`*^9}, {
   3.910886487455964*^9, 3.9108865246085014`*^9}, 3.9108866310894575`*^9, {
   3.9108884500527163`*^9, 3.91088845853407*^9}, {3.910888500917723*^9, 
   3.910888517587064*^9}, {3.910888607114299*^9, 3.9108886993550406`*^9}, {
   3.910888787479768*^9, 3.910888788051757*^9}, 3.9108888365214853`*^9, {
   3.9108889416726236`*^9, 3.910888966859831*^9}, {3.910889000863985*^9, 
   3.910889029755877*^9}, {3.910889097868157*^9, 3.910889129484561*^9}, {
   3.9108894417375317`*^9, 3.9108894586913214`*^9}, {3.910889523436384*^9, 
   3.9108895705873423`*^9}, {3.9108896113895493`*^9, 3.910889696436651*^9}, {
   3.9108897871082373`*^9, 3.910889852424917*^9}, {3.9108899246860776`*^9, 
   3.910889929432028*^9}, {3.9108900057939873`*^9, 3.910890010731218*^9}, {
   3.910890045995792*^9, 3.9108900605944653`*^9}, {3.9108901104011965`*^9, 
   3.9108902235484653`*^9}, 3.910890366971637*^9, {3.9108904340186453`*^9, 
   3.9108904406214886`*^9}, {3.910890477860799*^9, 3.910890493254973*^9}, {
   3.9108919978786097`*^9, 3.9108921078704033`*^9}, {3.9108921646045256`*^9, 
   3.9108921856610317`*^9}, {3.9108922200928836`*^9, 3.910892295582924*^9}, {
   3.910892373525034*^9, 3.910892386900787*^9}, {3.910892603647534*^9, 
   3.9108926516146984`*^9}, {3.9108929663756695`*^9, 3.910893001168992*^9}, {
   3.910902368119692*^9, 3.910902435819794*^9}, {3.910902616142697*^9, 
   3.910902622845441*^9}, {3.9109632270276165`*^9, 3.9109632491433525`*^9}, {
   3.910963279735193*^9, 3.910963323173728*^9}, {3.911060477254077*^9, 
   3.911060502550248*^9}, {3.9115079642348475`*^9, 3.9115079673899574`*^9}, {
   3.911508620446165*^9, 3.911508644749345*^9}, {3.9115087343169765`*^9, 
   3.9115087598165126`*^9}, {3.9115088988331594`*^9, 
   3.9115089333052273`*^9}, {3.911509006707953*^9, 3.911509057087068*^9}, {
   3.915484313690113*^9, 3.915484342816354*^9}, {3.915484934294574*^9, 
   3.915485036640052*^9}, {3.91548541399382*^9, 3.915485419496607*^9}, {
   3.9155522201782703`*^9, 3.915552227036832*^9}, 3.9156655476144257`*^9, {
   3.915665679286766*^9, 3.9156657488971434`*^9}, {3.9156658015672245`*^9, 
   3.915665856531108*^9}, {3.915665903788415*^9, 3.9156659323456745`*^9}, {
   3.915665971605772*^9, 3.915665971844879*^9}, {3.915666036721157*^9, 
   3.915666043052861*^9}, {3.9156660778230767`*^9, 3.915666128920495*^9}, {
   3.9156661639588437`*^9, 3.915666228738841*^9}, {3.91566629265258*^9, 
   3.9156663423512344`*^9}, {3.9156664176451664`*^9, 
   3.9156664352732487`*^9}, {3.9156664868925376`*^9, 
   3.9156665885614367`*^9}, {3.915666648428239*^9, 3.9156668284756346`*^9}, {
   3.9156668738857565`*^9, 3.9156669147663364`*^9}, {3.9156669713419886`*^9, 
   3.915666982686719*^9}, 3.9156670649382277`*^9, {3.9156671569517307`*^9, 
   3.9156671826320705`*^9}, {3.9156674333778496`*^9, 3.915667478306451*^9}, {
   3.915667518048832*^9, 3.9156675513260655`*^9}, {3.915667601329015*^9, 
   3.91566786206929*^9}, {3.915667894969236*^9, 3.9156679273496866`*^9}, {
   3.915667974361929*^9, 3.9156680154719*^9}, {3.915668068523157*^9, 
   3.915668180982405*^9}, {3.9156682686149206`*^9, 3.915668379060109*^9}, {
   3.9156684093163023`*^9, 3.9156685967202606`*^9}, {3.9156686632698774`*^9, 
   3.9156688255280676`*^9}, {3.9156688616673965`*^9, 
   3.9156688658312373`*^9}, {3.915670602853485*^9, 3.915670635175992*^9}},
 CellLabel->"In[13]:=",ExpressionUUID->"10c0f2bf-b991-4259-acb1-b978b3d9f399"]
}, Closed]],

Cell[CellGroupData[{

Cell["Find the index of a point in a point list.", "Subsubsection",
 CellChangeTimes->{{3.899295784777296*^9, 3.899295994734908*^9}, {
  3.899296031312354*^9, 3.8992960371332164`*^9}, {3.915475129215588*^9, 
  3.915475138627266*^9}, {3.9154752509693356`*^9, 3.9154753129208307`*^9}, {
  3.915475354086731*^9, 3.9154755052856717`*^9}, {3.915475547764888*^9, 
  3.9154755552142735`*^9}},ExpressionUUID->"cea81e48-a517-43b2-a1d4-\
9f43c1457e1a"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ClearAll", "[", "ptToIndex", "]"}], ";"}], "\[IndentingNewLine]", 
  
  RowBox[{"(*", 
   RowBox[{"--", 
    RowBox[{"-", " ", 
     RowBox[{"Input", ":", " ", 
      RowBox[{
       RowBox[{
       "a", " ", "single", " ", "point", " ", "and", " ", "a", " ", "point", 
        " ", "set", " ", 
        RowBox[{
         RowBox[{"list", " ", "--"}], "--"}]}], "-"}]}]}]}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"--", 
     RowBox[{"-", " ", 
      RowBox[{"Output", ":", " ", 
       RowBox[{
       "The", " ", "index", " ", "of", " ", "the", " ", "given", " ", "point",
         " ", "in", " ", "the", " ", "given", " ", "list", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
          "comparison", " ", "made", " ", "by", " ", "rounding", " ", "at", 
           " ", "the", " ", "level", " ", "of", " ", ".001"}], ")"}], ".", 
         " ", "\[IndentingNewLine]", "If"}], " ", "the", " ", "point", " ", 
        "is", " ", "not", " ", "in", " ", "the", " ", "list"}]}]}]}], ",", 
    " ", 
    RowBox[{
     StyleBox["Nothing",
      FontWeight->"Bold"], " ", "is", " ", "returned"}], ",", " ", 
    RowBox[{
    "which", " ", "Mathematica", " ", "drops", " ", "from", " ", "any", " ", 
     "enclosing", " ", 
     RowBox[{"list", ".", "\[IndentingNewLine]", "This"}], " ", "function", 
     " ", "is", " ", "primarily", " ", "for", " ", "inverting", " ", 
     "projections"}], ",", " ", 
    RowBox[{
     RowBox[{"since", " ", 
      RowBox[{"(", 
       RowBox[{"in", " ", "our", " ", "context"}], ")"}], " ", "the", " ", 
      "index", " ", "of", " ", "a", " ", "point", " ", "in", " ", "the", " ", 
      "projected", " ", "set", " ", "will", " ", "match", " ", "its", " ", 
      "index", " ", "in", " ", "the", " ", "pre"}], "-", 
     RowBox[{"projected", " ", 
      RowBox[{
       RowBox[{"set", " ", "--"}], "--"}]}], "-"}]}], "*)"}], "\n", 
  RowBox[{"(*", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ptToIndex", "[", 
    RowBox[{"pt_", ",", "superSet_"}], "]"}], " ", ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "index", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"index", " ", "=", " ", 
       RowBox[{"Flatten", "@", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"Round", "[", 
           RowBox[{"superSet", ",", 
            RowBox[{"10.", "^", 
             RowBox[{"-", "3"}]}]}], "]"}], ",", 
          RowBox[{"Round", "[", 
           RowBox[{"pt", ",", 
            RowBox[{"10.", "^", 
             RowBox[{"-", "3"}]}]}], "]"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"index", "==", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"index", "=", "Nothing"}], ",", 
        RowBox[{"index", " ", "=", " ", 
         RowBox[{"First", "@", "index"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "index", "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.8941294737155237`*^9, 3.894129584262308*^9}, {
   3.8941296193671246`*^9, 3.894129636771474*^9}, {3.8944616634919443`*^9, 
   3.8944616767842846`*^9}, {3.898958973094428*^9, 3.8989589805950375`*^9}, {
   3.8989671124312077`*^9, 3.8989671124342413`*^9}, {3.8989671612645564`*^9, 
   3.8989672089388237`*^9}, 3.89896733382018*^9, {3.8989674571034484`*^9, 
   3.8989674710477667`*^9}, {3.898968851504547*^9, 3.8989689676217804`*^9}, {
   3.898969044803418*^9, 3.898969056975024*^9}, {3.8989691475731335`*^9, 
   3.8989691789318895`*^9}, 3.898969235522375*^9, {3.898970292837315*^9, 
   3.898970292862179*^9}, 3.898990394794772*^9, {3.8992044004883337`*^9, 
   3.8992044426514883`*^9}, {3.8992896661416054`*^9, 3.899289850083294*^9}, {
   3.899289891619693*^9, 3.899290160006542*^9}, {3.8992902340416937`*^9, 
   3.8992902378255157`*^9}, {3.899290284528468*^9, 3.899290315283285*^9}, {
   3.8992903457507954`*^9, 3.8992903497485714`*^9}, {3.899290582789924*^9, 
   3.8992906981875143`*^9}, {3.899290732658476*^9, 3.899290868711325*^9}, {
   3.8992909153552275`*^9, 3.8992909989038315`*^9}, {3.899291282683879*^9, 
   3.899291369134114*^9}, {3.899291406463667*^9, 3.8992914246398783`*^9}, {
   3.899291455840021*^9, 3.8992914576608405`*^9}, {3.8992915037209244`*^9, 
   3.8992915155518007`*^9}, {3.8992915693943863`*^9, 
   3.8992915938077087`*^9}, {3.899291725416053*^9, 3.8992917793146963`*^9}, {
   3.899291876706521*^9, 3.8992919235128994`*^9}, {3.899292013844141*^9, 
   3.899292016707864*^9}, {3.8992920476318913`*^9, 3.899292129862335*^9}, {
   3.8992922654902754`*^9, 3.899292376119115*^9}, {3.89929246139557*^9, 
   3.8992924682329226`*^9}, {3.8992926033860626`*^9, 3.899292685726564*^9}, {
   3.899292747690163*^9, 3.8992927857051716`*^9}, {3.8992945403276625`*^9, 
   3.8992947785157156`*^9}, 3.8992956865292454`*^9},
 CellLabel->"In[17]:=",ExpressionUUID->"ae1e5d23-6058-491a-8d19-e8b774d09b52"]
}, Closed]],

Cell[CellGroupData[{

Cell["Facet normals of a 4D polytope", "Subsubsection",
 CellChangeTimes->{{3.915758331534645*^9, 
  3.915758338655379*^9}},ExpressionUUID->"8b45d703-dbde-4f91-9e05-\
48675c5eac05"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"facetNormals4D", "[", "convexPolytope4DVertices_", "]"}], " ", ":=",
    " ", 
   RowBox[{
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Mean", "@", 
         RowBox[{"convexPolytope4DVertices", "[", 
          RowBox[{"[", "#", "]"}], "]"}]}], "&"}], ",", 
       RowBox[{
        RowBox[{"ConvexPolytope", "[", "convexPolytope4DVertices", "]"}], "[", 
        RowBox[{"\"\<FaceVertexIndices\>\"", ",", "3"}], "]"}]}], " ", "]"}], 
     " ", "//", " ", 
     RowBox[{"Map", "@", "Normalize"}]}], " ", "//", " ", "Chop"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.915407132995057*^9, 3.915407220698419*^9}, {
  3.9154072528791146`*^9, 3.9154073022853355`*^9}, {3.915407453355631*^9, 
  3.915407466618801*^9}},
 CellLabel->"In[19]:=",ExpressionUUID->"bd079a84-7440-499f-93c3-6afb280933b8"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main", "Chapter",
 CellChangeTimes->{{3.8937680352863145`*^9, 3.8937680362949314`*^9}, {
  3.898361500245737*^9, 
  3.8983615008011665`*^9}},ExpressionUUID->"1fbd958d-fd94-4794-9e1d-\
2eb40549bd3a"],

Cell[CellGroupData[{

Cell["Gosset polytope", "Section",
 CellChangeTimes->{{3.6327757759443607`*^9, 3.632775779981124*^9}, {
  3.6327758569696493`*^9, 3.632775885832323*^9}, {3.892832455125064*^9, 
  3.8928324600086193`*^9}},ExpressionUUID->"73e85ae9-6372-459c-a5e5-\
811d750cea27"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"minvec", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"i", "<", "8"}], ",", 
    RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"j", "=", 
       RowBox[{"i", "+", "1"}]}], ",", 
      RowBox[{"j", "<=", "8"}], ",", 
      RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"u1", ",", "u2", ",", "u3", ",", "u4"}], "}"}], "=", 
        RowBox[{"Table", "[", 
         RowBox[{"0", ",", "4", ",", "8"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"u1", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"u2", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "1"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"u3", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"u4", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "1"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"minvec", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"minvec", ",", 
          RowBox[{"{", 
           RowBox[{"u1", ",", "u2", ",", "u3", ",", "u4"}], "}"}]}], 
         "]"}]}]}]}], "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"halves", "=", 
   RowBox[{"Tuples", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "/", "2"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "/", "2"}]}], "}"}], ",", "8"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"halves", "=", 
   RowBox[{"Select", "[", 
    RowBox[{"halves", ",", 
     RowBox[{
      RowBox[{"EvenQ", "[", 
       RowBox[{"Count", "[", 
        RowBox[{"#", ",", 
         RowBox[{
          RowBox[{"-", "1"}], "/", "2"}]}], "]"}], "]"}], "&"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minvec", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"minvec", ",", "halves"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.615645760236415*^9, {3.6156457997214146`*^9, 3.6156458011934147`*^9}, {
   3.8983390744204273`*^9, 3.898339150269208*^9}, {3.8983392341569395`*^9, 
   3.8983392422364807`*^9}, {3.8983393854455576`*^9, 
   3.8983394605512433`*^9}, {3.898339625054679*^9, 3.8983396600059576`*^9}, {
   3.8983399531364*^9, 3.898339953684852*^9}},
 CellLabel->"In[20]:=",ExpressionUUID->"99af1fe6-b4df-4c7f-bb99-07e2ac7c5d8f"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"GossetVerts", "=", "minvec"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"GossetVertsNCenter", "=", 
   RowBox[{"PrependTo", "[", 
    RowBox[{"minvec", ",", 
     RowBox[{"{", 
      RowBox[{
      "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
       "0"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "[", "GossetVertsNCenter", "]"}]}], "Input",
 CellChangeTimes->{{3.89283190149641*^9, 3.892831902991132*^9}, 
   3.892832624355321*^9, {3.915747582377898*^9, 3.915747582729488*^9}},
 CellLabel->"In[25]:=",ExpressionUUID->"f5fabf09-de9c-4bc8-be11-8ecf51b49faf"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"241", ",", "8"}], "}"}]], "Output",
 CellChangeTimes->{
  3.9157475873111553`*^9, 3.9157476203603525`*^9, 3.915747754827896*^9, {
   3.9157484389738684`*^9, 3.9157484674937954`*^9}, 3.915831030765611*^9, 
   3.9162689694037576`*^9, 3.916522662994088*^9, 3.9165258817907815`*^9},
 CellLabel->"Out[27]=",ExpressionUUID->"3cb9c6e8-f53e-4bda-9e23-a8c9b6ac6b2f"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Projection to 4D Elser-Sloane space (ES)", "Section",
 CellChangeTimes->{{3.6327757759443607`*^9, 3.632775779981124*^9}, {
   3.6327758569696493`*^9, 3.632775885832323*^9}, {3.8983625990003214`*^9, 
   3.8983625995958147`*^9}, {3.9154023745584865`*^9, 3.9154023976635084`*^9}, 
   3.9156694206949024`*^9},ExpressionUUID->"51ca42fd-ec15-4680-92d9-\
ad884b5da3af"],

Cell[CellGroupData[{

Cell["Orient Gosset for ES projection", "Subsection",
 CellChangeTimes->{{3.893418019594502*^9, 
  3.8934180414543405`*^9}},ExpressionUUID->"8166766d-d5c9-4a18-b038-\
8acca4e4253f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"nn", "=", "8"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"dimension", " ", "of", " ", "embedding", " ", "space"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dd", "=", "4"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"dimension", " ", "of", " ", "projection", " ", "subspace"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cc", "=", 
   RowBox[{"1", "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"4", "-", 
      RowBox[{"2", "*", 
       RowBox[{"1", "/", "\[CurlyPhi]"}]}]}], "]"}]}]}], ";", 
  RowBox[{"ct", "=", 
   RowBox[{"1", "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"4", "+", 
      RowBox[{"2", "*", "\[CurlyPhi]"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HE8", "=", 
   RowBox[{
    RowBox[{"1", "/", "2"}], "*", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", 
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"-", "1"}], ",", "1", ",", 
        RowBox[{"-", "1"}]}], "}"}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U11", "=", 
   RowBox[{"cc", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "4", "]"}], "-", 
      RowBox[{
       RowBox[{"1", "/", "\[CurlyPhi]"}], "*", "HE8"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U12", "=", 
   RowBox[{"ct", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "4", "]"}], "+", 
      RowBox[{"\[CurlyPhi]", "*", "HE8"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U21", "=", 
   RowBox[{"cc", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "4", "]"}], "+", 
      RowBox[{
       RowBox[{"1", "/", "\[CurlyPhi]"}], "*", "HE8"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U22", "=", 
   RowBox[{"ct", "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"IdentityMatrix", "[", "4", "]"}], "-", 
      RowBox[{"\[CurlyPhi]", "*", "HE8"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Umat", "=", 
   RowBox[{"ArrayFlatten", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"U11", ",", "U12"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"U21", ",", "U22"}], "}"}]}], "}"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rm", "=", 
   RowBox[{"IdentityMatrix", "[", "nn", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rm", "=", 
   RowBox[{
    RowBox[{"Transpose", "[", "Umat", "]"}], ".", "rm"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.632776862409873*^9, 3.632776863184731*^9}, {
   3.641242628367757*^9, 3.64124262889077*^9}, {3.8928323071317387`*^9, 
   3.892832364911133*^9}, {3.892837314954664*^9, 3.892837324986152*^9}, 
   3.8932555822163296`*^9, 3.8934178485367823`*^9, 3.893417901556346*^9, 
   3.915402523974275*^9, {3.9157473847309127`*^9, 3.915747384739691*^9}, {
   3.915747424739565*^9, 3.9157474972358155`*^9}},
 CellLabel->"In[28]:=",ExpressionUUID->"0b2a4286-60b9-4425-9083-f7f6665565e8"],

Cell[CellGroupData[{

Cell["Vertices of Gosset polytope in rotated lattice", "Subsubsection",
 CellChangeTimes->{{3.632777327674056*^9, 3.632777339308816*^9}, {
  3.641242604057197*^9, 3.6412426077562113`*^9}, {3.892832209639114*^9, 
  3.892832214566155*^9}, {3.892832389561743*^9, 
  3.8928324315570583`*^9}},ExpressionUUID->"d4a2eb29-1f50-42bb-af9c-\
a0abed12a4b8"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"GossetRot", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"rm", ".", 
      RowBox[{"GossetVerts", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "GossetVerts", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"GossetRot", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", "#", "]"}], "&"}], "/@", "GossetRot"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "[", "GossetRot", "]"}]}], "Input",
 CellChangeTimes->{{3.632777327674056*^9, 3.632777339308816*^9}, {
  3.641242604057197*^9, 3.6412426077562113`*^9}, {3.892832209639114*^9, 
  3.892832214566155*^9}, {3.892832389561743*^9, 3.892832407936264*^9}, {
  3.892832648919075*^9, 3.892832650639435*^9}, {3.915747543679309*^9, 
  3.9157475553567133`*^9}},
 CellLabel->"In[39]:=",ExpressionUUID->"89a17c7b-cd9b-46e7-8c2b-de93e7394117"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"240", ",", "8"}], "}"}]], "Output",
 CellChangeTimes->{
  3.9157475608848867`*^9, {3.915747595619994*^9, 3.915747623382553*^9}, 
   3.915747759369989*^9, {3.915748441985058*^9, 3.9157484710957375`*^9}, 
   3.915831034277899*^9, 3.916268972991191*^9, 3.916522666489238*^9, 
   3.9165258852830706`*^9},
 CellLabel->"Out[41]=",ExpressionUUID->"db35e724-be14-404c-a53c-b2aec7c4b97a"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Vertices projected to 4D ES, to two 600-cells", "Subsection",
 CellChangeTimes->{{3.632777327674056*^9, 3.632777339308816*^9}, {
  3.641242604057197*^9, 3.6412426077562113`*^9}, {3.892832209639114*^9, 
  3.892832214566155*^9}, {3.892832389561743*^9, 3.8928324315570583`*^9}, {
  3.892832476389106*^9, 3.892832485680717*^9}, {3.898350172427808*^9, 
  3.8983501803826675`*^9}, {3.89966223438468*^9, 
  3.8996622352675653`*^9}},ExpressionUUID->"a9e82749-82c0-4e4d-a92b-\
6f302cf49530"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"GossetRotProj", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"GossetRot", "[", 
      RowBox[{"[", 
       RowBox[{"j", ",", 
        RowBox[{
         RowBox[{"dd", "+", "1"}], ";;", "nn"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "GossetRot", "]"}]}], "}"}]}], "]"}]}], 
  RowBox[{"(*", 
   RowBox[{"//", "N"}], "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"GossetRotProj", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.63277747110175*^9, 3.632777485426279*^9}, 
   3.641242674570004*^9, {3.6417553218473673`*^9, 3.64175532253398*^9}, 
   3.645475525427753*^9, {3.7078569956738243`*^9, 3.707856996741199*^9}, {
   3.71916686102346*^9, 3.719166874588244*^9}, {3.719167101855503*^9, 
   3.7191671022662992`*^9}, {3.719168239668322*^9, 3.719168241534013*^9}, {
   3.738611004073086*^9, 3.738611004666114*^9}, {3.822166148835148*^9, 
   3.82216615375734*^9}, 3.826217901688211*^9, {3.892832493419621*^9, 
   3.892832510476243*^9}, {3.893424673946606*^9, 3.8934246852694626`*^9}, {
   3.8934247185430117`*^9, 3.8934247221556635`*^9}, 3.9154049835504065`*^9, {
   3.915669453810988*^9, 3.9156694627730513`*^9}},
 CellLabel->"In[42]:=",ExpressionUUID->"20ee3b41-b619-4c2a-9673-f89b38b20893"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"240", ",", "4"}], "}"}]], "Output",
 CellChangeTimes->{
  3.915669469112619*^9, 3.915670539942278*^9, {3.915747600552272*^9, 
   3.915747623420136*^9}, 3.9157477594397373`*^9, {3.915748442026326*^9, 
   3.9157484711447177`*^9}, 3.9158310343306255`*^9, 3.9162689730308685`*^9, 
   3.9165226665288515`*^9, 3.9165258853591204`*^9},
 CellLabel->"Out[43]=",ExpressionUUID->"b743f039-f3fd-4d27-ab24-41295c5c4dea"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
In 8D all vertices have the same magnitude.
Under this projection, they split into two 600-cells, with two different \
circumradii.
We select the smaller one as our 600-cell to work with.\
\>", "Item",
 CellChangeTimes->{{3.915669788061097*^9, 
  3.915669798237962*^9}},ExpressionUUID->"9f600ff3-2334-4aef-b312-\
f29f04ead1a5"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Norm", "[", 
    RowBox[{"N", "@", 
     RowBox[{"GossetRot", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"Norm", "[", 
    RowBox[{"N", "@", 
     RowBox[{"GossetRot", "[", 
      RowBox[{"[", "100", "]"}], "]"}]}], "]"}]}], 
  "}"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Norm", "[", 
    RowBox[{"N", "@", 
     RowBox[{"GossetRotProj", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"Norm", "[", 
    RowBox[{"N", "@", 
     RowBox[{"GossetRotProj", "[", 
      RowBox[{"[", "100", "]"}], "]"}]}], "]"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.892832562606576*^9, 3.892832570065488*^9}, {
   3.892832666669796*^9, 3.892832697517551*^9}, {3.893257405515532*^9, 
   3.893257454022188*^9}, {3.8932574967716103`*^9, 3.893257505427105*^9}, {
   3.893424757834543*^9, 3.8934247661305785`*^9}, {3.898341339022087*^9, 
   3.898341393273512*^9}, 3.915404991788216*^9, {3.915669482747769*^9, 
   3.9156695298786793`*^9}, {3.9156696449034815`*^9, 3.9156696480498734`*^9}},
 
 CellLabel->"In[44]:=",ExpressionUUID->"ff5ded62-7e20-4df2-bee8-28bdf389ad9c"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.414213562373095`", ",", "1.414213562373095`"}], "}"}]], "Output",\

 CellChangeTimes->{
  3.8989714875111217`*^9, 3.898989344492425*^9, 3.898989471254105*^9, 
   3.899288101905275*^9, 3.8992963016061816`*^9, 3.8992963441507187`*^9, 
   3.899297205375804*^9, 3.899297740307482*^9, 3.899336067045665*^9, 
   3.8993384200482764`*^9, 3.8993780241579957`*^9, 3.899466332210703*^9, 
   3.899480423268284*^9, 3.899480659638646*^9, 3.8995089186626673`*^9, 
   3.8995781458130507`*^9, 3.89958247631411*^9, 3.899661611718973*^9, 
   3.899893718720402*^9, 3.906216541611761*^9, 3.910201289565585*^9, 
   3.910263502274057*^9, 3.9103496148095303`*^9, 3.9104402776578565`*^9, 
   3.9107907915958347`*^9, 3.910828217268613*^9, 3.9109557918639793`*^9, 
   3.9109647164452486`*^9, 3.9110022787820616`*^9, 3.9114949066427283`*^9, 
   3.911652838074234*^9, 3.912085141447151*^9, 3.9121119330550327`*^9, 
   3.9123805748013735`*^9, 3.912602996838894*^9, 3.914865559445651*^9, 
   3.915197905431712*^9, 3.91520020074992*^9, 3.9154022229056287`*^9, 
   3.9154431395598927`*^9, 3.915475809675335*^9, 3.915541249927762*^9, 
   3.915551826290741*^9, 3.9155560228440056`*^9, 3.915559774002328*^9, 
   3.9156648988670316`*^9, 3.915668943626899*^9, 3.9156694836672373`*^9, 
   3.9156695309313173`*^9, 3.9156705399990997`*^9, {3.915747607700452*^9, 
   3.9157476234511833`*^9}, 3.915747759473295*^9, {3.915748442063873*^9, 
   3.9157484711758704`*^9}, 3.915831034357236*^9, 3.9162689730438933`*^9, 
   3.9165226665358944`*^9, 3.916525885421256*^9},
 CellLabel->"Out[44]=",ExpressionUUID->"4ea19a54-6177-46f5-bb15-a928bb05f7b4"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.743496068920369`", ",", "1.2030019100150913`"}], "}"}]], "Output",\

 CellChangeTimes->{
  3.8989714875111217`*^9, 3.898989344492425*^9, 3.898989471254105*^9, 
   3.899288101905275*^9, 3.8992963016061816`*^9, 3.8992963441507187`*^9, 
   3.899297205375804*^9, 3.899297740307482*^9, 3.899336067045665*^9, 
   3.8993384200482764`*^9, 3.8993780241579957`*^9, 3.899466332210703*^9, 
   3.899480423268284*^9, 3.899480659638646*^9, 3.8995089186626673`*^9, 
   3.8995781458130507`*^9, 3.89958247631411*^9, 3.899661611718973*^9, 
   3.899893718720402*^9, 3.906216541611761*^9, 3.910201289565585*^9, 
   3.910263502274057*^9, 3.9103496148095303`*^9, 3.9104402776578565`*^9, 
   3.9107907915958347`*^9, 3.910828217268613*^9, 3.9109557918639793`*^9, 
   3.9109647164452486`*^9, 3.9110022787820616`*^9, 3.9114949066427283`*^9, 
   3.911652838074234*^9, 3.912085141447151*^9, 3.9121119330550327`*^9, 
   3.9123805748013735`*^9, 3.912602996838894*^9, 3.914865559445651*^9, 
   3.915197905431712*^9, 3.91520020074992*^9, 3.9154022229056287`*^9, 
   3.9154431395598927`*^9, 3.915475809675335*^9, 3.915541249927762*^9, 
   3.915551826290741*^9, 3.9155560228440056`*^9, 3.915559774002328*^9, 
   3.9156648988670316`*^9, 3.915668943626899*^9, 3.9156694836672373`*^9, 
   3.9156695309313173`*^9, 3.9156705399990997`*^9, {3.915747607700452*^9, 
   3.9157476234511833`*^9}, 3.915747759473295*^9, {3.915748442063873*^9, 
   3.9157484711758704`*^9}, 3.915831034357236*^9, 3.9162689730438933`*^9, 
   3.9165226665358944`*^9, 3.916525885423798*^9},
 CellLabel->"Out[45]=",ExpressionUUID->"d7293dae-ccb0-4d5d-a54f-6684242be3c5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"C600", "=", 
    RowBox[{"N", "@", 
     RowBox[{"Select", "[", 
      RowBox[{"GossetRotProj", ",", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"N", "[", 
          RowBox[{"Norm", "[", "#", "]"}], "]"}], "<", "1"}], "&"}]}], 
      "]"}]}]}], ")"}], " ", "//", " ", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.89352807974889*^9, 3.8935280800691266`*^9}, {
   3.8989891565554113`*^9, 3.8989891615522165`*^9}, {3.915404862230647*^9, 
   3.915404868364679*^9}, 3.9154050475334744`*^9, 3.9156695694680867`*^9, {
   3.9156696096138086`*^9, 3.915669617737944*^9}, 3.915670066207077*^9, 
   3.9157476933474836`*^9},
 CellLabel->"In[46]:=",ExpressionUUID->"91cffd44-60b9-4d43-98dc-4bc336da095b"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"120", ",", "4"}], "}"}]], "Output",
 CellChangeTimes->{
  3.898971487544632*^9, 3.8989891649570875`*^9, 3.8989893445522165`*^9, 
   3.898989471301665*^9, 3.8992881019332757`*^9, 3.899296301641692*^9, 
   3.899296344177941*^9, 3.8992972054144*^9, 3.899297740343999*^9, 
   3.899336067077325*^9, 3.8993384200843706`*^9, 3.899378024192995*^9, 
   3.899466332248285*^9, 3.8994804233032846`*^9, 3.899480659686222*^9, 
   3.899508918688697*^9, 3.8995781458515058`*^9, 3.89958247636514*^9, 
   3.899661611752146*^9, 3.8998937187559166`*^9, 3.9062165416367617`*^9, 
   3.910201289590583*^9, 3.9102635023074903`*^9, 3.910349614834056*^9, 
   3.9104402776834435`*^9, 3.910790791621834*^9, 3.9108282173041344`*^9, 
   3.9109557919246655`*^9, 3.9109647164698596`*^9, 3.911002278806681*^9, 
   3.9114949067013435`*^9, 3.9116528381002345`*^9, 3.912085141472649*^9, 
   3.912111933093443*^9, 3.9123805748416777`*^9, 3.912602996876898*^9, 
   3.914865559479351*^9, 3.9151979054723697`*^9, 3.9152002007815332`*^9, 
   3.915402222945758*^9, 3.9154431396014338`*^9, 3.9154758097212234`*^9, 
   3.9155412499672804`*^9, 3.915551826336906*^9, 3.9155560228831673`*^9, 
   3.915559774035118*^9, 3.915664898958302*^9, 3.9156689436441092`*^9, 
   3.915669618951976*^9, 3.9156705400577126`*^9, 3.9157476234857883`*^9, 
   3.915747759508356*^9, {3.9157484421040387`*^9, 3.9157484712104354`*^9}, 
   3.915831034390461*^9, 3.916268973069455*^9, 3.9165226665589256`*^9, 
   3.9165258854915285`*^9},
 CellLabel->"Out[46]=",ExpressionUUID->"801b4f42-953b-4aa2-bac3-97583caad53f"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elser-Sloane space basis", "Item",
 CellChangeTimes->{{3.894647684621565*^9, 3.894647701138629*^9}, {
  3.898172070392007*^9, 3.898172073129614*^9}, {3.8983604294202356`*^9, 
  3.898360430092355*^9}, {3.8986291369585567`*^9, 3.8986291378519144`*^9}, {
  3.9157554753476467`*^9, 
  3.9157554812404284`*^9}},ExpressionUUID->"b97d4f58-bc48-45bb-8270-\
d3a62cab5c1e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"basisES", " ", "=", " ", 
   RowBox[{"MapThread", "[", 
    RowBox[{"Join", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{"0", ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "4"}], "}"}]}], "]"}], ",", 
       RowBox[{"Permutations", "[", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}]}], "}"}]}],
     " ", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.893255800021227*^9, 3.8932558035453577`*^9}, {
   3.8932558469965773`*^9, 3.893255865889877*^9}, {3.8932558999611254`*^9, 
   3.8932559391689687`*^9}, {3.893255994327957*^9, 3.893256125615147*^9}, {
   3.893256734372697*^9, 3.8932567381645775`*^9}, {3.8932569419890594`*^9, 
   3.8932569994920235`*^9}, {3.893257055471716*^9, 3.893257063670802*^9}, 
   3.893257733756735*^9, {3.8932578682502303`*^9, 3.893257871975005*^9}, 
   3.893257913760666*^9, {3.8933268634546432`*^9, 3.8933268709762497`*^9}, {
   3.8933289789319735`*^9, 3.8933289985116024`*^9}, {3.893330296596553*^9, 
   3.8933303028088093`*^9}, {3.8934174178777475`*^9, 
   3.8934174251735125`*^9}, {3.8934174862272387`*^9, 3.893417521530652*^9}, {
   3.894647728229718*^9, 3.894647730831812*^9}, {3.898341503095714*^9, 
   3.8983415046582413`*^9}, {3.89836042112673*^9, 3.898360422942584*^9}, {
   3.8988624837152853`*^9, 3.8988625112628117`*^9}, {3.8992892550878787`*^9, 
   3.899289276124056*^9}, 3.9156698386828856`*^9, 3.9157554717718983`*^9},
 CellLabel->"In[47]:=",ExpressionUUID->"818cdbed-0975-4466-b386-bc850ffc2ecd"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select some vertex as the pole in C600, and select a nearest neighbor;
sort vertices by distance from pole.\
\>", "Subsubsection",
 CellChangeTimes->{{3.8989624559497743`*^9, 3.89896249950146*^9}, {
   3.8989625764596987`*^9, 3.89896258148739*^9}, {3.915402893671769*^9, 
   3.915402896027979*^9}, 3.9156700570627284`*^9, {3.915670661074999*^9, 
   3.9156707262504745`*^9}, 
   3.9157476933564887`*^9},ExpressionUUID->"1ce956d3-c2b1-4d74-a9f6-\
e2c24e23b7be"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"poleVertex", " ", "=", " ", 
     RowBox[{"N", "@", 
      RowBox[{"C600", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], ";"}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"poleVertex", " ", "=", " ", 
    RowBox[{"First", "@", 
     RowBox[{"N", "@", 
      RowBox[{"MaximalBy", "[", 
       RowBox[{"C600", ",", 
        RowBox[{
         RowBox[{"#", ".", 
          RowBox[{"{", 
           RowBox[{"1.", ",", "0", ",", "0", ",", "0"}], "}"}]}], "&"}]}], 
       "]"}]}]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"C600", " ", "=", " ", 
     RowBox[{"SortBy", "[", 
      RowBox[{
       RowBox[{"N", "@", "C600"}], ",", 
       RowBox[{
        RowBox[{"Norm", "[", 
         RowBox[{"#", "-", "poleVertex"}], "]"}], "&"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"poleVertNeighbor", " ", "=", " ", 
     RowBox[{"C600", "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.8989651817228785`*^9, 3.8989652415953465`*^9}, {
   3.8989654863345656`*^9, 3.898965544354187*^9}, {3.8989655883173447`*^9, 
   3.8989656528307457`*^9}, 3.8989894318432703`*^9, {3.915670050498674*^9, 
   3.9156700505106525`*^9}, {3.9156701030360346`*^9, 
   3.9156701290920467`*^9}, {3.915747693362502*^9, 3.9157476933740697`*^9}},
 CellLabel->"In[48]:=",ExpressionUUID->"a6b1f83c-485a-48e7-a903-be424d643e21"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.7434960689203689`", ",", "0.`", ",", "0.`", ",", "0.`"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.898989433544649*^9, 3.8989894714262447`*^9, 3.8992881020338087`*^9, 
   3.899296301749773*^9, 3.899296344275784*^9, 3.8992972055192146`*^9, 
   3.8992977404512725`*^9, 3.899336067178688*^9, 3.899338420197235*^9, 
   3.8993780243029013`*^9, 3.8994663323569083`*^9, 3.8994804234188404`*^9, 
   3.8994806598306518`*^9, 3.899508918779707*^9, 3.8995781459366813`*^9, 
   3.899582476510325*^9, 3.899661611860799*^9, 3.89989371887094*^9, 
   3.906216541757979*^9, 3.910201289681539*^9, 3.9102635024189243`*^9, 
   3.9103496149251237`*^9, 3.9104402777775354`*^9, 3.9107907917160587`*^9, 
   3.9108282174178133`*^9, 3.9109557921367197`*^9, 3.910964716558544*^9, 
   3.911002278902425*^9, 3.9114949068642445`*^9, 3.9116528381919575`*^9, 
   3.912085141559574*^9, 3.9121119332010493`*^9, 3.9123805749755397`*^9, 
   3.9126029970078683`*^9, 3.914865559585289*^9, 3.9151979055879517`*^9, 
   3.9152002008880453`*^9, 3.915402223081314*^9, 3.915443139652646*^9, 
   3.9154758098188696`*^9, 3.9155412500211544`*^9, 3.915551826394611*^9, 
   3.9155560229309263`*^9, 3.915559774079199*^9, 3.9156648989995723`*^9, 
   3.9156689436632442`*^9, 3.915670540120907*^9, 3.91574762352748*^9, 
   3.915747759547984*^9, {3.91574844214909*^9, 3.915748471247013*^9}, 
   3.915831034428014*^9, 3.9162689730850086`*^9, 3.9165226665806923`*^9, 
   3.9165258855577765`*^9},
 CellLabel->"Out[48]=",ExpressionUUID->"7fb30628-f505-4ce1-bbc7-801449433140"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["From C600 vertices, define 5-fold isoclinic rotation in ES", \
"Subsection",
 CellChangeTimes->{{3.8983502152796335`*^9, 3.898350217569109*^9}, {
   3.899662249899629*^9, 3.899662261112939*^9}, {3.9154029089066305`*^9, 
   3.9154029223779335`*^9}, 3.9156700505206823`*^9, 
   3.915747693381341*^9},ExpressionUUID->"03250c36-16d8-4480-afb8-\
a5beff13fea9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Start", " ", "with", " ", "10"}], "-", 
    RowBox[{"fold", " ", "rotation", " ", "in", " ", "a", " ", "decagon", " ", 
     RowBox[{"plane", " ", "--"}], " ", "two", " ", "neighboring", " ", 
     RowBox[{"vertices", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"rot1ES", " ", "=", " ", 
     RowBox[{"RotationMatrix", "[", 
      RowBox[{"{", 
       RowBox[{"poleVertex", ",", "poleVertNeighbor"}], "}"}], "]"}]}], ";"}],
    " ", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"Get", " ", "5"}], "-", 
     RowBox[{
     "fold", " ", "rotation", " ", "by", " ", "doubling", " ", "the", " ", 
      "10"}], "-", 
     RowBox[{"fold", " ", 
      RowBox[{"one", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rot1ES", " ", "=", " ", 
     RowBox[{
      RowBox[{"rot1ES", ".", "rot1ES"}], " ", "//", " ", "Chop"}]}], ";"}], 
   " ", "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
     "Make", " ", "isoclinic", " ", "rotation", " ", "by", " ", "combining", 
      " ", "5"}], "-", 
     RowBox[{
     "fold", " ", "rotation", " ", "with", " ", "its", " ", "dual"}]}], 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rotIsoES", " ", "=", " ", 
     RowBox[{
      RowBox[{"rot1ES", ".", 
       RowBox[{"rotMatrixToDual4D", "[", "rot1ES", "]"}]}], "//", "Chop"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.915402960543377*^9, 3.9154030788153934`*^9}, {
  3.9154062566733465`*^9, 3.915406258296296*^9}, {3.915406609705755*^9, 
  3.915406610622629*^9}, {3.91567010304608*^9, 3.9156701290960455`*^9}, {
  3.9156707358687277`*^9, 3.915670745447976*^9}},
 CellLabel->"In[51]:=",ExpressionUUID->"a7aafcb0-91e6-4562-93e8-afd4b98d668d"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["600-cell structures", "Section",
 CellChangeTimes->{{3.893173320983095*^9, 3.893173324487362*^9}, 
   3.8934180983966227`*^9, {3.899204347465096*^9, 3.899204353860535*^9}, 
   3.8993378957508974`*^9, 
   3.9156701880866528`*^9},ExpressionUUID->"b4a2536c-e0c2-4b8b-ac5d-\
342220b257b7"],

Cell[CellGroupData[{

Cell["\<\
Just for reference, what are the angles between tetrahedron cell normals in \
the 600-cell?\
\>", "Item",
 CellChangeTimes->{{3.912087541445716*^9, 3.912087559063368*^9}, {
  3.912112260777844*^9, 3.9121122636076794`*^9}, {3.915403159782942*^9, 
  3.915403160967374*^9}},ExpressionUUID->"fbc8f8e4-cfb1-4a4c-a8e6-\
7adfe4099ec0"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C600TetInds", " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"ConvexPolytope", "[", "C600", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "[", "\"\<FaceVertexIndices\>\"", "]"}],
     "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"C600TetNormals", " ", "=", " ", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normalize", "@", 
       RowBox[{"Mean", "@", 
        RowBox[{"C600", "[", 
         RowBox[{"[", "#", "]"}], "]"}]}]}], "&"}], ",", "C600TetInds"}], 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.912111567632187*^9, 3.9121116010899873`*^9}, {
   3.9121117713360124`*^9, 3.9121118049744406`*^9}, 3.912112110172609*^9, {
   3.915670050525491*^9, 3.9156700505335226`*^9}, {3.915747693386346*^9, 
   3.91574769339686*^9}},
 CellLabel->"In[54]:=",ExpressionUUID->"62a84572-48ec-48c3-902a-45ed3a6998e8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Round", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VectorAngle", "[", 
          RowBox[{
           RowBox[{"C600TetNormals", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "]"}], "/", 
         "Degree"}], ",", 
        RowBox[{"10.", "^", 
         RowBox[{"-", "4"}]}]}], "]"}], "&"}], ",", "C600TetNormals"}], "]"}],
    "//", "Tally"}], "//", "Column"}]], "Input",
 CellChangeTimes->{{3.912087336934007*^9, 3.9120874257176285`*^9}, {
   3.9120874722893333`*^9, 3.912087485889993*^9}, {3.912087566740389*^9, 
   3.912087568412692*^9}, {3.9121121236813*^9, 3.912112125328459*^9}, {
   3.9156700505355225`*^9, 3.9156700505385237`*^9}, {3.915670877356496*^9, 
   3.9156709017707844`*^9}, 3.9156709319082513`*^9, {3.915747693399866*^9, 
   3.915747693401866*^9}},
 CellLabel->"In[56]:=",ExpressionUUID->"ab741433-ec84-40bb-bd2d-13d398715446"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"15.5225`", ",", "4"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"25.242800000000003`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"36.`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"41.409600000000005`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"44.4775`", ",", "4"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"49.1176`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"56.0122`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"60.`", ",", "32"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"66.1397`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"72.`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"75.52250000000001`", ",", "28"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"69.7882`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"81.1117`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"84.52040000000001`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"90.`", ",", "54"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"95.4796`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"98.8883`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"104.4775`", ",", "28"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"108.`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"110.21180000000001`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"113.86030000000001`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"120.`", ",", "32"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"123.98780000000001`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"130.88240000000002`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"135.5225`", ",", "4"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"138.59040000000002`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"144.`", ",", "24"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"154.7572`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"164.47750000000002`", ",", "4"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"180.`", ",", "1"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output",
 CellChangeTimes->{{3.9156709130670767`*^9, 3.9156709345283747`*^9}, 
   3.9157476238661118`*^9, 3.915747759951101*^9, {3.9157484424795322`*^9, 
   3.915748471661066*^9}, 3.915831034795771*^9, 3.916268973497264*^9, 
   3.9165226669908476`*^9, 3.916525885976302*^9},
 CellLabel->"Out[56]=",ExpressionUUID->"1e33a578-b2b5-4da1-8849-acf35905ff44"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[". . . and what are the angles between vertices?", "Item",
 CellChangeTimes->{{3.912087541445716*^9, 3.912087559063368*^9}, {
  3.912112260777844*^9, 3.9121122636076794`*^9}, {3.9123807869506164`*^9, 
  3.9123807986728373`*^9}},ExpressionUUID->"0816ea0b-7021-49e6-92b1-\
bc81c0e5812e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Round", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VectorAngle", "[", 
          RowBox[{
           RowBox[{"C600", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "]"}], "/", 
         "Degree"}], ",", 
        RowBox[{"10.", "^", 
         RowBox[{"-", "8"}]}]}], "]"}], "&"}], ",", "C600"}], "]"}], "//", 
   "Tally"}], "//", "Column"}]], "Input",
 CellChangeTimes->{{3.915670803813241*^9, 3.915670846195958*^9}, {
  3.915747693404871*^9, 3.9157476934072924`*^9}},
 CellLabel->"In[57]:=",ExpressionUUID->"b2cf49db-e9f3-429b-9f54-2230ee164998"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"36.`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"60.`", ",", "20"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"72.`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"90.`", ",", "30"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"108.`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"120.`", ",", "20"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"144.`", ",", "12"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"180.`", ",", "1"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output",
 CellChangeTimes->{{3.915670840875948*^9, 3.915670847375476*^9}, 
   3.915747623898628*^9, 3.9157477599776506`*^9, {3.9157484425211773`*^9, 
   3.9157484716860895`*^9}, 3.9158310348222857`*^9, 3.9162689735077734`*^9, 
   3.9165226670123687`*^9, 3.91652588601782*^9},
 CellLabel->"Out[57]=",ExpressionUUID->"ab2cce00-17c7-452f-a5a7-e433bb5c5196"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Select a vertex cap and a single tetrahedron.", "Subsection",
 CellChangeTimes->{{3.8889406915509615`*^9, 3.8889407392770786`*^9}, {
   3.8889409528240423`*^9, 3.8889409574171305`*^9}, 3.8914303746288147`*^9, {
   3.8989898413365617`*^9, 3.8989898523440466`*^9}, {3.8989898955483065`*^9, 
   3.898989909388343*^9}, {3.9154031850449715`*^9, 
   3.915403191634407*^9}},ExpressionUUID->"9d8732e1-01bd-490e-a89e-\
8cc401485be5"],

Cell["\<\
Since the C600 vertices are sorted, the first 12 + 1 will be a vertex cap (12 \
vertices of an icosahedron, plus the central vertex, which is not in the same \
affine 3D space as the 12).
Then get a \[OpenCurlyQuote]wheel\[CloseCurlyQuote] of 5 tetrahedra sharing \
an edge, and from them select one.\
\>", "Text",
 CellChangeTimes->{{3.89946691873199*^9, 3.8994669578439665`*^9}, {
  3.8994670049437013`*^9, 3.899467041091424*^9}, {3.915403211857349*^9, 
  3.9154032270419292`*^9}, {3.9154032686288385`*^9, 3.9154033253178406`*^9}, {
  3.915755516630803*^9, 
  3.9157555425269113`*^9}},ExpressionUUID->"916ec2b7-fdc6-4d6d-add2-\
566a5868b058"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "13", " ", "vertices", " ", "of", " ", "a", " ", "vertex", " ", "cap"}], 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"vCap", " ", "=", " ", 
     RowBox[{"C600", "[", 
      RowBox[{"[", 
       RowBox[{";;", "13"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
    "7", " ", "vertices", " ", "of", " ", "a", " ", "wheel", " ", "of", " ", 
     "5", " ", "tetrahedra", " ", "within", " ", "this", " ", "vertex", " ", 
     "cap"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"n5G", " ", "=", " ", 
     RowBox[{"PolygonVertexSort", "@", 
      RowBox[{"MinimalBy", "[", 
       RowBox[{
        RowBox[{"vCap", "[", 
         RowBox[{"[", 
          RowBox[{"3", ";;"}], "]"}], "]"}], ",", 
        RowBox[{
         RowBox[{"Round", "[", 
          RowBox[{
           RowBox[{"Norm", "[", 
            RowBox[{"#", "-", 
             RowBox[{"vCap", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
           RowBox[{"10.", "^", 
            RowBox[{"-", "4"}]}]}], "]"}], "&"}]}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
    "4", " ", "vertices", " ", "of", " ", "a", " ", "single", " ", 
     "tetrahedron", " ", "in", " ", "this", " ", "5", "G"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"mainTet", " ", "=", " ", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"vCap", "[", 
        RowBox[{"[", 
         RowBox[{";;", "2"}], "]"}], "]"}], ",", 
       RowBox[{"n5G", "[", 
        RowBox[{"[", 
         RowBox[{";;", "2"}], "]"}], "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{"Normal", " ", "from", " ", "the", " ", "600"}], "-", 
     RowBox[{
     "cell", " ", "centroid", " ", "to", " ", "this", " ", "tetrahedron"}]}], 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"mainTetNormal", " ", "=", " ", 
     RowBox[{"Mean", "@", "mainTet"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.8928404762487464`*^9, 3.8928405849861803`*^9}, {
   3.892840617746109*^9, 3.892840638683945*^9}, {3.8928407503510947`*^9, 
   3.892840905611626*^9}, {3.8928409498944097`*^9, 3.892841072958756*^9}, {
   3.892841190101823*^9, 3.8928411935188756`*^9}, {3.8928412270589857`*^9, 
   3.8928412555672255`*^9}, {3.892841297132055*^9, 3.892841372256328*^9}, {
   3.8928424783081036`*^9, 3.892842493991125*^9}, 3.8929031106605177`*^9, {
   3.8929031460089836`*^9, 3.8929031773490458`*^9}, {3.8934181617507954`*^9, 
   3.8934181737758317`*^9}, {3.898965963119591*^9, 3.8989659916362267`*^9}, 
   3.8989660232192564`*^9, {3.89896611744409*^9, 3.898966200064325*^9}, {
   3.8989899264326134`*^9, 3.8989899290294113`*^9}, {3.899336407863632*^9, 
   3.8993364087464266`*^9}, {3.8994670734544983`*^9, 
   3.8994670748586574`*^9}, {3.9154033737872744`*^9, 3.915403500137906*^9}, {
   3.9154035392324467`*^9, 3.9154035653205533`*^9}, {3.915403669433365*^9, 
   3.9154036705217876`*^9}, {3.915669979972006*^9, 3.9156699927375703`*^9}, 
   3.915670050545561*^9, {3.915670222034137*^9, 3.915670227756872*^9}, 
   3.9157476934102907`*^9},
 CellLabel->"In[58]:=",ExpressionUUID->"fb19ba39-c08b-4fae-8c91-7cc6f8a7ac80"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Select one 24-cell, then generate a set of 5 24-cells that partitions the \
vertices of C600\
\>", "Subsection",
 CellChangeTimes->{{3.8890346128592815`*^9, 3.889034615675828*^9}, {
   3.8896569037604775`*^9, 3.8896569610595217`*^9}, {3.889875753725093*^9, 
   3.889875758733642*^9}, {3.8902204423191614`*^9, 3.890220442687053*^9}, 
   3.890227959474351*^9, {3.8902534307936115`*^9, 3.8902534596304693`*^9}, 
   3.8902537093852434`*^9, 3.8902542141868286`*^9, {3.8913449274317923`*^9, 
   3.8913449965837593`*^9}, {3.8913453950780516`*^9, 3.8913453954952807`*^9}, 
   3.8993378493823833`*^9, {3.9156716232557273`*^9, 3.9156716590621166`*^9}, 
   3.915747693417328*^9, {3.9157482136943617`*^9, 
   3.9157482139247856`*^9}},ExpressionUUID->"342c6cfb-4651-486f-8e7d-\
36666e213c20"],

Cell[CellGroupData[{

Cell["\<\
Find 24-cell by layering the 600-cell parallel to the first tetrahedron\
\>", "Subsubsection",
 CellChangeTimes->{{3.8931564879923954`*^9, 3.8931564942764664`*^9}, {
   3.897652572893176*^9, 3.897652581770539*^9}, {3.8983581282072678`*^9, 
   3.898358140459672*^9}, {3.8993364413458633`*^9, 3.899336464906249*^9}, {
   3.899336542444311*^9, 3.8993365486030807`*^9}, {3.8994671076392784`*^9, 
   3.8994671252042456`*^9}, {3.899467337981901*^9, 3.8994673709916077`*^9}, {
   3.915200662390376*^9, 3.9152006633178787`*^9}, {3.915403683046759*^9, 
   3.915403693637785*^9}, 3.915748285908763*^9, {3.9157484006634502`*^9, 
   3.915748401655922*^9}},ExpressionUUID->"a815d779-560e-46f5-9d76-\
97213019f45c"],

Cell["The 600-cell layers parallel to one tetrahedron facet .", "Text",
 CellChangeTimes->{{3.898357375396351*^9, 3.898357382454221*^9}, 
   3.899336475935093*^9, {3.9154037056254296`*^9, 
   3.9154037074179087`*^9}},ExpressionUUID->"70ade3ad-2a99-49f5-a909-\
da2c9941c1fc"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"C600Layers", " ", "=", " ", 
    RowBox[{"SortBy", "[", 
     RowBox[{
      RowBox[{"GatherBy", "[", 
       RowBox[{"C600", ",", 
        RowBox[{
         RowBox[{"Round", "[", 
          RowBox[{
           RowBox[{"#", ".", "mainTetNormal"}], ",", 
           RowBox[{"10.", "^", 
            RowBox[{"-", "3"}]}]}], "]"}], "&"}]}], "]"}], " ", ",", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"First", "[", "#", "]"}], ".", "mainTetNormal"}], "&"}]}], 
     "]"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"layerHeights", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"#", ".", 
          RowBox[{"Normalize", "[", "mainTetNormal", "]"}]}], ")"}], "&"}], 
       " ", "@*", " ", "Mean"}], ")"}], "/@", " ", "C600Layers"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"Length", " ", "/@", " ", "C600Layers"}]}], "Input",
 CellChangeTimes->{{3.893169756149856*^9, 3.893169758362738*^9}, {
   3.8931702796995454`*^9, 3.8931703529757032`*^9}, {3.893170399129794*^9, 
   3.8931704163673153`*^9}, {3.893170499788173*^9, 3.8931706553233123`*^9}, {
   3.893170688764183*^9, 3.8931707229894485`*^9}, {3.8931709345457582`*^9, 
   3.893170966325918*^9}, {3.893171045812767*^9, 3.893171103911048*^9}, {
   3.8931711359853625`*^9, 3.8931712822524476`*^9}, {3.8931829441976385`*^9, 
   3.893182947369154*^9}, {3.8931829823601537`*^9, 3.893183026836734*^9}, {
   3.893183076757265*^9, 3.8931831140340185`*^9}, {3.8934182461373024`*^9, 
   3.8934182661176248`*^9}, {3.893768627720722*^9, 3.89376864546589*^9}, {
   3.8937691534529896`*^9, 3.893769192734983*^9}, {3.8939434502497168`*^9, 
   3.8939435514669085`*^9}, {3.893943622880253*^9, 3.8939436250860457`*^9}, 
   3.8939436834477196`*^9, 3.8939438529773245`*^9, {3.8939439261937237`*^9, 
   3.893943940157144*^9}, {3.8946293655174303`*^9, 3.894629395467299*^9}, 
   3.898989236254659*^9, 3.898989283897002*^9, {3.9156700505495586`*^9, 
   3.9156700505561*^9}, {3.915670230568755*^9, 3.9156702330702105`*^9}, {
   3.915671302046914*^9, 3.915671315148676*^9}, {3.915747693422838*^9, 
   3.915747693430354*^9}},
 CellLabel->"In[62]:=",ExpressionUUID->"35311f1d-d297-487c-a274-5f909ce15288"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "4", ",", "4", ",", "6", ",", "12", ",", "12", ",", "12", ",", "4", ",", 
   "12", ",", "4", ",", "12", ",", "12", ",", "12", ",", "6", ",", "4", ",", 
   "4"}], "}"}]], "Output",
 CellChangeTimes->{
  3.915671316956251*^9, 3.915747623946664*^9, 3.9157477600126867`*^9, {
   3.915748442569697*^9, 3.915748471719638*^9}, 3.9158310348558025`*^9, 
   3.916268973534296*^9, 3.9165226670413947`*^9, 3.9165258860724497`*^9},
 CellLabel->"Out[63]=",ExpressionUUID->"5bf68cfc-4188-41af-9162-590f1b36b794"]
}, Open  ]],

Cell["\<\
Layers 3 and 13 are upper and lower octahedra; Layer 8 is the cuboctahedron \
equator. These are all in parallel affine 3D spaces.
The vertices of layers 3, 8, and 13 together make a 24-cell (which has 24 \
vertices, as well as 24 facets).\
\>", "Text",
 CellChangeTimes->{{3.8937692051579866`*^9, 3.893769254920619*^9}, {
  3.8946294193203077`*^9, 3.8946294280102434`*^9}, {3.8983563555368423`*^9, 
  3.8983563575178747`*^9}, {3.898357544551324*^9, 3.898357565689469*^9}, {
  3.915403726845053*^9, 3.91540373202851*^9}, {3.9154039838323417`*^9, 
  3.9154040402972097`*^9}, {3.9157556053864813`*^9, 
  3.9157556198468575`*^9}},ExpressionUUID->"323067c0-e9f2-4d56-8dd3-\
a212e4f48a92"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"24", "-", 
    RowBox[{"cell", " ", "in", " ", "ES", " ", "space"}]}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"C24n1", " ", "=", " ", 
     RowBox[{"Join", "@@", 
      RowBox[{"C600Layers", "[", 
       RowBox[{"[", 
        RowBox[{"{", 
         RowBox[{"3", ",", "8", ",", "13"}], "}"}], "]"}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{"vertex", " ", "indices", " ", "of", " ", "the", " ", "24"}], 
     "-", "cell"}], " ", "*)"}], "\n", 
   RowBox[{
    RowBox[{"C24n1Inds", " ", "=", " ", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"ptToIndex", "[", 
         RowBox[{"#", ",", 
          RowBox[{"N", "@", "GossetRotProj"}]}], "]"}], "&"}], ",", "C24n1"}],
       "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
      RowBox[{"same", " ", "24"}], "-", "cell"}], ",", " ", 
     RowBox[{"lifted", " ", "to", " ", "8", "D"}]}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"C24n18D", " ", "=", " ", 
     RowBox[{"N", "@", 
      RowBox[{"GossetRot", "[", 
       RowBox[{"[", "C24n1Inds", "]"}], "]"}]}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.893169756149856*^9, 3.893169758362738*^9}, {
   3.8931702796995454`*^9, 3.8931703529757032`*^9}, {3.893170399129794*^9, 
   3.8931704163673153`*^9}, {3.893170499788173*^9, 3.8931706553233123`*^9}, {
   3.893170688764183*^9, 3.8931707229894485`*^9}, {3.8931709345457582`*^9, 
   3.893170966325918*^9}, {3.893171045812767*^9, 3.893171103911048*^9}, {
   3.8931711359853625`*^9, 3.8931712822524476`*^9}, {3.8931829441976385`*^9, 
   3.893182947369154*^9}, {3.8931829823601537`*^9, 3.893183026836734*^9}, {
   3.893183076757265*^9, 3.8931831140340185`*^9}, {3.8934182461373024`*^9, 
   3.8934182661176248`*^9}, {3.893768627720722*^9, 3.89376864546589*^9}, {
   3.8937691534529896`*^9, 3.893769192734983*^9}, {3.8944615834234543`*^9, 
   3.894461584644644*^9}, {3.8944616963696795`*^9, 3.8944617174598494`*^9}, {
   3.894461870373347*^9, 3.894461884053583*^9}, {3.894461941953245*^9, 
   3.8944619613824067`*^9}, {3.8962772438313336`*^9, 3.896277245275396*^9}, {
   3.896277282998388*^9, 3.896277322930066*^9}, 3.896435012414045*^9, {
   3.8964350879480505`*^9, 3.8964351067479753`*^9}, 3.896435325989939*^9, 
   3.896435472942235*^9, 3.896459599214779*^9, 3.898967112439242*^9, {
   3.8989673513371916`*^9, 3.89896737475898*^9}, {3.8989674049713683`*^9, 
   3.8989674072543736`*^9}, {3.8989675450514402`*^9, 3.8989675451812*^9}, 
   3.8993365147856064`*^9, 3.8993365626137457`*^9, {3.9154037491569843`*^9, 
   3.9154038877311616`*^9}, 3.915670050558642*^9, {3.9156702514816456`*^9, 
   3.9156702616772003`*^9}, 3.915747693434388*^9, {3.9157478711971397`*^9, 
   3.915747878998619*^9}, {3.915748143465275*^9, 3.9157481434758215`*^9}},
 CellLabel->"In[64]:=",ExpressionUUID->"fdd463f2-b54d-4ab6-b5ba-c5e63bfee9a6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Aside: Note that the 24-cell and the tetrahedron from the first layer ",
 StyleBox["lose their parallelism",
  FontSlant->"Italic"],
 " when lifted to 8D"
}], "Subsubsection",
 CellChangeTimes->{{3.8983573121862345`*^9, 3.898357337562481*^9}, {
   3.8983575995279016`*^9, 3.898357611050678*^9}, {3.899336994607152*^9, 
   3.8993370373806653`*^9}, {3.8993378024312677`*^9, 3.899337820168115*^9}, {
   3.899467329933853*^9, 3.8994673348145065`*^9}, {3.8994677492088375`*^9, 
   3.899467770716852*^9}, 3.8994678132959404`*^9, {3.899468044151078*^9, 
   3.899468080199815*^9}, {3.9154041222101727`*^9, 3.9154041342378793`*^9}, {
   3.9154051117004395`*^9, 
   3.915405116270915*^9}},ExpressionUUID->"90eeaa20-bdab-4aad-a4ee-\
b36753ae0e93"],

Cell["\<\
Consider the 24-cell layers and the original 600-cell tet facet (in ES, and \
lifted to 8D)\
\>", "Text",
 CellChangeTimes->{{3.8983573121862345`*^9, 3.898357337562481*^9}, {
  3.8983575995279016`*^9, 3.898357611050678*^9}, {3.899336994607152*^9, 
  3.8993370373806653`*^9}, {3.8993378024312677`*^9, 3.899337820168115*^9}, {
  3.899467329933853*^9, 3.8994673348145065`*^9}, {3.8994677492088375`*^9, 
  3.899467770716852*^9}, {3.8994678195147533`*^9, 
  3.899467834431328*^9}},ExpressionUUID->"7600f802-55f4-4b9e-8d96-\
dfae94c97bf1"],

Cell[CellGroupData[{

Cell["\<\
24-cell Layers and first-layer tetrahedron in ES \[LineSeparator](Are they \
parallel? -- translate to origin and check span)\
\>", "Item",
 CellChangeTimes->{{3.8993373191785707`*^9, 3.8993373245606065`*^9}, {
  3.8993373799449463`*^9, 3.899337385123185*^9}, {3.8993377066116295`*^9, 
  3.899337717741392*^9}, {3.899467415004958*^9, 3.899467431933756*^9}, {
  3.8994678713566885`*^9, 3.8994678791373296`*^9}, {3.915404154548073*^9, 
  3.9154041674261785`*^9}},ExpressionUUID->"b87d1904-b548-4eb9-acdd-\
7f1f6737dcab"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tet1", " ", "=", " ", 
   RowBox[{"C600Layers", "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"oct1", " ", "=", " ", 
   RowBox[{"C600Layers", "[", 
    RowBox[{"[", "3", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"cuboct1", " ", "=", " ", 
    RowBox[{"C600Layers", "[", 
     RowBox[{"[", "8", "]"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "tet1", "]"}]}], "&"}], ",", "tet1"}], "]"}], 
   "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "oct1", "]"}]}], "&"}], ",", "oct1"}], "]"}], 
   "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "cuboct1", "]"}]}], "&"}], ",", "cuboct1"}], 
    "]"}], "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", "  ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "tet1", "]"}]}], "&"}], ",", "tet1"}], "]"}], 
     ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "oct1", "]"}]}], "&"}], ",", "oct1"}], "]"}]}], 
    "]"}], "  ", "]"}], "[", "\"\<Dimension\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", "  ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "tet1", "]"}]}], "&"}], ",", "tet1"}], "]"}], 
     ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "cuboct1", "]"}]}], "&"}], ",", "cuboct1"}], 
      "]"}]}], "]"}], "  ", "]"}], "[", "\"\<Dimension\>\"", "]"}]}], "Input",\

 CellChangeTimes->{{3.8962763274845314`*^9, 3.8962764265415306`*^9}, {
   3.896276484190998*^9, 3.896276497597166*^9}, {3.896276536697822*^9, 
   3.8962765833515596`*^9}, {3.8962766370128365`*^9, 
   3.8962766897253723`*^9}, {3.896277523800494*^9, 3.8962775262912593`*^9}, {
   3.896436476865525*^9, 3.896436512252983*^9}, {3.896436547093547*^9, 
   3.8964365663507113`*^9}, {3.8964376495678005`*^9, 
   3.8964376590610943`*^9}, {3.898967112446243*^9, 3.8989671124492426`*^9}, {
   3.8989676061093273`*^9, 3.8989676254239826`*^9}, {3.8989676649045477`*^9, 
   3.8989676781527176`*^9}, 3.899337050645875*^9, {3.8993371150460997`*^9, 
   3.899337242755156*^9}, {3.899337434190097*^9, 3.8993374406437593`*^9}, 
   3.8993375218424935`*^9, {3.9156700505656424`*^9, 3.9156700505706444`*^9}, {
   3.915670277592685*^9, 3.9156702936890097`*^9}, {3.9157476934393883`*^9, 
   3.9157476934444237`*^9}},
 CellLabel->"In[67]:=",ExpressionUUID->"72e3771a-a26b-4b58-ae44-87abfd13790f"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{{3.8993372269059424`*^9, 3.89933724381315*^9}, 
   3.8993374633868914`*^9, 3.899338446931265*^9, 3.8993780245809956`*^9, 
   3.899466332685233*^9, 3.899480423695766*^9, 3.899480660174369*^9, 
   3.899508918987462*^9, 3.899578146200673*^9, 3.8995824787955236`*^9, 
   3.899661612147478*^9, 3.8998937191473117`*^9, 3.906216557010718*^9, 
   3.9102012899497643`*^9, 3.910263502732051*^9, 3.9103496151857457`*^9, 
   3.9104402780360036`*^9, 3.910790791978406*^9, 3.910828224953341*^9, 
   3.9109557925361004`*^9, 3.9109647168304663`*^9, 3.9110022791657057`*^9, 
   3.911494907290081*^9, 3.9116528385143366`*^9, 3.9120851418263316`*^9, 
   3.9121289868620305`*^9, 3.912380575687106*^9, 3.9126029976951904`*^9, 
   3.9148655601937385`*^9, 3.9151979064169436`*^9, 3.915200201483579*^9, 
   3.915402223789356*^9, 3.915443198550068*^9, 3.915475810479916*^9, 
   3.9155412506511393`*^9, 3.91555182706474*^9, 3.915556023503305*^9, 
   3.9155597746477947`*^9, 3.915664899605281*^9, 3.9156689443657475`*^9, 
   3.9156705408237658`*^9, 3.9156714529858847`*^9, 3.915747624014781*^9, 
   3.9157477600769434`*^9, {3.9157484426419945`*^9, 3.915748471783783*^9}, 
   3.9158310349204807`*^9, 3.9162689735903287`*^9, 3.916522667084078*^9, 
   3.9165258861236267`*^9},
 CellLabel->"Out[70]=",ExpressionUUID->"67fd5478-c882-4cd6-b799-cc57caf7640e"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{{3.8993372269059424`*^9, 3.89933724381315*^9}, 
   3.8993374633868914`*^9, 3.899338446931265*^9, 3.8993780245809956`*^9, 
   3.899466332685233*^9, 3.899480423695766*^9, 3.899480660174369*^9, 
   3.899508918987462*^9, 3.899578146200673*^9, 3.8995824787955236`*^9, 
   3.899661612147478*^9, 3.8998937191473117`*^9, 3.906216557010718*^9, 
   3.9102012899497643`*^9, 3.910263502732051*^9, 3.9103496151857457`*^9, 
   3.9104402780360036`*^9, 3.910790791978406*^9, 3.910828224953341*^9, 
   3.9109557925361004`*^9, 3.9109647168304663`*^9, 3.9110022791657057`*^9, 
   3.911494907290081*^9, 3.9116528385143366`*^9, 3.9120851418263316`*^9, 
   3.9121289868620305`*^9, 3.912380575687106*^9, 3.9126029976951904`*^9, 
   3.9148655601937385`*^9, 3.9151979064169436`*^9, 3.915200201483579*^9, 
   3.915402223789356*^9, 3.915443198550068*^9, 3.915475810479916*^9, 
   3.9155412506511393`*^9, 3.91555182706474*^9, 3.915556023503305*^9, 
   3.9155597746477947`*^9, 3.915664899605281*^9, 3.9156689443657475`*^9, 
   3.9156705408237658`*^9, 3.9156714529858847`*^9, 3.915747624014781*^9, 
   3.9157477600769434`*^9, {3.9157484426419945`*^9, 3.915748471783783*^9}, 
   3.9158310349204807`*^9, 3.9162689735903287`*^9, 3.916522667084078*^9, 
   3.9165258861246805`*^9},
 CellLabel->"Out[71]=",ExpressionUUID->"517895c2-c56e-4e4e-bd6c-7da89816e77a"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{{3.8993372269059424`*^9, 3.89933724381315*^9}, 
   3.8993374633868914`*^9, 3.899338446931265*^9, 3.8993780245809956`*^9, 
   3.899466332685233*^9, 3.899480423695766*^9, 3.899480660174369*^9, 
   3.899508918987462*^9, 3.899578146200673*^9, 3.8995824787955236`*^9, 
   3.899661612147478*^9, 3.8998937191473117`*^9, 3.906216557010718*^9, 
   3.9102012899497643`*^9, 3.910263502732051*^9, 3.9103496151857457`*^9, 
   3.9104402780360036`*^9, 3.910790791978406*^9, 3.910828224953341*^9, 
   3.9109557925361004`*^9, 3.9109647168304663`*^9, 3.9110022791657057`*^9, 
   3.911494907290081*^9, 3.9116528385143366`*^9, 3.9120851418263316`*^9, 
   3.9121289868620305`*^9, 3.912380575687106*^9, 3.9126029976951904`*^9, 
   3.9148655601937385`*^9, 3.9151979064169436`*^9, 3.915200201483579*^9, 
   3.915402223789356*^9, 3.915443198550068*^9, 3.915475810479916*^9, 
   3.9155412506511393`*^9, 3.91555182706474*^9, 3.915556023503305*^9, 
   3.9155597746477947`*^9, 3.915664899605281*^9, 3.9156689443657475`*^9, 
   3.9156705408237658`*^9, 3.9156714529858847`*^9, 3.915747624014781*^9, 
   3.9157477600769434`*^9, {3.9157484426419945`*^9, 3.915748471783783*^9}, 
   3.9158310349204807`*^9, 3.9162689735903287`*^9, 3.916522667084078*^9, 
   3.916525886126672*^9},
 CellLabel->"Out[72]=",ExpressionUUID->"336c6acc-0238-4db0-b412-a6811585ced6"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{{3.8993372269059424`*^9, 3.89933724381315*^9}, 
   3.8993374633868914`*^9, 3.899338446931265*^9, 3.8993780245809956`*^9, 
   3.899466332685233*^9, 3.899480423695766*^9, 3.899480660174369*^9, 
   3.899508918987462*^9, 3.899578146200673*^9, 3.8995824787955236`*^9, 
   3.899661612147478*^9, 3.8998937191473117`*^9, 3.906216557010718*^9, 
   3.9102012899497643`*^9, 3.910263502732051*^9, 3.9103496151857457`*^9, 
   3.9104402780360036`*^9, 3.910790791978406*^9, 3.910828224953341*^9, 
   3.9109557925361004`*^9, 3.9109647168304663`*^9, 3.9110022791657057`*^9, 
   3.911494907290081*^9, 3.9116528385143366`*^9, 3.9120851418263316`*^9, 
   3.9121289868620305`*^9, 3.912380575687106*^9, 3.9126029976951904`*^9, 
   3.9148655601937385`*^9, 3.9151979064169436`*^9, 3.915200201483579*^9, 
   3.915402223789356*^9, 3.915443198550068*^9, 3.915475810479916*^9, 
   3.9155412506511393`*^9, 3.91555182706474*^9, 3.915556023503305*^9, 
   3.9155597746477947`*^9, 3.915664899605281*^9, 3.9156689443657475`*^9, 
   3.9156705408237658`*^9, 3.9156714529858847`*^9, 3.915747624014781*^9, 
   3.9157477600769434`*^9, {3.9157484426419945`*^9, 3.915748471783783*^9}, 
   3.9158310349204807`*^9, 3.9162689735903287`*^9, 3.916522667084078*^9, 
   3.9165258861286325`*^9},
 CellLabel->"Out[73]=",ExpressionUUID->"f97612f7-2ef1-4f5a-bc39-97dc240c8cba"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{{3.8993372269059424`*^9, 3.89933724381315*^9}, 
   3.8993374633868914`*^9, 3.899338446931265*^9, 3.8993780245809956`*^9, 
   3.899466332685233*^9, 3.899480423695766*^9, 3.899480660174369*^9, 
   3.899508918987462*^9, 3.899578146200673*^9, 3.8995824787955236`*^9, 
   3.899661612147478*^9, 3.8998937191473117`*^9, 3.906216557010718*^9, 
   3.9102012899497643`*^9, 3.910263502732051*^9, 3.9103496151857457`*^9, 
   3.9104402780360036`*^9, 3.910790791978406*^9, 3.910828224953341*^9, 
   3.9109557925361004`*^9, 3.9109647168304663`*^9, 3.9110022791657057`*^9, 
   3.911494907290081*^9, 3.9116528385143366`*^9, 3.9120851418263316`*^9, 
   3.9121289868620305`*^9, 3.912380575687106*^9, 3.9126029976951904`*^9, 
   3.9148655601937385`*^9, 3.9151979064169436`*^9, 3.915200201483579*^9, 
   3.915402223789356*^9, 3.915443198550068*^9, 3.915475810479916*^9, 
   3.9155412506511393`*^9, 3.91555182706474*^9, 3.915556023503305*^9, 
   3.9155597746477947`*^9, 3.915664899605281*^9, 3.9156689443657475`*^9, 
   3.9156705408237658`*^9, 3.9156714529858847`*^9, 3.915747624014781*^9, 
   3.9157477600769434`*^9, {3.9157484426419945`*^9, 3.915748471783783*^9}, 
   3.9158310349204807`*^9, 3.9162689735903287`*^9, 3.916522667084078*^9, 
   3.9165258861296644`*^9},
 CellLabel->"Out[74]=",ExpressionUUID->"fe47171f-b6eb-4479-b58a-56e1dfd6d260"]
}, Open  ]]
}, Closed]],

Cell["Yes, each layer is 3D, and all are parallel. ", "Text",
 CellChangeTimes->{{3.899337267729081*^9, 3.8993372956422033`*^9}, 
   3.899337358784994*^9, {3.8993374785979514`*^9, 
   3.8993374823016443`*^9}},ExpressionUUID->"37d9abd2-f86e-4bed-9127-\
af30496eba82"],

Cell[CellGroupData[{

Cell["\<\
24-cell Layers and first-layer tetrahedron lifted to 8D \
\[LineSeparator](Still parallel? -- translate to origin and check span)\
\>", "Item",
 CellChangeTimes->{{3.899337363713145*^9, 3.89933737596332*^9}, {
  3.8993377250254865`*^9, 3.8993377256234455`*^9}, {3.8994674443827105`*^9, 
  3.899467454691757*^9}, {3.8994679027012496`*^9, 3.8994679169097137`*^9}, {
  3.915404190236024*^9, 
  3.9154041998247995`*^9}},ExpressionUUID->"65fdfd11-bdbc-4255-8df4-\
93f1c0d9f17d"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tet1Inds", " ", "=", " ", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"ptToIndex", "[", 
       RowBox[{"#", ",", 
        RowBox[{"N", "@", "GossetRotProj"}]}], "]"}], "&"}], ",", "tet1"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"oct1Inds", " ", "=", " ", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"ptToIndex", "[", 
       RowBox[{"#", ",", 
        RowBox[{"N", "@", "GossetRotProj"}]}], "]"}], "&"}], ",", "oct1"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cuboct1Inds", " ", "=", " ", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"ptToIndex", "[", 
       RowBox[{"#", ",", 
        RowBox[{"N", "@", "GossetRotProj"}]}], "]"}], "&"}], ",", "cuboct1"}],
     "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tet18D", " ", "=", " ", 
   RowBox[{"N", "@", 
    RowBox[{"GossetRot", "[", 
     RowBox[{"[", "tet1Inds", "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"oct18D", " ", "=", " ", 
   RowBox[{"N", "@", 
    RowBox[{"GossetRot", "[", 
     RowBox[{"[", "oct1Inds", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{
   RowBox[{"cuboct18D", " ", "=", " ", 
    RowBox[{"N", "@", 
     RowBox[{"GossetRot", "[", 
      RowBox[{"[", "cuboct1Inds", "]"}], "]"}]}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "tet18D", "]"}]}], "&"}], ",", "tet18D"}], "]"}], 
   "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "oct18D", "]"}]}], "&"}], ",", "oct18D"}], "]"}], 
   "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "-", 
       RowBox[{"Mean", "[", "cuboct18D", "]"}]}], "&"}], ",", "cuboct18D"}], 
    "]"}], "]"}], "[", "\"\<Dimension\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", "  ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "tet18D", "]"}]}], "&"}], ",", "tet18D"}], 
      "]"}], ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "oct18D", "]"}]}], "&"}], ",", "oct18D"}], 
      "]"}]}], "]"}], "  ", "]"}], "[", "\"\<Dimension\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", "  ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "tet18D", "]"}]}], "&"}], ",", "tet18D"}], 
      "]"}], ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "cuboct18D", "]"}]}], "&"}], ",", "cuboct18D"}],
       "]"}]}], "]"}], "  ", "]"}], "[", "\"\<Dimension\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VectorSpace", "[", "  ", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "oct18D", "]"}]}], "&"}], ",", "oct18D"}], 
      "]"}], ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "-", 
         RowBox[{"Mean", "[", "cuboct18D", "]"}]}], "&"}], ",", "cuboct18D"}],
       "]"}]}], "]"}], "  ", "]"}], "[", "\"\<Dimension\>\"", "]"}]}], "Input",\

 CellChangeTimes->{{3.8962763274845314`*^9, 3.8962764265415306`*^9}, {
   3.896276484190998*^9, 3.896276497597166*^9}, {3.896276536697822*^9, 
   3.8962765833515596`*^9}, {3.8962766370128365`*^9, 
   3.8962766897253723`*^9}, {3.896277523800494*^9, 3.8962775262912593`*^9}, {
   3.896436476865525*^9, 3.896436512252983*^9}, {3.896436547093547*^9, 
   3.8964365663507113`*^9}, {3.8964376495678005`*^9, 
   3.8964376590610943`*^9}, {3.898967112446243*^9, 3.8989671124492426`*^9}, {
   3.8989676061093273`*^9, 3.8989676254239826`*^9}, {3.8989676649045477`*^9, 
   3.8989676781527176`*^9}, 3.899337050645875*^9, {3.8993371150460997`*^9, 
   3.899337194267661*^9}, {3.899337442596197*^9, 3.899337448354491*^9}, {
   3.899337515623309*^9, 3.8993375598416843`*^9}, {3.8993375941740766`*^9, 
   3.899337605131168*^9}, {3.9156703001098056`*^9, 3.9156703234913588`*^9}},
 CellLabel->"In[75]:=",ExpressionUUID->"1b7e065d-1a84-4229-b79f-b9db98158d81"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.916525886173997*^9},
 CellLabel->"Out[81]=",ExpressionUUID->"f03ee251-da96-417d-98c4-5074cab4380a"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.9165258861759577`*^9},
 CellLabel->"Out[82]=",ExpressionUUID->"f035a491-24c0-4c28-8e65-21e24baad8d2"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.916525886176988*^9},
 CellLabel->"Out[83]=",ExpressionUUID->"616f96ed-88e1-41dc-be73-7bfc854eb3b7"],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.9165258861790085`*^9},
 CellLabel->"Out[84]=",ExpressionUUID->"f601e525-dd92-4970-b150-0e456247979f"],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.9165258861809893`*^9},
 CellLabel->"Out[85]=",ExpressionUUID->"f4f1520f-2d0e-4d40-a50b-4ce38ef111d8"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{
  3.89933757092048*^9, 3.899337607588689*^9, 3.8993384576166415`*^9, 
   3.8993780246323767`*^9, 3.8994663327477818`*^9, 3.8994804237437353`*^9, 
   3.899480660234409*^9, 3.899508919039773*^9, 3.8995781462497745`*^9, 
   3.89958247886964*^9, 3.8996616122116537`*^9, 3.899893719195921*^9, 
   3.906216557054246*^9, 3.9102012899926233`*^9, 3.9102635027756*^9, 
   3.9103496152282515`*^9, 3.9104402780794125`*^9, 3.9107907920213733`*^9, 
   3.910828224996498*^9, 3.910955792600149*^9, 3.9109647168724675`*^9, 
   3.9110022792082825`*^9, 3.911494907349665*^9, 3.9116528385686874`*^9, 
   3.9120851418689947`*^9, 3.912128986924305*^9, 3.9123805757536955`*^9, 
   3.912602997737195*^9, 3.91486556024139*^9, 3.915197906458989*^9, 
   3.9152002015226088`*^9, 3.9154022238641243`*^9, 3.9154431986369905`*^9, 
   3.9154758105412645`*^9, 3.915541250700246*^9, 3.915551827113375*^9, 
   3.9155560235461073`*^9, 3.915559774689577*^9, 3.915664899648242*^9, 
   3.915668944404925*^9, 3.91567054087644*^9, 3.915671453070865*^9, 
   3.915747624055233*^9, 3.9157477601171255`*^9, {3.915748442684023*^9, 
   3.915748471824376*^9}, 3.9158310349640045`*^9, 3.916268973645852*^9, 
   3.916522667126835*^9, 3.9165258861819906`*^9},
 CellLabel->"Out[86]=",ExpressionUUID->"0d7a8d18-0c61-44b0-9463-ed5c679bf974"]
}, Open  ]]
}, Closed]],

Cell["\<\
No:  After lifting to 8D, the two octahedra and the cuboctahedron are still \
parallel (so the 24-cell is still a 24-cell), 
but either of them with the tetrahedron spans a 6D space, hence no longer \
parallel.\
\>", "Text",
 CellChangeTimes->{{3.8964376725645423`*^9, 3.8964377145745993`*^9}, {
   3.897652663309761*^9, 3.8976527058081813`*^9}, {3.8983562512326283`*^9, 
   3.8983562644974227`*^9}, {3.8983566502890873`*^9, 3.89835666715584*^9}, {
   3.898356934546942*^9, 3.8983569856832685`*^9}, {3.898357024168748*^9, 
   3.898357047047304*^9}, 3.8983576572986603`*^9, {3.8983579493389206`*^9, 
   3.8983579870191884`*^9}, {3.899337628693014*^9, 3.899337689755729*^9}, {
   3.899467544769101*^9, 3.8994675647389135`*^9}, {3.899467604728143*^9, 
   3.8994676809391165`*^9}, {3.899467975167486*^9, 3.899468004615509*^9}, {
   3.8994711901707973`*^9, 3.899471214892294*^9}, {3.9154042239823775`*^9, 
   3.9154042698698344`*^9}, {3.915404337715547*^9, 3.915404388409951*^9}, {
   3.9154051430724363`*^9, 3.9154051467508435`*^9}, {3.915755642134761*^9, 
   3.9157557061221676`*^9}},ExpressionUUID->"1d0e5ec9-ba1f-4908-b40b-\
9a2910293b55"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
5 disjoint 24-cells, from isoclinic rotations (in ES) of the first 24-cell;
check that these partition the vertices of C600.\
\>", "Subsubsection",
 CellChangeTimes->{{3.8902535310371284`*^9, 3.890253586681969*^9}, {
   3.8902539023295813`*^9, 3.8902539576444416`*^9}, {3.89025400709629*^9, 
   3.890254024158964*^9}, {3.890254233575389*^9, 3.890254235051263*^9}, {
   3.890404838413845*^9, 3.8904048407957554`*^9}, {3.8906134249896164`*^9, 
   3.8906134267106915`*^9}, {3.8913429412626143`*^9, 
   3.8913429612948213`*^9}, {3.8937695443118925`*^9, 3.893769563944209*^9}, {
   3.8976527437460585`*^9, 3.8976527546439943`*^9}, {3.8989900326626606`*^9, 
   3.8989900345678024`*^9}, {3.8993378672717648`*^9, 
   3.8993378686900463`*^9}, {3.8994681091837177`*^9, 3.899468109589878*^9}, {
   3.8994681794981813`*^9, 3.8994681799593534`*^9}, {3.9154044058901587`*^9, 
   3.9154044215999804`*^9}, {3.915404570695779*^9, 3.915404571135147*^9}, {
   3.9154051689499345`*^9, 3.915405187621396*^9}, {3.9156715717079773`*^9, 
   3.915671576152376*^9}, {3.915671689740749*^9, 3.915671731558694*^9}, 
   3.915747693450392*^9},ExpressionUUID->"d7931742-65ff-45df-80d0-\
7839aca45925"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"C24s", " ", "=", " ", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"MatrixPower", "[", 
           RowBox[{"rotIsoES", ",", "k"}], "]"}], ".", "#"}], "&"}], ",", 
        "C24n1"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "0", ",", "4"}], "}"}]}], "]"}], " ", "//", " ", 
    "Chop"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ContainsExactly", "[", 
  RowBox[{
   RowBox[{"Round", "[", 
    RowBox[{"C600", ",", 
     RowBox[{"10.", "^", 
      RowBox[{"-", "8"}]}]}], "]"}], ",", 
   RowBox[{"Round", "[", 
    RowBox[{
     RowBox[{"Join", "@@", 
      RowBox[{"(", 
       RowBox[{"N", "@", "C24s"}], ")"}]}], ",", 
     RowBox[{"10.", "^", 
      RowBox[{"-", "8"}]}]}], "]"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.893173781794773*^9, 3.8931738791031904`*^9}, {
   3.89317398983232*^9, 3.893173990137767*^9}, {3.893174136219631*^9, 
   3.893174213860896*^9}, {3.89317428511945*^9, 3.8931743195530195`*^9}, {
   3.8931743616392174`*^9, 3.8931745031406507`*^9}, 3.8931746838970766`*^9, 
   3.8931749116481247`*^9, {3.8931749688178496`*^9, 3.8931750597211924`*^9}, {
   3.89317510217782*^9, 3.8931751296627455`*^9}, {3.8931752073186517`*^9, 
   3.893175227381457*^9}, 3.893175284752142*^9, {3.893176625862468*^9, 
   3.8931766753200235`*^9}, {3.893177012127039*^9, 3.8931770679835157`*^9}, {
   3.8931789065064793`*^9, 3.893178945489601*^9}, 3.8931789770045795`*^9, 
   3.893185429394788*^9, {3.8931864711693506`*^9, 3.8931864750061855`*^9}, {
   3.893186551936427*^9, 3.8931866001436357`*^9}, {3.8932486396011367`*^9, 
   3.893248660238866*^9}, {3.89324872337475*^9, 3.893248744290685*^9}, {
   3.893248781805313*^9, 3.8932488269792295`*^9}, {3.89324888697921*^9, 
   3.893248903378721*^9}, 3.893256177753038*^9, {3.893260781390047*^9, 
   3.8932608138114257`*^9}, {3.893260891811641*^9, 3.8932609137142563`*^9}, 
   3.8932629304216366`*^9, 3.893262975794715*^9, {3.8934184494141893`*^9, 
   3.8934184723593626`*^9}, 3.8934296351068344`*^9, 3.8934397302766323`*^9, {
   3.8935021654544864`*^9, 3.8935021661576643`*^9}, 3.8937816275777793`*^9, 
   3.8941297851489005`*^9, 3.894479897146206*^9, {3.8969020305003185`*^9, 
   3.896902032268259*^9}, {3.9154052056935787`*^9, 3.915405274271017*^9}, {
   3.9154054012875457`*^9, 3.9154054592786646`*^9}, {3.915405506348836*^9, 
   3.915405516381419*^9}, {3.9154055658962517`*^9, 3.915405593434419*^9}, 
   3.9154058252234163`*^9, 3.9154068212211127`*^9, 3.9154068687527924`*^9, 
   3.91567005057368*^9, {3.9156703268694663`*^9, 3.9156703297168937`*^9}, 
   3.915747693455393*^9, {3.9157478802095222`*^9, 3.915747882414863*^9}, 
   3.915748143479826*^9},
 CellLabel->"In[87]:=",ExpressionUUID->"e9209b3e-7c1f-4ebd-af36-e865590d5af1"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.9154068226998158`*^9, 3.915443266302867*^9, 3.9154758106138177`*^9, 
   3.9155412507542796`*^9, 3.9155518271660547`*^9, 3.915556023589881*^9, 
   3.91555977473162*^9, 3.9156648996804676`*^9, 3.915668944442753*^9, 
   3.9156705409340763`*^9, 3.9156714531255846`*^9, 3.915747624088408*^9, 
   3.915747760149165*^9, {3.915748442727116*^9, 3.9157484718578587`*^9}, 
   3.9157561027326417`*^9, 3.915831034992055*^9, 3.9162689736729193`*^9, 
   3.9165226671556807`*^9, 3.916525886226777*^9},
 CellLabel->"Out[88]=",ExpressionUUID->"ee685e65-ab33-4163-bed0-89680ad0c5c2"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "In C600, compare all cuboctahedra from the 5 24-cells, each rotated \n\t\
into the 3D space ",
 StyleBox["V",
  FontSlant->"Italic"],
 " of one \[OpenCurlyDoubleQuote]primary\[CloseCurlyDoubleQuote] \
cuboctahedron in 24-cell 1."
}], "Subsection",
 CellChangeTimes->{{3.8935224636622324`*^9, 3.893522497919166*^9}, {
   3.898357736176613*^9, 3.898357793628271*^9}, 3.8996623096184516`*^9, {
   3.9104403647181168`*^9, 3.9104403652766385`*^9}, {3.9104685452305884`*^9, 
   3.9104685472152004`*^9}, {3.914872252550089*^9, 3.914872337560424*^9}, {
   3.915404590439966*^9, 3.9154045909205728`*^9}, {3.9154046519614525`*^9, 
   3.915404752752578*^9}, {3.9155606585651684`*^9, 3.9155606717842865`*^9}, 
   3.9155607420235023`*^9, {3.915670339074956*^9, 3.9156703421568108`*^9}, {
   3.915671753492036*^9, 3.915671794524793*^9}, 3.915747693463005*^9, {
   3.9157557278205233`*^9, 
   3.915755728387619*^9}},ExpressionUUID->"07e12d27-0dde-4d90-8f64-\
c6a961f230fd"],

Cell["\<\
We find that 
\tWith twist in one direction (matching the isoclinic rotation that generated \
the 5 24-cells), they ALL rotate directly into the primary 4G; 
\tWith the twist in the opposite direction (isoclinically opposite rotation \
of the one that generated the 5 24-cells) they ALL rotate directly into 20G \
positions relative to the primary 4G; 
\tWithout the twist, they are chaotically jumbled (except for one set, whose \
normals lie at 2\[Pi]/5 in a single plane, and which actually forms a 20G \
without any twisting).\
\>", "Text",
 CellChangeTimes->{{3.9104683976795006`*^9, 3.9104685371345835`*^9}, {
   3.9148724092558403`*^9, 3.9148725711271544`*^9}, {3.91487262894115*^9, 
   3.9148726851828375`*^9}, 3.9148727190890493`*^9, {3.915404731343999*^9, 
   3.9154048152775955`*^9}, {3.915670346147852*^9, 3.9156703487188425`*^9}, {
   3.9156718070047626`*^9, 3.9156718170946913`*^9}, {3.915671864039754*^9, 
   3.915671921297532*^9}, {3.9157557466290073`*^9, 
   3.915755791305102*^9}},ExpressionUUID->"de2d3abe-0c47-4e74-ab8b-\
e35d2f3663b0"],

Cell[CellGroupData[{

Cell["All the facet normals of each 24-cell", "Subsubsection",
 CellChangeTimes->{
  3.910440368686304*^9, {3.91567195954265*^9, 3.915671968649459*^9}, 
   3.9156720090866346`*^9},ExpressionUUID->"88d8920c-9eeb-4c4a-bc3a-\
9ebcd7c21cbb"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"C24facetNormals", " ", "=", " ", 
    RowBox[{"facetNormals4D", " ", "/@", " ", "C24s"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"normal", " ", "to", " ", "V"}], ",", " ", 
    RowBox[{
     RowBox[{"the", " ", "primary", " ", "3"}], "-", "space"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"normalV", " ", "=", " ", 
   RowBox[{"C24facetNormals", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.9154073092401457`*^9, 3.9154074258324347`*^9}, {
   3.915407471091012*^9, 3.9154074743458047`*^9}, {3.9154075136436234`*^9, 
   3.915407529477708*^9}, {3.9154075695735216`*^9, 3.915407575111339*^9}, {
   3.9156703503594923`*^9, 3.915670351259709*^9}, 3.9156720318814187`*^9, {
   3.915747711019559*^9, 3.9157477110251293`*^9}, {3.9157558326651783`*^9, 
   3.915755897488543*^9}, {3.915756123435972*^9, 3.915756167656024*^9}},
 CellLabel->"In[89]:=",ExpressionUUID->"fcf4ff0f-c4a4-4f13-975a-92a6df4b91c6"],

Cell[CellGroupData[{

Cell["\<\
Just for reference, what are the angles between vertices in a single 24-cell?\
\
\>", "Item",
 CellChangeTimes->{{3.912087541445716*^9, 3.912087559063368*^9}, {
   3.912112241885609*^9, 3.9121122557479773`*^9}, {3.9121289068346386`*^9, 
   3.9121289074833956`*^9}, {3.912381456224593*^9, 3.912381460443931*^9}, 
   3.9123815209255095`*^9, {3.9123816592509966`*^9, 3.912381673130891*^9}, {
   3.9154442612270226`*^9, 
   3.915444276296586*^9}},ExpressionUUID->"8289e1ac-9997-4950-ad6b-\
c2b530fb5b81"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Round", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VectorAngle", "[", 
          RowBox[{
           RowBox[{"Normalize", "@", 
            RowBox[{"C24s", "[", 
             RowBox[{"[", 
              RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ",", "#"}], "]"}], "/",
          "Degree"}], ",", 
        RowBox[{"10.", "^", 
         RowBox[{"-", "4"}]}]}], "]"}], "&"}], ",", 
     RowBox[{"Normalize", " ", "/@", " ", 
      RowBox[{"C24s", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}], "//", "Tally"}], "//", 
  "Column"}]], "Input",
 CellChangeTimes->{{3.912087336934007*^9, 3.9120874257176285`*^9}, {
  3.9120874722893333`*^9, 3.912087485889993*^9}, {3.912087566740389*^9, 
  3.912087568412692*^9}, {3.9121121236813*^9, 3.912112125328459*^9}, {
  3.912112362654769*^9, 3.912112364292858*^9}, {3.912128805877402*^9, 
  3.9121288089914646`*^9}, {3.9123811440605373`*^9, 3.912381145477891*^9}, {
  3.912381384759616*^9, 3.9123814113831134`*^9}, {3.9123815341810493`*^9, 
  3.912381547925086*^9}, {3.9123816293460755`*^9, 3.9123816492657037`*^9}, {
  3.915443720638052*^9, 3.9154437281981564`*^9}, {3.9154438048536167`*^9, 
  3.9154438099224267`*^9}, {3.9156703564373713`*^9, 3.9156703578392935`*^9}, {
  3.91574771102913*^9, 3.9157477110310984`*^9}},
 CellLabel->"In[91]:=",ExpressionUUID->"05bff862-26a7-4d11-bcc3-9a48d6634c6b"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"60.`", ",", "8"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"90.`", ",", "6"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"120.`", ",", "8"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"180.`", ",", "1"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output",
 CellChangeTimes->{
  3.912381168474492*^9, 3.9123812432251873`*^9, {3.9123813870832677`*^9, 
   3.912381413229014*^9}, 3.9123815494230676`*^9, 3.9123816504593134`*^9, 
   3.912602998030336*^9, 3.914865560577623*^9, 3.9151979067376194`*^9, 
   3.9152002018143806`*^9, 3.9154022241557217`*^9, 3.915443770893633*^9, {
   3.9154438185467544`*^9, 3.915443835980423*^9}, 3.915475811179275*^9, 
   3.915541250942565*^9, 3.9155518273739367`*^9, 3.9155560237855444`*^9, 
   3.9155597749394875`*^9, 3.9156648998360496`*^9, 3.9156689445920324`*^9, 
   3.9156705410914607`*^9, 3.915671453278511*^9, 3.9157476242113695`*^9, 
   3.915747760341864*^9, {3.9157484428795795`*^9, 3.915748471997449*^9}, 
   3.915756175334934*^9, 3.915831035127796*^9, 3.916268973816039*^9, 
   3.9165226672738853`*^9, 3.9165258863633003`*^9},
 CellLabel->"Out[91]=",ExpressionUUID->"9429490f-a69d-4ece-8211-467cf764591e"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
What are the angles between facet normals of different 24-cells?\
\>", "Item",
 CellChangeTimes->{{3.912087541445716*^9, 3.912087559063368*^9}, {
   3.912112241885609*^9, 3.9121122557479773`*^9}, {3.9121289068346386`*^9, 
   3.9121289074833956`*^9}, {3.912381456224593*^9, 3.912381460443931*^9}, 
   3.9123815209255095`*^9, {3.9123816592509966`*^9, 3.912381673130891*^9}, {
   3.9154442612270226`*^9, 3.915444276296586*^9}, {3.915672069137813*^9, 
   3.9156720977781143`*^9}},ExpressionUUID->"2a736abd-e90e-44f6-b52c-\
8e20b06eab6f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"VectorAngle", "[", 
      RowBox[{"normalV", ",", "#"}], "]"}], "/", "Degree"}], "&"}], " ", ",", 
   " ", "C24facetNormals", ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.9154089035285416`*^9, 3.915408904103953*^9}, {
   3.9154089543318825`*^9, 3.915409002543888*^9}, 3.9154109074099274`*^9, 
   3.915670362100856*^9, 3.9157477110351305`*^9, 3.915755924018097*^9},
 CellLabel->"In[92]:=",ExpressionUUID->"1f77570d-da27-49f4-b450-4b451f85cb49"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.`", ",", "60.00000000000001`", ",", "59.99999999999999`", ",", 
     "59.99999999999999`", ",", "59.99999999999999`", ",", "90.`", ",", 
     "59.999999999999986`", ",", "59.999999999999986`", ",", "90.`", ",", 
     "59.999999999999986`", ",", "90.`", ",", "59.999999999999986`", ",", 
     "90.`", ",", "90.`", ",", "90.`", ",", "120.00000000000001`", ",", 
     "120.00000000000001`", ",", "120.00000000000001`", ",", 
     "120.00000000000001`", ",", "120.00000000000001`", ",", 
     "120.00000000000001`", ",", "120.00000000000001`", ",", 
     "119.99999999999999`", ",", "180.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "72.00000000000001`", ",", "120.00000000000001`", ",", "72.`", ",", 
     "90.`", ",", "35.99999999999998`", ",", "90.`", ",", 
     "120.00000000000001`", ",", "72.00000000000001`", ",", 
     "120.00000000000001`", ",", "90.`", ",", "144.`", ",", 
     "35.99999999999999`", ",", "35.999999999999986`", ",", 
     "59.99999999999997`", ",", "90.`", ",", "144.`", ",", "90.`", ",", 
     "107.99999999999999`", ",", "144.00000000000003`", ",", 
     "59.99999999999997`", ",", "90.`", ",", "108.`", ",", 
     "59.999999999999986`", ",", "107.99999999999999`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "144.`", ",", "144.00000000000003`", ",", "108.00000000000003`", ",", 
     "120.00000000000004`", ",", "90.00000000000003`", ",", 
     "89.99999999999999`", ",", "144.00000000000003`", ",", 
     "108.00000000000006`", ",", "108.`", ",", "120.00000000000007`", ",", 
     "119.99999999999999`", ",", "90.00000000000004`", ",", 
     "60.00000000000001`", ",", "72.`", ",", "90.00000000000001`", ",", 
     "89.99999999999996`", ",", "59.99999999999994`", ",", 
     "71.99999999999994`", ",", "89.99999999999997`", ",", 
     "35.99999999999998`", ",", "59.99999999999996`", ",", 
     "71.99999999999997`", ",", "35.99999999999997`", ",", 
     "35.999999999999986`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "143.99999999999997`", ",", "89.99999999999997`", ",", 
     "120.00000000000001`", ",", "108.00000000000004`", ",", 
     "144.00000000000009`", ",", "90.`", ",", "89.99999999999997`", ",", 
     "120.00000000000001`", ",", "71.99999999999993`", ",", 
     "108.00000000000004`", ",", "59.999999999999986`", ",", 
     "144.00000000000006`", ",", "120.00000000000001`", ",", 
     "108.00000000000007`", ",", "90.`", ",", "35.99999999999995`", ",", 
     "71.99999999999996`", ",", "59.999999999999986`", ",", 
     "35.99999999999993`", ",", "90.00000000000003`", ",", 
     "71.99999999999996`", ",", "59.999999999999986`", ",", 
     "90.00000000000003`", ",", "36.00000000000002`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "71.99999999999997`", ",", "35.99999999999996`", ",", 
     "89.99999999999999`", ",", "72.`", ",", "120.00000000000004`", ",", 
     "90.00000000000004`", ",", "35.99999999999991`", ",", 
     "89.99999999999994`", ",", "59.99999999999992`", ",", 
     "71.99999999999994`", ",", "35.999999999999936`", ",", 
     "119.99999999999999`", ",", "144.00000000000006`", ",", 
     "120.00000000000007`", ",", "89.99999999999997`", ",", 
     "60.00000000000001`", ",", "108.00000000000006`", ",", 
     "90.00000000000006`", ",", "59.99999999999996`", ",", 
     "144.0000000000001`", ",", "108.`", ",", "90.00000000000001`", ",", 
     "144.00000000000006`", ",", "108.00000000000003`"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.9154089875974994`*^9, 3.9154090040475717`*^9}, 
   3.915410927498255*^9, 3.9154440889264755`*^9, 3.915475811230982*^9, 
   3.915541250998719*^9, 3.915551827397519*^9, 3.9155560238040557`*^9, 
   3.9155597749885225`*^9, 3.9156648998726015`*^9, 3.915668944634346*^9, 
   3.915670541125513*^9, 3.915671453315175*^9, 3.915747624244501*^9, 
   3.915747760368372*^9, {3.9157484429187393`*^9, 3.9157484720269985`*^9}, 
   3.9157561781880755`*^9, 3.9158310351560087`*^9, 3.9162689738300476`*^9, 
   3.916522667294553*^9, 3.9165258864054823`*^9},
 CellLabel->"Out[92]=",ExpressionUUID->"a5a5a3b4-51da-4ae9-8f18-baa4fa84ce2d"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
24-cell facets come in parallel pairs, on opposite sides. 
Choose a single one from each pair.\
\>", "Subsubsection",
 CellChangeTimes->{{3.915672132052518*^9, 
  3.915672194641943*^9}},ExpressionUUID->"00ec375d-2422-451e-a73d-\
6d37cd1e45ca"],

Cell["\<\
This will give 12 normals for each 24-cell. 
Choose the ones on one side, i.e., that make acute angles with the first one.
(If they are perpendicular to the first one, choose them to be acute with the \
second one.)\
\>", "Text",
 CellChangeTimes->{{3.91567219912976*^9, 
  3.915672395935014*^9}},ExpressionUUID->"f73e1c57-b7e3-4822-912d-\
979d3f851938"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"C24facetNormalsReduced", " ", "=", " ", 
    RowBox[{"Map", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", "vectorList", "}"}], ",", 
        RowBox[{"Select", "[", 
         RowBox[{"vectorList", ",", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"#", ".", 
               RowBox[{"vectorList", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ">", "0.001"}], " ", "||", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"#", ".", 
                  RowBox[{"vectorList", "[", 
                   RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "<", "0.001"}], 
               " ", "&&", " ", 
               RowBox[{
                RowBox[{"#", ".", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"vectorList", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "+", 
                   RowBox[{".5", "*", 
                    RowBox[{"vectorList", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], ")"}]}], ">", "0"}]}],
               ")"}]}], ")"}], "&"}]}], "]"}]}], "]"}], "\[IndentingNewLine]",
       ",", "C24facetNormals"}], "]"}]}], ")"}], "//", 
  "Dimensions"}]], "Input",
 CellChangeTimes->{{3.9154091311019526`*^9, 3.915409400730632*^9}, {
  3.9156703643488865`*^9, 3.915670367252597*^9}, {3.9157477110391674`*^9, 
  3.915747711041168*^9}},
 CellLabel->"In[93]:=",ExpressionUUID->"fd21c092-f99d-4ac6-a6fc-7c0451f2e1f6"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "12", ",", "4"}], "}"}]], "Output",
 CellChangeTimes->{{3.9154093670167513`*^9, 3.9154094015730343`*^9}, 
   3.915410931439638*^9, 3.9154444141383953`*^9, 3.9154758112825804`*^9, 
   3.9155412510513735`*^9, 3.915551827446708*^9, 3.9155560238485746`*^9, 
   3.915559775036209*^9, 3.915664899900123*^9, 3.9156689446750174`*^9, 
   3.915670541157778*^9, 3.915671453346792*^9, 3.915747624273113*^9, 
   3.915747760398383*^9, {3.915748442959873*^9, 3.9157484720515237`*^9}, 
   3.9157561814964123`*^9, 3.9158310351813216`*^9, 3.916268973851559*^9, 
   3.9165226673146076`*^9, 3.916525886447192*^9},
 CellLabel->"Out[93]=",ExpressionUUID->"3545e5d0-1688-4c68-9790-f14d8155d85d"]
}, Open  ]],

Cell["Check angles the normals make with the primary normal", "Text",
 CellChangeTimes->{{3.915757712245077*^9, 
  3.9157577294585752`*^9}},ExpressionUUID->"cf071b19-3546-4677-934e-\
dc752e47279f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Map", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"VectorAngle", "[", 
       RowBox[{"normalV", ",", "#"}], "]"}], "/", "Degree"}], "&"}], " ", ",",
     " ", "C24facetNormalsReduced", ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "//", "Column"}]], "Input",
 CellChangeTimes->{{3.915409460021563*^9, 3.915409483967885*^9}, 
   3.915410915577889*^9, 3.915670370098188*^9, 3.915747711045169*^9, 
   3.915755924025279*^9, {3.9157577048565173`*^9, 3.915757708829197*^9}},
 CellLabel->"In[94]:=",ExpressionUUID->"49c1fb18-7376-4275-bcbf-921941fe11df"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{
      "0.`", ",", "60.00000000000001`", ",", "59.99999999999999`", ",", 
       "59.99999999999999`", ",", "59.99999999999999`", ",", "90.`", ",", 
       "59.999999999999986`", ",", "59.999999999999986`", ",", "90.`", ",", 
       "59.999999999999986`", ",", "90.`", ",", "59.999999999999986`"}], 
      "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
      "72.00000000000001`", ",", "120.00000000000001`", ",", "72.`", ",", 
       "90.`", ",", "35.99999999999998`", ",", "90.`", ",", 
       "120.00000000000001`", ",", "72.00000000000001`", ",", 
       "120.00000000000001`", ",", "90.`", ",", "144.`", ",", 
       "35.99999999999999`"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
      "144.`", ",", "144.00000000000003`", ",", "108.00000000000003`", ",", 
       "120.00000000000004`", ",", "90.00000000000003`", ",", 
       "89.99999999999999`", ",", "144.00000000000003`", ",", 
       "108.00000000000006`", ",", "108.`", ",", "120.00000000000007`", ",", 
       "119.99999999999999`", ",", "90.00000000000004`"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
      "143.99999999999997`", ",", "89.99999999999997`", ",", 
       "120.00000000000001`", ",", "108.00000000000004`", ",", 
       "144.00000000000009`", ",", "90.`", ",", "89.99999999999997`", ",", 
       "120.00000000000001`", ",", "71.99999999999993`", ",", 
       "108.00000000000004`", ",", "59.999999999999986`", ",", 
       "144.00000000000006`"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
      "71.99999999999997`", ",", "35.99999999999996`", ",", 
       "89.99999999999999`", ",", "72.`", ",", "120.00000000000004`", ",", 
       "90.00000000000004`", ",", "35.99999999999991`", ",", 
       "89.99999999999994`", ",", "59.99999999999992`", ",", 
       "71.99999999999994`", ",", "35.999999999999936`", ",", 
       "119.99999999999999`"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output",
 CellChangeTimes->{{3.915409446321806*^9, 3.915409493608817*^9}, 
   3.915410933342764*^9, 3.915444418260681*^9, 3.915475811336378*^9, 
   3.915541251100741*^9, 3.915551827466082*^9, 3.915556023866592*^9, 
   3.915559775084465*^9, 3.915664899934881*^9, 3.9156689446818943`*^9, 
   3.9156705411902914`*^9, 3.9156714533809805`*^9, 3.9157476243033123`*^9, 
   3.9157477604378557`*^9, {3.9157484430007725`*^9, 3.9157484720790153`*^9}, 
   3.915756183158807*^9, 3.915831035207345*^9, 3.9162689738645735`*^9, 
   3.916522667334632*^9, 3.916525886488345*^9},
 CellLabel->"Out[94]=",ExpressionUUID->"1ca73730-1191-4e18-a42c-de53cd401c67"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Rotate all the different cuboctahedra from other 24-cells into the primary \
3-space, ",
 StyleBox["V",
  FontSlant->"Italic"],
 ",\nand make 4Gs from them"
}], "Subsubsection",
 CellChangeTimes->{{3.910467966766608*^9, 3.910468011230531*^9}, {
   3.9107942497819457`*^9, 3.910794251189417*^9}, {3.9108291567850876`*^9, 
   3.910829157072466*^9}, {3.915409975892126*^9, 3.915409976580392*^9}, 
   3.915670379642125*^9, {3.915757376490283*^9, 3.9157574121994004`*^9}, {
   3.915757511072756*^9, 
   3.9157575242970037`*^9}},ExpressionUUID->"4c6cfb47-c642-41ce-9e91-\
0147daf1e696"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"C24cbctsAll", " ", "=", " ", 
   RowBox[{"MapThread", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"equatorFxn", "[", 
         RowBox[{"#1", ",", 
          RowBox[{"#2", "[", 
           RowBox[{"[", "k", "]"}], "]"}]}], "]"}], " ", ",", "  ", 
        RowBox[{"{", 
         RowBox[{"k", ",", "1", ",", 
          RowBox[{"Length", "@", "#2"}]}], "}"}]}], "]"}], "&"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"C24s", ",", "C24facetNormalsReduced"}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"C24cbctRotsToV", " ", "=", " ", 
   RowBox[{
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"RotationMatrix", "[", 
        RowBox[{"{", 
         RowBox[{"#", ",", "normalV"}], "}"}], "]"}], "&"}], ",", 
      "C24facetNormalsReduced", ",", 
      RowBox[{"{", "2", "}"}]}], "]"}], "//", "Chop"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cbctsInV", " ", "=", " ", 
   RowBox[{"MapThread", "[", 
    RowBox[{
     RowBox[{"Function", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"rotMat", ",", "ptset"}], "}"}], ",", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"rotMat", ".", "#"}], "&"}], ",", "ptset"}], "]"}]}], "]"}],
      ",", 
     RowBox[{"{", 
      RowBox[{"C24cbctRotsToV", ",", "C24cbctsAll"}], "}"}], ",", "2"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cbctsInV3D", " ", "=", " ", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"VectorProject", "[", 
       RowBox[{"#", ",", 
        RowBox[{"NullSpace", "[", 
         RowBox[{"{", "normalV", "}"}], "]"}]}], "]"}], "&"}], ",", 
     "cbctsInV"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.9154095984572725`*^9, 3.9154096398622117`*^9}, {
   3.9154097963781185`*^9, 3.915409822622795*^9}, {3.9154098552980065`*^9, 
   3.915409855580539*^9}, {3.9154099235118556`*^9, 3.915409946835614*^9}, {
   3.915410100824901*^9, 3.91541012754274*^9}, {3.915410158222758*^9, 
   3.9154101679423733`*^9}, 3.9154104283553176`*^9, {3.9154105212187304`*^9, 
   3.915410625585205*^9}, 3.915410828378759*^9, {3.915444922079528*^9, 
   3.915444948576927*^9}, {3.915444992064475*^9, 3.9154449988836193`*^9}, 
   3.9155603093270936`*^9, {3.9156703856127195`*^9, 3.915670387012766*^9}, {
   3.915747711055715*^9, 3.9157477110607824`*^9}, {3.915757437483904*^9, 
   3.9157574414360485`*^9}},
 CellLabel->"In[95]:=",ExpressionUUID->"d30da416-5e59-4422-975d-ba8e3fac867e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Twist", " ", "axes", " ", "of", " ", "each", " ", "cuboctahedron", " ", 
     "under", " ", "rotation", " ", "to", " ", "primary", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"used", " ", "for", " ", "compounding"}], ")"}], ".", 
      "\[IndentingNewLine]", "This"}], " ", "is", " ", "found", " ", "by", 
     " ", "simply", " ", "projecting", " ", "the", " ", 
     RowBox[{"cuboctahedron", "'"}], "s", " ", "3"}], "-", 
    RowBox[{
    "space", " ", "normal", " ", "into", " ", "the", " ", "primary", " ", 
     "3"}], "-", 
    RowBox[{"space", "."}]}], "  ", "*)"}], 
  RowBox[{
   RowBox[{
    RowBox[{"compoundingAxes", " ", "=", " ", 
     RowBox[{
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Total", "@", 
          RowBox[{"MapThread", "[", 
           RowBox[{"Times", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"NullSpace", "[", 
                RowBox[{"{", "normalV", "}"}], "]"}], ".", "#"}], ",", 
              RowBox[{"NullSpace", "[", 
               RowBox[{"{", "normalV", "}"}], "]"}]}], "}"}]}], "]"}]}], " ", 
         "&"}], ",", " ", "C24facetNormalsReduced", ",", 
        RowBox[{"{", "2", "}"}]}], "]"}], "//", "Chop"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"compoundingAxes3D", " ", "=", " ", 
     RowBox[{"Map", "[", 
      RowBox[{"Normalize", ",", 
       RowBox[{"VectorProject", "[", 
        RowBox[{"compoundingAxes", ",", 
         RowBox[{"NullSpace", "[", 
          RowBox[{"{", "normalV", "}"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.9154846276200933`*^9, 3.915484647345665*^9}, {
   3.915484681138975*^9, 3.91548470589782*^9}, 3.91555586783677*^9, 
   3.9156704029503317`*^9, 3.915747711076043*^9, {3.915749897754113*^9, 
   3.9157499598062043`*^9}, {3.915750667771738*^9, 3.9157506867173843`*^9}, {
   3.9157559240426702`*^9, 3.9157559240489783`*^9}},
 CellLabel->"In[99]:=",ExpressionUUID->"3db61cb7-7c64-4615-b716-d657bf8dc686"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
     "Ad", " ", "hoc", " ", "function", " ", "to", " ", "select", " ", "the", 
      " ", "correct", " ", "chiral", " ", "4", "G"}], ",", " ", 
     RowBox[{
      RowBox[{
      "depending", " ", "on", " ", "the", " ", "angle", " ", "its", " ", 
       "3"}], "-", 
      RowBox[{
      "space", " ", "normal", " ", "makes", " ", "\[IndentingNewLine]", 
       "with", " ", "the", " ", "primary", " ", "normal", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"multiple", " ", "of", " ", "36"}], ",", " ", "60", ",", 
          " ", 
          RowBox[{"or", " ", "90"}]}], ")"}], ".", " ", "\[IndentingNewLine]",
         "There"}], " ", "should", " ", "be", " ", "a", " ", "reason", " ", 
       "for", " ", "this"}]}], ",", " ", 
     RowBox[{"but", " ", "I", " ", 
      RowBox[{"haven", "'"}], "t", " ", "figured", " ", "it", " ", "out", " ", 
      RowBox[{"yet", ".", " ", "Future"}], " ", 
      RowBox[{"reserach", " ", ":"}]}]}], ")"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"chiralSgn", "::", "anamolousAngle"}], " ", "=", " ", 
     "\"\<Unexpected angle between given 4G normal and the primary \
normal.\>\""}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"chiralSgn", "[", "C24Normal_", "]"}], " ", ":=", "  ", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "angl", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"angl", " ", "=", " ", 
         RowBox[{"Round", "@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"ArcCos", "[", 
             RowBox[{"normalV", ".", "C24Normal"}], "]"}], "/", "Degree"}], 
           ")"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Mod", "[", 
            RowBox[{"angl", ",", "36"}], "]"}], "==", "0"}], ",", 
          RowBox[{"Return", "[", 
           RowBox[{"-", "1"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Mod", "[", 
            RowBox[{"angl", ",", "60"}], "]"}], "==", "0"}], ",", 
          RowBox[{"Return", "[", "1", "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"angl", "==", "90"}], ",", 
          RowBox[{"Return", "[", "1", "]"}], ",", "\[IndentingNewLine]", 
          "True", ",", 
          RowBox[{"Message", "[", 
           RowBox[{"chiralSgn", "::", "anamolousAngle"}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.911500084671338*^9, 3.9115001728348055`*^9}, {
   3.911500292852147*^9, 3.9115003719255896`*^9}, {3.9115004411728506`*^9, 
   3.911500499764719*^9}, {3.911500611027104*^9, 3.9115008852829046`*^9}, 
   3.9115010179424667`*^9, {3.9115010650599275`*^9, 3.911501066294717*^9}, 
   3.911509131130643*^9, 3.915444468236703*^9, 3.915670374182478*^9, {
   3.9157477110491734`*^9, 3.915747711052169*^9}, {3.915755924030279*^9, 
   3.9157559685930133`*^9}, {3.9157577487330904`*^9, 
   3.9157578957067957`*^9}, {3.9165234027460623`*^9, 3.916523402836007*^9}},
 CellLabel->
  "In[101]:=",ExpressionUUID->"21729fd1-292c-4630-bb40-3a1c0c841f42"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Create", " ", "4", "Gs", " ", "from", " ", "the", " ", "cuboctahedra"}], 
   " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"n4GsInV", " ", "=", " ", 
    RowBox[{"MapThread", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Quiet", "@", 
        RowBox[{"cbctTo4G4D", "[", 
         RowBox[{"#1", ",", "#2", ",", 
          RowBox[{
           RowBox[{"chiralSgn", "[", "#3", "]"}], "*", "normalV"}]}], "]"}]}],
        "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
       "cbctsInV", ",", "compoundingAxes", ",", "C24facetNormalsReduced"}], 
       "}"}], ",", "2"}], "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.9154756404577155`*^9, 3.9154757073621583`*^9}, {
   3.9154757508011827`*^9, 3.9154757626076107`*^9}, {3.9154758354076157`*^9, 
   3.9154759145995393`*^9}, {3.9154765541592407`*^9, 3.915476586391262*^9}, {
   3.9154767647385683`*^9, 3.915476876218735*^9}, {3.9154769554279118`*^9, 
   3.9154769697105536`*^9}, {3.9154770354505234`*^9, 
   3.9154770642911935`*^9}, {3.915484460532446*^9, 3.91548451020383*^9}, 
   3.915484592403299*^9, 3.915484733170061*^9, 3.91548485315759*^9, {
   3.9155519766295657`*^9, 3.9155519803658404`*^9}, {3.915553129342223*^9, 
   3.9155531294791174`*^9}, {3.9155534134427648`*^9, 3.915553432678824*^9}, {
   3.9155538158169575`*^9, 3.9155538161455936`*^9}, {3.9155538579455433`*^9, 
   3.915553881817735*^9}, {3.9155539148644896`*^9, 3.9155539408985767`*^9}, {
   3.9155549012513175`*^9, 3.915554915430703*^9}, 3.9155549660372596`*^9, {
   3.9155558632972555`*^9, 3.9155558650201597`*^9}, 3.915670411458685*^9, 
   3.9157477110800447`*^9, {3.9157511938982887`*^9, 3.9157511971407804`*^9}, {
   3.915753820894837*^9, 3.9157538219666185`*^9}, {3.915753989221301*^9, 
   3.915754015786106*^9}, 3.9157540806317515`*^9, {3.9157541542052493`*^9, 
   3.915754160667181*^9}, 3.915755924052978*^9, {3.915757471997488*^9, 
   3.9157574868558197`*^9}},
 CellLabel->
  "In[103]:=",ExpressionUUID->"b29a9626-6147-4602-acd4-709c18ebe158"],

Cell[TextData[{
 StyleBox["For ",
  FontColor->RGBColor[0.6, 0.4, 0.2]],
 StyleBox["n4GsInV[[1, 1]]",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0]],
 StyleBox[" there is no compounding axis defined: n4GsInV[[1, 1]] defines the \
primary space V, so there is no rotation to take it to V. \nThe default \
chirality 4G is therefore chosen, but that turns out to be the wrong one, so \
we replace it with the other one, ",
  FontColor->RGBColor[0.6, 0.4, 0.2]]
}], "Text",
 CellChangeTimes->{{3.910871896025712*^9, 3.9108718965094643`*^9}, {
  3.915753679044436*^9, 3.9157537947531195`*^9}, {3.915753860016724*^9, 
  3.9157539389211597`*^9}, {3.9157540025795593`*^9, 3.915754090853228*^9}, {
  3.9157541895819664`*^9, 3.9157542133737793`*^9}, {3.9157542590100794`*^9, 
  3.915754263373332*^9}, {3.915758565837535*^9, 
  3.915758567849348*^9}},ExpressionUUID->"dbe3d660-eedc-423b-a18f-\
cb6843f3b2cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"n4GsInV", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], " ", "=", " ", 
   RowBox[{"cbctTo4G4D", "[", 
    RowBox[{
     RowBox[{"cbctsInV", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "Null", ",", "Null", ",", 
     RowBox[{"-", "1"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->
  "In[104]:=",ExpressionUUID->"5226d951-8878-4071-bd39-47e663227e97"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Special", " ", "name", " ", "for", " ", "the", " ", "primary", " ", 
    "cuboctahedron", " ", "and", " ", "4", "G"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"CbctPrimary3D", " ", "=", " ", 
     RowBox[{"VectorProject", "[", 
      RowBox[{
       RowBox[{"C24cbctsAll", "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", 
       RowBox[{"NullSpace", "[", 
        RowBox[{"{", "normalV", "}"}], "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"n4GPrimary3D", " ", "=", " ", 
     RowBox[{"VectorProject", "[", 
      RowBox[{
       RowBox[{"n4GsInV", "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", 
       RowBox[{"NullSpace", "[", 
        RowBox[{"{", "normalV", "}"}], "]"}]}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.915445635398823*^9, 3.9154456935111933`*^9}, {
   3.9155525859831595`*^9, 3.915552652270465*^9}, {3.915552682882763*^9, 
   3.915552705128583*^9}, {3.915552738954566*^9, 3.9155527634051075`*^9}, {
   3.915552795438485*^9, 3.9155528156437297`*^9}, {3.9155528679948626`*^9, 
   3.915552877195171*^9}, 3.915747711085045*^9, {3.9157514467646456`*^9, 
   3.9157514726156044`*^9}, 3.9157515093643675`*^9, {3.9157515691918364`*^9, 
   3.9157515800546684`*^9}, 3.9157516352910132`*^9, {3.915755924057984*^9, 
   3.915755924059983*^9}, {3.915757549331791*^9, 3.9157576573796883`*^9}},
 CellLabel->
  "In[105]:=",ExpressionUUID->"bb42ae56-1d9b-450d-80c9-d99e66cfaf9c"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Display", "Subsection",
 CellChangeTimes->{{3.9157576617900205`*^9, 
  3.915757675075717*^9}},ExpressionUUID->"4841dc4c-9c56-401c-a8ac-\
412d49f7fd90"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"cbctSelections", " ", "=", " ", 
     RowBox[{"{", 
      RowBox[{"1", ",", "Cbct2", ",", "Cbct3", ",", "Cbct4", ",", "Cbct5"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Which", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"shapeChoice", " ", "==", " ", "\"\<4Gs\>\""}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"shapes", "=", "n4GsInV"}], ";", 
        RowBox[{"primaryShape", "=", "n4GPrimary3D"}], ";", 
        RowBox[{"mapDepth", "=", "2"}], ";", 
        RowBox[{"plottingFxn", " ", "=", " ", "Tetrahedron"}]}], "}"}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"shapeChoice", " ", "==", " ", "\"\<Cuboctahdera\>\""}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"shapes", "=", "cbctsInV"}], ";", 
        RowBox[{"primaryShape", "=", "CbctPrimary3D"}], ";", 
        RowBox[{"mapDepth", "=", "1"}], ";", 
        RowBox[{"plottingFxn", " ", "=", " ", "ConvexHullMesh"}]}], "}"}]}], 
     "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"C24cbctDualRots", " ", "=", " ", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"rotMatrixToDual4D", "[", 
         RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"C24cbctRotsToV", "[", 
          RowBox[{"[", 
           RowBox[{"k", ",", 
            RowBox[{"cbctSelections", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "1", ",", "5"}], "}"}]}], "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"shapesInVtwisted", " ", "=", " ", 
     RowBox[{"MapThread", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Function", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"rotMat", ",", "ptSets"}], "}"}], ",", 
           RowBox[{"Map", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"rotMat", ".", "#"}], "&"}], ",", "ptSets", ",", 
             RowBox[{"{", "mapDepth", "}"}]}], "]"}]}], "]"}], "[", 
         RowBox[{"#1", ",", "#2"}], "]"}], "&"}], ",", 
       RowBox[{"{", 
        RowBox[{"C24cbctDualRots", ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"shapes", "[", 
            RowBox[{"[", 
             RowBox[{"k", ",", 
              RowBox[{"cbctSelections", "[", 
               RowBox[{"[", "k", "]"}], "]"}]}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "1", ",", "5"}], "}"}]}], "]"}]}], "}"}]}], 
      "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"shapesInV3D", " ", "=", " ", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"VectorProject", "[", 
         RowBox[{"#", ",", 
          RowBox[{"NullSpace", "[", 
           RowBox[{"{", "normalV", "}"}], "]"}]}], "]"}], "&"}], ",", 
       "shapesInVtwisted", ",", 
       RowBox[{"{", "mapDepth", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"Column", "[", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{"Graphics3D", "[", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{"(*", "Magenta", "*)"}], 
         RowBox[{"Yellow", "\[IndentingNewLine]", ",", " ", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{"n4GList", ",", "1"}], "]"}], ",", 
            RowBox[{"plottingFxn", " ", "@", " ", 
             RowBox[{"(", 
              RowBox[{"1.01", "*", "primaryShape"}], ")"}]}]}], "]"}], 
          "\[IndentingNewLine]", ",", "Yellow", ",", " ", 
          RowBox[{"plottingFxn", " ", "/@", " ", 
           RowBox[{"shapesInV3D", "[", 
            RowBox[{"[", 
             RowBox[{"DeleteCases", "[", 
              RowBox[{"n4GList", ",", "1"}], "]"}], "]"}], "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"If", "[", 
           RowBox[{"showAxes", ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{"Tube", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"ctr3", ",", 
                    RowBox[{".8", "*", 
                    RowBox[{"compoundingAxes3D", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", 
                    RowBox[{"cbctSelections", "[", 
                    RowBox[{"[", "k", "]"}], "]"}]}], "]"}], "]"}]}]}], "}"}],
                   ",", ".015"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"k", ",", 
                  RowBox[{"DeleteCases", "[", 
                   RowBox[{"n4GList", ",", "1"}], "]"}]}], "}"}]}], "]"}]}], 
             "}"}]}], "]"}]}], "\[IndentingNewLine]", "}"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"ImageSize", "->", "Medium"}], "\[IndentingNewLine]", ",", 
        RowBox[{"PlotRange", "->", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".75"}], ",", ".75"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".75"}], ",", ".75"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".75"}], ",", ".75"}], "}"}]}], "}"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"Boxed", "->", "False"}], "\[IndentingNewLine]", ",", 
        RowBox[{"SphericalRegion", "->", "True"}], "\[IndentingNewLine]", ",", 
        RowBox[{"ViewPoint", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"3.277", ",", "0.756", ",", 
           RowBox[{"-", "0.378"}]}], "}"}]}], "\[IndentingNewLine]", ",", 
        RowBox[{"ViewVertical", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0.410", ",", "0.905", ",", "0.113"}], "}"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"ViewAngle", "->", ".4"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", "}"}], "]"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"C24cbctDualRots", ",", "None"}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{"shapes", ",", "None"}], "}"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"primaryShape", ",", "None"}], "}"}], "\[IndentingNewLine]", ",", 
   
   RowBox[{"{", 
    RowBox[{"mapDepth", ",", "None"}], "}"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"plottingFxn", ",", "None"}], "}"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"shapesInVtwisted", ",", "None"}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{"shapesInV3D", ",", "None"}], "}"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{",", 
     RowBox[{"{", 
      RowBox[{"cmpdAxes3D", ",", "None"}], "}"}]}], "*)"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "shapeChoice", ",", "\"\<4Gs\>\"", ",", "\"\<Shapes to show\>\""}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<4Gs\>\"", ",", "\"\<Cuboctahdera\>\""}], "}"}]}], "}"}], 
   "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "showAxes", ",", "False", ",", "\"\<Show Compounding Axes\>\""}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}]}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"n4GList", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], ",", 
       "\"\<Show Cuboctahedron/4G from Selected 24-cells\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], ",", 
     "TogglerBar"}], "}"}], "\[IndentingNewLine]", ",", 
   RowBox[{
   "Text", "[", "\"\<Select which Cuboctahedron/4G from each 24-cell\>\"", 
    "]"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Cbct2", ",", "1", ",", "\"\<4G/Cuboct from 24-cell 2\>\""}], 
      "}"}], ",", "1", ",", "12", ",", "1"}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Cbct3", ",", "1", ",", "\"\<4G/Cuboct from 24-cell 3\>\""}], 
      "}"}], ",", "1", ",", "12", ",", "1"}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Cbct4", ",", "1", ",", "\"\<4G/Cuboct from 24-cell 4\>\""}], 
      "}"}], ",", "1", ",", "12", ",", "1"}], "}"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Cbct5", ",", "1", ",", "\"\<4G/Cuboct from 24-cell 5\>\""}], 
      "}"}], ",", "1", ",", "12", ",", "1"}], "}"}], "\[IndentingNewLine]", 
   ",", "Delimiter", "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "0", ",", "\"\<Twist fraction\>\""}], "}"}], 
     ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], "\[IndentingNewLine]", ",", 
   RowBox[{"LabelStyle", "->", 
    RowBox[{"{", "11", "}"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"SaveDefinitions", "->", "True"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{{3.910894820456312*^9, 3.910894906299671*^9}, {
   3.9108949414416533`*^9, 3.910895061526923*^9}, 3.9108960752842693`*^9, {
   3.910896126952634*^9, 3.9108961325270996`*^9}, {3.9108961653704634`*^9, 
   3.910896178063634*^9}, {3.9108962592935658`*^9, 3.9108962631706667`*^9}, {
   3.9108963485311728`*^9, 3.9108963617198925`*^9}, {3.9108964512539873`*^9, 
   3.910896483394081*^9}, {3.910896625993469*^9, 3.9108967002037396`*^9}, {
   3.910896754254157*^9, 3.910896793658245*^9}, {3.9108968353554535`*^9, 
   3.9108968890940914`*^9}, {3.9108969418775053`*^9, 3.910896979602131*^9}, {
   3.910897061904909*^9, 3.910897075122732*^9}, {3.9108971080809093`*^9, 
   3.9108971322092967`*^9}, {3.9108972571945267`*^9, 
   3.9108972707277203`*^9}, {3.910898967928178*^9, 3.910899002685068*^9}, {
   3.910899754492032*^9, 3.9108997552432594`*^9}, 3.9109569651006966`*^9, {
   3.9109570375896873`*^9, 3.9109571800505457`*^9}, {3.9109575604616117`*^9, 
   3.910957570123988*^9}, {3.9109584443165073`*^9, 3.910958455230523*^9}, 
   3.9109585379284954`*^9, {3.9109585920720806`*^9, 3.9109586307334037`*^9}, {
   3.9109586635446014`*^9, 3.9109586791009817`*^9}, 3.910958807617885*^9, {
   3.911056437501968*^9, 3.911056437642454*^9}, 3.911056578523185*^9, 
   3.911056630635018*^9, 3.9110566794363537`*^9, {3.9110567257403092`*^9, 
   3.911056725871578*^9}, {3.91105681590096*^9, 3.9110568159771013`*^9}, {
   3.911056889042301*^9, 3.9110568891643457`*^9}, {3.9110584050909147`*^9, 
   3.9110584052453184`*^9}, {3.9110588878720207`*^9, 3.911058888025629*^9}, {
   3.911059103145014*^9, 3.9110591130086975`*^9}, 3.9110591698447323`*^9, 
   3.9110592218745995`*^9, 3.911059263137459*^9, {3.9110593048414154`*^9, 
   3.911059304980448*^9}, {3.911059355957342*^9, 3.9110593569357986`*^9}, {
   3.911495760112678*^9, 3.911495760248454*^9}, 3.9114959665585146`*^9, 
   3.9114959975946407`*^9, {3.9114960764741344`*^9, 3.911496112893132*^9}, {
   3.9114961922809167`*^9, 3.911496254685687*^9}, {3.9114963152255716`*^9, 
   3.9114963647040467`*^9}, 3.911497158014125*^9, {3.911497246030676*^9, 
   3.91149727900556*^9}, 3.911498050384465*^9, 3.911498177217743*^9, 
   3.9114983836359234`*^9, 3.9114989614480457`*^9, 3.911499736311598*^9, 
   3.911499840660499*^9, 3.9115018764407387`*^9, {3.9115055385644903`*^9, 
   3.911505538667306*^9}, {3.9115055996764927`*^9, 3.9115056513448515`*^9}, {
   3.911505900803226*^9, 3.9115059124822636`*^9}, {3.9115060183229895`*^9, 
   3.911506024652553*^9}, {3.9115063268700113`*^9, 3.9115063331335373`*^9}, 
   3.912559224411136*^9, {3.9125592666697083`*^9, 3.9125592712434864`*^9}, {
   3.912559493826095*^9, 3.9125595422159815`*^9}, {3.912559686533162*^9, 
   3.912559715234943*^9}, {3.912559881518701*^9, 3.912559916662823*^9}, {
   3.9125599528732505`*^9, 3.9125600453358717`*^9}, {3.9125600830145173`*^9, 
   3.912560110619684*^9}, {3.9126271851778812`*^9, 3.912627202713244*^9}, {
   3.9154113595751133`*^9, 3.9154113960617056`*^9}, {3.915411466127329*^9, 
   3.915411579928837*^9}, {3.9154117214865327`*^9, 3.915411781745887*^9}, {
   3.9154118249088564`*^9, 3.9154118750704355`*^9}, {3.915411918415908*^9, 
   3.915411919579522*^9}, {3.915411951322055*^9, 3.9154120443063145`*^9}, 
   3.9154121080493712`*^9, 3.915412149380085*^9, {3.9154453454760103`*^9, 
   3.915445370696869*^9}, {3.915445403877271*^9, 3.915445505944993*^9}, {
   3.9154455484458055`*^9, 3.915445572702056*^9}, 3.9154457009763203`*^9, {
   3.915445757868154*^9, 3.915445804266374*^9}, {3.9154462928522825`*^9, 
   3.9154462944456844`*^9}, {3.9154463756591606`*^9, 
   3.9154463962788563`*^9}, {3.915446471030855*^9, 3.915446500999067*^9}, {
   3.915446542978367*^9, 3.9154465761061945`*^9}, {3.91544661118421*^9, 
   3.915446611445791*^9}, {3.915447886647625*^9, 3.9154479307295165`*^9}, {
   3.9154479925168056`*^9, 3.9154480045916576`*^9}, {3.915448151518607*^9, 
   3.9154481668291817`*^9}, 3.9154483412797546`*^9, 3.9154483984864817`*^9, {
   3.9154485809272404`*^9, 3.915448593890604*^9}, {3.915448689934389*^9, 
   3.9154486905606976`*^9}, {3.9154487245812407`*^9, 3.9154487433480225`*^9}, 
   3.915448880822561*^9, {3.915449150044938*^9, 3.915449164757327*^9}, 
   3.9154493178130465`*^9, 3.915449397660627*^9, 3.9154770220806046`*^9, {
   3.9154796148022537`*^9, 3.915479623260956*^9}, {3.9154796813264394`*^9, 
   3.9154796834607897`*^9}, 3.9154797312999487`*^9, 3.915479792974825*^9, {
   3.9154798345038385`*^9, 3.9154798359124775`*^9}, {3.915480021684892*^9, 
   3.915480024986182*^9}, 3.915552314161566*^9, {3.9155525413775663`*^9, 
   3.9155525553298016`*^9}, 3.9155528939892416`*^9, 3.9155529619072995`*^9, 
   3.915555032733306*^9, {3.9155550907608232`*^9, 3.915555117502822*^9}, {
   3.91555523755208*^9, 3.9155552500944133`*^9}, {3.9155552921850157`*^9, 
   3.915555302393127*^9}, {3.9155566807922153`*^9, 3.915556698302235*^9}, {
   3.915556855861268*^9, 3.9155569476891575`*^9}, {3.9155569815239415`*^9, 
   3.915556983023034*^9}, {3.915557022646041*^9, 3.91555711588719*^9}, {
   3.915557919124617*^9, 3.915557964906913*^9}, {3.9155579997233076`*^9, 
   3.9155580123072987`*^9}, 3.915558212986166*^9, {3.915558322699921*^9, 
   3.915558348253763*^9}, {3.9155583938932104`*^9, 3.9155584174919786`*^9}, {
   3.9155584788184958`*^9, 3.91555849487239*^9}, {3.9155591317693024`*^9, 
   3.915559167726294*^9}, {3.915559201600144*^9, 3.9155592583919773`*^9}, {
   3.9155594998180275`*^9, 3.9155595103041096`*^9}, {3.91555954245637*^9, 
   3.915559543158424*^9}, {3.9155595806793776`*^9, 3.915559595998962*^9}, {
   3.9155597108164177`*^9, 3.9155597308961535`*^9}, {3.9155598278332295`*^9, 
   3.9155598848648057`*^9}, {3.915560478832279*^9, 3.9155605190568447`*^9}, {
   3.9155605539528446`*^9, 3.915560559744742*^9}, {3.9155609742827787`*^9, 
   3.915561106232915*^9}, {3.9155611916182337`*^9, 3.915561225442152*^9}, {
   3.9156704183764935`*^9, 3.9156704514907074`*^9}, {3.9157477110912*^9, 
   3.9157477111132097`*^9}, {3.915748435500563*^9, 3.9157484369875174`*^9}, {
   3.915749164954647*^9, 3.915749211040168*^9}, 3.9157494795797615`*^9, {
   3.9157495729024544`*^9, 3.915749683227516*^9}, {3.91574971750282*^9, 
   3.915749778826683*^9}, 3.9157498176996317`*^9, {3.9157498570086546`*^9, 
   3.9157498667724323`*^9}, {3.915749976932053*^9, 3.9157500347446136`*^9}, {
   3.9157501451087084`*^9, 3.9157501490122476`*^9}, 3.915750299720761*^9, 
   3.9157504444375544`*^9, {3.9157504945827446`*^9, 3.9157505169776635`*^9}, {
   3.915750569079303*^9, 3.9157505734738226`*^9}, 3.9157506090548553`*^9, 
   3.9157506415939784`*^9, {3.915750901636833*^9, 3.915750902595148*^9}, {
   3.91575100214756*^9, 3.91575113603566*^9}, 3.9157512445069427`*^9, {
   3.915751277663027*^9, 3.915751303452973*^9}, {3.915751335228569*^9, 
   3.9157513413087378`*^9}, {3.91575137159354*^9, 3.915751395911289*^9}, {
   3.9157514265196223`*^9, 3.9157514282953844`*^9}, {3.9157515019897704`*^9, 
   3.9157515022101026`*^9}, {3.9157516153347034`*^9, 3.915751659513386*^9}, {
   3.9157516983752203`*^9, 3.9157518486879654`*^9}, 3.91575188404547*^9, {
   3.9157519163522863`*^9, 3.915751958450523*^9}, {3.915752006384119*^9, 
   3.9157520871986337`*^9}, {3.915752121337578*^9, 3.915752144750229*^9}, {
   3.9157522282992525`*^9, 3.9157522307529974`*^9}, {3.915752421577854*^9, 
   3.915752468154069*^9}, {3.91575253099961*^9, 3.915752573789648*^9}, {
   3.915752647738429*^9, 3.9157527061363044`*^9}, {3.9157527771801133`*^9, 
   3.9157527829375687`*^9}, {3.915752818225774*^9, 3.915752927884008*^9}, 
   3.9157529610345254`*^9, {3.915753074063204*^9, 3.9157530741280007`*^9}, {
   3.915753114058798*^9, 3.915753147084404*^9}, 3.9157531885599365`*^9, {
   3.9157532387554007`*^9, 3.9157532443232875`*^9}, {3.915753362221464*^9, 
   3.9157534329792967`*^9}, {3.9157535693907747`*^9, 3.9157535851808615`*^9}, 
   3.915755924067988*^9, 3.9157560451794643`*^9, {3.915756297291854*^9, 
   3.915756340233361*^9}, {3.915756396587946*^9, 3.91575683339143*^9}, {
   3.9157568711402965`*^9, 3.9157571461625657`*^9}, {3.9157572579896183`*^9, 
   3.9157573004891996`*^9}, {3.91575808733784*^9, 3.915758095230329*^9}, {
   3.9157581746773443`*^9, 3.915758237312252*^9}, {3.9158337052366934`*^9, 
   3.915833719806097*^9}, {3.91626901240974*^9, 3.9162690165634775`*^9}, 
   3.916269402705904*^9, {3.916522622095584*^9, 3.916522628612273*^9}},
 CellLabel->
  "In[107]:=",ExpressionUUID->"c512e144-3e2f-438d-8f17-f184e444b24d"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`C24cbctDualRots$$ = {{{1., 0, 0, 0}, {
      0, 1., 0, 0}, {0, 0, 1., 0}, {0, 0, 0, 1.}}, {{
      1.0000000000000002`, 0, 0, 0}, {0, 1., 0, 0}, {
      0, 0, 0.9999999999999999, 0}, {0, 0, 0, 1.}}, {{
      0.9999999999999999, 0, 0, 0}, {0, 1., 0, 0}, {
      0, 0, 0.9999999999999997, 0}, {0, 0, 0, 1.0000000000000002`}}, {{
      1., 0, 0, 0}, {0, 0.9999999999999998, 0, 0}, {0, 0, 1., 0}, {
      0, 0, 0, 1.}}, {{1.0000000000000002`, 0, 0, 0}, {0, 1., 0, 0}, {
      0, 0, 0.9999999999999999, 0}, {
      0, 0, 0, 0.9999999999999999}}}, $CellContext`Cbct2$$ = 
    1, $CellContext`Cbct3$$ = 1, $CellContext`Cbct4$$ = 
    1, $CellContext`Cbct5$$ = 1, $CellContext`mapDepth$$ = 
    2, $CellContext`n4GList$$ = {1, 2, 3, 4, 5}, $CellContext`plottingFxn$$ = 
    Tetrahedron, $CellContext`primaryShape$$ = CompressedData["
1:eJxTTMoPSmVmYGBggWIQO8T2Q2HCpln2xuo3PgqKPrH/9XOr973Sm/t/Qmh7
NPn9aOpg8vsZcADPpR7v6uwf77+54ot+d8ZZe5Vv01x2nLoO4+9XhfI9oOpg
9sDcAbMfl/kw/VB77G9B7YHZB5W3h5pv/xPVXLh/CJhvD/MH1Hy4fTB/QP1l
j2YuzB84zQcAFd2l0A==
    "], $CellContext`shapeChoice$$ = 
    "4Gs", $CellContext`shapes$$ = {{{{{
        0.2297529205473612, 0, 0.6015009550075456, -0.37174803446018445`}, {
        0.2297529205473612, 0.37174803446018445`, 0, -0.6015009550075456}, {
        0.2297529205473612, 0.6015009550075456, 0.37174803446018445`, 0}, {0, 
        0, 0, 0}}, {{
        0, -0.6015009550075456, 0.2297529205473612, -0.37174803446018445`}, {
        0, -0.2297529205473612, -0.37174803446018445`, -0.6015009550075456}, \
{-0.2297529205473612, -0.6015009550075456, -0.37174803446018445`, 0}, {0, 0, 
        0, 0}}, {{
        0, -0.37174803446018445`, 0.6015009550075456, 0.2297529205473612}, {
        0, 0.2297529205473612, 0.37174803446018445`, 
         0.6015009550075456}, {-0.2297529205473612, -0.37174803446018445`, 0, 
         0.6015009550075456}, {0, 0, 0, 0}}, {{
        0, 0.37174803446018445`, -0.6015009550075456, -0.2297529205473612}, {
        0, 0.6015009550075456, -0.2297529205473612, 
         0.37174803446018445`}, {-0.2297529205473612, 0, -0.6015009550075456, 
         0.37174803446018445`}, {0, 0, 0, 0}}}, {{{
        0, -0.6015009550075454, 0.2297529205473612, -0.37174803446018445`}, {
        0, -0.22975292054736113`, -0.37174803446018445`, \
-0.6015009550075456}, {0.22975292054736116`, 0.04733170463760791, 
        0.27708462518496907`, -0.6488326596451535}, {0, 0, 0, 0}}, {{
        0, -0.37174803446018445`, 0.6015009550075456, 0.22975292054736124`}, {
        0, 0.22975292054736113`, 0.37174803446018445`, 0.6015009550075456}, {
        0.22975292054736118`, 0.27708462518496907`, 
        0.6488326596451535, -0.04733170463760783}, {0, 0, 0, 0}}, {{
        0, 0.37174803446018445`, -0.6015009550075456, -0.22975292054736124`}, \
{0, 0.6015009550075454, -0.2297529205473612, 0.37174803446018445`}, {
        0.22975292054736118`, 0.6488326596451535, 
        0.04733170463760786, -0.27708462518496907`}, {0, 0, 0, 
        0}}, {{-0.22975292054736118`, -0.6488326596451535, \
-0.04733170463760786, 
        0.27708462518496907`}, {-0.22975292054736118`, -0.27708462518496907`, \
-0.6488326596451535, 
        0.04733170463760783}, {-0.22975292054736116`, -0.04733170463760791, \
-0.27708462518496907`, 0.6488326596451535}, {0, 0, 0, 0}}}, {{{
        0.07658430684912032, 
        0.6488326596451539, -0.3536689320340896, -0.02925260221151252}, {
        0.22975292054736118`, 0.3717480344601845, 0, -0.6015009550075456}, {
        0.2297529205473612, 0.6015009550075459, 0.3717480344601847, 0}, {0, 0,
         0, 0}}, {{-0.1531686136982409, 
        0.04733170463760798, -0.7254169664942742, -0.02925260221151249}, {
        0, -0.22975292054736135`, -0.3717480344601845, -0.6015009550075456}, \
{-0.2297529205473612, -0.6015009550075459, -0.3717480344601847, 0}, {0, 0, 0, 
        0}}, {{0.15316861369824084`, -0.27708462518496935`, 
        0.3536689320340897, -0.5722483527960331}, {
        0.1531686136982409, -0.04733170463760798, 0.7254169664942742, 
        0.02925260221151249}, {-0.07658430684912032, -0.6488326596451539, 
        0.3536689320340896, 0.02925260221151252}, {0, 0, 0, 
        0}}, {{-0.15316861369824084`, 
        0.27708462518496935`, -0.3536689320340897, 0.5722483527960331}, {
        0, 0.22975292054736135`, 0.3717480344601845, 
         0.6015009550075456}, {-0.22975292054736118`, -0.3717480344601845, 0, 
         0.6015009550075456}, {0, 0, 0, 0}}}, {{{0.0765843068491204, 
        0.029252602211512567`, 0.6488326596451537, 0.3536689320340897}, {
        0.22975292054736124`, 0, 0.6015009550075456, -0.37174803446018445`}, {
        0.2297529205473612, 0.6015009550075456, 0.37174803446018445`, 0}, {0, 
        0, 0, 0}}, {{-0.15316861369824086`, 0.02925260221151253, 
        0.04733170463760805, 0.7254169664942741}, {
        0, 0.6015009550075456, -0.2297529205473612, 
         0.3717480344601844}, {-0.22975292054736124`, 0, -0.6015009550075456, 
         0.37174803446018445`}, {0, 0, 0, 0}}, {{
        0.15316861369824086`, -0.02925260221151253, -0.04733170463760805, \
-0.7254169664942741}, {0.15316861369824078`, 
        0.5722483527960331, -0.27708462518496924`, -0.3536689320340898}, \
{-0.0765843068491204, -0.029252602211512567`, -0.6488326596451537, \
-0.3536689320340897}, {0, 0, 0, 
        0}}, {{-0.15316861369824078`, -0.5722483527960331, 
        0.27708462518496924`, 0.3536689320340898}, {
        0, -0.6015009550075456, 
         0.2297529205473612, -0.3717480344601844}, {-0.2297529205473612, \
-0.6015009550075456, -0.37174803446018445`, 0}, {0, 0, 0, 0}}}, {{{
        0.15316861369824092`, 0.3536689320340897, 0.572248352796033, 
        0.2770846251849691}, {
        0.22975292054736127`, 0, 0.6015009550075455, -0.3717480344601845}, {
        0, -0.3717480344601845, 0.6015009550075454, 0.2297529205473613}, {0, 
        0, 0, 0}}, {{0.1531686136982409, 
        0.7254169664942742, -0.02925260221151247, 0.04733170463760786}, {
        0.22975292054736118`, 0.37174803446018456`, 0, -0.6015009550075457}, {
        0, 0.3717480344601845, -0.6015009550075454, -0.2297529205473613}, {0, 
        0, 0, 0}}, {{0.07658430684912029, -0.35366893203408967`, 
        0.029252602211512498`, -0.6488326596451537}, {-0.1531686136982409, \
-0.7254169664942742, 
        0.02925260221151247, -0.04733170463760786}, {-0.15316861369824092`, \
-0.3536689320340897, -0.572248352796033, -0.2770846251849691}, {0, 0, 0, 
        0}}, {{-0.07658430684912029, 
        0.35366893203408967`, -0.029252602211512498`, 
        0.6488326596451537}, {-0.22975292054736118`, -0.37174803446018456`, 0,
          0.6015009550075457}, {-0.22975292054736127`, 0, -0.6015009550075455,
          0.3717480344601845}, {0, 0, 0, 0}}}, {{{
        0, 0.22975292054736124`, 0.37174803446018445`, 
         0.601500955007546}, {-0.2297529205473612, -0.37174803446018423`, 0, 
         0.601500955007546}, {
        0, -0.3717480344601844, 0.6015009550075456, 0.22975292054736113`}, {0,
         0, 0, 0}}, {{
        0.2297529205473611, 0.6015009550075454, 0.37174803446018445`, 0}, {
        0.22975292054736124`, 0, 0.6015009550075456, -0.3717480344601849}, {
        0.2297529205473612, 0.37174803446018423`, 0, -0.601500955007546}, {0, 
        0, 0, 0}}, {{-0.2297529205473611, -0.6015009550075454, \
-0.37174803446018445`, 0}, {
        0, -0.6015009550075455, 0.22975292054736124`, -0.3717480344601848}, {
        0, -0.22975292054736124`, -0.37174803446018445`, -0.601500955007546}, \
{0, 0, 0, 0}}, {{
        0, 0.6015009550075455, -0.22975292054736124`, 
         0.3717480344601848}, {-0.22975292054736124`, 0, -0.6015009550075456, 
         0.3717480344601849}, {
        0, 0.3717480344601844, -0.6015009550075456, -0.22975292054736113`}, {
        0, 0, 0, 0}}}, {{{0.0765843068491204, -0.3536689320340898, 
        0.02925260221151251, -0.6488326596451535}, {
        0.2297529205473612, 0, 0.6015009550075456, -0.3717480344601844}, {
        0.22975292054736127`, 0.37174803446018445`, 0, -0.6015009550075454}, {
        0, 0, 0, 0}}, {{-0.15316861369824084`, -0.7254169664942741, 
        0.02925260221151249, -0.047331704637608045`}, {
        0, -0.3717480344601844, 0.6015009550075456, 
         0.22975292054736116`}, {-0.22975292054736127`, -0.37174803446018445`,
          0, 0.6015009550075454}, {0, 0, 0, 0}}, {{0.15316861369824078`, 
        0.3536689320340898, 0.572248352796033, 0.2770846251849692}, {
        0.15316861369824084`, 0.7254169664942741, -0.02925260221151249, 
        0.047331704637608045`}, {-0.0765843068491204, 
        0.3536689320340898, -0.02925260221151251, 0.6488326596451535}, {0, 0, 
        0, 0}}, {{-0.15316861369824078`, -0.3536689320340898, \
-0.572248352796033, -0.2770846251849692}, {
        0, 
         0.3717480344601844, -0.6015009550075456, -0.22975292054736116`}, \
{-0.2297529205473612, 0, -0.6015009550075456, 0.3717480344601844}, {0, 0, 0, 
        0}}}, {{{
        0.22975292054736113`, 0, 0.6015009550075455, -0.3717480344601844}, {
        0.22975292054736118`, 0.6015009550075456, 0.37174803446018434`, 0}, {
        0.07658430684912021, 0.0292526022115126, 0.6488326596451535, 
        0.35366893203408956`}, {0, 0, 0, 0}}, {{
        0, -0.6015009550075456, 
         0.2297529205473612, -0.37174803446018445`}, {-0.22975292054736118`, \
-0.6015009550075456, -0.37174803446018434`, 
         0}, {-0.15316861369824092`, -0.572248352796033, 0.2770846251849691, 
        0.35366893203408956`}, {0, 0, 0, 0}}, {{
        0, 0.6015009550075456, -0.2297529205473612, 
         0.37174803446018445`}, {-0.22975292054736113`, 
         0, -0.6015009550075455, 0.3717480344601844}, {-0.15316861369824086`, 
        0.029252602211512616`, 0.04733170463760792, 0.725416966494274}, {0, 0,
         0, 0}}, {{
        0.15316861369824086`, -0.029252602211512616`, -0.04733170463760792, \
-0.725416966494274}, {0.15316861369824092`, 
        0.572248352796033, -0.2770846251849691, -0.35366893203408956`}, \
{-0.07658430684912021, -0.0292526022115126, -0.6488326596451535, \
-0.35366893203408956`}, {0, 0, 0, 0}}}, {{{
        0, -0.2297529205473612, -0.3717480344601848, -0.6015009550075456}, {
        0, 0.3717480344601845, -0.601500955007546, -0.22975292054736127`}, {
        0.2297529205473611, 0.3717480344601845, 0, -0.6015009550075454}, {0, 
        0, 0, 0}}, {{
        0, -0.6015009550075456, 0.22975292054736116`, -0.3717480344601844}, {
        0.22975292054736118`, 0, 0.601500955007546, -0.37174803446018423`}, {
        0, -0.3717480344601845, 0.601500955007546, 0.22975292054736127`}, {0, 
        0, 0, 0}}, {{
        0, 0.6015009550075456, -0.22975292054736116`, 0.3717480344601844}, {
        0.2297529205473613, 0.6015009550075456, 0.37174803446018484`, 0}, {
        0, 0.2297529205473612, 0.3717480344601848, 0.6015009550075456}, {0, 0,
         0, 0}}, {{-0.2297529205473613, -0.6015009550075456, \
-0.37174803446018484`, 0}, {-0.22975292054736118`, 0, -0.601500955007546, 
         0.37174803446018423`}, {-0.2297529205473611, -0.3717480344601845, 0, 
         0.6015009550075454}, {0, 0, 0, 0}}}, {{{
        0.1531686136982407, -0.047331704637607865`, 0.725416966494274, 
        0.0292526022115126}, {
        0.22975292054736116`, 0.6015009550075457, 0.37174803446018434`, 0}, {
        0, 0.22975292054736124`, 0.3717480344601845, 0.6015009550075456}, {0, 
        0, 0, 0}}, {{-0.0765843068491204, -0.6488326596451537, 
        0.35366893203408967`, 
        0.029252602211512574`}, {-0.22975292054736116`, -0.6015009550075457, \
-0.37174803446018434`, 0}, {-0.22975292054736116`, -0.3717480344601845, 0, 
         0.6015009550075456}, {0, 0, 0, 0}}, {{0.0765843068491204, 
        0.6488326596451537, -0.35366893203408967`, -0.029252602211512574`}, \
{-0.1531686136982407, 
        0.047331704637607865`, -0.725416966494274, -0.0292526022115126}, \
{-0.15316861369824075`, 0.2770846251849691, -0.35366893203408956`, 
        0.572248352796033}, {0, 0, 0, 0}}, {{
        0.15316861369824075`, -0.2770846251849691, 
        0.35366893203408956`, -0.572248352796033}, {
        0.22975292054736116`, 0.3717480344601845, 0, -0.6015009550075456}, {
        0, -0.22975292054736124`, -0.3717480344601845, -0.6015009550075456}, {
        0, 0, 0, 0}}}, {{{
        0, -0.601500955007546, 
         0.22975292054736127`, -0.37174803446018445`}, \
{-0.22975292054736116`, -0.601500955007546, -0.3717480344601843, 0}, {
        0, -0.22975292054736116`, -0.37174803446018445`, \
-0.6015009550075456}, {0, 0, 0, 0}}, {{
        0.2297529205473611, 0, 0.6015009550075455, -0.3717480344601845}, {
        0.22975292054736127`, 0.3717480344601849, 0, -0.6015009550075456}, {
        0.22975292054736116`, 0.601500955007546, 0.3717480344601843, 0}, {0, 
        0, 0, 0}}, {{-0.2297529205473611, 0, -0.6015009550075455, 
         0.3717480344601845}, {
        0, 0.3717480344601848, -0.6015009550075457, -0.2297529205473612}, {
        0, 0.601500955007546, -0.22975292054736127`, 0.37174803446018445`}, {
        0, 0, 0, 0}}, {{
        0, -0.3717480344601848, 0.6015009550075457, 
         0.2297529205473612}, {-0.22975292054736127`, -0.3717480344601849, 0, 
         0.6015009550075456}, {
        0, 0.22975292054736116`, 0.37174803446018445`, 0.6015009550075456}, {
        0, 0, 0, 0}}}, {{{0.22975292054736135`, 0.04733170463760797, 
        0.2770846251849692, -0.6488326596451536}, {
        0, -0.6015009550075456, 
         0.22975292054736118`, -0.37174803446018445`}, {
        0, -0.2297529205473612, -0.37174803446018445`, -0.6015009550075456}, {
        0, 0, 0, 0}}, {{0.22975292054736135`, 0.6488326596451536, 
        0.047331704637607976`, -0.2770846251849692}, {
        0, 0.37174803446018445`, -0.6015009550075455, -0.2297529205473612}, {
        0, 0.6015009550075456, -0.22975292054736118`, 0.37174803446018445`}, {
        0, 0, 0, 
        0}}, {{-0.22975292054736135`, -0.6488326596451536, \
-0.047331704637607976`, 
        0.2770846251849692}, {-0.22975292054736135`, -0.2770846251849692, \
-0.6488326596451536, 
        0.04733170463760797}, {-0.22975292054736135`, -0.04733170463760797, \
-0.2770846251849692, 0.6488326596451536}, {0, 0, 0, 0}}, {{
        0.22975292054736135`, 0.2770846251849692, 
        0.6488326596451536, -0.04733170463760797}, {
        0, -0.37174803446018445`, 0.6015009550075455, 0.2297529205473612}, {
        0, 0.2297529205473612, 0.37174803446018445`, 0.6015009550075456}, {0, 
        0, 0, 0}}}}, {{{{0.18587401723009248`, 
        0.22975292054736168`, -0.11487646027368059`, -0.6724985119639572}, {
        0.07099755695641177, 
        0.6015009550075462, -0.4156269377774533, -0.11487646027368069`}, {
        0.25687157418650425`, 0.6015009550075466, 
        0.3007504775037733, -0.18587401723009236`}, {0, 0, 0, 0}}, {{
        0.11487646027368083`, -0.37174803446018456`, 
        0.3007504775037728, -0.5576220516902766}, {
        0.18587401723009245`, 0, 
         0.7163774152812266, -0.07099755695641166}, {-0.07099755695641177, \
-0.6015009550075462, 0.4156269377774533, 0.11487646027368069`}, {0, 0, 0, 
        0}}, {{-0.11487646027368083`, 
        0.37174803446018456`, -0.3007504775037728, 0.5576220516902766}, {
        0.0709975569564118, 0.371748034460185, 0.41562693777745374`, 
        0.48662449473386493`}, {-0.18587401723009248`, -0.22975292054736168`, 
        0.11487646027368059`, 0.6724985119639572}, {0, 0, 0, 
        0}}, {{-0.0709975569564118, -0.371748034460185, \
-0.41562693777745374`, -0.48662449473386493`}, {-0.18587401723009245`, 
         0, -0.7163774152812266, 
         0.07099755695641166}, {-0.25687157418650425`, -0.6015009550075466, \
-0.3007504775037733, 0.18587401723009236`}, {0, 0, 0, 
        0}}}, {{{-0.18587401723009245`, -0.07099755695641169, 0, 
         0.7163774152812267}, {-0.2568715741865034, -0.18587401723009234`, \
-0.6015009550075457, 0.30075047750377304`}, {-0.07099755695641172, 
        0.4866244947338654, -0.37174803446018423`, 0.4156269377774535}, {0, 0,
         0, 0}}, {{0.07099755695641126, 0.11487646027368048`, 
        0.601500955007546, 0.41562693777745385`}, {0.2568715741865034, 
        0.18587401723009234`, 0.6015009550075457, -0.30075047750377304`}, {
        0.18587401723009178`, 0.6724985119639577, 0.2297529205473615, 
        0.11487646027368048`}, {0, 0, 0, 
        0}}, {{-0.07099755695641126, -0.11487646027368048`, \
-0.601500955007546, -0.41562693777745385`}, {
        0.18587401723009245`, 0.07099755695641169, 0, -0.7163774152812267}, {
        0.11487646027368065`, 
        0.557622051690277, -0.37174803446018445`, -0.3007504775037731}, {0, 0,
         0, 0}}, {{-0.11487646027368065`, -0.557622051690277, 
        0.37174803446018445`, 
        0.3007504775037731}, {-0.18587401723009178`, -0.6724985119639577, \
-0.2297529205473615, -0.11487646027368048`}, {
        0.07099755695641172, -0.4866244947338654, 
        0.37174803446018423`, -0.4156269377774535}, {0, 0, 0, 0}}}, {{{
        0.1858740172300922, 0.7163774152812267, 0.07099755695641184, 
         0}, {-0.07099755695641191, 
        0.41562693777745346`, -0.11487646027368063`, 0.601500955007546}, {
        0.11487646027368038`, 0.30075047750377293`, 0.557622051690277, 
        0.3717480344601847}, {0, 0, 0, 0}}, {{0.07099755695641163, 
        0.4156269377774536, -0.486624494733865, -0.3717480344601847}, \
{-0.1858740172300924, 0.11487646027368043`, -0.6724985119639573, 
        0.22975292054736143`}, {-0.11487646027368038`, -0.30075047750377293`, \
-0.557622051690277, -0.3717480344601847}, {0, 0, 0, 0}}, {{0.2568715741865041,
         0.30075047750377315`, 0.1858740172300925, -0.6015009550075462}, {
        0.1858740172300924, -0.11487646027368043`, 
        0.6724985119639573, -0.22975292054736143`}, {
        0.07099755695641191, -0.41562693777745346`, 
        0.11487646027368063`, -0.601500955007546}, {0, 0, 0, 
        0}}, {{-0.2568715741865041, -0.30075047750377315`, \
-0.1858740172300925, 
        0.6015009550075462}, {-0.07099755695641163, -0.4156269377774536, 
        0.486624494733865, 
        0.3717480344601847}, {-0.1858740172300922, -0.7163774152812267, \
-0.07099755695641184, 0}, {0, 0, 0, 0}}}, {{{0.27881102584513806`, 
        0.4730651679142942, 0.3362492559819788, -0.37174803446018456`}, {
        0.17231469041052075`, -0.22137279570829826`, 
        0.5795615033489115, -0.3717480344601847}, {0.15037523875188621`, 
        0.2078134688887267, 0.6589391851443864, 0.22975292054736096`}, {0, 0, 
        0, 0}}, {{0.12843578709325187`, 
        0.2652516990255674, -0.32268992916240746`, -0.6015009550075455}, {
        0.02193945165863445, -0.42918626459702514`, -0.07937768179547465, \
-0.6015009550075456}, {-0.15037523875188621`, -0.2078134688887267, \
-0.6589391851443864, -0.22975292054736096`}, {0, 0, 0, 0}}, {{
        0.10649633543461762`, 0.6944379636225926, -0.24331224736693266`, 
         0}, {-0.02193945165863445, 0.42918626459702514`, 0.07937768179547465,
         0.6015009550075456}, {-0.17231469041052075`, 
        0.22137279570829826`, -0.5795615033489115, 0.3717480344601847}, {0, 0,
         0, 0}}, {{-0.10649633543461762`, -0.6944379636225926, 
         0.24331224736693266`, 
         0}, {-0.12843578709325187`, -0.2652516990255674, 
        0.32268992916240746`, 
        0.6015009550075455}, {-0.27881102584513806`, -0.4730651679142942, \
-0.3362492559819788, 0.37174803446018456`}, {0, 0, 0, 0}}}, {{{
        0.28112723538466244`, 0.41562693777745396`, 
        0.37174803446018445`, -0.4034991071783743}, {0.1344997023927919, 
        0.11487646027368079`, -0.2297529205473615, -0.6846263425630362}, {
        0.14662753299187034`, -0.300750477503773, 
        0.37174803446018434`, -0.5501266401702449}, {0, 0, 0, 0}}, {{
        0.13449970239279196`, 0.7163774152812269, 0, 
         0.14662753299187054`}, {-0.01212783059907846, 
        0.41562693777745363`, -0.6015009550075457, -0.13449970239279144`}, \
{-0.14662753299187034`, 0.300750477503773, -0.37174803446018434`, 
        0.5501266401702449}, {0, 0, 0, 
        0}}, {{-0.14662753299187029`, -0.3007504775037731, \
-0.6015009550075461, -0.281127235384662}, {-0.13449970239279196`, \
-0.7163774152812269, 
         0, -0.14662753299187054`}, {-0.28112723538466244`, \
-0.41562693777745396`, -0.37174803446018445`, 0.4034991071783743}, {0, 0, 0, 
        0}}, {{0.14662753299187029`, 0.3007504775037731, 0.6015009550075461, 
        0.281127235384662}, {0.01212783059907846, -0.41562693777745363`, 
        0.6015009550075457, 
        0.13449970239279144`}, {-0.1344997023927919, -0.11487646027368079`, 
        0.2297529205473615, 0.6846263425630362}, {0, 0, 0, 0}}}, {{{
        0.2788110258451386, 0.4156269377774538, 
        0.31430980432334377`, -0.4511257162556594}, {0.12843578709325207`, 
        0.7163774152812267, -0.1503752387518867, 0.02193945165863423}, {
        0.10649633543461755`, 
        0.30075047750377315`, -0.39368748611881954`, -0.5440627248707058}, {0,
         0, 0, 0}}, {{0.15037523875188635`, -0.30075047750377293`, 
        0.46468504307523045`, -0.47306516791429365`}, \
{-0.021939451658634472`, -0.41562693777745363`, -0.2433122473669329, \
-0.5660021765293399}, {-0.12843578709325207`, -0.7163774152812267, 
        0.1503752387518867, -0.02193945165863423}, {0, 0, 0, 
        0}}, {{-0.15037523875188635`, 
        0.30075047750377293`, -0.46468504307523045`, 
        0.47306516791429365`}, {-0.1723146904105212, -0.1148764602736807, \
-0.7079972904421634, -0.09293700861504622}, {-0.2788110258451386, \
-0.4156269377774538, -0.31430980432334377`, 0.4511257162556594}, {0, 0, 0, 
        0}}, {{0.1723146904105212, 0.1148764602736807, 0.7079972904421634, 
        0.09293700861504622}, {0.021939451658634472`, 0.41562693777745363`, 
        0.2433122473669329, 
        0.5660021765293399}, {-0.10649633543461755`, -0.30075047750377315`, 
        0.39368748611881954`, 0.5440627248707058}, {0, 0, 0, 0}}}, {{{
        0.2568715741865035, 0.3007504775037731, 
        0.18587401723009225`, -0.6015009550075457}, {
        0.07099755695641191, -0.41562693777745346`, 
        0.11487646027368069`, -0.6015009550075455}, {
        0.18587401723009228`, -0.1148764602736805, 
        0.6724985119639577, -0.22975292054736124`}, {0, 0, 0, 0}}, {{
        0.18587401723009175`, 0.7163774152812266, 0.0709975569564116, 
         0}, {-0.07099755695641191, 
        0.41562693777745346`, -0.11487646027368069`, 0.6015009550075455}, {
        0.11487646027368073`, 0.30075047750377315`, 0.5576220516902768, 
        0.3717480344601844}, {0, 0, 0, 
        0}}, {{-0.18587401723009175`, -0.7163774152812266, \
-0.0709975569564116, 
         0}, {-0.2568715741865035, -0.3007504775037731, -0.18587401723009225`,
         0.6015009550075457}, {-0.07099755695641102, -0.4156269377774532, 
        0.4866244947338654, 0.37174803446018456`}, {0, 0, 0, 0}}, {{
        0.07099755695641102, 
        0.4156269377774532, -0.4866244947338654, -0.37174803446018456`}, \
{-0.11487646027368073`, -0.30075047750377315`, -0.5576220516902768, \
-0.3717480344601844}, {-0.18587401723009228`, 
        0.1148764602736805, -0.6724985119639577, 0.22975292054736124`}, {0, 0,
         0, 0}}}, {{{0.11487646027368038`, 
        0.5576220516902769, -0.37174803446018445`, -0.3007504775037727}, {
        0.18587401723009256`, 0.07099755695641174, 
         0, -0.716377415281226}, {-0.07099755695641191, \
-0.11487646027368069`, -0.6015009550075463, -0.41562693777745296`}, {0, 0, 0, 
        0}}, {{0.18587401723009253`, 0.6724985119639577, 0.22975292054736177`,
         0.11487646027368026`}, {0.25687157418650425`, 0.18587401723009248`, 
        0.6015009550075469, -0.30075047750377315`}, {0.07099755695641191, 
        0.11487646027368069`, 0.6015009550075463, 0.41562693777745296`}, {0, 
        0, 0, 0}}, {{0.07099755695641172, -0.48662449473386526`, 
        0.371748034460185, -0.4156269377774533}, {-0.18587401723009253`, \
-0.6724985119639577, -0.22975292054736177`, -0.11487646027368026`}, \
{-0.11487646027368038`, -0.5576220516902769, 0.37174803446018445`, 
        0.3007504775037727}, {0, 0, 0, 0}}, {{-0.07099755695641172, 
        0.48662449473386526`, -0.371748034460185, 
        0.4156269377774533}, {-0.25687157418650425`, -0.18587401723009248`, \
-0.6015009550075469, 
        0.30075047750377315`}, {-0.18587401723009256`, -0.07099755695641174, 
         0, 0.716377415281226}, {0, 0, 0, 0}}}, {{{0.07099755695641169, 
        0.601500955007546, -0.4156269377774533, -0.11487646027368037`}, {
        0.25687157418650386`, 0.6015009550075464, 
        0.30075047750377326`, -0.1858740172300925}, {0.18587401723009195`, 
        0.22975292054736132`, -0.1148764602736804, -0.6724985119639576}, {0, 
        0, 0, 0}}, {{-0.18587401723009211`, 0, -0.7163774152812266, 
         0.07099755695641186}, {-0.07099755695641186, -0.37174803446018495`, \
-0.4156269377774535, -0.4866244947338655}, {-0.25687157418650386`, \
-0.6015009550075464, -0.30075047750377326`, 0.1858740172300925}, {0, 0, 0, 
        0}}, {{0.18587401723009211`, 0, 
         0.7163774152812266, -0.07099755695641186}, {
        0.1148764602736804, -0.3717480344601847, 
        0.30075047750377293`, -0.557622051690277}, {-0.07099755695641169, \
-0.601500955007546, 0.4156269377774533, 0.11487646027368037`}, {0, 0, 0, 
        0}}, {{-0.1148764602736804, 0.3717480344601847, -0.30075047750377293`,
         0.557622051690277}, {0.07099755695641186, 0.37174803446018495`, 
        0.4156269377774535, 
        0.4866244947338655}, {-0.18587401723009195`, -0.22975292054736132`, 
        0.1148764602736804, 0.6724985119639576}, {0, 0, 0, 0}}}, {{{
        0.1503752387518861, 
        0.4730651679142937, -0.30075047750377254`, -0.4646850430752312}, \
{-0.021939451658634507`, 0.5660021765293402, -0.4156269377774533, 
        0.24331224736693252`}, {-0.12843578709325212`, 
        0.021939451658634007`, -0.7163774152812262, -0.15037523875188658`}, {
        0, 0, 0, 0}}, {{0.27881102584513817`, 0.4511257162556598, 
        0.4156269377774537, -0.3143098043233444}, {0.10649633543461766`, 
        0.5440627248707062, 0.30075047750377315`, 0.3936874861188191}, {
        0.12843578709325212`, -0.021939451658634007`, 0.7163774152812262, 
        0.15037523875188658`}, {0, 0, 0, 0}}, {{-0.17231469041052072`, 
        0.09293700861504631, -0.1148764602736808, 
        0.7079972904421635}, {-0.27881102584513817`, -0.4511257162556598, \
-0.4156269377774537, 
        0.3143098043233444}, {-0.1503752387518861, -0.4730651679142937, 
        0.30075047750377254`, 0.4646850430752312}, {0, 0, 0, 0}}, {{
        0.17231469041052072`, -0.09293700861504631, 
        0.1148764602736808, -0.7079972904421635}, {-0.10649633543461766`, \
-0.5440627248707062, -0.30075047750377315`, -0.3936874861188191}, {
        0.021939451658634507`, -0.5660021765293402, 
        0.4156269377774533, -0.24331224736693252`}, {0, 0, 0, 0}}}, {{{
        0.18587401723009234`, 
        0.22975292054736082`, -0.11487646027368012`, -0.6724985119639577}, {
        0.2568715741865034, 0.6015009550075461, 
        0.30075047750377304`, -0.1858740172300928}, {0.07099755695641125, 
        0.6015009550075461, -0.4156269377774532, -0.11487646027368084`}, {0, 
        0, 0, 0}}, {{-0.07099755695641119, -0.371748034460185, \
-0.41562693777745324`, -0.4866244947338648}, {-0.1858740172300923, 
         0, -0.7163774152812264, 
         0.07099755695641202}, {-0.2568715741865034, -0.6015009550075461, \
-0.30075047750377304`, 0.1858740172300928}, {0, 0, 0, 0}}, {{
        0.07099755695641119, 0.371748034460185, 0.41562693777745324`, 
        0.4866244947338648}, {-0.11487646027368076`, 
        0.37174803446018523`, -0.3007504775037728, 
        0.5576220516902771}, {-0.18587401723009234`, -0.22975292054736082`, 
        0.11487646027368012`, 0.6724985119639577}, {0, 0, 0, 0}}, {{
        0.11487646027368076`, -0.37174803446018523`, 
        0.3007504775037728, -0.5576220516902771}, {
        0.1858740172300923, 0, 
         0.7163774152812264, -0.07099755695641202}, {-0.07099755695641125, \
-0.6015009550075461, 0.4156269377774532, 0.11487646027368084`}, {0, 0, 0, 
        0}}}, {{{0.1466275329918702, 
        0.5501266401702452, -0.3007504775037726, -0.3717480344601847}, {
        0.2811272353846622, 0.4034991071783748, 
        0.41562693777745374`, -0.37174803446018434`}, {0.13449970239279174`, 
        0.6846263425630369, 0.11487646027368079`, 0.2297529205473615}, {0, 0, 
        0, 0}}, {{-0.134499702392792, 
         0.14662753299187037`, -0.7163774152812263, 0}, {-0.1466275329918706, 
        0.2811272353846622, -0.3007504775037728, 
        0.6015009550075459}, {-0.2811272353846622, -0.4034991071783748, \
-0.41562693777745374`, 0.37174803446018434`}, {0, 0, 0, 0}}, {{
        0.134499702392792, -0.14662753299187037`, 0.7163774152812263, 
         0}, {-0.012127830599078598`, 0.1344997023927917, 0.4156269377774536, 
        0.6015009550075461}, {-0.1466275329918702, -0.5501266401702452, 
        0.3007504775037726, 0.3717480344601847}, {0, 0, 0, 0}}, {{
        0.012127830599078598`, -0.1344997023927917, -0.4156269377774536, \
-0.6015009550075461}, {0.1466275329918706, -0.2811272353846622, 
        0.3007504775037728, -0.6015009550075459}, {-0.13449970239279174`, \
-0.6846263425630369, -0.11487646027368079`, -0.2297529205473615}, {0, 0, 0, 
        0}}}}, {{{{
        0, 0.4866244947338651, -0.5576220516902766, -0.07099755695641174}, {
        0, 0.557622051690277, -0.07099755695641133, 0.48662449473386493`}, {
        0.2297529205473616, 0.672498511963958, 
        0.11487646027368087`, -0.18587401723009264`}, {0, 0, 0, 0}}, {{
        0, -0.07099755695641166, -0.4866244947338651, -0.5576220516902766}, {
        0.22975292054736127`, 0.11487646027368068`, 
        0.18587401723009256`, -0.6724985119639573}, {
        0, -0.557622051690277, 0.07099755695641133, -0.48662449473386493`}, {
        0, 0, 0, 0}}, {{
        0, 0.07099755695641166, 0.4866244947338651, 0.5576220516902766}, {
        0.22975292054736118`, 0.18587401723009267`, 
        0.6724985119639574, -0.11487646027368092`}, {
        0, -0.4866244947338651, 0.5576220516902766, 0.07099755695641174}, {0, 
        0, 0, 0}}, {{-0.22975292054736118`, -0.18587401723009267`, \
-0.6724985119639574, 
        0.11487646027368092`}, {-0.22975292054736127`, -0.11487646027368068`, \
-0.18587401723009256`, 
        0.6724985119639573}, {-0.2297529205473616, -0.672498511963958, \
-0.11487646027368087`, 0.18587401723009264`}, {0, 0, 0, 0}}}, {{{
        0.07099755695641105, 
        0.6015009550075462, -0.4156269377774544, -0.11487646027368034`}, {
        0.25687157418650475`, 0.6015009550075463, 
        0.30075047750377215`, -0.18587401723009167`}, {0.18587401723009228`, 
        0.2297529205473614, -0.1148764602736807, -0.6724985119639573}, {0, 0, 
        0, 0}}, {{-0.11487646027368123`, 
        0.37174803446018473`, -0.3007504775037739, 0.557622051690277}, {
        0.07099755695641224, 0.3717480344601851, 0.41562693777745296`, 
        0.4866244947338651}, {-0.18587401723009228`, -0.2297529205473614, 
        0.1148764602736807, 0.6724985119639573}, {0, 0, 0, 
        0}}, {{-0.18587401723009367`, 0, -0.7163774152812267, 
         0.07099755695641158}, {-0.07099755695641224, -0.3717480344601851, \
-0.41562693777745296`, -0.4866244947338651}, {-0.25687157418650475`, \
-0.6015009550075463, -0.30075047750377215`, 0.18587401723009167`}, {0, 0, 0, 
        0}}, {{0.18587401723009367`, 0, 
         0.7163774152812267, -0.07099755695641158}, {
        0.11487646027368123`, -0.37174803446018473`, 
        0.3007504775037739, -0.557622051690277}, {-0.07099755695641105, \
-0.6015009550075462, 0.4156269377774544, 0.11487646027368034`}, {0, 0, 0, 
        0}}}, {{{0.03175107271818977, 
        0.557622051690278, -0.4866244947338655, -0.06350214543638044}, {
        0.15412294451190178`, 0.672498511963958, 0.18587401723009284`, 
        0.2054972593492027}, {0.24937616266647145`, 0.48662449473386593`, 
        0.07099755695641156, -0.49875232533294456`}, {0, 0, 0, 
        0}}, {{-0.21762508994828195`, 
        0.07099755695641191, -0.5576220516902771, 
        0.43525017989656417`}, {-0.09525321815457008, 0.1858740172300921, 
        0.11487646027368117`, 
        0.7042495846821473}, {-0.24937616266647145`, -0.48662449473386593`, \
-0.07099755695641156, 0.49875232533294456`}, {0, 0, 0, 
        0}}, {{-0.12237187179371226`, -0.11487646027368047`, \
-0.6724985119639584, -0.26899940478558326`}, {
        0.09525321815457008, -0.1858740172300921, -0.11487646027368117`, \
-0.7042495846821473}, {-0.15412294451190178`, -0.672498511963958, \
-0.18587401723009284`, -0.2054972593492027}, {0, 0, 0, 0}}, {{
        0.12237187179371226`, 0.11487646027368047`, 0.6724985119639584, 
        0.26899940478558326`}, {0.21762508994828195`, -0.07099755695641191, 
        0.5576220516902771, -0.43525017989656417`}, {-0.03175107271818977, \
-0.557622051690278, 0.4866244947338655, 0.06350214543638044}, {0, 0, 0, 
        0}}}, {{{-0.07099755695641263, 
        0.41562693777745435`, -0.1148764602736802, 0.6015009550075462}, {
        0.18587401723009164`, 0.716377415281227, 0.070997556956412, 0}, {
        0.1148764602736797, 0.3007504775037737, 0.5576220516902771, 
        0.3717480344601849}, {0, 0, 0, 0}}, {{-0.18587401723009223`, 
        0.11487646027368077`, -0.6724985119639573, 0.22975292054736143`}, {
        0.07099755695641193, 
        0.41562693777745324`, -0.48662449473386493`, -0.37174803446018523`}, \
{-0.1148764602736797, -0.3007504775037737, -0.5576220516902771, \
-0.3717480344601849}, {0, 0, 0, 
        0}}, {{-0.2568715741865042, -0.30075047750377293`, \
-0.18587401723009242`, 
        0.6015009550075469}, {-0.07099755695641193, -0.41562693777745324`, 
        0.48662449473386493`, 
        0.37174803446018523`}, {-0.18587401723009164`, -0.716377415281227, \
-0.070997556956412, 0}, {0, 0, 0, 0}}, {{0.2568715741865042, 
        0.30075047750377293`, 0.18587401723009242`, -0.6015009550075469}, {
        0.18587401723009223`, -0.11487646027368077`, 
        0.6724985119639573, -0.22975292054736143`}, {
        0.07099755695641263, -0.41562693777745435`, 
        0.1148764602736802, -0.6015009550075462}, {0, 0, 0, 0}}}, {{{
        0.13681591193231468`, 
        0.6724985119639584, -0.2433122473669324, -0.15037523875188646`}, \
{-0.1284357870932519, 0.18587401723009234`, -0.707997290442163, 
        0.02193945165863444}, {0.07937768179547439, 
        0.11487646027368048`, -0.39368748611881876`, -0.6150602818271169}, {0,
         0, 0, 0}}, {{0.057438230136840684`, 0.557622051690278, 
        0.15037523875188666`, 0.46468504307523073`}, {-0.2078134688887261, 
        0.07099755695641183, -0.31430980432334427`, 
        0.6369997334857517}, {-0.07937768179547439, -0.11487646027368048`, 
        0.39368748611881876`, 0.6150602818271169}, {0, 0, 0, 0}}, {{
        0.26525169902556667`, 0.4866244947338661, 
        0.4646850430752306, -0.172314690410521}, {
        0.2078134688887261, -0.07099755695641183, 
        0.31430980432334427`, -0.6369997334857517}, {
        0.1284357870932519, -0.18587401723009234`, 
        0.707997290442163, -0.02193945165863444}, {0, 0, 0, 
        0}}, {{-0.26525169902556667`, -0.4866244947338661, \
-0.4646850430752306, 
        0.172314690410521}, {-0.057438230136840684`, -0.557622051690278, \
-0.15037523875188666`, -0.46468504307523073`}, {-0.13681591193231468`, \
-0.6724985119639584, 0.2433122473669324, 0.15037523875188646`}, {0, 0, 0, 
        0}}}, {{{-0.07099755695641172, 
        0.3362492559819793, -0.6589391851443868, -0.02193945165863373}, \
{-0.18587401723009206`, 0.16393456557145836`, -0.33624925598197963`, 
        0.615060281827118}, {-0.2568715741865043, -0.3581887076406137, \
-0.5795615033489122, 0.15037523875188671`}, {0, 0, 0, 0}}, {{
        0.18587401723009248`, 
        0.6944379636225926, -0.07937768179547489, -0.1723146904105206}, {
        0.07099755695641202, 0.522123273212072, 0.24331224736693244`, 
        0.46468504307523095`}, {0.2568715741865043, 0.3581887076406137, 
        0.5795615033489122, -0.15037523875188671`}, {0, 0, 0, 
        0}}, {{-0.11487646027368059`, -0.17231469041052078`, 
        0.3226899291624073, 
        0.6369997334857513}, {-0.18587401723009248`, -0.6944379636225926, 
        0.07937768179547489, 0.1723146904105206}, {
        0.07099755695641172, -0.3362492559819793, 0.6589391851443868, 
        0.02193945165863373}, {0, 0, 0, 0}}, {{0.11487646027368059`, 
        0.17231469041052078`, -0.3226899291624073, -0.6369997334857513}, \
{-0.07099755695641202, -0.522123273212072, -0.24331224736693244`, \
-0.46468504307523095`}, {0.18587401723009206`, -0.16393456557145836`, 
        0.33624925598197963`, -0.615060281827118}, {0, 0, 0, 0}}}, {{{
        0.0709975569564111, -0.4156269377774537, 
        0.11487646027368148`, -0.6015009550075457}, {0.25687157418650397`, 
        0.3007504775037725, 0.1858740172300929, -0.6015009550075466}, {
        0.18587401723009195`, -0.11487646027368043`, 
        0.6724985119639582, -0.22975292054736163`}, {0, 0, 0, 
        0}}, {{-0.11487646027368059`, -0.30075047750377315`, \
-0.5576220516902766, -0.37174803446018445`}, {0.070997556956412, 
        0.41562693777745296`, -0.48662449473386504`, -0.371748034460185}, \
{-0.18587401723009195`, 0.11487646027368043`, -0.6724985119639582, 
        0.22975292054736163`}, {0, 0, 0, 
        0}}, {{-0.1858740172300929, -0.7163774152812263, -0.07099755695641163,
          0}, {-0.070997556956412, -0.41562693777745296`, 
        0.48662449473386504`, 
        0.371748034460185}, {-0.25687157418650397`, -0.3007504775037725, \
-0.1858740172300929, 0.6015009550075466}, {0, 0, 0, 0}}, {{
        0.1858740172300929, 0.7163774152812263, 0.07099755695641163, 0}, {
        0.11487646027368059`, 0.30075047750377315`, 0.5576220516902766, 
        0.37174803446018445`}, {-0.0709975569564111, 
        0.4156269377774537, -0.11487646027368148`, 0.6015009550075457}, {0, 0,
         0, 0}}}, {{{0.15412294451190178`, 
        0.6407474392457679, -0.23724833206739238`, -0.2493761626664725}, \
{-0.08312538755549057, 
        0.10274862967460166`, -0.6921217540830682, -0.23724833206739313`}, \
{-0.09525321815456972, 0.4352501798965643, -0.4548734220156748, 
        0.3838758650592643}, {0, 0, 0, 0}}, {{0.2493761626664716, 
        0.20549725934920382`, 0.21762508994828272`, -0.6332520277257367}, {
        0.01212783059907925, -0.33250155022196304`, -0.2372483320673934, \
-0.6211241971266576}, {0.09525321815456972, -0.4352501798965643, 
        0.4548734220156748, -0.3838758650592643}, {0, 0, 0, 0}}, {{
        0.2372483320673922, 0.5379988095711666, 
        0.45487342201567577`, -0.0121278305990793}, {-0.01212783059907925, 
        0.33250155022196304`, 0.2372483320673934, 0.6211241971266576}, {
        0.08312538755549057, -0.10274862967460166`, 0.6921217540830682, 
        0.23724833206739313`}, {0, 0, 0, 
        0}}, {{-0.2372483320673922, -0.5379988095711666, \
-0.45487342201567577`, 
        0.0121278305990793}, {-0.2493761626664716, -0.20549725934920382`, \
-0.21762508994828272`, 
        0.6332520277257367}, {-0.15412294451190178`, -0.6407474392457679, 
        0.23724833206739238`, 0.2493761626664725}, {0, 0, 0, 
        0}}}, {{{-0.06350214543637983, 
        0.2886226469046934, -0.6724985119639579, -0.11487646027368117`}, \
{-0.10274862967460227`, -0.36425262294015354`, -0.4866244947338658, \
-0.4156269377774542}, {-0.2689994047855835, -0.2811272353846622, \
-0.5576220516902763, 0.30075047750377315`}, {0, 0, 0, 0}}, {{
        0.2054972593492035, 
        0.5697498822893557, -0.11487646027368173`, -0.41562693777745424`}, {
        0.16625077511098102`, -0.08312538755549123, 
        0.07099755695641052, -0.7163774152812272}, {0.2689994047855835, 
        0.2811272353846622, 0.5576220516902763, -0.30075047750377315`}, {0, 0,
         0, 0}}, {{-0.039246484238222745`, -0.6528752698448469, 
        0.18587401723009217`, -0.30075047750377293`}, {-0.2054972593492035, \
-0.5697498822893557, 0.11487646027368173`, 0.41562693777745424`}, {
        0.06350214543637983, -0.2886226469046934, 0.6724985119639579, 
        0.11487646027368117`}, {0, 0, 0, 0}}, {{0.039246484238222745`, 
        0.6528752698448469, -0.18587401723009217`, 
        0.30075047750377293`}, {-0.16625077511098102`, 
        0.08312538755549123, -0.07099755695641052, 0.7163774152812272}, {
        0.10274862967460227`, 0.36425262294015354`, 0.4866244947338658, 
        0.4156269377774542}, {0, 0, 0, 0}}}, {{{0.07099755695641108, 
        0.3717480344601848, 0.4156269377774543, 
        0.4866244947338646}, {-0.18587401723009228`, -0.22975292054736146`, 
        0.11487646027368105`, 0.6724985119639573}, {-0.11487646027368004`, 
        0.3717480344601847, -0.30075047750377276`, 0.5576220516902768}, {0, 0,
         0, 0}}, {{
        0.18587401723009153`, 0, 
         0.7163774152812272, -0.07099755695641181}, {-0.07099755695641183, \
-0.6015009550075464, 0.4156269377774541, 0.11487646027368062`}, {
        0.11487646027368004`, -0.3717480344601847, 
        0.30075047750377276`, -0.5576220516902768}, {0, 0, 0, 0}}, {{
        0.2568715741865035, 0.6015009550075462, 
        0.3007504775037733, -0.1858740172300925}, {0.07099755695641183, 
        0.6015009550075464, -0.4156269377774541, -0.11487646027368062`}, {
        0.18587401723009228`, 
        0.22975292054736146`, -0.11487646027368105`, -0.6724985119639573}, {0,
         0, 0, 0}}, {{-0.2568715741865035, -0.6015009550075462, \
-0.3007504775037733, 
        0.1858740172300925}, {-0.18587401723009153`, 0, -0.7163774152812272, 
         0.07099755695641181}, {-0.07099755695641108, -0.3717480344601848, \
-0.4156269377774543, -0.4866244947338646}, {0, 0, 0, 
        0}}}, {{{-0.2297529205473613, -0.11487646027368142`, \
-0.18587401723009175`, 
        0.6724985119639576}, {-0.22975292054736182`, -0.6724985119639579, \
-0.11487646027368048`, 
        0.18587401723009228`}, {-0.22975292054736118`, -0.1858740172300923, \
-0.6724985119639574, 0.11487646027368047`}, {0, 0, 0, 0}}, {{
        0, 0.07099755695641077, 0.4866244947338656, 0.5576220516902775}, {
        0, -0.4866244947338658, 0.557622051690277, 0.07099755695641177}, {
        0.22975292054736118`, 0.1858740172300923, 
        0.6724985119639574, -0.11487646027368047`}, {0, 0, 0, 0}}, {{
        0, -0.5576220516902765, 0.07099755695641172, -0.4866244947338657}, {
        0, -0.07099755695641077, -0.4866244947338656, -0.5576220516902775}, {
        0.2297529205473613, 0.11487646027368142`, 
        0.18587401723009175`, -0.6724985119639576}, {0, 0, 0, 0}}, {{
        0, 0.5576220516902765, -0.07099755695641172, 0.4866244947338657}, {
        0, 0.4866244947338658, -0.557622051690277, -0.07099755695641177}, {
        0.22975292054736182`, 0.6724985119639579, 
        0.11487646027368048`, -0.18587401723009228`}, {0, 0, 0, 0}}}, {{{
        0.12843578709325124`, 0.7079972904421639, 0.02193945165863498, 
        0.18587401723009195`}, {0.26525169902556694`, 0.46468504307523173`, 
        0.17231469041052141`, -0.48662449473386526`}, {0.2078134688887268, 
        0.3143098043233454, 0.6369997334857525, 0.07099755695641119}, {0, 0, 
        0, 0}}, {{-0.13681591193231557`, 
        0.2433122473669326, -0.15037523875188685`, 
        0.6724985119639573}, {-0.0574382301368401, -0.15037523875188616`, 
        0.4646850430752307, 
        0.5576220516902766}, {-0.26525169902556694`, -0.46468504307523173`, \
-0.17231469041052141`, 0.48662449473386526`}, {0, 0, 0, 0}}, {{
        0.13681591193231557`, -0.2433122473669326, 
        0.15037523875188685`, -0.6724985119639573}, {
        0.07937768179547577, -0.3936874861188185, 
        0.6150602818271174, -0.11487646027368086`}, {-0.12843578709325124`, \
-0.7079972904421639, -0.02193945165863498, -0.18587401723009195`}, {0, 0, 0, 
        0}}, {{-0.07937768179547577, 0.3936874861188185, -0.6150602818271174, 
        0.11487646027368086`}, {0.0574382301368401, 
        0.15037523875188616`, -0.4646850430752307, -0.5576220516902766}, \
{-0.2078134688887268, -0.3143098043233454, -0.6369997334857525, \
-0.07099755695641119}, {0, 0, 0, 
        0}}}}, {{{{-0.2568715741865041, -0.3007504775037746, \
-0.18587401723009284`, 
        0.6015009550075452}, {-0.07099755695641219, -0.41562693777745435`, 
        0.48662449473386493`, 
        0.37174803446018384`}, {-0.18587401723009295`, -0.7163774152812282, \
-0.07099755695641208, 0}, {0, 0, 0, 0}}, {{-0.18587401723009198`, 
        0.11487646027367975`, -0.6724985119639572, 0.2297529205473613}, {
        0.07099755695641219, 
        0.41562693777745435`, -0.48662449473386493`, -0.37174803446018384`}, \
{-0.11487646027368092`, -0.30075047750377365`, -0.5576220516902767, \
-0.37174803446018456`}, {0, 0, 0, 0}}, {{
        0.18587401723009198`, -0.11487646027367975`, 
        0.6724985119639572, -0.2297529205473613}, {0.2568715741865041, 
        0.3007504775037746, 0.18587401723009284`, -0.6015009550075452}, {
        0.07099755695641097, -0.41562693777745335`, 
        0.1148764602736805, -0.6015009550075459}, {0, 0, 0, 
        0}}, {{-0.07099755695641097, 
        0.41562693777745335`, -0.1148764602736805, 0.6015009550075459}, {
        0.11487646027368092`, 0.30075047750377365`, 0.5576220516902767, 
        0.37174803446018456`}, {
        0.18587401723009295`, 0.7163774152812282, 0.07099755695641208, 0}, {0,
         0, 0, 0}}}, {{{-0.27881102584513817`, -0.3362492559819802, \
-0.371748034460185, 
        0.4730651679142943}, {-0.10649633543461651`, 0.2433122473669313, 0, 
         0.6944379636225928}, {-0.12843578709325149`, 
        0.3226899291624069, -0.6015009550075454, 0.26525169902556744`}, {0, 0,
         0, 0}}, {{-0.1503752387518869, -0.6589391851443873, 
        0.22975292054736052`, 0.2078134688887267}, {
        0.021939451658634784`, -0.07937768179547589, 0.6015009550075453, 
        0.42918626459702525`}, {0.12843578709325149`, -0.3226899291624069, 
        0.6015009550075454, -0.26525169902556744`}, {0, 0, 0, 
        0}}, {{-0.17231469041052205`, -0.5795615033489117, \
-0.37174803446018456`, -0.22137279570829874`}, {-0.021939451658634784`, 
        0.07937768179547589, -0.6015009550075453, -0.42918626459702525`}, {
        0.10649633543461651`, -0.2433122473669313, 0, -0.6944379636225928}, {
        0, 0, 0, 0}}, {{0.17231469041052205`, 0.5795615033489117, 
        0.37174803446018456`, 0.22137279570829874`}, {0.1503752387518869, 
        0.6589391851443873, -0.22975292054736052`, -0.2078134688887267}, {
        0.27881102584513817`, 0.3362492559819802, 
        0.371748034460185, -0.4730651679142943}, {0, 0, 0, 
        0}}}, {{{-0.2297529205473616, -0.6724985119639584, \
-0.11487646027368088`, 
        0.18587401723009273`}, {-0.22975292054736257`, -0.11487646027368024`, \
-0.1858740172300932, 
        0.6724985119639582}, {-0.2297529205473624, -0.18587401723009278`, \
-0.6724985119639577, 0.11487646027368065`}, {0, 0, 0, 0}}, {{
        0, -0.48662449473386565`, 0.5576220516902767, 0.07099755695641133}, {
        0, 0.07099755695641209, 0.4866244947338646, 0.5576220516902777}, {
        0.2297529205473624, 0.18587401723009278`, 
        0.6724985119639577, -0.11487646027368065`}, {0, 0, 0, 0}}, {{
        0, 0.5576220516902781, -0.07099755695641205, 0.48662449473386615`}, {
        0, 0.48662449473386565`, -0.5576220516902767, -0.07099755695641133}, {
        0.2297529205473616, 0.6724985119639584, 
        0.11487646027368088`, -0.18587401723009273`}, {0, 0, 0, 0}}, {{
        0, -0.5576220516902781, 0.07099755695641205, -0.48662449473386615`}, {
        0, -0.07099755695641209, -0.4866244947338646, -0.5576220516902777}, {
        0.22975292054736257`, 0.11487646027368024`, 
        0.1858740172300932, -0.6724985119639582}, {0, 0, 0, 
        0}}}, {{{-0.2689994047855844, -0.5576220516902773, \
-0.30075047750377365`, 
        0.28112723538466133`}, {-0.10274862967460238`, -0.48662449473386643`, 
        0.4156269377774542, 
        0.3642526229401525}, {-0.06350214543638152, -0.672498511963958, 
        0.11487646027368004`, -0.2886226469046943}, {0, 0, 0, 
        0}}, {{-0.20549725934920274`, 
        0.11487646027368112`, -0.415626937777454, 
        0.5697498822893556}, {-0.03924648423822108, 0.1858740172300912, 
        0.3007504775037739, 0.6528752698448468}, {0.06350214543638152, 
        0.672498511963958, -0.11487646027368004`, 0.2886226469046943}, {0, 0, 
        0, 0}}, {{0.16625077511098169`, 0.07099755695641047, 
        0.7163774152812281, 0.08312538755549098}, {
        0.20549725934920274`, -0.11487646027368112`, 
        0.415626937777454, -0.5697498822893556}, {0.2689994047855844, 
        0.5576220516902773, 0.30075047750377365`, -0.28112723538466133`}, {0, 
        0, 0, 0}}, {{-0.16625077511098169`, -0.07099755695641047, \
-0.7163774152812281, -0.08312538755549098}, {
        0.03924648423822108, -0.1858740172300912, -0.3007504775037739, \
-0.6528752698448468}, {0.10274862967460238`, 
        0.48662449473386643`, -0.4156269377774542, -0.3642526229401525}, {0, 
        0, 0, 0}}}, {{{-0.18587401723009223`, -0.07099755695641136, 0, 
         0.7163774152812272}, {-0.2568715741865028, -0.18587401723009295`, \
-0.6015009550075456, 0.3007504775037736}, {-0.07099755695641176, 
        0.48662449473386526`, -0.3717480344601843, 0.4156269377774527}, {0, 0,
         0, 0}}, {{-0.11487646027368052`, -0.557622051690277, 
        0.371748034460186, 
        0.30075047750377387`}, {-0.18587401723009112`, -0.6724985119639582, \
-0.22975292054736124`, -0.11487646027367907`}, {
        0.07099755695641176, -0.48662449473386526`, 
        0.3717480344601843, -0.4156269377774527}, {0, 0, 0, 0}}, {{
        0.07099755695641086, 0.11487646027368142`, 0.6015009550075469, 
        0.41562693777745363`}, {0.18587401723009112`, 0.6724985119639582, 
        0.22975292054736124`, 0.11487646027367907`}, {0.2568715741865028, 
        0.18587401723009295`, 0.6015009550075456, -0.3007504775037736}, {0, 0,
         0, 0}}, {{-0.07099755695641086, -0.11487646027368142`, \
-0.6015009550075469, -0.41562693777745363`}, {0.11487646027368052`, 
        0.557622051690277, -0.371748034460186, -0.30075047750377387`}, {
        0.18587401723009223`, 0.07099755695641136, 0, -0.7163774152812272}, {
        0, 0, 0, 
        0}}}, {{{-0.27881102584513845`, -0.3143098043233452, \
-0.45112571625565945`, 
        0.4156269377774542}, {-0.17231469041052117`, -0.7079972904421643, 
        0.09293700861504628, 
        0.11487646027368081`}, {-0.1503752387518866, -0.46468504307523156`, \
-0.47306516791429487`, -0.3007504775037738}, {0, 0, 0, 
        0}}, {{-0.12843578709325182`, 0.15037523875188658`, 
        0.021939451658635145`, 
        0.7163774152812276}, {-0.021939451658634528`, -0.24331224736693274`, 
        0.5660021765293404, 0.41562693777745474`}, {0.1503752387518866, 
        0.46468504307523156`, 0.47306516791429487`, 0.3007504775037738}, {0, 
        0, 0, 0}}, {{-0.1064963354346173, 
        0.39368748611881893`, -0.5440627248707051, 0.30075047750377293`}, {
        0.021939451658634528`, 
        0.24331224736693274`, -0.5660021765293404, -0.41562693777745474`}, {
        0.17231469041052117`, 
        0.7079972904421643, -0.09293700861504628, -0.11487646027368081`}, {0, 
        0, 0, 0}}, {{0.1064963354346173, -0.39368748611881893`, 
        0.5440627248707051, -0.30075047750377293`}, {
        0.12843578709325182`, -0.15037523875188658`, -0.021939451658635145`, \
-0.7163774152812276}, {0.27881102584513845`, 0.3143098043233452, 
        0.45112571625565945`, -0.4156269377774542}, {0, 0, 0, 
        0}}}, {{{-0.2568715741865045, -0.5795615033489117, \
-0.15037523875188583`, 
        0.35818870764061317`}, {-0.18587401723009211`, -0.3362492559819803, \
-0.615060281827118, -0.1639345655714578}, {-0.07099755695641252, \
-0.6589391851443873, 0.021939451658633646`, -0.33624925598197897`}, {0, 0, 0, 
        0}}, {{-0.1858740172300922, 
        0.07937768179547594, -0.17231469041051975`, 
        0.694437963622592}, {-0.11487646027367993`, 
        0.32268992916240674`, -0.6369997334857516, 0.172314690410521}, {
        0.07099755695641252, 0.6589391851443873, -0.021939451658633646`, 
        0.33624925598197897`}, {0, 0, 0, 0}}, {{0.07099755695641233, 
        0.243312247366931, -0.46468504307523173`, -0.5221232732120709}, {
        0.1858740172300922, -0.07937768179547594, 
        0.17231469041051975`, -0.694437963622592}, {0.2568715741865045, 
        0.5795615033489117, 0.15037523875188583`, -0.35818870764061317`}, {0, 
        0, 0, 0}}, {{-0.07099755695641233, -0.243312247366931, 
        0.46468504307523173`, 0.5221232732120709}, {
        0.11487646027367993`, -0.32268992916240674`, 
        0.6369997334857516, -0.172314690410521}, {0.18587401723009211`, 
        0.3362492559819803, 0.615060281827118, 0.1639345655714578}, {0, 0, 0, 
        0}}}, {{{-0.25687157418650447`, -0.1858740172300925, \
-0.6015009550075461, 
        0.30075047750377254`}, {-0.18587401723009173`, -0.07099755695641193, 
         0, 0.7163774152812266}, {-0.07099755695641205, 
        0.4866244947338661, -0.3717480344601846, 0.4156269377774531}, {0, 0, 
        0, 0}}, {{-0.18587401723009253`, -0.6724985119639585, \
-0.22975292054736168`, -0.11487646027368084`}, {-0.11487646027368006`, \
-0.5576220516902781, 0.37174803446018556`, 0.3007504775037735}, {
        0.07099755695641205, -0.4866244947338661, 
        0.3717480344601846, -0.4156269377774531}, {0, 0, 0, 
        0}}, {{-0.07099755695641302, -0.11487646027368044`, \
-0.6015009550075467, -0.4156269377774541}, {0.11487646027368006`, 
        0.5576220516902781, -0.37174803446018556`, -0.3007504775037735}, {
        0.18587401723009173`, 0.07099755695641193, 0, -0.7163774152812266}, {
        0, 0, 0, 0}}, {{0.07099755695641302, 0.11487646027368044`, 
        0.6015009550075467, 0.4156269377774541}, {0.18587401723009253`, 
        0.6724985119639585, 0.22975292054736168`, 0.11487646027368084`}, {
        0.25687157418650447`, 0.1858740172300925, 
        0.6015009550075461, -0.30075047750377254`}, {0, 0, 0, 
        0}}}, {{{-0.25687157418650364`, -0.30075047750377343`, \
-0.18587401723009292`, 
        0.6015009550075459}, {-0.18587401723009253`, -0.7163774152812276, \
-0.07099755695641179, 0}, {-0.07099755695641227, -0.4156269377774536, 
        0.486624494733866, 0.37174803446018373`}, {0, 0, 0, 
        0}}, {{-0.18587401723009161`, 
        0.11487646027368068`, -0.6724985119639589, 
        0.22975292054736204`}, {-0.11487646027368104`, -0.3007504775037736, \
-0.5576220516902777, -0.371748034460184}, {0.07099755695641227, 
        0.4156269377774536, -0.486624494733866, -0.37174803446018373`}, {0, 0,
         0, 0}}, {{-0.07099755695641097, 
        0.4156269377774541, -0.11487646027368142`, 0.6015009550075459}, {
        0.11487646027368104`, 0.3007504775037736, 0.5576220516902777, 
        0.371748034460184}, {
        0.18587401723009253`, 0.7163774152812276, 0.07099755695641179, 0}, {0,
         0, 0, 0}}, {{0.07099755695641097, -0.4156269377774541, 
        0.11487646027368142`, -0.6015009550075459}, {
        0.18587401723009161`, -0.11487646027368068`, 
        0.6724985119639589, -0.22975292054736204`}, {0.25687157418650364`, 
        0.30075047750377343`, 0.18587401723009292`, -0.6015009550075459}, {0, 
        0, 0, 0}}}, {{{-0.12237187179371406`, -0.6724985119639577, 
        0.2689994047855827, 
        0.11487646027368055`}, {-0.15412294451190267`, -0.1858740172300915, 
        0.20549725934920265`, 0.672498511963958}, {
        0.09525321815456965, -0.11487646027368131`, 0.7042495846821476, 
        0.18587401723009117`}, {0, 0, 0, 
        0}}, {{-0.2176250899482841, -0.5576220516902769, \
-0.43525017989656495`, -0.07099755695641098}, {-0.24937616266647275`, \
-0.07099755695641033, -0.49875232533294495`, 
        0.48662449473386704`}, {-0.09525321815456965, 
        0.11487646027368131`, -0.7042495846821476, -0.18587401723009117`}, {0,
         0, 0, 0}}, {{0.03175107271818897, -0.4866244947338663, 
        0.06350214543637972, -0.5576220516902777}, {0.24937616266647275`, 
        0.07099755695641033, 0.49875232533294495`, -0.48662449473386704`}, {
        0.15412294451190267`, 
        0.1858740172300915, -0.20549725934920265`, -0.672498511963958}, {0, 0,
         0, 0}}, {{-0.03175107271818897, 
        0.4866244947338663, -0.06350214543637972, 0.5576220516902777}, {
        0.2176250899482841, 0.5576220516902769, 0.43525017989656495`, 
        0.07099755695641098}, {0.12237187179371406`, 
        0.6724985119639577, -0.2689994047855827, -0.11487646027368055`}, {0, 
        0, 0, 0}}}, {{{-0.19491356844314014`, -0.5102903470526696, 
        0.16220816491128837`, 
        0.4775849435208174}, {-0.2624583240792139, -0.4573718925223528, \
-0.5012507958396216, 0.15316861369824072`}, {-0.20953986954889692`, 
        0.05291845453031763, -0.2917109262907251, 0.6488326596451532}, {0, 0, 
        0, 0}}, {{0.014626301105756932`, -0.5632088015829873, 
        0.45391909120201307`, -0.17124771612433604`}, \
{-0.052918454530317155`, -0.5102903470526698, -0.20953986954889675`, \
-0.4956640459469125}, {0.20953986954889692`, -0.05291845453031763, 
        0.2917109262907251, -0.6488326596451532}, {0, 0, 0, 0}}, {{
        0.06754475563607366, -0.052918454530317544`, 0.6634589607509099, 
        0.3244163298225764}, {0.052918454530317155`, 0.5102903470526698, 
        0.20953986954889675`, 0.4956640459469125}, {0.2624583240792139, 
        0.4573718925223528, 0.5012507958396216, -0.15316861369824072`}, {0, 0,
         0, 0}}, {{-0.06754475563607366, 
        0.052918454530317544`, -0.6634589607509099, -0.3244163298225764}, \
{-0.014626301105756932`, 0.5632088015829873, -0.45391909120201307`, 
        0.17124771612433604`}, {0.19491356844314014`, 
        0.5102903470526696, -0.16220816491128837`, -0.4775849435208174}, {0, 
        0, 0, 0}}}, {{{0.2297529205473601, 0.11487646027368137`, 
        0.18587401723009223`, -0.6724985119639584}, {
        0, -0.07099755695641166, -0.4866244947338653, -0.5576220516902777}, {
        0, -0.557622051690277, 0.07099755695641168, -0.4866244947338666}, {0, 
        0, 0, 0}}, {{0.22975292054735977`, 0.6724985119639584, 
        0.11487646027368069`, -0.1858740172300919}, {
        0, 0.48662449473386515`, -0.5576220516902763, -0.07099755695641088}, {
        0, 0.557622051690277, -0.07099755695641168, 0.4866244947338666}, {0, 
        0, 0, 0}}, {{0.2297529205473602, 0.18587401723009295`, 
        0.6724985119639575, -0.11487646027368122`}, {
        0, -0.48662449473386515`, 0.5576220516902763, 0.07099755695641088}, {
        0, 0.07099755695641166, 0.4866244947338653, 0.5576220516902777}, {0, 
        0, 0, 0}}, {{-0.2297529205473602, -0.18587401723009295`, \
-0.6724985119639575, 
        0.11487646027368122`}, {-0.22975292054735977`, -0.6724985119639584, \
-0.11487646027368069`, 
        0.1858740172300919}, {-0.2297529205473601, -0.11487646027368137`, \
-0.18587401723009223`, 0.6724985119639584}, {0, 0, 0, 
        0}}}}, {{{{-0.11487646027368047`, -0.5576220516902781, 
        0.3717480344601834, 0.30075047750377265`}, {
        0.07099755695641126, -0.4866244947338662, 
        0.37174803446018334`, -0.4156269377774534}, {-0.18587401723009267`, \
-0.6724985119639589, -0.2297529205473625, -0.11487646027368092`}, {0, 0, 0, 
        0}}, {{-0.18587401723009211`, -0.07099755695641155, 0, 
         0.716377415281226}, {-0.07099755695641126, 
        0.4866244947338662, -0.37174803446018334`, 
        0.4156269377774534}, {-0.25687157418650375`, -0.18587401723009261`, \
-0.601500955007546, 0.300750477503773}, {0, 0, 0, 0}}, {{
        0.18587401723009211`, 0.07099755695641155, 0, -0.716377415281226}, {
        0.11487646027368047`, 
        0.5576220516902781, -0.3717480344601834, -0.30075047750377265`}, \
{-0.07099755695641187, -0.1148764602736807, -0.6015009550075462, \
-0.41562693777745296`}, {0, 0, 0, 0}}, {{0.07099755695641187, 
        0.1148764602736807, 0.6015009550075462, 0.41562693777745296`}, {
        0.25687157418650375`, 0.18587401723009261`, 
        0.601500955007546, -0.300750477503773}, {0.18587401723009267`, 
        0.6724985119639589, 0.2297529205473625, 0.11487646027368092`}, {0, 0, 
        0, 0}}}, {{{-0.24937616266647236`, -0.6015009550075471, \
-0.1148764602736812, 
        0.3399969617419946}, {-0.0317510727181884, -0.37174803446018584`, 
        0.5576220516902767, 
        0.3203737196228841}, {-0.15412294451190234`, 0, 0.0709975569564116, 
         0.7238728268012578}, {0, 0, 0, 
        0}}, {{-0.09525321815457004, -0.6015009550075473, \
-0.18587401723009306`, -0.38387586505926313`}, {
        0.12237187179371357`, -0.37174803446018595`, 
        0.4866244947338646, -0.4034991071783737}, {
        0.15412294451190234`, 0, -0.0709975569564116, -0.7238728268012578}, {
        0, 0, 0, 0}}, {{0.21762508994828417`, 0.22975292054736107`, 
        0.6724985119639579, -0.019623242119110346`}, {0.09525321815457004, 
        0.6015009550075473, 0.18587401723009306`, 0.38387586505926313`}, {
        0.24937616266647236`, 0.6015009550075471, 
        0.1148764602736812, -0.3399969617419946}, {0, 0, 0, 
        0}}, {{-0.21762508994828417`, -0.22975292054736107`, \
-0.6724985119639579, 0.019623242119110346`}, {-0.12237187179371357`, 
        0.37174803446018595`, -0.4866244947338646, 0.4034991071783737}, {
        0.0317510727181884, 
        0.37174803446018584`, -0.5576220516902767, -0.3203737196228841}, {0, 
        0, 0, 0}}}, {{{
        0.07099755695641094, -0.02193945165863259, -0.33624925598197936`, \
-0.6589391851443879}, {-0.11487646027368237`, -0.6369997334857521, \
-0.17231469041052191`, -0.32268992916240813`}, {-0.18587401723009317`, \
-0.17231469041051994`, -0.6944379636225926, -0.07937768179547672}, {0, 0, 0, 
        0}}, {{0.2568715741865039, 0.15037523875188705`, 
        0.35818870764061317`, -0.5795615033489114}, {
        0.0709975569564108, -0.46468504307523173`, 
        0.5221232732120703, -0.24331224736693194`}, {0.18587401723009317`, 
        0.17231469041051994`, 0.6944379636225926, 0.07937768179547672}, {0, 0,
         0, 0}}, {{-0.18587401723009306`, -0.6150602818271192, 
        0.16393456557145708`, 
        0.33624925598197913`}, {-0.2568715741865039, -0.15037523875188705`, \
-0.35818870764061317`, 0.5795615033489114}, {-0.07099755695641094, 
        0.02193945165863259, 0.33624925598197936`, 0.6589391851443879}, {0, 0,
         0, 0}}, {{0.18587401723009306`, 
        0.6150602818271192, -0.16393456557145708`, -0.33624925598197913`}, \
{-0.0709975569564108, 0.46468504307523173`, -0.5221232732120703, 
        0.24331224736693194`}, {0.11487646027368237`, 0.6369997334857521, 
        0.17231469041052191`, 0.32268992916240813`}, {0, 0, 0, 
        0}}}, {{{-0.22975292054736282`, -0.11487646027367873`, \
-0.18587401723009264`, 
        0.6724985119639584}, {-0.229752920547363, -0.6724985119639578, \
-0.11487646027368137`, 
        0.18587401723009236`}, {-0.22975292054736338`, -0.18587401723009125`, \
-0.6724985119639576, 0.11487646027368192`}, {0, 0, 0, 0}}, {{
        0, 0.07099755695641216, 0.48662449473386526`, 0.5576220516902763}, {
        0, -0.4866244947338664, 0.5576220516902761, 0.07099755695641066}, {
        0.22975292054736338`, 0.18587401723009125`, 
        0.6724985119639576, -0.11487646027368192`}, {0, 0, 0, 0}}, {{
        0, -0.5576220516902787, 0.07099755695641077, -0.486624494733866}, {
        0, -0.07099755695641216, -0.48662449473386526`, -0.5576220516902763}, \
{0.22975292054736282`, 0.11487646027367873`, 
        0.18587401723009264`, -0.6724985119639584}, {0, 0, 0, 0}}, {{
        0, 0.5576220516902787, -0.07099755695641077, 0.486624494733866}, {
        0, 0.4866244947338664, -0.5576220516902761, -0.07099755695641066}, {
        0.229752920547363, 0.6724985119639578, 
        0.11487646027368137`, -0.18587401723009236`}, {0, 0, 0, 
        0}}}, {{{-0.11487646027367966`, 
        0.37174803446018684`, -0.3007504775037726, 0.5576220516902772}, {
        0.07099755695641309, 0.37174803446018345`, 0.4156269377774546, 
        0.4866244947338648}, {-0.18587401723009234`, -0.2297529205473615, 
        0.11487646027368006`, 0.6724985119639569}, {0, 0, 0, 
        0}}, {{-0.18587401723009328`, 0, -0.716377415281227, 
         0.07099755695641224}, {-0.25687157418650547`, -0.6015009550075449, \
-0.3007504775037741, 
        0.1858740172300919}, {-0.07099755695641309, -0.37174803446018345`, \
-0.4156269377774546, -0.4866244947338648}, {0, 0, 0, 0}}, {{
        0.18587401723009328`, 0, 
         0.716377415281227, -0.07099755695641224}, {-0.07099755695641255, \
-0.6015009550075485, 0.41562693777745285`, 0.11487646027367954`}, {
        0.11487646027367966`, -0.37174803446018684`, 
        0.3007504775037726, -0.5576220516902772}, {0, 0, 0, 0}}, {{
        0.07099755695641255, 
        0.6015009550075485, -0.41562693777745285`, -0.11487646027367954`}, {
        0.25687157418650547`, 0.6015009550075449, 
        0.3007504775037741, -0.1858740172300919}, {0.18587401723009234`, 
        0.2297529205473615, -0.11487646027368006`, -0.6724985119639569}, {0, 
        0, 0, 0}}}, {{{-0.20781346888872704`, -0.6369997334857531, 
        0.07099755695641177, 
        0.314309804323344}, {-0.07937768179547508, -0.6150602818271188, \
-0.11487646027368059`, -0.3936874861188187}, {
        0.05743823013684016, -0.4646850430752324, 
        0.5576220516902783, -0.1503752387518869}, {0, 0, 0, 
        0}}, {{-0.12843578709325182`, -0.021939451658634562`, 
        0.18587401723009234`, 0.7079972904421628}, {0.07937768179547508, 
        0.6150602818271188, 0.11487646027368059`, 0.3936874861188187}, {
        0.1368159119323153, 0.15037523875188633`, 0.6724985119639597, 
        0.24331224736693205`}, {0, 0, 0, 0}}, {{0.12843578709325182`, 
        0.021939451658634562`, -0.18587401723009234`, -0.7079972904421628}, {
        0.20781346888872704`, 
        0.6369997334857531, -0.07099755695641177, -0.314309804323344}, {
        0.265251699025567, 0.17231469041052072`, 
        0.4866244947338671, -0.4646850430752307}, {0, 0, 0, 
        0}}, {{-0.265251699025567, -0.17231469041052072`, -0.4866244947338671,
         0.4646850430752307}, {-0.1368159119323153, -0.15037523875188633`, \
-0.6724985119639597, -0.24331224736693205`}, {-0.05743823013684016, 
        0.4646850430752324, -0.5576220516902783, 0.1503752387518869}, {0, 0, 
        0, 0}}}, {{{-0.06350214543638169, -0.6332520277257363, 
        0.38387586505926363`, 
        0.019623242119110353`}, {-0.26899940478558326`, -0.5501266401702464, \
-0.2493761626664728, 
        0.3399969617419946}, {-0.10274862967460212`, -0.5818777128884364, \
-0.23724833206739418`, -0.38387586505926313`}, {0, 0, 0, 0}}, {{
        0.039246484238221024`, -0.05137431483730061, 0.6211241971266575, 
        0.40349910717837345`}, {-0.1662507751109813, 
        0.03175107271818983, -0.012127830599078923`, 0.7238728268012578}, {
        0.10274862967460212`, 0.5818777128884364, 0.23724833206739418`, 
        0.38387586505926313`}, {0, 0, 0, 0}}, {{-0.20549725934920182`, 
        0.08312538755549038, -0.6332520277257362, 
        0.32037371962288436`}, {-0.039246484238221024`, 
        0.05137431483730061, -0.6211241971266575, -0.40349910717837345`}, {
        0.06350214543638169, 
        0.6332520277257363, -0.38387586505926363`, -0.019623242119110353`}, {
        0, 0, 0, 0}}, {{0.20549725934920182`, -0.08312538755549038, 
        0.6332520277257362, -0.32037371962288436`}, {
        0.1662507751109813, -0.03175107271818983, 
        0.012127830599078923`, -0.7238728268012578}, {0.26899940478558326`, 
        0.5501266401702464, 0.2493761626664728, -0.3399969617419946}, {0, 0, 
        0, 0}}}, {{{-0.021939451658635645`, -0.6015009550075457, 
        0.42918626459702547`, -0.07937768179547633}, {
        0.10649633543461795`, 0, 0.6944379636225921, 
         0.24331224736693233`}, {-0.17231469041052125`, -0.37174803446018523`,
         0.2213727957082969, 0.5795615033489119}, {0, 0, 0, 
        0}}, {{-0.12843578709325376`, -0.6015009550075466, \
-0.2652516990255671, -0.32268992916240824`}, {-0.27881102584513917`, \
-0.371748034460186, -0.47306516791429526`, 
        0.33624925598197924`}, {-0.10649633543461795`, 
         0, -0.6944379636225921, -0.24331224736693233`}, {0, 0, 0, 0}}, {{
        0.12843578709325376`, 0.6015009550075466, 0.2652516990255671, 
        0.32268992916240824`}, {-0.15037523875188574`, 
        0.22975292054736057`, -0.20781346888872845`, 0.6589391851443882}, {
        0.021939451658635645`, 0.6015009550075457, -0.42918626459702547`, 
        0.07937768179547633}, {0, 0, 0, 0}}, {{
        0.15037523875188574`, -0.22975292054736057`, 
        0.20781346888872845`, -0.6589391851443882}, {0.27881102584513917`, 
        0.371748034460186, 0.47306516791429526`, -0.33624925598197924`}, {
        0.17231469041052125`, 
        0.37174803446018523`, -0.2213727957082969, -0.5795615033489119}, {0, 
        0, 0, 0}}}, {{{-0.2624583240792116, -0.5012507958396226, \
-0.15316861369824158`, 
        0.45737189252235366`}, {-0.2095398695488965, -0.2917109262907266, \
-0.6488326596451538, -0.052918454530316766`}, {-0.1949135684431386, 
        0.1622081649112886, -0.4775849435208174, 0.5102903470526701}, {0, 0, 
        0, 0}}, {{-0.052918454530315184`, -0.20953986954889608`, 
        0.4956640459469122, 0.5102903470526702}, {0.2095398695488965, 
        0.2917109262907266, 0.6488326596451538, 0.052918454530316766`}, {
        0.014626301105758022`, 0.4539190912020147, 0.17124771612433642`, 
        0.5632088015829868}, {0, 0, 0, 0}}, {{0.052918454530315184`, 
        0.20953986954889608`, -0.4956640459469122, -0.5102903470526702}, {
        0.2624583240792116, 0.5012507958396226, 
        0.15316861369824158`, -0.45737189252235366`}, {0.06754475563607341, 
        0.6634589607509109, -0.32441632982257573`, 0.05291845453031667}, {0, 
        0, 0, 0}}, {{-0.06754475563607341, -0.6634589607509109, 
        0.32441632982257573`, -0.05291845453031667}, {-0.014626301105758022`, \
-0.4539190912020147, -0.17124771612433642`, -0.5632088015829868}, {
        0.1949135684431386, -0.1622081649112886, 
        0.4775849435208174, -0.5102903470526701}, {0, 0, 0, 
        0}}}, {{{-0.07099755695641347, -0.6015009550075459, 
        0.4156269377774541, 0.11487646027368108`}, {
        0.18587401723009206`, 0, 0.7163774152812269, -0.07099755695641197}, {
        0.11487646027368037`, -0.37174803446018545`, 
        0.3007504775037714, -0.5576220516902775}, {0, 0, 0, 
        0}}, {{-0.25687157418650564`, -0.6015009550075465, \
-0.30075047750377276`, 
        0.18587401723009264`}, {-0.07099755695641205, -0.3717480344601861, \
-0.4156269377774553, -0.4866244947338657}, {-0.18587401723009206`, 
         0, -0.7163774152812269, 0.07099755695641197}, {0, 0, 0, 0}}, {{
        0.25687157418650564`, 0.6015009550075465, 
        0.30075047750377276`, -0.18587401723009264`}, {0.18587401723009409`, 
        0.22975292054736013`, -0.11487646027368278`, -0.6724985119639584}, {
        0.07099755695641347, 
        0.6015009550075459, -0.4156269377774541, -0.11487646027368108`}, {0, 
        0, 0, 0}}, {{-0.18587401723009409`, -0.22975292054736013`, 
        0.11487646027368278`, 0.6724985119639584}, {0.07099755695641205, 
        0.3717480344601861, 0.4156269377774553, 
        0.4866244947338657}, {-0.11487646027368037`, 
        0.37174803446018545`, -0.3007504775037714, 0.5576220516902775}, {0, 0,
         0, 0}}}, {{{-0.08312538755548952, -0.5697498822893572, \
-0.20549725934920438`, -0.4231223492974853}, {-0.23724833206739393`, \
-0.28112723538466317`, -0.6407474392457677, 0.08312538755549119}, {
        0.01212783059908043, 
        0.0831253875554907, -0.5379988095711663, -0.5062477368529756}, {0, 0, 
        0, 0}}, {{0.15412294451190384`, -0.2886226469046942, 
        0.43525017989656334`, -0.5062477368529763}, {0.23724833206739393`, 
        0.28112723538466317`, 0.6407474392457677, -0.08312538755549119}, {
        0.24937616266647444`, 0.36425262294015315`, 
        0.10274862967460167`, -0.589373124408467}, {0, 0, 0, 
        0}}, {{-0.15412294451190384`, 
        0.2886226469046942, -0.43525017989656334`, 0.5062477368529763}, {
        0.08312538755548952, 0.5697498822893572, 0.20549725934920438`, 
        0.4231223492974853}, {0.09525321815457051, 
        0.6528752698448475, -0.3325015502219619, -0.08312538755549086}, {0, 0,
         0, 0}}, {{-0.09525321815457051, -0.6528752698448475, 
        0.3325015502219619, 
        0.08312538755549086}, {-0.24937616266647444`, -0.36425262294015315`, \
-0.10274862967460167`, 
        0.589373124408467}, {-0.01212783059908043, -0.0831253875554907, 
        0.5379988095711663, 0.5062477368529756}, {0, 0, 0, 
        0}}}, {{{-0.22975292054736246`, -0.6724985119639575, \
-0.11487646027368072`, 
        0.18587401723009161`}, {-0.22975292054736116`, -0.11487646027368074`, \
-0.18587401723009206`, 
        0.672498511963957}, {-0.22975292054736174`, -0.1858740172300938, \
-0.6724985119639589, 0.1148764602736799}, {0, 0, 0, 0}}, {{
        0, -0.5576220516902766, 0.07099755695641179, -0.4866244947338653}, {
        0.22975292054736116`, 0.11487646027368074`, 
        0.18587401723009206`, -0.672498511963957}, {
        0, -0.0709975569564131, -0.4866244947338666, -0.557622051690277}, {0, 
        0, 0, 0}}, {{
        0, 0.5576220516902766, -0.07099755695641179, 0.4866244947338653}, {
        0.22975292054736246`, 0.6724985119639575, 
        0.11487646027368072`, -0.18587401723009161`}, {
        0, 0.4866244947338637, -0.5576220516902783, -0.0709975569564119}, {0, 
        0, 0, 0}}, {{
        0, -0.4866244947338637, 0.5576220516902783, 0.0709975569564119}, {
        0, 0.0709975569564131, 0.4866244947338666, 0.557622051690277}, {
        0.22975292054736174`, 0.1858740172300938, 
        0.6724985119639589, -0.1148764602736799}, {0, 0, 0, 
        0}}}}}, $CellContext`shapesInV3D$$ = CompressedData["
1:eJyFlE1IlFEUhqWMQIxs4aKFoKG1yX6wQBN7MYPMQFNaOGZhhiG6MMtgIpB+
iOiHUCrLVDQhcdVCEEWiA0EQlhBkOWpppekmwqCCEaKEea6QcJmBj+Hynfuc
933vuV9KZV1JVWxMTMyapSeWZ/XScyRnob6i76EytoR+bEic1WK4/9DUuXEL
R/614r2tqHPvLcbzO/g4/3ujZmy89+f2m9UjSv3dsn9weMytLY11PnWuj9Ph
+vv4bj99NEEf14/3gq/w/9xlP1H4cj7gL/dzPvClFVznw8u/XDPdObbujboP
VyTkrRq0p6WVHbM75+x43dnTjZdmFMhe6O9OnLDqcEHutaBZzWJVw4Fts7pY
dGVxfdI7XS+IPzZx9ZU3/z0Na3PTfr1Hz6iawpn9b/d9sVBFcWNy6z3Vvsgq
DbTOq7dtJJh+Z8D1NfqKvl798JmPUYOv4hvf8gcKQ+o52fasK2lSu2/tjf/Q
NSX8Gn6FXy8fjsExOE6/od/QL/Ix8jHy8fKPJt9NmEyd1oPM+oy40GcL5mUF
vm7ts47IWgHWZdT1Fv1tf1I2p+TyzJ7muOf69Ge0JDbFn/8Z9pdG9lsbfTay
fzyyX48iXNfX6Cv6RuMLvuDrI9xO9G6K9DP8Gn6FX28+cAyOwRH6Df2GfpeP
kY+Rj5ffwjndZ47Lme8+znsXc5PDPF3gvIeZ13nm2MevZc6GuFfnuW/wBd/g
Wy5z3Mw9yeb+ROEbfME3fAlfwpeVc58LuVdl3Dcfn3pRb9QLnUKn0OnyEfmI
fLz8DnxuZn863EJy2UHunZzHa3JswucQ/n38ds7pJTpOoQ++4Bt83WYOTpBX
kByj8AVf8A1fwpfhy2rI0X3nSjgPH596ue8x9U6n0Cl0unxEPiIfr/5/b4jq
XA==
    "], $CellContext`shapesInVtwisted$$ = CompressedData["
1:eJyVlU1IVGEUhmdhLdrkNtwkyIBQBv1AuegVmlCkoCLEahOBFG6mIqEQWxSt
QphFhVMQtAhsG0QRyFmFUAwVs7BMtAiFfg0J18XMcy7dE3KvHwzD3O875z3v
c853p/1M+dhQS6FQ2PD30/LP5/3k7x03z9VUYPU97Pt5VZ/VsXqn9OzljPk+
v+O5ZD/EJefyrpjP9X2f5+bPw/nkd4zLu0JeRS6RQzj3n67vr1Nf0VfMF/S0
xjnL24euxrlpfcXfTON7yp7UytW2G4s2emDficVtj3UY/dHXB58/qM7aAudW
X1Uv3z7/RkfZP9L4rquzmTezD9+aeVSEXy/xd/de2LXp3SfbTn1+fqi793jl
+5IuNuvy+oz6jPpEfZn+0Tf0hb6hrzH8d7F/nfyDW2+1fuhY4Pm0wc/gJ/hl
6pPfyG/kN/K7/4Qj/g3/gr/B3+BvnYFb1jqFnwm4X6Eu378Pj0Gen+T8EnMz
hd8v+N+ds/++vJ/u+x51zJJ/nrw+r0+Zz1CfUZ+ob736SV/R5z7U3E/S1xXm
JvAz+Al+mfzJb87L7x35hX/Dv/m84d/5G/wN/p4vU3+F+TnL3HSTv8R9nqAv
4+m51Bbuw37OX2IuK2FuslYb8dX0vbE5uKOv8fS9M/TtF34HqN/nppjzfwB9
VdPvPaFv8BF8BB/Bx3qoz9+Lb/HRz36Wfo+/x/BHvBEv/Al/yVwW0/wFf1Vy
zp0v15uEWyt5S+gNkK8M983sj+F3D/17Qd82PmpyWw73Y63VTvxwmJvT+EHf
0Bf6hr5+ML87yePc+6k/p76GAzf0k37CR/DhftTtGvV9xO8h9Efgk6VPvIgX
8RpJ+xP+fK74rjt/wV/w13LO/78/KOS8zA==
    "], $CellContext`showAxes$$ = False, $CellContext`\[Alpha]$$ = 0, 
    Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
    Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
    1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`C24cbctDualRots$$], 0}, {
      Hold[$CellContext`shapes$$], 0}, {
      Hold[$CellContext`primaryShape$$], 0}, {
      Hold[$CellContext`mapDepth$$], 0}, {
      Hold[$CellContext`plottingFxn$$], 0}, {
      Hold[$CellContext`shapesInVtwisted$$], 0}, {
      Hold[$CellContext`shapesInV3D$$], 0}, {{
       Hold[$CellContext`shapeChoice$$], "4Gs", "Shapes to show"}, {
      "4Gs", "Cuboctahdera"}}, {{
       Hold[$CellContext`showAxes$$], False, "Show Compounding Axes"}, {
      True, False}}, {{
       Hold[$CellContext`n4GList$$], {1, 2, 3, 4, 5}, 
       "Show Cuboctahedron/4G from Selected 24-cells"}, {1, 2, 3, 4, 5}}, {
      Hold[
       Text["Select which Cuboctahedron/4G from each 24-cell"]], 
      Manipulate`Dump`ThisIsNotAControl}, {{
       Hold[$CellContext`Cbct2$$], 1, "4G/Cuboct from 24-cell 2"}, 1, 12, 
      1}, {{
       Hold[$CellContext`Cbct3$$], 1, "4G/Cuboct from 24-cell 3"}, 1, 12, 
      1}, {{
       Hold[$CellContext`Cbct4$$], 1, "4G/Cuboct from 24-cell 4"}, 1, 12, 
      1}, {{
       Hold[$CellContext`Cbct5$$], 1, "4G/Cuboct from 24-cell 5"}, 1, 12, 
      1}, {{
       Hold[$CellContext`\[Alpha]$$], 0, "Twist fraction"}, -1, 1}}, 
    Typeset`size$$ = {270., {132.85052490234375`, 137.14947509765625`}}, 
    Typeset`update$$ = 0, Typeset`initDone$$, Typeset`skipInitDone$$ = False}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`C24cbctDualRots$$ = 
        0, $CellContext`Cbct2$$ = 1, $CellContext`Cbct3$$ = 
        1, $CellContext`Cbct4$$ = 1, $CellContext`Cbct5$$ = 
        1, $CellContext`mapDepth$$ = 
        0, $CellContext`n4GList$$ = {1, 2, 3, 4, 
         5}, $CellContext`plottingFxn$$ = 0, $CellContext`primaryShape$$ = 
        0, $CellContext`shapeChoice$$ = "4Gs", $CellContext`shapes$$ = 
        0, $CellContext`shapesInV3D$$ = 0, $CellContext`shapesInVtwisted$$ = 
        0, $CellContext`showAxes$$ = False, $CellContext`\[Alpha]$$ = 0}, 
      "ControllerVariables" :> {}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, 
      "Body" :> ($CellContext`cbctSelections = {
         1, $CellContext`Cbct2$$, $CellContext`Cbct3$$, $CellContext`Cbct4$$, \
$CellContext`Cbct5$$}; 
       Which[$CellContext`shapeChoice$$ == 
         "4Gs", {$CellContext`shapes$$ = $CellContext`n4GsInV; \
$CellContext`primaryShape$$ = $CellContext`n4GPrimary3D; \
$CellContext`mapDepth$$ = 2; $CellContext`plottingFxn$$ = 
           Tetrahedron}, $CellContext`shapeChoice$$ == 
         "Cuboctahdera", {$CellContext`shapes$$ = $CellContext`cbctsInV; \
$CellContext`primaryShape$$ = $CellContext`CbctPrimary3D; \
$CellContext`mapDepth$$ = 1; $CellContext`plottingFxn$$ = 
           ConvexHullMesh}]; $CellContext`C24cbctDualRots$$ = 
        Map[$CellContext`rotMatrixToDual4D[#, $CellContext`\[Alpha]$$]& , 
          Table[
           Part[$CellContext`C24cbctRotsToV, $CellContext`k, 
            
            Part[$CellContext`cbctSelections, $CellContext`k]], \
{$CellContext`k, 1, 5}]]; $CellContext`shapesInVtwisted$$ = 
        MapThread[Function[{$CellContext`rotMat$, $CellContext`ptSets$}, 
           Map[
           Dot[$CellContext`rotMat$, #]& , $CellContext`ptSets$, \
{$CellContext`mapDepth$$}]][#, #2]& , {$CellContext`C24cbctDualRots$$, 
           Table[
            Part[$CellContext`shapes$$, $CellContext`k, 
             
             Part[$CellContext`cbctSelections, $CellContext`k]], \
{$CellContext`k, 1, 5}]}]; $CellContext`shapesInV3D$$ = 
        Map[Vector`VectorProject[#, 
           
           NullSpace[{$CellContext`normalV}]]& , \
$CellContext`shapesInVtwisted$$, {$CellContext`mapDepth$$}]; Column[{
          Graphics3D[{Yellow, 
            If[
             MemberQ[$CellContext`n4GList$$, 1], 
             $CellContext`plottingFxn$$[1.01 $CellContext`primaryShape$$]], 
            Yellow, 
            Map[$CellContext`plottingFxn$$, 
             Part[$CellContext`shapesInV3D$$, 
              DeleteCases[$CellContext`n4GList$$, 1]]], 
            If[$CellContext`showAxes$$, {Blue, 
              Table[
               
               Tube[{$CellContext`ctr3, 0.8 
                 Part[$CellContext`compoundingAxes3D, $CellContext`k, 
                   Part[$CellContext`cbctSelections, $CellContext`k]]}, 
                0.015], {$CellContext`k, 
                DeleteCases[$CellContext`n4GList$$, 1]}]}]}, ImageSize -> 
           Medium, PlotRange -> {{-0.75, 0.75}, {-0.75, 0.75}, {-0.75, 0.75}},
            Boxed -> False, SphericalRegion -> True, 
           ViewPoint -> {3.277, 0.756, -0.378}, 
           ViewVertical -> {0.41, 0.905, 0.113}, ViewAngle -> 0.4]}]), 
      "Specifications" :> {{$CellContext`C24cbctDualRots$$, 0, ControlType -> 
         None}, {$CellContext`shapes$$, 0, ControlType -> 
         None}, {$CellContext`primaryShape$$, 0, ControlType -> 
         None}, {$CellContext`mapDepth$$, 0, ControlType -> 
         None}, {$CellContext`plottingFxn$$, 0, ControlType -> 
         None}, {$CellContext`shapesInVtwisted$$, 0, ControlType -> 
         None}, {$CellContext`shapesInV3D$$, 0, ControlType -> 
         None}, {{$CellContext`shapeChoice$$, "4Gs", "Shapes to show"}, {
         "4Gs", "Cuboctahdera"}}, {{$CellContext`showAxes$$, False, 
          "Show Compounding Axes"}, {
         True, False}}, {{$CellContext`n4GList$$, {1, 2, 3, 4, 5}, 
          "Show Cuboctahedron/4G from Selected 24-cells"}, {1, 2, 3, 4, 5}, 
         ControlType -> TogglerBar}, 
        Text[
        "Select which Cuboctahedron/4G from each 24-cell"], \
{{$CellContext`Cbct2$$, 1, "4G/Cuboct from 24-cell 2"}, 1, 12, 
         1}, {{$CellContext`Cbct3$$, 1, "4G/Cuboct from 24-cell 3"}, 1, 12, 
         1}, {{$CellContext`Cbct4$$, 1, "4G/Cuboct from 24-cell 4"}, 1, 12, 
         1}, {{$CellContext`Cbct5$$, 1, "4G/Cuboct from 24-cell 5"}, 1, 12, 
         1}, Delimiter, {{$CellContext`\[Alpha]$$, 0, "Twist fraction"}, -1, 
         1}}, "Options" :> {LabelStyle -> {11}}, "DefaultOptions" :> {}],
     ImageSizeCache->{647., {156.85052490234375`, 161.14947509765625`}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({$CellContext`cbctSelections = {1, 1, 1, 1, 
        1}, $CellContext`n4GsInV = {{{{{
            0.2297529205473612, 0, 
             0.6015009550075456, -0.37174803446018445`}, {
            0.2297529205473612, 0.37174803446018445`, 
             0, -0.6015009550075456}, {
            0.2297529205473612, 0.6015009550075456, 0.37174803446018445`, 
             0}, {0, 0, 0, 0}}, {{
            0, -0.6015009550075456, 
             0.2297529205473612, -0.37174803446018445`}, {
            0, -0.2297529205473612, -0.37174803446018445`, \
-0.6015009550075456}, {-0.2297529205473612, -0.6015009550075456, \
-0.37174803446018445`, 0}, {0, 0, 0, 0}}, {{
            0, -0.37174803446018445`, 0.6015009550075456, 
             0.2297529205473612}, {
            0, 0.2297529205473612, 0.37174803446018445`, 
             0.6015009550075456}, {-0.2297529205473612, -0.37174803446018445`,
              0, 0.6015009550075456}, {0, 0, 0, 0}}, {{
            0, 0.37174803446018445`, -0.6015009550075456, \
-0.2297529205473612}, {
            0, 0.6015009550075456, -0.2297529205473612, 
             0.37174803446018445`}, {-0.2297529205473612, 
             0, -0.6015009550075456, 0.37174803446018445`}, {0, 0, 0, 
            0}}}, {{{
            0, -0.6015009550075454, 
             0.2297529205473612, -0.37174803446018445`}, {
            0, -0.22975292054736113`, -0.37174803446018445`, \
-0.6015009550075456}, {0.22975292054736116`, 0.04733170463760791, 
            0.27708462518496907`, -0.6488326596451535}, {0, 0, 0, 0}}, {{
            0, -0.37174803446018445`, 0.6015009550075456, 
             0.22975292054736124`}, {
            0, 0.22975292054736113`, 0.37174803446018445`, 
             0.6015009550075456}, {0.22975292054736118`, 0.27708462518496907`,
             0.6488326596451535, -0.04733170463760783}, {0, 0, 0, 0}}, {{
            0, 0.37174803446018445`, -0.6015009550075456, \
-0.22975292054736124`}, {
            0, 0.6015009550075454, -0.2297529205473612, 
             0.37174803446018445`}, {0.22975292054736118`, 0.6488326596451535,
             0.04733170463760786, -0.27708462518496907`}, {0, 0, 0, 
            0}}, {{-0.22975292054736118`, -0.6488326596451535, \
-0.04733170463760786, 
            0.27708462518496907`}, {-0.22975292054736118`, \
-0.27708462518496907`, -0.6488326596451535, 
            0.04733170463760783}, {-0.22975292054736116`, \
-0.04733170463760791, -0.27708462518496907`, 0.6488326596451535}, {0, 0, 0, 
            0}}}, {{{0.07658430684912032, 
            0.6488326596451539, -0.3536689320340896, -0.02925260221151252}, {
            0.22975292054736118`, 0.3717480344601845, 
             0, -0.6015009550075456}, {
            0.2297529205473612, 0.6015009550075459, 0.3717480344601847, 0}, {
            0, 0, 0, 0}}, {{-0.1531686136982409, 
            0.04733170463760798, -0.7254169664942742, -0.02925260221151249}, {
            0, -0.22975292054736135`, -0.3717480344601845, \
-0.6015009550075456}, {-0.2297529205473612, -0.6015009550075459, \
-0.3717480344601847, 0}, {0, 0, 0, 0}}, {{
            0.15316861369824084`, -0.27708462518496935`, 
            0.3536689320340897, -0.5722483527960331}, {
            0.1531686136982409, -0.04733170463760798, 0.7254169664942742, 
            0.02925260221151249}, {-0.07658430684912032, -0.6488326596451539, 
            0.3536689320340896, 0.02925260221151252}, {0, 0, 0, 
            0}}, {{-0.15316861369824084`, 
            0.27708462518496935`, -0.3536689320340897, 0.5722483527960331}, {
            0, 0.22975292054736135`, 0.3717480344601845, 
             0.6015009550075456}, {-0.22975292054736118`, -0.3717480344601845,
              0, 0.6015009550075456}, {0, 0, 0, 0}}}, {{{0.0765843068491204, 
            0.029252602211512567`, 0.6488326596451537, 0.3536689320340897}, {
            0.22975292054736124`, 0, 
             0.6015009550075456, -0.37174803446018445`}, {
            0.2297529205473612, 0.6015009550075456, 0.37174803446018445`, 
             0}, {0, 0, 0, 0}}, {{-0.15316861369824086`, 0.02925260221151253, 
            0.04733170463760805, 0.7254169664942741}, {
            0, 0.6015009550075456, -0.2297529205473612, 
             0.3717480344601844}, {-0.22975292054736124`, 
             0, -0.6015009550075456, 0.37174803446018445`}, {0, 0, 0, 0}}, {{
            0.15316861369824086`, -0.02925260221151253, -0.04733170463760805, \
-0.7254169664942741}, {0.15316861369824078`, 
            0.5722483527960331, -0.27708462518496924`, -0.3536689320340898}, \
{-0.0765843068491204, -0.029252602211512567`, -0.6488326596451537, \
-0.3536689320340897}, {0, 0, 0, 
            0}}, {{-0.15316861369824078`, -0.5722483527960331, 
            0.27708462518496924`, 0.3536689320340898}, {
            0, -0.6015009550075456, 
             0.2297529205473612, -0.3717480344601844}, {-0.2297529205473612, \
-0.6015009550075456, -0.37174803446018445`, 0}, {0, 0, 0, 0}}}, {{{
            0.15316861369824092`, 0.3536689320340897, 0.572248352796033, 
            0.2770846251849691}, {
            0.22975292054736127`, 0, 
             0.6015009550075455, -0.3717480344601845}, {
            0, -0.3717480344601845, 0.6015009550075454, 0.2297529205473613}, {
            0, 0, 0, 0}}, {{0.1531686136982409, 
            0.7254169664942742, -0.02925260221151247, 0.04733170463760786}, {
            0.22975292054736118`, 0.37174803446018456`, 
             0, -0.6015009550075457}, {
            0, 0.3717480344601845, -0.6015009550075454, -0.2297529205473613}, \
{0, 0, 0, 0}}, {{0.07658430684912029, -0.35366893203408967`, 
            0.029252602211512498`, -0.6488326596451537}, \
{-0.1531686136982409, -0.7254169664942742, 
            0.02925260221151247, -0.04733170463760786}, \
{-0.15316861369824092`, -0.3536689320340897, -0.572248352796033, \
-0.2770846251849691}, {0, 0, 0, 0}}, {{-0.07658430684912029, 
            0.35366893203408967`, -0.029252602211512498`, 
            0.6488326596451537}, {-0.22975292054736118`, \
-0.37174803446018456`, 0, 0.6015009550075457}, {-0.22975292054736127`, 
             0, -0.6015009550075455, 0.3717480344601845}, {0, 0, 0, 0}}}, {{{
            0, 0.22975292054736124`, 0.37174803446018445`, 
             0.601500955007546}, {-0.2297529205473612, -0.37174803446018423`, 
             0, 0.601500955007546}, {
            0, -0.3717480344601844, 0.6015009550075456, 
             0.22975292054736113`}, {0, 0, 0, 0}}, {{
            0.2297529205473611, 0.6015009550075454, 0.37174803446018445`, 
             0}, {0.22975292054736124`, 0, 
             0.6015009550075456, -0.3717480344601849}, {
            0.2297529205473612, 0.37174803446018423`, 
             0, -0.601500955007546}, {0, 0, 0, 
            0}}, {{-0.2297529205473611, -0.6015009550075454, \
-0.37174803446018445`, 0}, {
            0, -0.6015009550075455, 
             0.22975292054736124`, -0.3717480344601848}, {
            0, -0.22975292054736124`, -0.37174803446018445`, \
-0.601500955007546}, {0, 0, 0, 0}}, {{
            0, 0.6015009550075455, -0.22975292054736124`, 
             0.3717480344601848}, {-0.22975292054736124`, 
             0, -0.6015009550075456, 0.3717480344601849}, {
            0, 0.3717480344601844, -0.6015009550075456, \
-0.22975292054736113`}, {0, 0, 0, 0}}}, {{{
            0.0765843068491204, -0.3536689320340898, 
            0.02925260221151251, -0.6488326596451535}, {
            0.2297529205473612, 0, 0.6015009550075456, -0.3717480344601844}, {
            0.22975292054736127`, 0.37174803446018445`, 
             0, -0.6015009550075454}, {0, 0, 0, 
            0}}, {{-0.15316861369824084`, -0.7254169664942741, 
            0.02925260221151249, -0.047331704637608045`}, {
            0, -0.3717480344601844, 0.6015009550075456, 
             0.22975292054736116`}, {-0.22975292054736127`, \
-0.37174803446018445`, 0, 0.6015009550075454}, {0, 0, 0, 0}}, {{
            0.15316861369824078`, 0.3536689320340898, 0.572248352796033, 
            0.2770846251849692}, {0.15316861369824084`, 
            0.7254169664942741, -0.02925260221151249, 
            0.047331704637608045`}, {-0.0765843068491204, 
            0.3536689320340898, -0.02925260221151251, 0.6488326596451535}, {0,
             0, 0, 0}}, {{-0.15316861369824078`, -0.3536689320340898, \
-0.572248352796033, -0.2770846251849692}, {
            0, 0.3717480344601844, -0.6015009550075456, \
-0.22975292054736116`}, {-0.2297529205473612, 0, -0.6015009550075456, 
             0.3717480344601844}, {0, 0, 0, 0}}}, {{{
            0.22975292054736113`, 0, 
             0.6015009550075455, -0.3717480344601844}, {
            0.22975292054736118`, 0.6015009550075456, 0.37174803446018434`, 
             0}, {0.07658430684912021, 0.0292526022115126, 0.6488326596451535,
             0.35366893203408956`}, {0, 0, 0, 0}}, {{
            0, -0.6015009550075456, 
             0.2297529205473612, -0.37174803446018445`}, \
{-0.22975292054736118`, -0.6015009550075456, -0.37174803446018434`, 
             0}, {-0.15316861369824092`, -0.572248352796033, 
            0.2770846251849691, 0.35366893203408956`}, {0, 0, 0, 0}}, {{
            0, 0.6015009550075456, -0.2297529205473612, 
             0.37174803446018445`}, {-0.22975292054736113`, 
             0, -0.6015009550075455, 
             0.3717480344601844}, {-0.15316861369824086`, 
            0.029252602211512616`, 0.04733170463760792, 0.725416966494274}, {
            0, 0, 0, 0}}, {{
            0.15316861369824086`, -0.029252602211512616`, \
-0.04733170463760792, -0.725416966494274}, {0.15316861369824092`, 
            0.572248352796033, -0.2770846251849691, -0.35366893203408956`}, \
{-0.07658430684912021, -0.0292526022115126, -0.6488326596451535, \
-0.35366893203408956`}, {0, 0, 0, 0}}}, {{{
            0, -0.2297529205473612, -0.3717480344601848, \
-0.6015009550075456}, {
            0, 0.3717480344601845, -0.601500955007546, \
-0.22975292054736127`}, {
            0.2297529205473611, 0.3717480344601845, 0, -0.6015009550075454}, {
            0, 0, 0, 0}}, {{
            0, -0.6015009550075456, 
             0.22975292054736116`, -0.3717480344601844}, {
            0.22975292054736118`, 0, 
             0.601500955007546, -0.37174803446018423`}, {
            0, -0.3717480344601845, 0.601500955007546, 
             0.22975292054736127`}, {0, 0, 0, 0}}, {{
            0, 0.6015009550075456, -0.22975292054736116`, 
             0.3717480344601844}, {
            0.2297529205473613, 0.6015009550075456, 0.37174803446018484`, 
             0}, {0, 0.2297529205473612, 0.3717480344601848, 
             0.6015009550075456}, {0, 0, 0, 
            0}}, {{-0.2297529205473613, -0.6015009550075456, \
-0.37174803446018484`, 0}, {-0.22975292054736118`, 0, -0.601500955007546, 
             0.37174803446018423`}, {-0.2297529205473611, -0.3717480344601845,
              0, 0.6015009550075454}, {0, 0, 0, 0}}}, {{{
            0.1531686136982407, -0.047331704637607865`, 0.725416966494274, 
            0.0292526022115126}, {
            0.22975292054736116`, 0.6015009550075457, 0.37174803446018434`, 
             0}, {0, 0.22975292054736124`, 0.3717480344601845, 
             0.6015009550075456}, {0, 0, 0, 
            0}}, {{-0.0765843068491204, -0.6488326596451537, 
            0.35366893203408967`, 
            0.029252602211512574`}, {-0.22975292054736116`, \
-0.6015009550075457, -0.37174803446018434`, 
             0}, {-0.22975292054736116`, -0.3717480344601845, 0, 
             0.6015009550075456}, {0, 0, 0, 0}}, {{0.0765843068491204, 
            0.6488326596451537, -0.35366893203408967`, \
-0.029252602211512574`}, {-0.1531686136982407, 
            0.047331704637607865`, -0.725416966494274, -0.0292526022115126}, \
{-0.15316861369824075`, 0.2770846251849691, -0.35366893203408956`, 
            0.572248352796033}, {0, 0, 0, 0}}, {{
            0.15316861369824075`, -0.2770846251849691, 
            0.35366893203408956`, -0.572248352796033}, {
            0.22975292054736116`, 0.3717480344601845, 
             0, -0.6015009550075456}, {
            0, -0.22975292054736124`, -0.3717480344601845, \
-0.6015009550075456}, {0, 0, 0, 0}}}, {{{
            0, -0.601500955007546, 
             0.22975292054736127`, -0.37174803446018445`}, \
{-0.22975292054736116`, -0.601500955007546, -0.3717480344601843, 0}, {
            0, -0.22975292054736116`, -0.37174803446018445`, \
-0.6015009550075456}, {0, 0, 0, 0}}, {{
            0.2297529205473611, 0, 0.6015009550075455, -0.3717480344601845}, {
            0.22975292054736127`, 0.3717480344601849, 
             0, -0.6015009550075456}, {
            0.22975292054736116`, 0.601500955007546, 0.3717480344601843, 0}, {
            0, 0, 0, 
            0}}, {{-0.2297529205473611, 0, -0.6015009550075455, 
             0.3717480344601845}, {
            0, 0.3717480344601848, -0.6015009550075457, -0.2297529205473612}, \
{0, 0.601500955007546, -0.22975292054736127`, 0.37174803446018445`}, {0, 0, 0,
             0}}, {{0, -0.3717480344601848, 0.6015009550075457, 
             0.2297529205473612}, {-0.22975292054736127`, -0.3717480344601849,
              0, 0.6015009550075456}, {
            0, 0.22975292054736116`, 0.37174803446018445`, 
             0.6015009550075456}, {0, 0, 0, 0}}}, {{{0.22975292054736135`, 
            0.04733170463760797, 0.2770846251849692, -0.6488326596451536}, {
            0, -0.6015009550075456, 
             0.22975292054736118`, -0.37174803446018445`}, {
            0, -0.2297529205473612, -0.37174803446018445`, \
-0.6015009550075456}, {0, 0, 0, 0}}, {{0.22975292054736135`, 
            0.6488326596451536, 0.047331704637607976`, -0.2770846251849692}, {
            0, 0.37174803446018445`, -0.6015009550075455, \
-0.2297529205473612}, {
            0, 0.6015009550075456, -0.22975292054736118`, 
             0.37174803446018445`}, {0, 0, 0, 
            0}}, {{-0.22975292054736135`, -0.6488326596451536, \
-0.047331704637607976`, 
            0.2770846251849692}, {-0.22975292054736135`, -0.2770846251849692, \
-0.6488326596451536, 
            0.04733170463760797}, {-0.22975292054736135`, \
-0.04733170463760797, -0.2770846251849692, 0.6488326596451536}, {0, 0, 0, 
            0}}, {{0.22975292054736135`, 0.2770846251849692, 
            0.6488326596451536, -0.04733170463760797}, {
            0, -0.37174803446018445`, 0.6015009550075455, 
             0.2297529205473612}, {
            0, 0.2297529205473612, 0.37174803446018445`, 
             0.6015009550075456}, {0, 0, 0, 0}}}}, {{{{0.18587401723009248`, 
            0.22975292054736168`, -0.11487646027368059`, \
-0.6724985119639572}, {0.07099755695641177, 
            0.6015009550075462, -0.4156269377774533, -0.11487646027368069`}, {
            0.25687157418650425`, 0.6015009550075466, 
            0.3007504775037733, -0.18587401723009236`}, {0, 0, 0, 0}}, {{
            0.11487646027368083`, -0.37174803446018456`, 
            0.3007504775037728, -0.5576220516902766}, {
            0.18587401723009245`, 0, 
             0.7163774152812266, -0.07099755695641166}, \
{-0.07099755695641177, -0.6015009550075462, 0.4156269377774533, 
            0.11487646027368069`}, {0, 0, 0, 0}}, {{-0.11487646027368083`, 
            0.37174803446018456`, -0.3007504775037728, 0.5576220516902766}, {
            0.0709975569564118, 0.371748034460185, 0.41562693777745374`, 
            
            0.48662449473386493`}, {-0.18587401723009248`, \
-0.22975292054736168`, 0.11487646027368059`, 0.6724985119639572}, {0, 0, 0, 
            0}}, {{-0.0709975569564118, -0.371748034460185, \
-0.41562693777745374`, -0.48662449473386493`}, {-0.18587401723009245`, 
             0, -0.7163774152812266, 
             0.07099755695641166}, {-0.25687157418650425`, \
-0.6015009550075466, -0.3007504775037733, 0.18587401723009236`}, {0, 0, 0, 
            0}}}, {{{-0.18587401723009245`, -0.07099755695641169, 0, 
             0.7163774152812267}, {-0.2568715741865034, \
-0.18587401723009234`, -0.6015009550075457, 
            0.30075047750377304`}, {-0.07099755695641172, 
            0.4866244947338654, -0.37174803446018423`, 0.4156269377774535}, {
            0, 0, 0, 0}}, {{0.07099755695641126, 0.11487646027368048`, 
            0.601500955007546, 0.41562693777745385`}, {0.2568715741865034, 
            0.18587401723009234`, 
            0.6015009550075457, -0.30075047750377304`}, {0.18587401723009178`,
             0.6724985119639577, 0.2297529205473615, 0.11487646027368048`}, {
            0, 0, 0, 
            0}}, {{-0.07099755695641126, -0.11487646027368048`, \
-0.601500955007546, -0.41562693777745385`}, {
            0.18587401723009245`, 0.07099755695641169, 
             0, -0.7163774152812267}, {0.11487646027368065`, 
            0.557622051690277, -0.37174803446018445`, -0.3007504775037731}, {
            0, 0, 0, 0}}, {{-0.11487646027368065`, -0.557622051690277, 
            0.37174803446018445`, 
            0.3007504775037731}, {-0.18587401723009178`, -0.6724985119639577, \
-0.2297529205473615, -0.11487646027368048`}, {
            0.07099755695641172, -0.4866244947338654, 
            0.37174803446018423`, -0.4156269377774535}, {0, 0, 0, 0}}}, {{{
            0.1858740172300922, 0.7163774152812267, 0.07099755695641184, 
             0}, {-0.07099755695641191, 
            0.41562693777745346`, -0.11487646027368063`, 0.601500955007546}, {
            0.11487646027368038`, 0.30075047750377293`, 0.557622051690277, 
            0.3717480344601847}, {0, 0, 0, 0}}, {{0.07099755695641163, 
            0.4156269377774536, -0.486624494733865, -0.3717480344601847}, \
{-0.1858740172300924, 0.11487646027368043`, -0.6724985119639573, 
            0.22975292054736143`}, {-0.11487646027368038`, \
-0.30075047750377293`, -0.557622051690277, -0.3717480344601847}, {0, 0, 0, 
            0}}, {{0.2568715741865041, 0.30075047750377315`, 
            0.1858740172300925, -0.6015009550075462}, {
            0.1858740172300924, -0.11487646027368043`, 
            0.6724985119639573, -0.22975292054736143`}, {
            0.07099755695641191, -0.41562693777745346`, 
            0.11487646027368063`, -0.601500955007546}, {0, 0, 0, 
            0}}, {{-0.2568715741865041, -0.30075047750377315`, \
-0.1858740172300925, 
            0.6015009550075462}, {-0.07099755695641163, -0.4156269377774536, 
            0.486624494733865, 
            0.3717480344601847}, {-0.1858740172300922, -0.7163774152812267, \
-0.07099755695641184, 0}, {0, 0, 0, 0}}}, {{{0.27881102584513806`, 
            0.4730651679142942, 0.3362492559819788, -0.37174803446018456`}, {
            0.17231469041052075`, -0.22137279570829826`, 
            0.5795615033489115, -0.3717480344601847}, {0.15037523875188621`, 
            0.2078134688887267, 0.6589391851443864, 0.22975292054736096`}, {0,
             0, 0, 0}}, {{0.12843578709325187`, 
            0.2652516990255674, -0.32268992916240746`, -0.6015009550075455}, {
            0.02193945165863445, -0.42918626459702514`, -0.07937768179547465, \
-0.6015009550075456}, {-0.15037523875188621`, -0.2078134688887267, \
-0.6589391851443864, -0.22975292054736096`}, {0, 0, 0, 0}}, {{
            0.10649633543461762`, 0.6944379636225926, -0.24331224736693266`, 
             0}, {-0.02193945165863445, 0.42918626459702514`, 
            0.07937768179547465, 0.6015009550075456}, {-0.17231469041052075`, 
            0.22137279570829826`, -0.5795615033489115, 0.3717480344601847}, {
            0, 0, 0, 
            0}}, {{-0.10649633543461762`, -0.6944379636225926, 
             0.24331224736693266`, 
             0}, {-0.12843578709325187`, -0.2652516990255674, 
            0.32268992916240746`, 
            0.6015009550075455}, {-0.27881102584513806`, -0.4730651679142942, \
-0.3362492559819788, 0.37174803446018456`}, {0, 0, 0, 0}}}, {{{
            0.28112723538466244`, 0.41562693777745396`, 
            0.37174803446018445`, -0.4034991071783743}, {0.1344997023927919, 
            0.11487646027368079`, -0.2297529205473615, -0.6846263425630362}, {
            0.14662753299187034`, -0.300750477503773, 
            0.37174803446018434`, -0.5501266401702449}, {0, 0, 0, 0}}, {{
            0.13449970239279196`, 0.7163774152812269, 0, 
             0.14662753299187054`}, {-0.01212783059907846, 
            0.41562693777745363`, -0.6015009550075457, \
-0.13449970239279144`}, {-0.14662753299187034`, 
            0.300750477503773, -0.37174803446018434`, 0.5501266401702449}, {0,
             0, 0, 0}}, {{-0.14662753299187029`, -0.3007504775037731, \
-0.6015009550075461, -0.281127235384662}, {-0.13449970239279196`, \
-0.7163774152812269, 
             0, -0.14662753299187054`}, {-0.28112723538466244`, \
-0.41562693777745396`, -0.37174803446018445`, 0.4034991071783743}, {0, 0, 0, 
            0}}, {{0.14662753299187029`, 0.3007504775037731, 
            0.6015009550075461, 0.281127235384662}, {
            0.01212783059907846, -0.41562693777745363`, 0.6015009550075457, 
            0.13449970239279144`}, {-0.1344997023927919, \
-0.11487646027368079`, 0.2297529205473615, 0.6846263425630362}, {0, 0, 0, 
            0}}}, {{{0.2788110258451386, 0.4156269377774538, 
            0.31430980432334377`, -0.4511257162556594}, {0.12843578709325207`,
             0.7163774152812267, -0.1503752387518867, 0.02193945165863423}, {
            0.10649633543461755`, 
            0.30075047750377315`, -0.39368748611881954`, \
-0.5440627248707058}, {0, 0, 0, 0}}, {{
            0.15037523875188635`, -0.30075047750377293`, 
            0.46468504307523045`, -0.47306516791429365`}, \
{-0.021939451658634472`, -0.41562693777745363`, -0.2433122473669329, \
-0.5660021765293399}, {-0.12843578709325207`, -0.7163774152812267, 
            0.1503752387518867, -0.02193945165863423}, {0, 0, 0, 
            0}}, {{-0.15037523875188635`, 
            0.30075047750377293`, -0.46468504307523045`, 
            0.47306516791429365`}, {-0.1723146904105212, -0.1148764602736807, \
-0.7079972904421634, -0.09293700861504622}, {-0.2788110258451386, \
-0.4156269377774538, -0.31430980432334377`, 0.4511257162556594}, {0, 0, 0, 
            0}}, {{0.1723146904105212, 0.1148764602736807, 0.7079972904421634,
             0.09293700861504622}, {0.021939451658634472`, 
            0.41562693777745363`, 0.2433122473669329, 
            0.5660021765293399}, {-0.10649633543461755`, \
-0.30075047750377315`, 0.39368748611881954`, 0.5440627248707058}, {0, 0, 0, 
            0}}}, {{{0.2568715741865035, 0.3007504775037731, 
            0.18587401723009225`, -0.6015009550075457}, {
            0.07099755695641191, -0.41562693777745346`, 
            0.11487646027368069`, -0.6015009550075455}, {
            0.18587401723009228`, -0.1148764602736805, 
            0.6724985119639577, -0.22975292054736124`}, {0, 0, 0, 0}}, {{
            0.18587401723009175`, 0.7163774152812266, 0.0709975569564116, 
             0}, {-0.07099755695641191, 
            0.41562693777745346`, -0.11487646027368069`, 
            0.6015009550075455}, {0.11487646027368073`, 0.30075047750377315`, 
            0.5576220516902768, 0.3717480344601844}, {0, 0, 0, 
            0}}, {{-0.18587401723009175`, -0.7163774152812266, \
-0.0709975569564116, 
             0}, {-0.2568715741865035, -0.3007504775037731, \
-0.18587401723009225`, 
            0.6015009550075457}, {-0.07099755695641102, -0.4156269377774532, 
            0.4866244947338654, 0.37174803446018456`}, {0, 0, 0, 0}}, {{
            0.07099755695641102, 
            0.4156269377774532, -0.4866244947338654, -0.37174803446018456`}, \
{-0.11487646027368073`, -0.30075047750377315`, -0.5576220516902768, \
-0.3717480344601844}, {-0.18587401723009228`, 
            0.1148764602736805, -0.6724985119639577, 0.22975292054736124`}, {
            0, 0, 0, 0}}}, {{{0.11487646027368038`, 
            0.5576220516902769, -0.37174803446018445`, -0.3007504775037727}, {
            0.18587401723009256`, 0.07099755695641174, 
             0, -0.716377415281226}, {-0.07099755695641191, \
-0.11487646027368069`, -0.6015009550075463, -0.41562693777745296`}, {0, 0, 0, 
            0}}, {{0.18587401723009253`, 0.6724985119639577, 
            0.22975292054736177`, 0.11487646027368026`}, {
            0.25687157418650425`, 0.18587401723009248`, 
            0.6015009550075469, -0.30075047750377315`}, {0.07099755695641191, 
            0.11487646027368069`, 0.6015009550075463, 0.41562693777745296`}, {
            0, 0, 0, 0}}, {{0.07099755695641172, -0.48662449473386526`, 
            0.371748034460185, -0.4156269377774533}, {-0.18587401723009253`, \
-0.6724985119639577, -0.22975292054736177`, -0.11487646027368026`}, \
{-0.11487646027368038`, -0.5576220516902769, 0.37174803446018445`, 
            0.3007504775037727}, {0, 0, 0, 0}}, {{-0.07099755695641172, 
            0.48662449473386526`, -0.371748034460185, 
            0.4156269377774533}, {-0.25687157418650425`, \
-0.18587401723009248`, -0.6015009550075469, 
            0.30075047750377315`}, {-0.18587401723009256`, \
-0.07099755695641174, 0, 0.716377415281226}, {0, 0, 0, 0}}}, {{{
            0.07099755695641169, 
            0.601500955007546, -0.4156269377774533, -0.11487646027368037`}, {
            0.25687157418650386`, 0.6015009550075464, 
            0.30075047750377326`, -0.1858740172300925}, {0.18587401723009195`,
             0.22975292054736132`, -0.1148764602736804, -0.6724985119639576}, \
{0, 0, 0, 0}}, {{-0.18587401723009211`, 0, -0.7163774152812266, 
             0.07099755695641186}, {-0.07099755695641186, \
-0.37174803446018495`, -0.4156269377774535, -0.4866244947338655}, \
{-0.25687157418650386`, -0.6015009550075464, -0.30075047750377326`, 
            0.1858740172300925}, {0, 0, 0, 0}}, {{
            0.18587401723009211`, 0, 
             0.7163774152812266, -0.07099755695641186}, {
            0.1148764602736804, -0.3717480344601847, 
            0.30075047750377293`, -0.557622051690277}, {-0.07099755695641169, \
-0.601500955007546, 0.4156269377774533, 0.11487646027368037`}, {0, 0, 0, 
            0}}, {{-0.1148764602736804, 
            0.3717480344601847, -0.30075047750377293`, 0.557622051690277}, {
            0.07099755695641186, 0.37174803446018495`, 0.4156269377774535, 
            0.4866244947338655}, {-0.18587401723009195`, \
-0.22975292054736132`, 0.1148764602736804, 0.6724985119639576}, {0, 0, 0, 
            0}}}, {{{0.1503752387518861, 
            0.4730651679142937, -0.30075047750377254`, -0.4646850430752312}, \
{-0.021939451658634507`, 0.5660021765293402, -0.4156269377774533, 
            0.24331224736693252`}, {-0.12843578709325212`, 
            0.021939451658634007`, -0.7163774152812262, \
-0.15037523875188658`}, {0, 0, 0, 0}}, {{0.27881102584513817`, 
            0.4511257162556598, 0.4156269377774537, -0.3143098043233444}, {
            0.10649633543461766`, 0.5440627248707062, 0.30075047750377315`, 
            0.3936874861188191}, {
            0.12843578709325212`, -0.021939451658634007`, 0.7163774152812262, 
            0.15037523875188658`}, {0, 0, 0, 0}}, {{-0.17231469041052072`, 
            0.09293700861504631, -0.1148764602736808, 
            0.7079972904421635}, {-0.27881102584513817`, -0.4511257162556598, \
-0.4156269377774537, 
            0.3143098043233444}, {-0.1503752387518861, -0.4730651679142937, 
            0.30075047750377254`, 0.4646850430752312}, {0, 0, 0, 0}}, {{
            0.17231469041052072`, -0.09293700861504631, 
            0.1148764602736808, -0.7079972904421635}, {-0.10649633543461766`, \
-0.5440627248707062, -0.30075047750377315`, -0.3936874861188191}, {
            0.021939451658634507`, -0.5660021765293402, 
            0.4156269377774533, -0.24331224736693252`}, {0, 0, 0, 0}}}, {{{
            0.18587401723009234`, 
            0.22975292054736082`, -0.11487646027368012`, \
-0.6724985119639577}, {0.2568715741865034, 0.6015009550075461, 
            0.30075047750377304`, -0.1858740172300928}, {0.07099755695641125, 
            0.6015009550075461, -0.4156269377774532, -0.11487646027368084`}, {
            0, 0, 0, 
            0}}, {{-0.07099755695641119, -0.371748034460185, \
-0.41562693777745324`, -0.4866244947338648}, {-0.1858740172300923, 
             0, -0.7163774152812264, 
             0.07099755695641202}, {-0.2568715741865034, -0.6015009550075461, \
-0.30075047750377304`, 0.1858740172300928}, {0, 0, 0, 0}}, {{
            0.07099755695641119, 0.371748034460185, 0.41562693777745324`, 
            0.4866244947338648}, {-0.11487646027368076`, 
            0.37174803446018523`, -0.3007504775037728, 
            0.5576220516902771}, {-0.18587401723009234`, \
-0.22975292054736082`, 0.11487646027368012`, 0.6724985119639577}, {0, 0, 0, 
            0}}, {{0.11487646027368076`, -0.37174803446018523`, 
            0.3007504775037728, -0.5576220516902771}, {
            0.1858740172300923, 0, 
             0.7163774152812264, -0.07099755695641202}, \
{-0.07099755695641125, -0.6015009550075461, 0.4156269377774532, 
            0.11487646027368084`}, {0, 0, 0, 0}}}, {{{0.1466275329918702, 
            0.5501266401702452, -0.3007504775037726, -0.3717480344601847}, {
            0.2811272353846622, 0.4034991071783748, 
            0.41562693777745374`, -0.37174803446018434`}, {
            0.13449970239279174`, 0.6846263425630369, 0.11487646027368079`, 
            0.2297529205473615}, {0, 0, 0, 
            0}}, {{-0.134499702392792, 
             0.14662753299187037`, -0.7163774152812263, 
             0}, {-0.1466275329918706, 
            0.2811272353846622, -0.3007504775037728, 
            0.6015009550075459}, {-0.2811272353846622, -0.4034991071783748, \
-0.41562693777745374`, 0.37174803446018434`}, {0, 0, 0, 0}}, {{
            0.134499702392792, -0.14662753299187037`, 0.7163774152812263, 
             0}, {-0.012127830599078598`, 0.1344997023927917, 
            0.4156269377774536, 
            0.6015009550075461}, {-0.1466275329918702, -0.5501266401702452, 
            0.3007504775037726, 0.3717480344601847}, {0, 0, 0, 0}}, {{
            0.012127830599078598`, -0.1344997023927917, -0.4156269377774536, \
-0.6015009550075461}, {0.1466275329918706, -0.2811272353846622, 
            0.3007504775037728, -0.6015009550075459}, {-0.13449970239279174`, \
-0.6846263425630369, -0.11487646027368079`, -0.2297529205473615}, {0, 0, 0, 
            0}}}}, {{{{
            0, 0.4866244947338651, -0.5576220516902766, \
-0.07099755695641174}, {
            0, 0.557622051690277, -0.07099755695641133, 
             0.48662449473386493`}, {0.2297529205473616, 0.672498511963958, 
            0.11487646027368087`, -0.18587401723009264`}, {0, 0, 0, 0}}, {{
            0, -0.07099755695641166, -0.4866244947338651, \
-0.5576220516902766}, {0.22975292054736127`, 0.11487646027368068`, 
            0.18587401723009256`, -0.6724985119639573}, {
            0, -0.557622051690277, 
             0.07099755695641133, -0.48662449473386493`}, {0, 0, 0, 0}}, {{
            0, 0.07099755695641166, 0.4866244947338651, 0.5576220516902766}, {
            0.22975292054736118`, 0.18587401723009267`, 
            0.6724985119639574, -0.11487646027368092`}, {
            0, -0.4866244947338651, 0.5576220516902766, 
             0.07099755695641174}, {0, 0, 0, 
            0}}, {{-0.22975292054736118`, -0.18587401723009267`, \
-0.6724985119639574, 
            0.11487646027368092`}, {-0.22975292054736127`, \
-0.11487646027368068`, -0.18587401723009256`, 
            0.6724985119639573}, {-0.2297529205473616, -0.672498511963958, \
-0.11487646027368087`, 0.18587401723009264`}, {0, 0, 0, 0}}}, {{{
            0.07099755695641105, 
            0.6015009550075462, -0.4156269377774544, -0.11487646027368034`}, {
            0.25687157418650475`, 0.6015009550075463, 
            0.30075047750377215`, -0.18587401723009167`}, {
            0.18587401723009228`, 
            0.2297529205473614, -0.1148764602736807, -0.6724985119639573}, {0,
             0, 0, 0}}, {{-0.11487646027368123`, 
            0.37174803446018473`, -0.3007504775037739, 0.557622051690277}, {
            0.07099755695641224, 0.3717480344601851, 0.41562693777745296`, 
            0.4866244947338651}, {-0.18587401723009228`, -0.2297529205473614, 
            0.1148764602736807, 0.6724985119639573}, {0, 0, 0, 
            0}}, {{-0.18587401723009367`, 0, -0.7163774152812267, 
             0.07099755695641158}, {-0.07099755695641224, \
-0.3717480344601851, -0.41562693777745296`, -0.4866244947338651}, \
{-0.25687157418650475`, -0.6015009550075463, -0.30075047750377215`, 
            0.18587401723009167`}, {0, 0, 0, 0}}, {{
            0.18587401723009367`, 0, 
             0.7163774152812267, -0.07099755695641158}, {
            0.11487646027368123`, -0.37174803446018473`, 
            0.3007504775037739, -0.557622051690277}, {-0.07099755695641105, \
-0.6015009550075462, 0.4156269377774544, 0.11487646027368034`}, {0, 0, 0, 
            0}}}, {{{0.03175107271818977, 
            0.557622051690278, -0.4866244947338655, -0.06350214543638044}, {
            0.15412294451190178`, 0.672498511963958, 0.18587401723009284`, 
            0.2054972593492027}, {0.24937616266647145`, 0.48662449473386593`, 
            0.07099755695641156, -0.49875232533294456`}, {0, 0, 0, 
            0}}, {{-0.21762508994828195`, 
            0.07099755695641191, -0.5576220516902771, 
            0.43525017989656417`}, {-0.09525321815457008, 0.1858740172300921, 
            0.11487646027368117`, 
            0.7042495846821473}, {-0.24937616266647145`, \
-0.48662449473386593`, -0.07099755695641156, 0.49875232533294456`}, {0, 0, 0, 
            0}}, {{-0.12237187179371226`, -0.11487646027368047`, \
-0.6724985119639584, -0.26899940478558326`}, {
            0.09525321815457008, -0.1858740172300921, -0.11487646027368117`, \
-0.7042495846821473}, {-0.15412294451190178`, -0.672498511963958, \
-0.18587401723009284`, -0.2054972593492027}, {0, 0, 0, 0}}, {{
            0.12237187179371226`, 0.11487646027368047`, 0.6724985119639584, 
            0.26899940478558326`}, {
            0.21762508994828195`, -0.07099755695641191, 
            
            0.5576220516902771, -0.43525017989656417`}, \
{-0.03175107271818977, -0.557622051690278, 0.4866244947338655, 
            0.06350214543638044}, {0, 0, 0, 0}}}, {{{-0.07099755695641263, 
            0.41562693777745435`, -0.1148764602736802, 0.6015009550075462}, {
            0.18587401723009164`, 0.716377415281227, 0.070997556956412, 0}, {
            0.1148764602736797, 0.3007504775037737, 0.5576220516902771, 
            0.3717480344601849}, {0, 0, 0, 0}}, {{-0.18587401723009223`, 
            0.11487646027368077`, -0.6724985119639573, 
            0.22975292054736143`}, {0.07099755695641193, 
            0.41562693777745324`, -0.48662449473386493`, \
-0.37174803446018523`}, {-0.1148764602736797, -0.3007504775037737, \
-0.5576220516902771, -0.3717480344601849}, {0, 0, 0, 
            0}}, {{-0.2568715741865042, -0.30075047750377293`, \
-0.18587401723009242`, 
            0.6015009550075469}, {-0.07099755695641193, -0.41562693777745324`,
             0.48662449473386493`, 
            0.37174803446018523`}, {-0.18587401723009164`, \
-0.716377415281227, -0.070997556956412, 0}, {0, 0, 0, 0}}, {{
            0.2568715741865042, 0.30075047750377293`, 
            0.18587401723009242`, -0.6015009550075469}, {
            0.18587401723009223`, -0.11487646027368077`, 
            0.6724985119639573, -0.22975292054736143`}, {
            0.07099755695641263, -0.41562693777745435`, 
            0.1148764602736802, -0.6015009550075462}, {0, 0, 0, 0}}}, {{{
            0.13681591193231468`, 
            0.6724985119639584, -0.2433122473669324, -0.15037523875188646`}, \
{-0.1284357870932519, 0.18587401723009234`, -0.707997290442163, 
            0.02193945165863444}, {0.07937768179547439, 
            0.11487646027368048`, -0.39368748611881876`, \
-0.6150602818271169}, {0, 0, 0, 0}}, {{0.057438230136840684`, 
            0.557622051690278, 0.15037523875188666`, 
            0.46468504307523073`}, {-0.2078134688887261, 
            0.07099755695641183, -0.31430980432334427`, 
            0.6369997334857517}, {-0.07937768179547439, -0.11487646027368048`,
             0.39368748611881876`, 0.6150602818271169}, {0, 0, 0, 0}}, {{
            0.26525169902556667`, 0.4866244947338661, 
            0.4646850430752306, -0.172314690410521}, {
            0.2078134688887261, -0.07099755695641183, 
            0.31430980432334427`, -0.6369997334857517}, {
            0.1284357870932519, -0.18587401723009234`, 
            0.707997290442163, -0.02193945165863444}, {0, 0, 0, 
            0}}, {{-0.26525169902556667`, -0.4866244947338661, \
-0.4646850430752306, 
            0.172314690410521}, {-0.057438230136840684`, -0.557622051690278, \
-0.15037523875188666`, -0.46468504307523073`}, {-0.13681591193231468`, \
-0.6724985119639584, 0.2433122473669324, 0.15037523875188646`}, {0, 0, 0, 
            0}}}, {{{-0.07099755695641172, 
            0.3362492559819793, -0.6589391851443868, -0.02193945165863373}, \
{-0.18587401723009206`, 0.16393456557145836`, -0.33624925598197963`, 
            0.615060281827118}, {-0.2568715741865043, -0.3581887076406137, \
-0.5795615033489122, 0.15037523875188671`}, {0, 0, 0, 0}}, {{
            0.18587401723009248`, 
            0.6944379636225926, -0.07937768179547489, -0.1723146904105206}, {
            0.07099755695641202, 0.522123273212072, 0.24331224736693244`, 
            0.46468504307523095`}, {0.2568715741865043, 0.3581887076406137, 
            0.5795615033489122, -0.15037523875188671`}, {0, 0, 0, 
            0}}, {{-0.11487646027368059`, -0.17231469041052078`, 
            0.3226899291624073, 
            0.6369997334857513}, {-0.18587401723009248`, -0.6944379636225926, 
            0.07937768179547489, 0.1723146904105206}, {
            0.07099755695641172, -0.3362492559819793, 0.6589391851443868, 
            0.02193945165863373}, {0, 0, 0, 0}}, {{0.11487646027368059`, 
            0.17231469041052078`, -0.3226899291624073, -0.6369997334857513}, \
{-0.07099755695641202, -0.522123273212072, -0.24331224736693244`, \
-0.46468504307523095`}, {0.18587401723009206`, -0.16393456557145836`, 
            0.33624925598197963`, -0.615060281827118}, {0, 0, 0, 0}}}, {{{
            0.0709975569564111, -0.4156269377774537, 
            0.11487646027368148`, -0.6015009550075457}, {0.25687157418650397`,
             0.3007504775037725, 0.1858740172300929, -0.6015009550075466}, {
            0.18587401723009195`, -0.11487646027368043`, 
            0.6724985119639582, -0.22975292054736163`}, {0, 0, 0, 
            0}}, {{-0.11487646027368059`, -0.30075047750377315`, \
-0.5576220516902766, -0.37174803446018445`}, {0.070997556956412, 
            
            0.41562693777745296`, -0.48662449473386504`, -0.371748034460185}, \
{-0.18587401723009195`, 0.11487646027368043`, -0.6724985119639582, 
            0.22975292054736163`}, {0, 0, 0, 
            0}}, {{-0.1858740172300929, -0.7163774152812263, \
-0.07099755695641163, 0}, {-0.070997556956412, -0.41562693777745296`, 
            0.48662449473386504`, 
            0.371748034460185}, {-0.25687157418650397`, -0.3007504775037725, \
-0.1858740172300929, 0.6015009550075466}, {0, 0, 0, 0}}, {{
            0.1858740172300929, 0.7163774152812263, 0.07099755695641163, 0}, {
            0.11487646027368059`, 0.30075047750377315`, 0.5576220516902766, 
            0.37174803446018445`}, {-0.0709975569564111, 
            0.4156269377774537, -0.11487646027368148`, 0.6015009550075457}, {
            0, 0, 0, 0}}}, {{{0.15412294451190178`, 
            0.6407474392457679, -0.23724833206739238`, -0.2493761626664725}, \
{-0.08312538755549057, 
            0.10274862967460166`, -0.6921217540830682, \
-0.23724833206739313`}, {-0.09525321815456972, 
            0.4352501798965643, -0.4548734220156748, 0.3838758650592643}, {0, 
            0, 0, 0}}, {{0.2493761626664716, 0.20549725934920382`, 
            0.21762508994828272`, -0.6332520277257367}, {
            0.01212783059907925, -0.33250155022196304`, -0.2372483320673934, \
-0.6211241971266576}, {0.09525321815456972, -0.4352501798965643, 
            0.4548734220156748, -0.3838758650592643}, {0, 0, 0, 0}}, {{
            0.2372483320673922, 0.5379988095711666, 
            0.45487342201567577`, -0.0121278305990793}, {-0.01212783059907925,
             0.33250155022196304`, 0.2372483320673934, 0.6211241971266576}, {
            0.08312538755549057, -0.10274862967460166`, 0.6921217540830682, 
            0.23724833206739313`}, {0, 0, 0, 
            0}}, {{-0.2372483320673922, -0.5379988095711666, \
-0.45487342201567577`, 
            0.0121278305990793}, {-0.2493761626664716, -0.20549725934920382`, \
-0.21762508994828272`, 
            0.6332520277257367}, {-0.15412294451190178`, -0.6407474392457679, 
            0.23724833206739238`, 0.2493761626664725}, {0, 0, 0, 
            0}}}, {{{-0.06350214543637983, 
            0.2886226469046934, -0.6724985119639579, -0.11487646027368117`}, \
{-0.10274862967460227`, -0.36425262294015354`, -0.4866244947338658, \
-0.4156269377774542}, {-0.2689994047855835, -0.2811272353846622, \
-0.5576220516902763, 0.30075047750377315`}, {0, 0, 0, 0}}, {{
            0.2054972593492035, 
            0.5697498822893557, -0.11487646027368173`, \
-0.41562693777745424`}, {0.16625077511098102`, -0.08312538755549123, 
            0.07099755695641052, -0.7163774152812272}, {0.2689994047855835, 
            0.2811272353846622, 0.5576220516902763, -0.30075047750377315`}, {
            0, 0, 0, 0}}, {{-0.039246484238222745`, -0.6528752698448469, 
            0.18587401723009217`, -0.30075047750377293`}, \
{-0.2054972593492035, -0.5697498822893557, 0.11487646027368173`, 
            0.41562693777745424`}, {0.06350214543637983, -0.2886226469046934, 
            0.6724985119639579, 0.11487646027368117`}, {0, 0, 0, 0}}, {{
            0.039246484238222745`, 0.6528752698448469, -0.18587401723009217`, 
            0.30075047750377293`}, {-0.16625077511098102`, 
            0.08312538755549123, -0.07099755695641052, 0.7163774152812272}, {
            0.10274862967460227`, 0.36425262294015354`, 0.4866244947338658, 
            0.4156269377774542}, {0, 0, 0, 0}}}, {{{0.07099755695641108, 
            0.3717480344601848, 0.4156269377774543, 
            0.4866244947338646}, {-0.18587401723009228`, \
-0.22975292054736146`, 0.11487646027368105`, 
            0.6724985119639573}, {-0.11487646027368004`, 
            0.3717480344601847, -0.30075047750377276`, 0.5576220516902768}, {
            0, 0, 0, 0}}, {{
            0.18587401723009153`, 0, 
             0.7163774152812272, -0.07099755695641181}, \
{-0.07099755695641183, -0.6015009550075464, 0.4156269377774541, 
            0.11487646027368062`}, {0.11487646027368004`, -0.3717480344601847,
             0.30075047750377276`, -0.5576220516902768}, {0, 0, 0, 0}}, {{
            0.2568715741865035, 0.6015009550075462, 
            0.3007504775037733, -0.1858740172300925}, {0.07099755695641183, 
            0.6015009550075464, -0.4156269377774541, -0.11487646027368062`}, {
            0.18587401723009228`, 
            0.22975292054736146`, -0.11487646027368105`, \
-0.6724985119639573}, {0, 0, 0, 
            0}}, {{-0.2568715741865035, -0.6015009550075462, \
-0.3007504775037733, 
            0.1858740172300925}, {-0.18587401723009153`, 
             0, -0.7163774152812272, 
             0.07099755695641181}, {-0.07099755695641108, \
-0.3717480344601848, -0.4156269377774543, -0.4866244947338646}, {0, 0, 0, 
            0}}}, {{{-0.2297529205473613, -0.11487646027368142`, \
-0.18587401723009175`, 
            0.6724985119639576}, {-0.22975292054736182`, -0.6724985119639579, \
-0.11487646027368048`, 
            0.18587401723009228`}, {-0.22975292054736118`, \
-0.1858740172300923, -0.6724985119639574, 0.11487646027368047`}, {0, 0, 0, 
            0}}, {{0, 0.07099755695641077, 0.4866244947338656, 
             0.5576220516902775}, {
            0, -0.4866244947338658, 0.557622051690277, 0.07099755695641177}, {
            0.22975292054736118`, 0.1858740172300923, 
            0.6724985119639574, -0.11487646027368047`}, {0, 0, 0, 0}}, {{
            0, -0.5576220516902765, 
             0.07099755695641172, -0.4866244947338657}, {
            0, -0.07099755695641077, -0.4866244947338656, \
-0.5576220516902775}, {0.2297529205473613, 0.11487646027368142`, 
            0.18587401723009175`, -0.6724985119639576}, {0, 0, 0, 0}}, {{
            0, 0.5576220516902765, -0.07099755695641172, 
             0.4866244947338657}, {
            0, 0.4866244947338658, -0.557622051690277, -0.07099755695641177}, \
{0.22975292054736182`, 0.6724985119639579, 
            0.11487646027368048`, -0.18587401723009228`}, {0, 0, 0, 0}}}, {{{
            0.12843578709325124`, 0.7079972904421639, 0.02193945165863498, 
            0.18587401723009195`}, {0.26525169902556694`, 
            0.46468504307523173`, 
            0.17231469041052141`, -0.48662449473386526`}, {0.2078134688887268,
             0.3143098043233454, 0.6369997334857525, 0.07099755695641119}, {0,
             0, 0, 0}}, {{-0.13681591193231557`, 
            0.2433122473669326, -0.15037523875188685`, 
            0.6724985119639573}, {-0.0574382301368401, -0.15037523875188616`, 
            0.4646850430752307, 
            0.5576220516902766}, {-0.26525169902556694`, \
-0.46468504307523173`, -0.17231469041052141`, 0.48662449473386526`}, {0, 0, 0,
             0}}, {{0.13681591193231557`, -0.2433122473669326, 
            0.15037523875188685`, -0.6724985119639573}, {
            0.07937768179547577, -0.3936874861188185, 
            0.6150602818271174, -0.11487646027368086`}, \
{-0.12843578709325124`, -0.7079972904421639, -0.02193945165863498, \
-0.18587401723009195`}, {0, 0, 0, 0}}, {{-0.07937768179547577, 
            0.3936874861188185, -0.6150602818271174, 0.11487646027368086`}, {
            0.0574382301368401, 
            0.15037523875188616`, -0.4646850430752307, -0.5576220516902766}, \
{-0.2078134688887268, -0.3143098043233454, -0.6369997334857525, \
-0.07099755695641119}, {0, 0, 0, 
            0}}}}, {{{{-0.2568715741865041, -0.3007504775037746, \
-0.18587401723009284`, 
            0.6015009550075452}, {-0.07099755695641219, -0.41562693777745435`,
             0.48662449473386493`, 
            0.37174803446018384`}, {-0.18587401723009295`, \
-0.7163774152812282, -0.07099755695641208, 0}, {0, 0, 0, 
            0}}, {{-0.18587401723009198`, 
            0.11487646027367975`, -0.6724985119639572, 0.2297529205473613}, {
            0.07099755695641219, 
            0.41562693777745435`, -0.48662449473386493`, \
-0.37174803446018384`}, {-0.11487646027368092`, -0.30075047750377365`, \
-0.5576220516902767, -0.37174803446018456`}, {0, 0, 0, 0}}, {{
            0.18587401723009198`, -0.11487646027367975`, 
            0.6724985119639572, -0.2297529205473613}, {0.2568715741865041, 
            0.3007504775037746, 0.18587401723009284`, -0.6015009550075452}, {
            0.07099755695641097, -0.41562693777745335`, 
            0.1148764602736805, -0.6015009550075459}, {0, 0, 0, 
            0}}, {{-0.07099755695641097, 
            0.41562693777745335`, -0.1148764602736805, 0.6015009550075459}, {
            0.11487646027368092`, 0.30075047750377365`, 0.5576220516902767, 
            0.37174803446018456`}, {
            0.18587401723009295`, 0.7163774152812282, 0.07099755695641208, 
             0}, {0, 0, 0, 
            0}}}, {{{-0.27881102584513817`, -0.3362492559819802, \
-0.371748034460185, 
            0.4730651679142943}, {-0.10649633543461651`, 0.2433122473669313, 
             0, 0.6944379636225928}, {-0.12843578709325149`, 
            0.3226899291624069, -0.6015009550075454, 0.26525169902556744`}, {
            0, 0, 0, 0}}, {{-0.1503752387518869, -0.6589391851443873, 
            0.22975292054736052`, 0.2078134688887267}, {
            0.021939451658634784`, -0.07937768179547589, 0.6015009550075453, 
            0.42918626459702525`}, {0.12843578709325149`, -0.3226899291624069,
             0.6015009550075454, -0.26525169902556744`}, {0, 0, 0, 
            0}}, {{-0.17231469041052205`, -0.5795615033489117, \
-0.37174803446018456`, -0.22137279570829874`}, {-0.021939451658634784`, 
            0.07937768179547589, -0.6015009550075453, -0.42918626459702525`}, \
{0.10649633543461651`, -0.2433122473669313, 0, -0.6944379636225928}, {0, 0, 0,
             0}}, {{0.17231469041052205`, 0.5795615033489117, 
            0.37174803446018456`, 0.22137279570829874`}, {0.1503752387518869, 
            0.6589391851443873, -0.22975292054736052`, -0.2078134688887267}, {
            0.27881102584513817`, 0.3362492559819802, 
            0.371748034460185, -0.4730651679142943}, {0, 0, 0, 
            0}}}, {{{-0.2297529205473616, -0.6724985119639584, \
-0.11487646027368088`, 
            0.18587401723009273`}, {-0.22975292054736257`, \
-0.11487646027368024`, -0.1858740172300932, 
            0.6724985119639582}, {-0.2297529205473624, -0.18587401723009278`, \
-0.6724985119639577, 0.11487646027368065`}, {0, 0, 0, 0}}, {{
            0, -0.48662449473386565`, 0.5576220516902767, 
             0.07099755695641133}, {
            0, 0.07099755695641209, 0.4866244947338646, 0.5576220516902777}, {
            0.2297529205473624, 0.18587401723009278`, 
            0.6724985119639577, -0.11487646027368065`}, {0, 0, 0, 0}}, {{
            0, 0.5576220516902781, -0.07099755695641205, 
             0.48662449473386615`}, {
            0, 0.48662449473386565`, -0.5576220516902767, \
-0.07099755695641133}, {0.2297529205473616, 0.6724985119639584, 
            0.11487646027368088`, -0.18587401723009273`}, {0, 0, 0, 0}}, {{
            0, -0.5576220516902781, 
             0.07099755695641205, -0.48662449473386615`}, {
            0, -0.07099755695641209, -0.4866244947338646, \
-0.5576220516902777}, {0.22975292054736257`, 0.11487646027368024`, 
            0.1858740172300932, -0.6724985119639582}, {0, 0, 0, 
            0}}}, {{{-0.2689994047855844, -0.5576220516902773, \
-0.30075047750377365`, 
            0.28112723538466133`}, {-0.10274862967460238`, \
-0.48662449473386643`, 0.4156269377774542, 
            0.3642526229401525}, {-0.06350214543638152, -0.672498511963958, 
            0.11487646027368004`, -0.2886226469046943}, {0, 0, 0, 
            0}}, {{-0.20549725934920274`, 
            0.11487646027368112`, -0.415626937777454, 
            0.5697498822893556}, {-0.03924648423822108, 0.1858740172300912, 
            0.3007504775037739, 0.6528752698448468}, {0.06350214543638152, 
            0.672498511963958, -0.11487646027368004`, 0.2886226469046943}, {0,
             0, 0, 0}}, {{0.16625077511098169`, 0.07099755695641047, 
            0.7163774152812281, 0.08312538755549098}, {
            0.20549725934920274`, -0.11487646027368112`, 
            0.415626937777454, -0.5697498822893556}, {0.2689994047855844, 
            0.5576220516902773, 
            0.30075047750377365`, -0.28112723538466133`}, {0, 0, 0, 
            0}}, {{-0.16625077511098169`, -0.07099755695641047, \
-0.7163774152812281, -0.08312538755549098}, {
            0.03924648423822108, -0.1858740172300912, -0.3007504775037739, \
-0.6528752698448468}, {0.10274862967460238`, 
            0.48662449473386643`, -0.4156269377774542, -0.3642526229401525}, {
            0, 0, 0, 
            0}}}, {{{-0.18587401723009223`, -0.07099755695641136, 0, 
             0.7163774152812272}, {-0.2568715741865028, \
-0.18587401723009295`, -0.6015009550075456, 
            0.3007504775037736}, {-0.07099755695641176, 
            0.48662449473386526`, -0.3717480344601843, 0.4156269377774527}, {
            0, 0, 0, 0}}, {{-0.11487646027368052`, -0.557622051690277, 
            0.371748034460186, 
            0.30075047750377387`}, {-0.18587401723009112`, \
-0.6724985119639582, -0.22975292054736124`, -0.11487646027367907`}, {
            0.07099755695641176, -0.48662449473386526`, 
            0.3717480344601843, -0.4156269377774527}, {0, 0, 0, 0}}, {{
            0.07099755695641086, 0.11487646027368142`, 0.6015009550075469, 
            0.41562693777745363`}, {0.18587401723009112`, 0.6724985119639582, 
            0.22975292054736124`, 0.11487646027367907`}, {0.2568715741865028, 
            0.18587401723009295`, 0.6015009550075456, -0.3007504775037736}, {
            0, 0, 0, 
            0}}, {{-0.07099755695641086, -0.11487646027368142`, \
-0.6015009550075469, -0.41562693777745363`}, {0.11487646027368052`, 
            0.557622051690277, -0.371748034460186, -0.30075047750377387`}, {
            0.18587401723009223`, 0.07099755695641136, 
             0, -0.7163774152812272}, {0, 0, 0, 
            0}}}, {{{-0.27881102584513845`, -0.3143098043233452, \
-0.45112571625565945`, 
            0.4156269377774542}, {-0.17231469041052117`, -0.7079972904421643, 
            0.09293700861504628, 
            0.11487646027368081`}, {-0.1503752387518866, \
-0.46468504307523156`, -0.47306516791429487`, -0.3007504775037738}, {0, 0, 0, 
            0}}, {{-0.12843578709325182`, 0.15037523875188658`, 
            0.021939451658635145`, 
            0.7163774152812276}, {-0.021939451658634528`, \
-0.24331224736693274`, 0.5660021765293404, 0.41562693777745474`}, {
            0.1503752387518866, 0.46468504307523156`, 0.47306516791429487`, 
            0.3007504775037738}, {0, 0, 0, 0}}, {{-0.1064963354346173, 
            0.39368748611881893`, -0.5440627248707051, 
            0.30075047750377293`}, {0.021939451658634528`, 
            0.24331224736693274`, -0.5660021765293404, \
-0.41562693777745474`}, {0.17231469041052117`, 
            0.7079972904421643, -0.09293700861504628, -0.11487646027368081`}, \
{0, 0, 0, 0}}, {{0.1064963354346173, -0.39368748611881893`, 
            0.5440627248707051, -0.30075047750377293`}, {
            0.12843578709325182`, -0.15037523875188658`, \
-0.021939451658635145`, -0.7163774152812276}, {0.27881102584513845`, 
            0.3143098043233452, 0.45112571625565945`, -0.4156269377774542}, {
            0, 0, 0, 
            0}}}, {{{-0.2568715741865045, -0.5795615033489117, \
-0.15037523875188583`, 
            0.35818870764061317`}, {-0.18587401723009211`, \
-0.3362492559819803, -0.615060281827118, -0.1639345655714578}, \
{-0.07099755695641252, -0.6589391851443873, 
            0.021939451658633646`, -0.33624925598197897`}, {0, 0, 0, 
            0}}, {{-0.1858740172300922, 
            0.07937768179547594, -0.17231469041051975`, 
            0.694437963622592}, {-0.11487646027367993`, 
            0.32268992916240674`, -0.6369997334857516, 0.172314690410521}, {
            0.07099755695641252, 0.6589391851443873, -0.021939451658633646`, 
            0.33624925598197897`}, {0, 0, 0, 0}}, {{0.07099755695641233, 
            0.243312247366931, -0.46468504307523173`, -0.5221232732120709}, {
            0.1858740172300922, -0.07937768179547594, 
            0.17231469041051975`, -0.694437963622592}, {0.2568715741865045, 
            0.5795615033489117, 
            0.15037523875188583`, -0.35818870764061317`}, {0, 0, 0, 
            0}}, {{-0.07099755695641233, -0.243312247366931, 
            0.46468504307523173`, 0.5221232732120709}, {
            0.11487646027367993`, -0.32268992916240674`, 
            0.6369997334857516, -0.172314690410521}, {0.18587401723009211`, 
            0.3362492559819803, 0.615060281827118, 0.1639345655714578}, {0, 0,
             0, 0}}}, {{{-0.25687157418650447`, -0.1858740172300925, \
-0.6015009550075461, 
            0.30075047750377254`}, {-0.18587401723009173`, \
-0.07099755695641193, 0, 0.7163774152812266}, {-0.07099755695641205, 
            0.4866244947338661, -0.3717480344601846, 0.4156269377774531}, {0, 
            0, 0, 0}}, {{-0.18587401723009253`, -0.6724985119639585, \
-0.22975292054736168`, -0.11487646027368084`}, {-0.11487646027368006`, \
-0.5576220516902781, 0.37174803446018556`, 0.3007504775037735}, {
            0.07099755695641205, -0.4866244947338661, 
            0.3717480344601846, -0.4156269377774531}, {0, 0, 0, 
            0}}, {{-0.07099755695641302, -0.11487646027368044`, \
-0.6015009550075467, -0.4156269377774541}, {0.11487646027368006`, 
            0.5576220516902781, -0.37174803446018556`, -0.3007504775037735}, {
            0.18587401723009173`, 0.07099755695641193, 
             0, -0.7163774152812266}, {0, 0, 0, 0}}, {{0.07099755695641302, 
            0.11487646027368044`, 0.6015009550075467, 0.4156269377774541}, {
            0.18587401723009253`, 0.6724985119639585, 0.22975292054736168`, 
            0.11487646027368084`}, {0.25687157418650447`, 0.1858740172300925, 
            0.6015009550075461, -0.30075047750377254`}, {0, 0, 0, 
            0}}}, {{{-0.25687157418650364`, -0.30075047750377343`, \
-0.18587401723009292`, 
            0.6015009550075459}, {-0.18587401723009253`, -0.7163774152812276, \
-0.07099755695641179, 0}, {-0.07099755695641227, -0.4156269377774536, 
            0.486624494733866, 0.37174803446018373`}, {0, 0, 0, 
            0}}, {{-0.18587401723009161`, 
            0.11487646027368068`, -0.6724985119639589, 
            0.22975292054736204`}, {-0.11487646027368104`, \
-0.3007504775037736, -0.5576220516902777, -0.371748034460184}, {
            0.07099755695641227, 
            0.4156269377774536, -0.486624494733866, -0.37174803446018373`}, {
            0, 0, 0, 0}}, {{-0.07099755695641097, 
            0.4156269377774541, -0.11487646027368142`, 0.6015009550075459}, {
            0.11487646027368104`, 0.3007504775037736, 0.5576220516902777, 
            0.371748034460184}, {
            0.18587401723009253`, 0.7163774152812276, 0.07099755695641179, 
             0}, {0, 0, 0, 0}}, {{0.07099755695641097, -0.4156269377774541, 
            0.11487646027368142`, -0.6015009550075459}, {
            0.18587401723009161`, -0.11487646027368068`, 
            0.6724985119639589, -0.22975292054736204`}, {0.25687157418650364`,
             0.30075047750377343`, 
            0.18587401723009292`, -0.6015009550075459}, {0, 0, 0, 
            0}}}, {{{-0.12237187179371406`, -0.6724985119639577, 
            0.2689994047855827, 
            0.11487646027368055`}, {-0.15412294451190267`, \
-0.1858740172300915, 0.20549725934920265`, 0.672498511963958}, {
            0.09525321815456965, -0.11487646027368131`, 0.7042495846821476, 
            0.18587401723009117`}, {0, 0, 0, 
            0}}, {{-0.2176250899482841, -0.5576220516902769, \
-0.43525017989656495`, -0.07099755695641098}, {-0.24937616266647275`, \
-0.07099755695641033, -0.49875232533294495`, 
            0.48662449473386704`}, {-0.09525321815456965, 
            0.11487646027368131`, -0.7042495846821476, \
-0.18587401723009117`}, {0, 0, 0, 0}}, {{
            0.03175107271818897, -0.4866244947338663, 
            0.06350214543637972, -0.5576220516902777}, {0.24937616266647275`, 
            0.07099755695641033, 
            0.49875232533294495`, -0.48662449473386704`}, {
            0.15412294451190267`, 
            0.1858740172300915, -0.20549725934920265`, -0.672498511963958}, {
            0, 0, 0, 0}}, {{-0.03175107271818897, 
            0.4866244947338663, -0.06350214543637972, 0.5576220516902777}, {
            0.2176250899482841, 0.5576220516902769, 0.43525017989656495`, 
            0.07099755695641098}, {0.12237187179371406`, 
            0.6724985119639577, -0.2689994047855827, -0.11487646027368055`}, {
            0, 0, 0, 0}}}, {{{-0.19491356844314014`, -0.5102903470526696, 
            0.16220816491128837`, 
            0.4775849435208174}, {-0.2624583240792139, -0.4573718925223528, \
-0.5012507958396216, 0.15316861369824072`}, {-0.20953986954889692`, 
            0.05291845453031763, -0.2917109262907251, 0.6488326596451532}, {0,
             0, 0, 0}}, {{0.014626301105756932`, -0.5632088015829873, 
            0.45391909120201307`, -0.17124771612433604`}, \
{-0.052918454530317155`, -0.5102903470526698, -0.20953986954889675`, \
-0.4956640459469125}, {0.20953986954889692`, -0.05291845453031763, 
            0.2917109262907251, -0.6488326596451532}, {0, 0, 0, 0}}, {{
            0.06754475563607366, -0.052918454530317544`, 0.6634589607509099, 
            0.3244163298225764}, {0.052918454530317155`, 0.5102903470526698, 
            0.20953986954889675`, 0.4956640459469125}, {0.2624583240792139, 
            0.4573718925223528, 0.5012507958396216, -0.15316861369824072`}, {
            0, 0, 0, 0}}, {{-0.06754475563607366, 
            0.052918454530317544`, -0.6634589607509099, -0.3244163298225764}, \
{-0.014626301105756932`, 0.5632088015829873, -0.45391909120201307`, 
            0.17124771612433604`}, {0.19491356844314014`, 
            0.5102903470526696, -0.16220816491128837`, -0.4775849435208174}, {
            0, 0, 0, 0}}}, {{{0.2297529205473601, 0.11487646027368137`, 
            0.18587401723009223`, -0.6724985119639584}, {
            0, -0.07099755695641166, -0.4866244947338653, \
-0.5576220516902777}, {
            0, -0.557622051690277, 
             0.07099755695641168, -0.4866244947338666}, {0, 0, 0, 0}}, {{
            0.22975292054735977`, 0.6724985119639584, 
            0.11487646027368069`, -0.1858740172300919}, {
            0, 0.48662449473386515`, -0.5576220516902763, \
-0.07099755695641088}, {
            0, 0.557622051690277, -0.07099755695641168, 0.4866244947338666}, {
            0, 0, 0, 0}}, {{0.2297529205473602, 0.18587401723009295`, 
            0.6724985119639575, -0.11487646027368122`}, {
            0, -0.48662449473386515`, 0.5576220516902763, 
             0.07099755695641088}, {
            0, 0.07099755695641166, 0.4866244947338653, 0.5576220516902777}, {
            0, 0, 0, 
            0}}, {{-0.2297529205473602, -0.18587401723009295`, \
-0.6724985119639575, 
            0.11487646027368122`}, {-0.22975292054735977`, \
-0.6724985119639584, -0.11487646027368069`, 
            0.1858740172300919}, {-0.2297529205473601, -0.11487646027368137`, \
-0.18587401723009223`, 0.6724985119639584}, {0, 0, 0, 
            0}}}}, {{{{-0.11487646027368047`, -0.5576220516902781, 
            0.3717480344601834, 0.30075047750377265`}, {
            0.07099755695641126, -0.4866244947338662, 
            0.37174803446018334`, -0.4156269377774534}, \
{-0.18587401723009267`, -0.6724985119639589, -0.2297529205473625, \
-0.11487646027368092`}, {0, 0, 0, 
            0}}, {{-0.18587401723009211`, -0.07099755695641155, 0, 
             0.716377415281226}, {-0.07099755695641126, 
            0.4866244947338662, -0.37174803446018334`, 
            0.4156269377774534}, {-0.25687157418650375`, \
-0.18587401723009261`, -0.601500955007546, 0.300750477503773}, {0, 0, 0, 
            0}}, {{0.18587401723009211`, 0.07099755695641155, 
             0, -0.716377415281226}, {0.11487646027368047`, 
            0.5576220516902781, -0.3717480344601834, -0.30075047750377265`}, \
{-0.07099755695641187, -0.1148764602736807, -0.6015009550075462, \
-0.41562693777745296`}, {0, 0, 0, 0}}, {{0.07099755695641187, 
            0.1148764602736807, 0.6015009550075462, 0.41562693777745296`}, {
            0.25687157418650375`, 0.18587401723009261`, 
            0.601500955007546, -0.300750477503773}, {0.18587401723009267`, 
            0.6724985119639589, 0.2297529205473625, 0.11487646027368092`}, {0,
             0, 0, 0}}}, {{{-0.24937616266647236`, -0.6015009550075471, \
-0.1148764602736812, 
            0.3399969617419946}, {-0.0317510727181884, -0.37174803446018584`, 
            0.5576220516902767, 
            0.3203737196228841}, {-0.15412294451190234`, 0, 
             0.0709975569564116, 0.7238728268012578}, {0, 0, 0, 
            0}}, {{-0.09525321815457004, -0.6015009550075473, \
-0.18587401723009306`, -0.38387586505926313`}, {
            0.12237187179371357`, -0.37174803446018595`, 
            0.4866244947338646, -0.4034991071783737}, {
            0.15412294451190234`, 
             0, -0.0709975569564116, -0.7238728268012578}, {0, 0, 0, 0}}, {{
            0.21762508994828417`, 0.22975292054736107`, 
            0.6724985119639579, -0.019623242119110346`}, {0.09525321815457004,
             0.6015009550075473, 0.18587401723009306`, 
            0.38387586505926313`}, {0.24937616266647236`, 0.6015009550075471, 
            0.1148764602736812, -0.3399969617419946}, {0, 0, 0, 
            0}}, {{-0.21762508994828417`, -0.22975292054736107`, \
-0.6724985119639579, 0.019623242119110346`}, {-0.12237187179371357`, 
            0.37174803446018595`, -0.4866244947338646, 0.4034991071783737}, {
            0.0317510727181884, 
            0.37174803446018584`, -0.5576220516902767, -0.3203737196228841}, {
            0, 0, 0, 0}}}, {{{
            0.07099755695641094, -0.02193945165863259, -0.33624925598197936`, \
-0.6589391851443879}, {-0.11487646027368237`, -0.6369997334857521, \
-0.17231469041052191`, -0.32268992916240813`}, {-0.18587401723009317`, \
-0.17231469041051994`, -0.6944379636225926, -0.07937768179547672}, {0, 0, 0, 
            0}}, {{0.2568715741865039, 0.15037523875188705`, 
            0.35818870764061317`, -0.5795615033489114}, {
            0.0709975569564108, -0.46468504307523173`, 
            0.5221232732120703, -0.24331224736693194`}, {0.18587401723009317`,
             0.17231469041051994`, 0.6944379636225926, 0.07937768179547672}, {
            0, 0, 0, 0}}, {{-0.18587401723009306`, -0.6150602818271192, 
            0.16393456557145708`, 
            0.33624925598197913`}, {-0.2568715741865039, \
-0.15037523875188705`, -0.35818870764061317`, 
            0.5795615033489114}, {-0.07099755695641094, 0.02193945165863259, 
            0.33624925598197936`, 0.6589391851443879}, {0, 0, 0, 0}}, {{
            0.18587401723009306`, 
            0.6150602818271192, -0.16393456557145708`, \
-0.33624925598197913`}, {-0.0709975569564108, 
            0.46468504307523173`, -0.5221232732120703, 
            0.24331224736693194`}, {0.11487646027368237`, 0.6369997334857521, 
            0.17231469041052191`, 0.32268992916240813`}, {0, 0, 0, 
            0}}}, {{{-0.22975292054736282`, -0.11487646027367873`, \
-0.18587401723009264`, 
            0.6724985119639584}, {-0.229752920547363, -0.6724985119639578, \
-0.11487646027368137`, 
            0.18587401723009236`}, {-0.22975292054736338`, \
-0.18587401723009125`, -0.6724985119639576, 0.11487646027368192`}, {0, 0, 0, 
            0}}, {{0, 0.07099755695641216, 0.48662449473386526`, 
             0.5576220516902763}, {
            0, -0.4866244947338664, 0.5576220516902761, 
             0.07099755695641066}, {0.22975292054736338`, 
            0.18587401723009125`, 
            0.6724985119639576, -0.11487646027368192`}, {0, 0, 0, 0}}, {{
            0, -0.5576220516902787, 
             0.07099755695641077, -0.486624494733866}, {
            0, -0.07099755695641216, -0.48662449473386526`, \
-0.5576220516902763}, {0.22975292054736282`, 0.11487646027367873`, 
            0.18587401723009264`, -0.6724985119639584}, {0, 0, 0, 0}}, {{
            0, 0.5576220516902787, -0.07099755695641077, 0.486624494733866}, {
            0, 0.4866244947338664, -0.5576220516902761, \
-0.07099755695641066}, {0.229752920547363, 0.6724985119639578, 
            0.11487646027368137`, -0.18587401723009236`}, {0, 0, 0, 
            0}}}, {{{-0.11487646027367966`, 
            0.37174803446018684`, -0.3007504775037726, 0.5576220516902772}, {
            0.07099755695641309, 0.37174803446018345`, 0.4156269377774546, 
            0.4866244947338648}, {-0.18587401723009234`, -0.2297529205473615, 
            0.11487646027368006`, 0.6724985119639569}, {0, 0, 0, 
            0}}, {{-0.18587401723009328`, 0, -0.716377415281227, 
             0.07099755695641224}, {-0.25687157418650547`, \
-0.6015009550075449, -0.3007504775037741, 
            0.1858740172300919}, {-0.07099755695641309, \
-0.37174803446018345`, -0.4156269377774546, -0.4866244947338648}, {0, 0, 0, 
            0}}, {{0.18587401723009328`, 0, 
             0.716377415281227, -0.07099755695641224}, {-0.07099755695641255, \
-0.6015009550075485, 0.41562693777745285`, 0.11487646027367954`}, {
            0.11487646027367966`, -0.37174803446018684`, 
            0.3007504775037726, -0.5576220516902772}, {0, 0, 0, 0}}, {{
            0.07099755695641255, 
            0.6015009550075485, -0.41562693777745285`, \
-0.11487646027367954`}, {0.25687157418650547`, 0.6015009550075449, 
            0.3007504775037741, -0.1858740172300919}, {0.18587401723009234`, 
            0.2297529205473615, -0.11487646027368006`, -0.6724985119639569}, {
            0, 0, 0, 0}}}, {{{-0.20781346888872704`, -0.6369997334857531, 
            0.07099755695641177, 
            0.314309804323344}, {-0.07937768179547508, -0.6150602818271188, \
-0.11487646027368059`, -0.3936874861188187}, {
            0.05743823013684016, -0.4646850430752324, 
            0.5576220516902783, -0.1503752387518869}, {0, 0, 0, 
            0}}, {{-0.12843578709325182`, -0.021939451658634562`, 
            0.18587401723009234`, 0.7079972904421628}, {0.07937768179547508, 
            0.6150602818271188, 0.11487646027368059`, 0.3936874861188187}, {
            0.1368159119323153, 0.15037523875188633`, 0.6724985119639597, 
            0.24331224736693205`}, {0, 0, 0, 0}}, {{0.12843578709325182`, 
            0.021939451658634562`, -0.18587401723009234`, \
-0.7079972904421628}, {0.20781346888872704`, 
            0.6369997334857531, -0.07099755695641177, -0.314309804323344}, {
            0.265251699025567, 0.17231469041052072`, 
            0.4866244947338671, -0.4646850430752307}, {0, 0, 0, 
            0}}, {{-0.265251699025567, -0.17231469041052072`, \
-0.4866244947338671, 
            0.4646850430752307}, {-0.1368159119323153, -0.15037523875188633`, \
-0.6724985119639597, -0.24331224736693205`}, {-0.05743823013684016, 
            0.4646850430752324, -0.5576220516902783, 0.1503752387518869}, {0, 
            0, 0, 0}}}, {{{-0.06350214543638169, -0.6332520277257363, 
            0.38387586505926363`, 
            0.019623242119110353`}, {-0.26899940478558326`, \
-0.5501266401702464, -0.2493761626664728, 
            0.3399969617419946}, {-0.10274862967460212`, -0.5818777128884364, \
-0.23724833206739418`, -0.38387586505926313`}, {0, 0, 0, 0}}, {{
            0.039246484238221024`, -0.05137431483730061, 0.6211241971266575, 
            0.40349910717837345`}, {-0.1662507751109813, 
            0.03175107271818983, -0.012127830599078923`, 
            0.7238728268012578}, {0.10274862967460212`, 0.5818777128884364, 
            0.23724833206739418`, 0.38387586505926313`}, {0, 0, 0, 
            0}}, {{-0.20549725934920182`, 
            0.08312538755549038, -0.6332520277257362, 
            0.32037371962288436`}, {-0.039246484238221024`, 
            0.05137431483730061, -0.6211241971266575, -0.40349910717837345`}, \
{0.06350214543638169, 
            0.6332520277257363, -0.38387586505926363`, \
-0.019623242119110353`}, {0, 0, 0, 0}}, {{
            0.20549725934920182`, -0.08312538755549038, 
            0.6332520277257362, -0.32037371962288436`}, {
            0.1662507751109813, -0.03175107271818983, 
            0.012127830599078923`, -0.7238728268012578}, {
            0.26899940478558326`, 0.5501266401702464, 
            0.2493761626664728, -0.3399969617419946}, {0, 0, 0, 
            0}}}, {{{-0.021939451658635645`, -0.6015009550075457, 
            0.42918626459702547`, -0.07937768179547633}, {
            0.10649633543461795`, 0, 0.6944379636225921, 
             0.24331224736693233`}, {-0.17231469041052125`, \
-0.37174803446018523`, 0.2213727957082969, 0.5795615033489119}, {0, 0, 0, 
            0}}, {{-0.12843578709325376`, -0.6015009550075466, \
-0.2652516990255671, -0.32268992916240824`}, {-0.27881102584513917`, \
-0.371748034460186, -0.47306516791429526`, 
            0.33624925598197924`}, {-0.10649633543461795`, 
             0, -0.6944379636225921, -0.24331224736693233`}, {0, 0, 0, 0}}, {{
            0.12843578709325376`, 0.6015009550075466, 0.2652516990255671, 
            0.32268992916240824`}, {-0.15037523875188574`, 
            0.22975292054736057`, -0.20781346888872845`, 
            0.6589391851443882}, {0.021939451658635645`, 
            0.6015009550075457, -0.42918626459702547`, 0.07937768179547633}, {
            0, 0, 0, 0}}, {{0.15037523875188574`, -0.22975292054736057`, 
            0.20781346888872845`, -0.6589391851443882}, {0.27881102584513917`,
             0.371748034460186, 
            0.47306516791429526`, -0.33624925598197924`}, {
            0.17231469041052125`, 
            0.37174803446018523`, -0.2213727957082969, -0.5795615033489119}, {
            0, 0, 0, 
            0}}}, {{{-0.2624583240792116, -0.5012507958396226, \
-0.15316861369824158`, 
            0.45737189252235366`}, {-0.2095398695488965, -0.2917109262907266, \
-0.6488326596451538, -0.052918454530316766`}, {-0.1949135684431386, 
            0.1622081649112886, -0.4775849435208174, 0.5102903470526701}, {0, 
            0, 0, 0}}, {{-0.052918454530315184`, -0.20953986954889608`, 
            0.4956640459469122, 0.5102903470526702}, {0.2095398695488965, 
            0.2917109262907266, 0.6488326596451538, 0.052918454530316766`}, {
            0.014626301105758022`, 0.4539190912020147, 0.17124771612433642`, 
            0.5632088015829868}, {0, 0, 0, 0}}, {{0.052918454530315184`, 
            0.20953986954889608`, -0.4956640459469122, -0.5102903470526702}, {
            0.2624583240792116, 0.5012507958396226, 
            0.15316861369824158`, -0.45737189252235366`}, {
            0.06754475563607341, 0.6634589607509109, -0.32441632982257573`, 
            0.05291845453031667}, {0, 0, 0, 
            0}}, {{-0.06754475563607341, -0.6634589607509109, 
            0.32441632982257573`, -0.05291845453031667}, \
{-0.014626301105758022`, -0.4539190912020147, -0.17124771612433642`, \
-0.5632088015829868}, {0.1949135684431386, -0.1622081649112886, 
            0.4775849435208174, -0.5102903470526701}, {0, 0, 0, 
            0}}}, {{{-0.07099755695641347, -0.6015009550075459, 
            0.4156269377774541, 0.11487646027368108`}, {
            0.18587401723009206`, 0, 
             0.7163774152812269, -0.07099755695641197}, {
            0.11487646027368037`, -0.37174803446018545`, 
            0.3007504775037714, -0.5576220516902775}, {0, 0, 0, 
            0}}, {{-0.25687157418650564`, -0.6015009550075465, \
-0.30075047750377276`, 
            0.18587401723009264`}, {-0.07099755695641205, \
-0.3717480344601861, -0.4156269377774553, -0.4866244947338657}, \
{-0.18587401723009206`, 0, -0.7163774152812269, 0.07099755695641197}, {0, 0, 
            0, 0}}, {{0.25687157418650564`, 0.6015009550075465, 
            0.30075047750377276`, -0.18587401723009264`}, {
            0.18587401723009409`, 
            0.22975292054736013`, -0.11487646027368278`, \
-0.6724985119639584}, {0.07099755695641347, 
            0.6015009550075459, -0.4156269377774541, -0.11487646027368108`}, {
            0, 0, 0, 0}}, {{-0.18587401723009409`, -0.22975292054736013`, 
            0.11487646027368278`, 0.6724985119639584}, {0.07099755695641205, 
            0.3717480344601861, 0.4156269377774553, 
            0.4866244947338657}, {-0.11487646027368037`, 
            0.37174803446018545`, -0.3007504775037714, 0.5576220516902775}, {
            0, 0, 0, 
            0}}}, {{{-0.08312538755548952, -0.5697498822893572, \
-0.20549725934920438`, -0.4231223492974853}, {-0.23724833206739393`, \
-0.28112723538466317`, -0.6407474392457677, 0.08312538755549119}, {
            0.01212783059908043, 
            0.0831253875554907, -0.5379988095711663, -0.5062477368529756}, {0,
             0, 0, 0}}, {{0.15412294451190384`, -0.2886226469046942, 
            0.43525017989656334`, -0.5062477368529763}, {0.23724833206739393`,
             0.28112723538466317`, 
            0.6407474392457677, -0.08312538755549119}, {0.24937616266647444`, 
            0.36425262294015315`, 0.10274862967460167`, -0.589373124408467}, {
            0, 0, 0, 0}}, {{-0.15412294451190384`, 
            0.2886226469046942, -0.43525017989656334`, 0.5062477368529763}, {
            0.08312538755548952, 0.5697498822893572, 0.20549725934920438`, 
            0.4231223492974853}, {0.09525321815457051, 
            0.6528752698448475, -0.3325015502219619, -0.08312538755549086}, {
            0, 0, 0, 0}}, {{-0.09525321815457051, -0.6528752698448475, 
            0.3325015502219619, 
            0.08312538755549086}, {-0.24937616266647444`, \
-0.36425262294015315`, -0.10274862967460167`, 
            0.589373124408467}, {-0.01212783059908043, -0.0831253875554907, 
            0.5379988095711663, 0.5062477368529756}, {0, 0, 0, 
            0}}}, {{{-0.22975292054736246`, -0.6724985119639575, \
-0.11487646027368072`, 
            0.18587401723009161`}, {-0.22975292054736116`, \
-0.11487646027368074`, -0.18587401723009206`, 
            0.672498511963957}, {-0.22975292054736174`, -0.1858740172300938, \
-0.6724985119639589, 0.1148764602736799}, {0, 0, 0, 0}}, {{
            0, -0.5576220516902766, 
             0.07099755695641179, -0.4866244947338653}, {0.22975292054736116`,
             0.11487646027368074`, 
            0.18587401723009206`, -0.672498511963957}, {
            0, -0.0709975569564131, -0.4866244947338666, -0.557622051690277}, \
{0, 0, 0, 0}}, {{
            0, 0.5576220516902766, -0.07099755695641179, 
             0.4866244947338653}, {0.22975292054736246`, 0.6724985119639575, 
            0.11487646027368072`, -0.18587401723009161`}, {
            0, 0.4866244947338637, -0.5576220516902783, -0.0709975569564119}, \
{0, 0, 0, 0}}, {{
            0, -0.4866244947338637, 0.5576220516902783, 0.0709975569564119}, {
            0, 0.0709975569564131, 0.4866244947338666, 0.557622051690277}, {
            0.22975292054736174`, 0.1858740172300938, 
            0.6724985119639589, -0.1148764602736799}, {0, 0, 0, 
            0}}}}}, $CellContext`n4GPrimary3D = CompressedData["
1:eJxTTMoPSmVmYGBggWIQO8T2Q2HCpln2xuo3PgqKPrH/9XOr973Sm/t/Qmh7
NPn9aOpg8vsZcADPpR7v6uwf77+54ot+d8ZZe5Vv01x2nLoO4+9XhfI9oOpg
9sDcAbMfl/kw/VB77G9B7YHZB5W3h5pv/xPVXLh/CJhvD/MH1Hy4fTB/QP1l
j2YuzB84zQcAFd2l0A==
        "], $CellContext`cbctsInV = CompressedData["
1:eJylnAt0VtWxx6NFbLVSRKrYomIpqIjoRWsBPzuK8hJRBLEUUZRiS9Ur+KJo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        "], $CellContext`CbctPrimary3D = {{0.026071078256845695`, 
        0.6275720332643914, -0.3978191127170302}, {0.39781911271703013`, 
        0.026071078256845695`, -0.6275720332643914}, {0.6275720332643914, 
        0.3978191127170302, -0.026071078256845695`}, {-0.6015009550075457, 
        0.2297529205473612, -0.37174803446018445`}, {-0.3717480344601845, 
        0.6015009550075457, 
        0.22975292054736124`}, {-0.2297529205473612, -0.3717480344601845, \
-0.6015009550075456}, {0.2297529205473612, 0.3717480344601845, 
        0.6015009550075456}, {
        0.3717480344601845, -0.6015009550075457, -0.22975292054736124`}, {
        0.6015009550075457, -0.2297529205473612, 
        0.37174803446018445`}, {-0.6275720332643914, -0.3978191127170302, 
        0.026071078256845695`}, {-0.39781911271703013`, \
-0.026071078256845695`, 
        0.6275720332643914}, {-0.026071078256845695`, -0.6275720332643914, 
        0.3978191127170302}}, $CellContext`rotMatrixToDual4D[
         Pattern[$CellContext`rotationMatrix, 
          Blank[]], 
         Optional[
          Pattern[$CellContext`angleFraction, 
           Blank[]], 1]] := 
       Module[{$CellContext`evals, $CellContext`evecs, \
$CellContext`firstCmplxIndex, $CellContext`angles, $CellContext`planeVecs, \
$CellContext`dualMatrix}, If[
           Or[Dimensions[$CellContext`rotationMatrix] != {4, 4}, 
            Not[
             OrthogonalMatrixQ[$CellContext`rotationMatrix]], Sign[
              Det[$CellContext`rotationMatrix]] < 0], 
           Return[
            Message[
             
             MessageName[$CellContext`rotMatrixToDual4D, 
              "NotRotation"]]]]; {$CellContext`evals, $CellContext`evecs} = 
          Chop[
            
            Eigensystem[$CellContext`rotationMatrix]]; \
$CellContext`firstCmplxIndex = Flatten[
            Position[$CellContext`evals, 
             PatternTest[
              Blank[], Im[#] != 0& ]]]; If[$CellContext`firstCmplxIndex == {}, 
           Return[$CellContext`rotationMatrix], $CellContext`firstCmplxIndex = 
           First[$CellContext`firstCmplxIndex]]; {$CellContext`evals, \
$CellContext`evecs} = 
          Map[RotateLeft[#, $CellContext`firstCmplxIndex - 
             1]& , {$CellContext`evals, $CellContext`evecs}]; \
$CellContext`angles = Map[Arg, $CellContext`evals]; If[
           And[Chop[
              Dot[
               Part[$CellContext`evecs, 1], 
               Part[$CellContext`evecs, 1]]] == 0, Round[
              Part[$CellContext`evecs, 1], 10.^(-10)] == Conjugate[
              Round[
               Part[$CellContext`evecs, 2], 
               10.^(-10)]]], $CellContext`planeVecs = Sqrt[2] {
              Im[
               Part[$CellContext`evecs, 1]], 
              Re[
               Part[$CellContext`evecs, 1]]}, 
           Return[
            Print[
            "Hmm...First complex EVec is not null, or not followed by its \
conjugate."]]]; If[Chop[
             Dot[
              Part[$CellContext`evecs, 3], 
              Part[$CellContext`evecs, 3]]] == 
           0, $CellContext`planeVecs = Join[$CellContext`planeVecs, Sqrt[2] {
                Im[
                 Part[$CellContext`evecs, 3]], 
                Re[
                 Part[$CellContext`evecs, 3]]}]; 
           If[Abs[Part[$CellContext`angles, 1] - 
               Part[$CellContext`angles, 3]] < 
             10^(-6), $CellContext`planeVecs = Chop[
               Orthogonalize[$CellContext`planeVecs]]], If[
             Not[
              Element[
               Part[$CellContext`evecs, {3, 4}], Reals]], 
             Part[$CellContext`evecs, {3, 4}] = Re[
                Part[$CellContext`evecs, {3, 4}]] + Im[
                Part[$CellContext`evecs, {3, 4}]]]; $CellContext`planeVecs = 
            Join[$CellContext`planeVecs, 
              Chop[
               Orthogonalize[
                Part[$CellContext`evecs, {3, 4}]]]]; 
           If[Det[$CellContext`planeVecs] < 0, 
             Part[$CellContext`planeVecs, {3, 4}] = 
             Part[$CellContext`planeVecs, {4, 3}]]]; $CellContext`dualMatrix = 
          Chop[
            Dot[
             
             RotationMatrix[$CellContext`angleFraction 
              Part[$CellContext`angles, 3], 
              Part[$CellContext`planeVecs, {1, 2}]], 
             
             RotationMatrix[$CellContext`angleFraction 
              Part[$CellContext`angles, 1], 
              Part[$CellContext`planeVecs, {3, 4}]]]]; 
         Return[$CellContext`dualMatrix]], 
       TagSet[$CellContext`rotMatrixToDual4D, 
        MessageName[$CellContext`rotMatrixToDual4D, "NotRotation"], 
        "Input matrix is not a 4D rotation."], $CellContext`C24cbctRotsToV = \
{{{{1., 0, 0, 0}, {0, 1., 0, 0}, {0, 0, 1., 0}, {0, 0, 0, 1.}}, {{
          0.5, -0.5000000000000002, -0.5000000000000002, 
          0.5000000000000002}, {0.5000000000000002, 
          0.8333333333333333, -0.16666666666666657`, 0.16666666666666652`}, {
          0.5000000000000002, -0.16666666666666657`, 0.8333333333333334, 
          0.16666666666666657`}, {-0.5000000000000002, 0.16666666666666652`, 
          0.16666666666666657`, 0.8333333333333335}}, {{0.5636610018750179, 
          0.5393446629166319, -0.6030056647916497, 
          0.16666666666666696`}, {-0.26967233145831626`, 0.8333333333333337, 
          0.43633899812498267`, -0.20601132958329835`}, {
          0.7060113295832988, -0.06366100187501766, 0.666666666666667, 
          0.23032766854168418`}, {-0.33333333333333354`, 0.10300566479164933`,
           0.03934466291663147, 0.9363389981249826}}, {{
          0.5636610018750178, -0.16666666666666688`, 0.5393446629166319, 
          0.6030056647916495}, {0.3333333333333335, 
          0.9363389981249826, -0.10300566479164924`, 
          0.03934466291663147}, {-0.26967233145831626`, 0.20601132958329837`, 
          0.8333333333333335, -0.43633899812498256`}, {-0.7060113295832988, 
          0.23032766854168416`, 0.06366100187501783, 0.6666666666666667}}, {{
          0.5636610018750178, 0.7060113295832987, 0.3333333333333335, 
          0.2696723314583161}, {-0.6030056647916496, 
          0.666666666666667, -0.03934466291663141, -0.4363389981249827}, \
{-0.1666666666666669, -0.2303276685416841, 
          0.9363389981249823, -0.20601132958329837`}, {-0.5393446629166319, 
          0.06366100187501766, 0.10300566479164927`, 0.8333333333333335}}, {{
          0.0954915028125262, 0.4045084971874738, -0.09549150281252651, 
          0.9045084971874744}, {0.09549150281252614, 0.9045084971874735, 
          0.09549150281252627, -0.40450849718747384`}, {
          0.4045084971874738, -0.09549150281252639, 0.9045084971874737, 
          0.09549150281252614}, {-0.9045084971874744, 0.09549150281252648, 
          0.4045084971874738, 0.0954915028125262}}, {{
          0.5636610018750178, -0.6030056647916495, -0.16666666666666688`, \
-0.5393446629166319}, {0.7060113295832988, 
          0.6666666666666667, -0.23032766854168416`, 0.06366100187501789}, {
          0.3333333333333335, -0.039344662916631475`, 0.9363389981249824, 
          0.10300566479164928`}, {
          0.26967233145831626`, -0.4363389981249825, -0.20601132958329837`, 
          0.8333333333333333}}, {{0.5636610018750178, 
          0.3333333333333335, -0.26967233145831626`, -0.7060113295832987}, \
{-0.16666666666666677`, 0.9363389981249824, 0.20601132958329832`, 
          0.23032766854168424`}, {0.5393446629166317, -0.1030056647916493, 
          0.8333333333333331, 0.0636610018750177}, {0.6030056647916494, 
          0.03934466291663152, -0.43633899812498245`, 0.6666666666666669}}, {{
          0.09549150281252625, -0.09549150281252643, -0.9045084971874744, \
-0.4045084971874737}, {0.40450849718747384`, 
          0.9045084971874737, -0.09549150281252615, 0.09549150281252633}, {
          0.9045084971874744, -0.4045084971874738, 0.09549150281252625, 
          0.0954915028125265}, {-0.09549150281252611, -0.09549150281252633, \
-0.40450849718747384`, 0.9045084971874735}}, {{
          0.5636610018750179, -0.26967233145831615`, 
          0.7060113295832986, -0.33333333333333337`}, {0.5393446629166319, 
          0.8333333333333335, -0.06366100187501766, 
          0.10300566479164924`}, {-0.6030056647916495, 0.43633899812498245`, 
          0.6666666666666669, 0.03934466291663155}, {
          0.16666666666666682`, -0.20601132958329832`, 0.23032766854168424`, 
          0.9363389981249826}}, {{0.09549150281252622, -0.9045084971874744, 
          0.4045084971874737, 0.09549150281252647}, {0.9045084971874744, 
          0.09549150281252625, -0.09549150281252647, 0.4045084971874738}, {
          0.0954915028125261, 0.40450849718747384`, 
          0.9045084971874736, -0.09549150281252633}, {-0.4045084971874738, 
          0.09549150281252616, 0.09549150281252633, 0.9045084971874738}}, {{
          0.5000000000000004, 0.5000000000000003, 
          0.5000000000000003, -0.5000000000000003}, {-0.5000000000000002, 
          0.8333333333333335, -0.1666666666666665, 
          0.16666666666666652`}, {-0.5000000000000002, -0.1666666666666665, 
          0.8333333333333335, 0.16666666666666652`}, {0.5000000000000002, 
          0.16666666666666652`, 0.16666666666666652`, 
          0.8333333333333335}}}, {{{
           0.3454915028125265, -0.9045084971874745, -0.2500000000000003, 0}, {
           0.8090169943749481, 0.40450849718747384`, -0.3454915028125264, 
           0.25000000000000006`}, {0.40450849718747406`, -0.0954915028125262, 
           0.9045084971874741, 
           0.09549150281252634}, {-0.24999999999999997`, -0.09549150281252655,
             0, 0.9635254915624211}}, {{-0.40450849718747406`, \
-0.7135254915624213, 0.40450849718747345`, 0.40450849718747356`}, {
           0.6545084971874737, 0, -0.09549150281252654, 0.7500000000000003}, {
           0.3090169943749476, 0.24999999999999967`, 
           0.9045084971874738, -0.1545084971874734}, {-0.5590169943749475, 
           0.654508497187474, 0.09549150281252644, 0.5000000000000001}}, {{
           0.3454915028125263, -0.25000000000000033`, 0, 
            0.9045084971874738}, {0.40450849718747384`, 
           0.9045084971874735, -0.09549150281252641, 0.09549150281252641}, {
           0.25000000000000017`, 0, 
            0.9635254915624212, -0.09549150281252619}, {-0.8090169943749475, 
           0.3454915028125263, 0.25000000000000006`, 0.4045084971874739}}, {{
           0.1432372542187895, -0.5295084971874743, 
           0.7317627457812106, -0.4045084971874737}, {0.9340169943749478, 
           0.3272542485937367, 0.10676274578121042`, 
           0.09549150281252633}, {-0.32725424859373675`, 0.7022542485937368, 
           0.6250000000000003, 0.0954915028125262}, {
           0, -0.3454915028125264, 0.24999999999999983`, 
            0.9045084971874738}}, {{0.8354101966249688, -0.3454915028125261, 
           0.40450849718747334`, 0.13819660112501042`}, {0.40450849718747367`,
            0.9045084971874741, -0.09549150281252641, 
           0.09549150281252616}, {-0.3090169943749471, 0.2499999999999998, 
           0.9045084971874741, -0.1545084971874735}, {-0.20729490168751555`, 
            0, 0.09549150281252633, 0.973606797749979}}, {{
           0.1432372542187894, 0, 0.9340169943749478, 0.327254248593737}, {
           0.4045084971874738, 0.9045084971874737, -0.09549150281252639, 
           0.09549150281252627}, {-0.5295084971874741, 0.3454915028125264, 
           0.3272542485937367, -0.7022542485937369}, {-0.7317627457812106, 
           0.25, -0.1067627457812104, 
           0.625}}, {{-0.4045084971874741, -0.404508497187474, \
-0.7135254915624208, -0.4045084971874736}, {0.5590169943749473, 
           0.4999999999999996, -0.6545084971874742, 0.0954915028125263}, {
           0.6545084971874734, -0.7500000000000002, 0, 
            0.09549150281252636}, {-0.30901699437494756`, \
-0.15450849718747361`, -0.2499999999999997, 0.9045084971874737}}, {{
           0.3454915028125265, 0, -0.9045084971874744, 0.2500000000000002}, {
           0.2500000000000001, 0.9635254915624211, 0.09549150281252625, 0}, {
           0.8090169943749483, -0.2500000000000001, 0.4045084971874739, 
           0.3454915028125263}, {-0.40450849718747384`, 0.09549150281252611, 
           0.095491502812526, 0.9045084971874735}}, {{-0.40450849718747406`, 
           0.40450849718747384`, -0.4045084971874735, 0.7135254915624212}, {
           0.3090169943749476, 0.904508497187474, 
           0.15450849718747375`, -0.2499999999999999}, {
           0.5590169943749475, -0.09549150281252622, 0.4999999999999999, 
           0.6545084971874741}, {-0.6545084971874737, 0.09549150281252636, 
            0.7500000000000003, 0}}, {{
           0.14323725421878955`, -0.3272542485937373, 
            0, -0.9340169943749477}, {0.731762745781211, 
           0.6249999999999999, -0.2500000000000001, -0.10676274578121059`}, {
           0.4045084971874738, -0.09549150281252622, 0.9045084971874737, 
           0.09549150281252629}, {
           0.5295084971874735, -0.7022542485937371, -0.34549150281252633`, 
           0.32725424859373736`}}, {{-0.5590169943749479, 0.09549150281252616,
           0.5, -0.6545084971874738}, {0.8090169943749477, 
          0.09549150281252539, 0.15450849718747378`, -0.5590169943749479}, {
          0.15450849718747392`, 0.4045084971874737, 0.7499999999999998, 
          0.4999999999999999}, {0.09549150281252583, -0.9045084971874742, 
          0.4045084971874736, 
          0.09549150281252644}}, {{
           0.8354101966249685, -0.13819660112501062`, -0.3454915028125261, \
-0.4045084971874736}, {
           0.20729490168751594`, 0.9736067977499792, 0, 0.09549150281252644},
            {0.40450849718747384`, -0.09549150281252633, 0.904508497187474, 
           0.0954915028125263}, {
           0.30901699437494745`, -0.15450849718747364`, -0.24999999999999994`,
            0.9045084971874742}}}, {{{-0.7135254915624214, \
-0.6545084971874736, 0, -0.2499999999999999}, {
           0.40450849718747356`, -0.5590169943749472, -0.6545084971874738, 
           0.3090169943749477}, {0.40450849718747367`, -0.5000000000000003, 
           0.7499999999999999, 0.15450849718747384`}, {-0.4045084971874737, 
           0.09549150281252664, 0.09549150281252643, 
           0.9045084971874737}}, {{-0.7135254915624217, 0.4045084971874737, 
           0.40450849718747306`, -0.4045084971874735}, {0.24999999999999964`, 
           0.9045084971874738, -0.3090169943749481, 
           0.15450849718747384`}, {-0.6545084971874735, -0.09549150281252727, \
-0.5590169943749479, 0.5000000000000002}, {
           0, 0.09549150281252661, 0.6545084971874737, 
            0.7500000000000001}}, {{-0.12811529493745313`, \
-0.4045084971874744, 0.9045084971874737, 0.042705098312484174`}, {
           0.8090169943749478, 0.34549150281252544`, 0.2500000000000001, 
           0.40450849718747406`}, {-0.24999999999999972`, 0.8090169943749478, 
           0.3454915028125266, -0.4045084971874736}, {-0.5163118960624634, 
            0.2500000000000001, 0, 
            0.8190983005625053}}, {{-0.3090169943749483, 0.15450849718747345`,
           0.25, -0.9045084971874741}, {0.5590169943749479, 
          0.8090169943749469, -0.15450849718747398`, -0.095491502812527}, {
          0.4045084971874741, -0.0954915028125265, 0.9045084971874737, 
          0.09549150281252597}, {
          0.6545084971874734, -0.5590169943749482, -0.3090169943749478, \
-0.4045084971874743}}, {{0.1249999999999996, -0.8090169943749481, 
           0.17274575140626308`, -0.5477457514062628}, {
           0.9635254915624215, 0.09549150281252566, -0.25000000000000017`, 
            0}, {0.23176274578121073`, 0.1545084971874737, 0.9522542485937368,
            0.12499999999999989`}, {
           0.04774575140626275, -0.5590169943749472, -0.029508497187473677`, 
           0.8272542485937371}}, {{
           0.09549150281252666, 0, 0.7500000000000002, -0.654508497187474}, {
           0.5590169943749478, 0.8272542485937369, -0.029508497187473753`, 
           0.047745751406263304`}, {-0.654508497187474, 0.43401699437494773`, 
           0.4522542485937374, 0.422745751406263}, {
           0.5000000000000004, -0.35676274578121075`, 0.4817627457812105, 
           0.625}}, {{-0.5590169943749477, 0.8090169943749476, 
          0.15450849718747373`, 0.09549150281252572}, {0.09549150281252625, 
          0.09549150281252555, 0.40450849718747417`, -0.9045084971874737}, {
          0.5000000000000001, 0.15450849718747417`, 0.7499999999999999, 
          0.404508497187474}, {-0.6545084971874736, -0.5590169943749477, 0.5, 
          0.09549150281252716}}, {{-0.239918693812443, -0.18090169943749518`, 
          0.11180339887498933`, 0.9472135954999581}, {0.47360679774997916`, 
          0.8190983005625049, -0.13819660112501067`, 0.2927050983124849}, {
          0.36180339887498963`, -0.04270509831248431, 
          0.9309016994374948, -0.02639320225002069}, {-0.7663118960624633, 
          0.5427050983124847, 
          0.31909830056250543`, -0.1281152949374531}}, {{-0.1281152949374531, 
           0.5163118960624637, 0.8090169943749478, 
           0.24999999999999978`}, {-0.04270509831248498, 
           0.8190983005625051, -0.4045084971874743, -0.40450849718747417`}, \
{-0.4045084971874745, -0.25000000000000067`, 
           0.34549150281252594`, -0.8090169943749475}, {-0.904508497187474, 
            0, -0.24999999999999994`, 
            0.34549150281252683`}}, {{-0.30901699437494823`, 
          0.4045084971874737, -0.6545084971874737, -0.5590169943749473}, {
          0.2500000000000003, 0.9045084971874735, 0.3090169943749473, 
          0.15450849718747361`}, {
          0.9045084971874734, -0.0954915028125263, -0.4045084971874748, \
-0.09549150281252627}, {-0.15450849718747423`, 
          0.09549150281252627, -0.5590169943749472, 
          0.8090169943749476}}, {{-0.40450849718747384`, 0.654508497187474, 
           0.30901699437494745`, -0.5590169943749475}, {-0.7135254915624216, 
            0, 0.2500000000000001, 0.6545084971874733}, {
           0.4045084971874738, -0.09549150281252632, 0.9045084971874738, 
           0.09549150281252633}, {0.40450849718747334`, 
           0.7499999999999998, -0.15450849718747356`, 0.5000000000000007}}, {{
          0.01823725421878887, -0.6067627457812115, -0.7795084971874738, 
          0.15450849718747378`}, {0.6067627457812113, 
          0.6249999999999992, -0.4227457514062637, 0.25000000000000044`}, {
          0.6840169943749477, -0.48176274578121075`, 0.4522542485937371, 
          0.30901699437494745`}, {-0.4045084971874739, 0.09549150281252639, 
          0.09549150281252629, 
          0.9045084971874737}}}, {{{-0.7135254915624215, 0.4045084971874744, 
           0.404508497187474, -0.40450849718747384`}, {-0.6545084971874742, \
-0.5590169943749482, -0.4999999999999999, 0.09549150281252552}, {
           0, -0.6545084971874738, 0.75, 
            0.09549150281252602}, {-0.2500000000000004, 0.309016994374947, 
           0.15450849718747356`, 0.9045084971874737}}, {{
          0.018237254218790015`, 0.684016994374948, 
          0.4045084971874737, -0.6067627457812105}, {-0.7795084971874746, 
          0.4522542485937364, -0.09549150281252602, 
          0.4227457514062625}, {-0.15450849718747378`, -0.3090169943749475, 
          0.9045084971874737, 0.24999999999999964`}, {0.6067627457812099, 
          0.4817627457812105, 0.0954915028125261, 
          0.6250000000000009}}, {{-0.40450849718747434`, 0.30901699437494745`,
            0.5590169943749477, -0.654508497187474}, {0.40450849718747406`, 
           0.9045084971874736, -0.09549150281252622, 
           0.09549150281252616}, {-0.40450849718747367`, 0.15450849718747384`,
            0.5, 0.7500000000000003}, {
           0.7135254915624214, -0.2500000000000002, 0.6545084971874741, 
            0}}, {{-0.1281152949374535, 
           0.8090169943749482, -0.2499999999999994, -0.5163118960624631}, \
{-0.4045084971874745, 0.34549150281252483`, 0.8090169943749481, 
           0.2499999999999998}, {
           0.9045084971874737, 0.25000000000000056`, 0.3454915028125267, 0}, {
           0.042705098312483786`, 0.404508497187474, -0.4045084971874731, 
           0.8190983005625057}}, {{-0.5590169943749486, 
          0.15450849718747336`, -0.09549150281252639, -0.8090169943749472}, {
          0.5000000000000001, 
          0.7499999999999991, -0.40450849718747517`, -0.15450849718747467`}, {
          0.654508497187473, -0.5000000000000012, 
          0.09549150281252498, -0.5590169943749478}, {-0.09549150281252783, \
-0.4045084971874744, -0.9045084971874734, 
          0.09549150281252722}}, {{0.12499999999999997`, 
           0.23176274578121053`, -0.04774575140626364, -0.9635254915624214}, {
           0.17274575140626316`, 0.9522542485937368, 0.029508497187473892`, 
           0.25}, {0.5477457514062637, -0.125, 0.827254248593737, 0}, {
           0.8090169943749475, -0.15450849718747375`, -0.559016994374948, 
           0.09549150281252654}}, {{
           0.09549150281252661, 0.7500000000000008, 0.6545084971874737, 
            0}, {-0.6545084971874753, 
           0.45225424859373625`, -0.4227457514062628, -0.43401699437494706`}, \
{-0.5000000000000003, -0.4817627457812107, 
           0.6250000000000007, -0.3567627457812097}, {-0.5590169943749466, 
           0.02950849718747428, 0.04774575140626369, 
           0.8272542485937376}}, {{-0.3090169943749479, 0.24999999999999994`, 
          0.9045084971874738, -0.15450849718747403`}, {0.4045084971874738, 
          0.9045084971874737, -0.09549150281252611, 
          0.0954915028125265}, {-0.654508497187474, 
          0.30901699437494756`, -0.40450849718747445`, -0.5590169943749472}, \
{-0.5590169943749473, 0.15450849718747378`, -0.0954915028125261, 
          0.8090169943749478}}, {{0.3454915028125275, 0.24999999999999997`, 
           0.8090169943749479, -0.4045084971874739}, {
           0, 0.9635254915624212, -0.24999999999999994`, 
            0.09549150281252639}, {-0.9045084971874738, 0.0954915028125263, 
           0.40450849718747495`, 0.09549150281252614}, {
           0.2500000000000007, 0, 0.345491502812526, 
            0.9045084971874738}}, {{-0.12811529493745322`, 
           0.9045084971874746, -0.04270509831248328, 
           0.40450849718747295`}, {-0.250000000000001, 0.3454915028125248, 
           0.40450849718747356`, -0.8090169943749483}, {
           0.516311896062463, 0, 0.8190983005625058, 
            0.24999999999999942`}, {-0.8090169943749479, -0.2500000000000005, 
           0.40450849718747334`, 0.3454915028125266}}, {{0.5636610018750177, 
          0.8211751638541402, -0.0833333333333327, -0.03183050093750908}, \
{-0.7378418305208074, 0.5318305009375088, 
          0.34084749531245556`, -0.23784183052080682`}, {
          0.3211751638541396, -0.12267799624996517`, 0.9363389981249828, 
          0.07117516385414017}, {-0.1863389981249821, 0.16666666666666674`, 
          0.012158169479192946`, 
          0.9681694990624914}}, {{-0.7135254915624222, 0, 0.2500000000000006, 
            0.6545084971874731}, {0.40450849718747367`, 
           0.7499999999999991, -0.15450849718747378`, 0.5000000000000013}, {
           0.40450849718747384`, -0.09549150281252615, 0.9045084971874737, 
           0.09549150281252589}, {-0.40450849718747317`, 0.6545084971874746, 
           0.30901699437494684`, -0.5590169943749479}}}, {{{
           0.34549150281252694`, 0.8090169943749473, 
           0.4045084971874739, -0.24999999999999992`}, {-0.9045084971874734, 
           0.4045084971874745, -0.0954915028125264, -0.09549150281252619}, \
{-0.2500000000000006, -0.34549150281252616`, 0.9045084971874736, 0}, {
           0, 0.2499999999999999, 0.09549150281252634, 
            0.9635254915624212}}, {{0.8354101966249691, 
           0.4045084971874735, -0.3090169943749469, -0.20729490168751585`}, \
{-0.3454915028125265, 0.904508497187474, 0.24999999999999944`, 0}, {
           0.4045084971874729, -0.09549150281252655, 0.9045084971874744, 
           0.09549150281252626}, {0.1381966011250109, 
           0.09549150281252614, -0.15450849718747345`, 
           0.9736067977499789}}, {{
           0.0954915028125264, 0.6545084971874745, 
            0, -0.7499999999999998}, {-0.5000000000000014, 0.624999999999999, 
           0.3567627457812106, 0.4817627457812105}, {
           0.5590169943749466, -0.047745751406263345`, 0.8272542485937374, 
           0.029508497187474215`}, {0.6545084971874737, 
           0.4227457514062636, -0.4340169943749465, 0.4522542485937377}}, {{
           0.40450849718747395`, 0.7500000000000008, -0.15450849718747253`, 
           0.4999999999999992}, {-0.40450849718747517`, 0.6545084971874727, 
           0.30901699437494706`, -0.5590169943749477}, {
           0.4045084971874727, -0.09549150281252664, 0.904508497187474, 
           0.09549150281252543}, {-0.713525491562421, 0, 0.2499999999999994, 
            0.6545084971874742}}, {{-0.30901699437494823`, 0.9045084971874735,
           0.15450849718747547`, -0.2500000000000002}, {-0.654508497187475, \
-0.4045084971874757, 0.5590169943749446, -0.30901699437494795`}, {
          0.5590169943749458, 0.0954915028125241, 0.8090169943749489, 
          0.15450849718747306`}, {-0.40450849718747406`, 0.09549150281252601, 
          0.09549150281252647, 
          0.9045084971874735}}, {{0.12499999999999933`, 
           0.5477457514062638, -0.8090169943749479, -0.17274575140626305`}, \
{-0.04774575140626348, 0.8272542485937366, 
           0.559016994374948, -0.02950849718747385}, {
           0.9635254915624216, 0, 0.09549150281252584, 
            0.2500000000000002}, {-0.23176274578121078`, 
           0.12500000000000014`, -0.15450849718747356`, 
           0.9522542485937368}}, {{0.8190983005625062, 0.11180339887498947`, 
          0.5163118960624621, -0.22360679774997896`}, {-0.042705098312484625`,
           0.9899186938124425, -0.12811529493745238`, 
          0.04270509831248415}, {-0.542705098312483, 0.08541019662496811, 
          0.8354101966249695, -0.016311896062463055`}, {
          0.18090169943749515`, -0.01631189606246325, 0.13819660112500998`, 
          0.9736067977499789}}, {{0.01823725421879031, 0.4045084971874742, 
          0.6067627457812109, -0.6840169943749478}, {-0.1545084971874749, 
          0.9045084971874736, -0.25000000000000017`, 
          0.30901699437494756`}, {-0.606762745781211, -0.09549150281252666, 
          0.6250000000000004, 0.48176274578121003`}, {0.779508497187474, 
          0.09549150281252666, 0.4227457514062627, 0.45225424859373786`}}, {{
          0.5636610018750186, -0.08333333333333415, 
          0.0318305009375078, -0.8211751638541397}, {0.3211751638541406, 
          0.9363389981249824, -0.07117516385414038, 0.12267799624996481`}, {
          0.18633899812498297`, -0.012158169479193091`, 0.9681694990624912, 
          0.16666666666666627`}, {
          0.7378418305208062, -0.3408474953124562, -0.2378418305208074, 
          0.5318305009375102}}, {{0.4045084971874746, 0.4045084971874744, 
           0.7135254915624213, 0.40450849718747317`}, {-0.15450849718747509`, 
           0.9045084971874736, -0.2500000000000002, -0.30901699437494756`}, \
{-0.5000000000000012, -0.09549150281252675, 
           0.654508497187474, -0.5590169943749468}, {-0.7499999999999993, 
            0.09549150281252622, 0, 0.654508497187475}}, {{
          0.8190983005625061, -0.18090169943749504`, -0.04270509831248482, 
          0.5427050983124833}, {0.223606797749979, 
          0.9736067977499789, -0.04270509831248419, -0.016311896062463138`}, {
          0.11180339887498962`, 0.01631189606246314, 
          0.9899186938124421, -0.08541019662496813}, {-0.5163118960624621, 
          0.13819660112501003`, 0.12811529493745236`, 
          0.8354101966249694}}, {{-0.4045084971874737, 0.5590169943749478, 
           0.6545084971874737, -0.3090169943749475}, {-0.40450849718747467`, 
           0.4999999999999988, -0.7500000000000002, -0.15450849718747398`}, \
{-0.7135254915624208, -0.6545084971874744, 
            0, -0.24999999999999983`}, {-0.4045084971874737, 
           0.09549150281252633, 0.0954915028125264, 0.9045084971874737}}}}, 
       Vector`VectorProject[
         Pattern[Vector`Private`subspace, 
          Blank[]]][
         Pattern[Vector`Private`points, 
          Blank[]]] := 
       Vector`VectorProject[Vector`Private`points, Vector`Private`subspace], 
       Vector`VectorProject[{}, 
         Pattern[Vector`Private`space, 
          Blank[]]] := {}, Vector`VectorProject[
         Pattern[Vector`Private`points, 
          Blank[]], {}] := Map[{}& , Vector`Private`points, {-2}], 
       Vector`VectorProject[
         PatternTest[
          Pattern[Vector`Private`points, 
           Blank[]], ArrayQ], 
         PatternTest[
          Pattern[Vector`Private`vector, 
           Blank[]], VectorQ]] := 
       Dot[Vector`Private`points, Vector`Private`vector], Vector`VectorProject[
         PatternTest[
          Pattern[Vector`Private`points, 
           Blank[]], ArrayQ], 
         PatternTest[
          Pattern[Vector`Private`subspace, 
           Blank[]], MatrixQ]] := Dot[Vector`Private`points, 
         Transpose[Vector`Private`subspace]], Vector`VectorProject[
         Pattern[Vector`Private`points, 
          Blank[]], 
         PatternTest[
          Pattern[Vector`Private`subspace, 
           Blank[]], MatrixQ]] := 
       Map[Vector`VectorProject[#, Vector`Private`subspace]& , 
         Vector`Private`points], Vector`VectorProject[
         Pattern[Vector`Private`points, 
          Blank[]], 0] := Vector`VectorProject[Vector`Private`points, {}], 
       TagSet[Vector`VectorProject, 
        MessageName[Vector`VectorProject, "usage"], 
        "VectorProject[points, basis] projects points to the subpsace defined \
by basis."], $CellContext`normalV = {-0.9256147934109582, 
        0.21850801222441055`, 
        0.21850801222441055`, -0.21850801222441055`}, $CellContext`ctr3 = {0, 
        0, 0}, $CellContext`compoundingAxes3D = CompressedData["
1:eJx1U2tIk2EUXq7hSKGyTMcmVEpCCUEaWmGnmhsaIqSRs/ph1may7CL9SMH+
WlA66C44aWpJGWJ/ung5eCkViyydpb9CM53RQMkyEsw4zytMaPBx9nHO8z6X
836b8s5l2rUajUa3+IQuPv/+/+9nindZHhaPkqpGqfwow1/vqv3MCbW5sSEe
H8U27ynPmfvAqKT68dJXc7wFc6o/nGqdudXXw0MNMXHlMe9oRf6V6fYCv8LR
svOWeB7jXfEPBuJpRM5dwi3jX5pT/F7BM3xyFCr80qQ59lv/z++0L7n1d81Z
L6XUhJQV+15yftLW2YUdU9S0eTbSZ3jO6736V2s2jrIZc0dlTuE589jIblu6
j6rmQ59lZYzyp5Kdq8Pm+qj3TqXR2D1ONVpLhMk8zo5d7pnSzgH+iP4hwbEb
uEjhoVPgbxF+yg7ko72ig19DXzhwDsGp82g4kIfqRQflQ0c39J2Rd+rB+wPM
KT7l+wjyKUJV/gn9vmV+neBpG7o3fSD6PZ3v2l9Wcb+FgrItjSXVfnL6gsLj
TH6qe3HjycG8CvpVFbb2Wp2XBjzBttxVXkrTNxe2Oic4qje1rcgwpvRzIfR2
Ch9bwG9DTl+h6yL44Ys9wGeJDl4JHU+h7zTy60Xu65CrLnJh8s/tMdqefeGu
1TdBiT8uO3Ou9/MUeJKxj23gT5Q5TsDcfITgB3HuBtwnO/b8NlAfnYC/S6gq
12r0b0perHdLXgXIMRN5GSQvZsmRHchdK34Zfvkk9qjysiLHBuR0HPxd0Ie8
uA55pSDHdOCS4H8C56mahnxy0Fd+3fCDPVE0dHdg/6nihxrV3jDXDnww9Lig
xw49OsmFVS6VyKsUfr9A12HoNmNPWtkzqz134lwP8tPgvpTiXr2BjiroUzlc
hc9x8Bix7zzcrw58t38BoEnpRg==
        "]}; Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellChangeTimes->{
  3.9162689745077486`*^9, 3.916269017861496*^9, {3.916269074841487*^9, 
   3.9162690756327877`*^9}, 3.9162694064197493`*^9, 3.9162694835546465`*^9, 
   3.9165226678817806`*^9, {3.916522917518612*^9, 3.916522933215148*^9}, {
   3.9165258766431503`*^9, 3.916525887024312*^9}},
 CellLabel->
  "Out[107]=",ExpressionUUID->"57c17075-b72d-4f6c-afac-c2d44699a73b"]
}, Open  ]],

Cell[TextData[{
 "This graphic shows the results of 5 cuboctahedra (or 4-Groups, selectable) \
from the 5 respective 24-cells all rotated into ",
 StyleBox["V",
  FontSlant->"Italic"],
 ". You can choose which of the 12 cuboctahedra to view from each 24-cell, \
and which of those 5 to view (except for the primary, which is fixed, and \
always shown). You can slide the rotation \[Alpha] from left- to \
right-isoclinic, to see how at one extreme all cuboctahedra coincide, at the \
other extreme they all combine in the icosahedrally symmetric compound, and \
in the middle (simple direct rotation) they form an incoherent jumble. \n\n\
The default values in the graphic select the first cuboctahedron from each \
24-cell. This is a special case: even with no twist (\[Alpha] = 0), the \
compound is a 20-Group or C5C (depending on whether one chooses to show 4Gs \
or cuboctahedra) . \n\nFor all other cases, the shapes form a chaotic jumble \
when rotated directly into V with no twist . But with the left twist \
(\[Alpha] = 1) they all converge to a single 4G/cuboctahedron; with the right \
twist (\[Alpha] = -1), they form a right-handed 20-Group. If the original \
compound of 24-cells had been constructed using a right-isoclinic rotation, \
and then the 4Gs/cuboctahedra were compounded using a left-isoclinic \
rotation, we would get a left-handed 20-Group ."
}], "Text",
 CellChangeTimes->{{3.915757151296363*^9, 3.9157572015754766`*^9}, {
  3.91575732938165*^9, 3.9157573509992633`*^9}, {3.9158314309619813`*^9, 
  3.9158314774154496`*^9}, {3.9162686313881097`*^9, 3.9162686322141867`*^9}, {
  3.9162690932694974`*^9, 3.916269378083884*^9}, {3.916269430491534*^9, 
  3.9162694772074766`*^9}},ExpressionUUID->"ba02451b-1f5e-4916-95f8-\
27414a9b488d"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1137, 709.5},
WindowMargins->{{106.5, Automatic}, {-3, Automatic}},
DockedCells->{},
TaggingRules-><|"TryRealOnly" -> False|>,
Magnification:>1. Inherited,
FrontEndVersion->"13.3 for Microsoft Windows (64-bit) (June 3, 2023)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"bdb3d655-2c83-4117-b0a3-85a7deb20186"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 943, 19, 121, "Title",ExpressionUUID->"88b4d7db-17fc-49b5-b855-b70064b435c4"],
Cell[1526, 43, 762, 21, 118, "Text",ExpressionUUID->"dfd051c7-62e2-429d-a929-1491a1b035c2"],
Cell[2291, 66, 2797, 61, 143, "Text",ExpressionUUID->"f06b0642-2092-4b5c-a375-a290bef69f1d",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[5113, 131, 156, 3, 50, "Section",ExpressionUUID->"c212aa7d-65ac-4002-9761-9922237546c3"],
Cell[5272, 136, 4600, 95, 392, "Text",ExpressionUUID->"d0ffc935-ae7b-48b0-8757-0c2d8dca3b1b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9909, 236, 394, 6, 51, "Chapter",ExpressionUUID->"e373e991-a9a6-4a1b-8868-f838cb9c4c44"],
Cell[CellGroupData[{
Cell[10328, 246, 189, 3, 50, "Section",ExpressionUUID->"f52d77d8-d75d-40fd-9812-eb54a92c9a08"],
Cell[10520, 251, 809, 12, 36, "Input",ExpressionUUID->"e390329a-fd8c-4ec5-b350-ef4f3c2821c6"],
Cell[11332, 265, 833, 18, 50, "Input",ExpressionUUID->"9e241e0b-ac14-4a1f-9f00-450f79a71112"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12202, 288, 231, 4, 50, "Section",ExpressionUUID->"bc872a82-b50d-49a7-98ae-61c7ce0538ab"],
Cell[CellGroupData[{
Cell[12458, 296, 1022, 16, 34, "Subsubsection",ExpressionUUID->"7979290d-83a9-4fed-871a-7055b0b6c3a3"],
Cell[13483, 314, 4474, 96, 107, "Input",ExpressionUUID->"6d9e5cca-721c-47f4-8283-c628706f3580"]
}, Closed]],
Cell[CellGroupData[{
Cell[17994, 415, 1044, 15, 29, "Subsubsection",ExpressionUUID->"fe30af3d-0573-4a1b-ba3e-3ad4c1686918"],
Cell[19041, 432, 1891, 40, 136, "Text",ExpressionUUID->"49460a08-eaac-410c-bc26-0326715b00f8"],
Cell[20935, 474, 18362, 424, 578, "Input",ExpressionUUID->"a84d1679-450b-468a-a8d1-f0a8245c4bd3"]
}, Closed]],
Cell[CellGroupData[{
Cell[39334, 903, 587, 8, 29, "Subsubsection",ExpressionUUID->"067ea792-2558-4a78-ab09-cf262c60d79d"],
Cell[39924, 913, 721, 12, 45, "Text",ExpressionUUID->"474ac226-6772-48d1-9791-093c25206665"],
Cell[40648, 927, 28626, 583, 982, "Input",ExpressionUUID->"10c0f2bf-b991-4259-acb1-b978b3d9f399"]
}, Closed]],
Cell[CellGroupData[{
Cell[69311, 1515, 442, 6, 29, "Subsubsection",ExpressionUUID->"cea81e48-a517-43b2-a1d4-9f43c1457e1a"],
Cell[69756, 1523, 5032, 108, 179, "Input",ExpressionUUID->"ae1e5d23-6058-491a-8d19-e8b774d09b52"]
}, Closed]],
Cell[CellGroupData[{
Cell[74825, 1636, 181, 3, 29, "Subsubsection",ExpressionUUID->"8b45d703-dbde-4f91-9e05-48675c5eac05"],
Cell[75009, 1641, 879, 22, 21, "Input",ExpressionUUID->"bd079a84-7440-499f-93c3-6afb280933b8"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[75949, 1670, 204, 4, 51, "Chapter",ExpressionUUID->"1fbd958d-fd94-4794-9e1d-2eb40549bd3a"],
Cell[CellGroupData[{
Cell[76178, 1678, 261, 4, 50, "Section",ExpressionUUID->"73e85ae9-6372-459c-a5e5-811d750cea27"],
Cell[76442, 1684, 3215, 92, 179, "Input",ExpressionUUID->"99af1fe6-b4df-4c7f-bb99-07e2ac7c5d8f"],
Cell[CellGroupData[{
Cell[79682, 1780, 644, 14, 50, "Input",ExpressionUUID->"f5fabf09-de9c-4bc8-be11-8ecf51b49faf"],
Cell[80329, 1796, 397, 7, 70, "Output",ExpressionUUID->"3cb9c6e8-f53e-4bda-9e23-a8c9b6ac6b2f"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[80775, 1809, 368, 5, 40, "Section",ExpressionUUID->"51ca42fd-ec15-4680-92d9-ad884b5da3af"],
Cell[CellGroupData[{
Cell[81168, 1818, 181, 3, 41, "Subsection",ExpressionUUID->"8166766d-d5c9-4a18-b038-8acca4e4253f"],
Cell[81352, 1823, 3496, 107, 164, "Input",ExpressionUUID->"0b2a4286-60b9-4425-9083-f7f6665565e8"],
Cell[CellGroupData[{
Cell[84873, 1934, 345, 5, 34, "Subsubsection",ExpressionUUID->"d4a2eb29-1f50-42bb-af9c-a0abed12a4b8"],
Cell[CellGroupData[{
Cell[85243, 1943, 983, 24, 50, "Input",ExpressionUUID->"89a17c7b-cd9b-46e7-8c2b-de93e7394117"],
Cell[86229, 1969, 419, 8, 70, "Output",ExpressionUUID->"db35e724-be14-404c-a53c-b2aec7c4b97a"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[86709, 1984, 488, 7, 30, "Subsection",ExpressionUUID->"a9e82749-82c0-4e4d-a92b-6f302cf49530"],
Cell[CellGroupData[{
Cell[87222, 1995, 1313, 26, 36, "Input",ExpressionUUID->"20ee3b41-b619-4c2a-9673-f89b38b20893"],
Cell[88538, 2023, 447, 8, 70, "Output",ExpressionUUID->"b743f039-f3fd-4d27-ab24-41295c5c4dea"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89022, 2036, 337, 8, 58, "Item",ExpressionUUID->"9f600ff3-2334-4aef-b312-f29f04ead1a5"],
Cell[CellGroupData[{
Cell[89384, 2048, 1175, 29, 36, "Input",ExpressionUUID->"ff5ded62-7e20-4df2-bee8-28bdf389ad9c"],
Cell[90562, 2079, 1641, 25, 70, "Output",ExpressionUUID->"4ea19a54-6177-46f5-bb15-a928bb05f7b4"],
Cell[92206, 2106, 1642, 25, 70, "Output",ExpressionUUID->"d7293dae-ccb0-4d5d-a54f-6684242be3c5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93885, 2136, 757, 17, 21, "Input",ExpressionUUID->"91cffd44-60b9-4d43-98dc-4bc336da095b"],
Cell[94645, 2155, 1586, 24, 70, "Output",ExpressionUUID->"801b4f42-953b-4aa2-bac3-97583caad53f"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[96280, 2185, 369, 6, 25, "Item",ExpressionUUID->"b97d4f58-bc48-45bb-8270-d3a62cab5c1e"],
Cell[96652, 2193, 1642, 31, 21, "Input",ExpressionUUID->"818cdbed-0975-4466-b386-bc850ffc2ecd"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98331, 2229, 469, 9, 53, "Subsubsection",ExpressionUUID->"1ce956d3-c2b1-4d74-a9f6-e2c24e23b7be"],
Cell[CellGroupData[{
Cell[98825, 2242, 1475, 39, 79, "Input",ExpressionUUID->"a6b1f83c-485a-48e7-a903-be424d643e21"],
Cell[100303, 2283, 1563, 24, 70, "Output",ExpressionUUID->"7fb30628-f505-4ce1-bbc7-801449433140"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[101927, 2314, 361, 6, 30, "Subsection",ExpressionUUID->"03250c36-16d8-4480-afb8-a5beff13fea9"],
Cell[102291, 2322, 1834, 46, 93, "Input",ExpressionUUID->"a7aafcb0-91e6-4562-93e8-afd4b98d668d"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[104174, 2374, 291, 5, 50, "Section",ExpressionUUID->"b4a2536c-e0c2-4b8b-ac5d-342220b257b7"],
Cell[CellGroupData[{
Cell[104490, 2383, 338, 7, 25, "Item",ExpressionUUID->"fbc8f8e4-cfb1-4a4c-a8e6-7adfe4099ec0"],
Cell[104831, 2392, 923, 23, 36, "Input",ExpressionUUID->"62a84572-48ec-48c3-902a-45ed3a6998e8"],
Cell[CellGroupData[{
Cell[105779, 2419, 973, 23, 21, "Input",ExpressionUUID->"ab741433-ec84-40bb-bd2d-13d398715446"],
Cell[106755, 2444, 2862, 104, 70, "Output",ExpressionUUID->"1e33a578-b2b5-4da1-8849-acf35905ff44"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[109666, 2554, 290, 4, 23, "Item",ExpressionUUID->"0816ea0b-7021-49e6-92b1-bc81c0e5812e"],
Cell[CellGroupData[{
Cell[109981, 2562, 676, 19, 21, "Input",ExpressionUUID->"b2cf49db-e9f3-429b-9f54-2230ee164998"],
Cell[110660, 2583, 1172, 38, 70, "Output",ExpressionUUID->"ab2cce00-17c7-452f-a5a7-e433bb5c5196"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[111881, 2627, 430, 6, 30, "Subsection",ExpressionUUID->"9d8732e1-01bd-490e-a89e-8cc401485be5"],
Cell[112314, 2635, 654, 12, 45, "Text",ExpressionUUID->"916ec2b7-fdc6-4d6d-add2-566a5868b058"],
Cell[112971, 2649, 3272, 76, 122, "Input",ExpressionUUID->"fb19ba39-c08b-4fae-8c91-7cc6f8a7ac80"]
}, Closed]],
Cell[CellGroupData[{
Cell[116280, 2730, 789, 13, 30, "Subsection",ExpressionUUID->"342c6cfb-4651-486f-8e7d-36666e213c20"],
Cell[CellGroupData[{
Cell[117094, 2747, 711, 11, 34, "Subsubsection",ExpressionUUID->"a815d779-560e-46f5-9d76-97213019f45c"],
Cell[117808, 2760, 274, 4, 27, "Text",ExpressionUUID->"70ade3ad-2a99-49f5-a909-da2c9941c1fc"],
Cell[CellGroupData[{
Cell[118107, 2768, 2295, 47, 50, "Input",ExpressionUUID->"35311f1d-d297-487c-a274-5f909ce15288"],
Cell[120405, 2817, 539, 10, 70, "Output",ExpressionUUID->"5bf68cfc-4188-41af-9162-590f1b36b794"]
}, Open  ]],
Cell[120959, 2830, 694, 12, 45, "Text",ExpressionUUID->"323067c0-e9f2-4d56-8dd3-a212e4f48a92"],
Cell[121656, 2844, 2980, 61, 93, "Input",ExpressionUUID->"fdd463f2-b54d-4ab6-b5ba-c5e63bfee9a6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[124673, 2910, 754, 14, 34, "Subsubsection",ExpressionUUID->"90eeaa20-bdab-4aad-a4ee-b36753ae0e93"],
Cell[125430, 2926, 543, 10, 27, "Text",ExpressionUUID->"7600f802-55f4-4b9e-8d96-dfae94c97bf1"],
Cell[CellGroupData[{
Cell[125998, 2940, 528, 9, 41, "Item",ExpressionUUID->"b87d1904-b548-4eb9-acdd-7f1f6737dcab"],
Cell[CellGroupData[{
Cell[126551, 2953, 3248, 86, 136, "Input",ExpressionUUID->"72e3771a-a26b-4b58-ae44-87abfd13790f"],
Cell[129802, 3041, 1373, 19, 70, "Output",ExpressionUUID->"67fd5478-c882-4cd6-b799-cc57caf7640e"],
Cell[131178, 3062, 1373, 19, 70, "Output",ExpressionUUID->"517895c2-c56e-4e4e-bd6c-7da89816e77a"],
Cell[132554, 3083, 1371, 19, 70, "Output",ExpressionUUID->"336c6acc-0238-4db0-b412-a6811585ced6"],
Cell[133928, 3104, 1373, 19, 70, "Output",ExpressionUUID->"f97612f7-2ef1-4f5a-bc39-97dc240c8cba"],
Cell[135304, 3125, 1373, 19, 70, "Output",ExpressionUUID->"fe47171f-b6eb-4479-b58a-56e1dfd6d260"]
}, Open  ]]
}, Closed]],
Cell[136704, 3148, 266, 4, 25, "Text",ExpressionUUID->"37d9abd2-f86e-4bed-9127-af30496eba82"],
Cell[CellGroupData[{
Cell[136995, 3156, 483, 9, 41, "Item",ExpressionUUID->"65fdfd11-bdbc-4255-8df4-93f1c0d9f17d"],
Cell[CellGroupData[{
Cell[137503, 3169, 4705, 134, 193, "Input",ExpressionUUID->"1b7e065d-1a84-4229-b79f-b9db98158d81"],
Cell[142211, 3305, 1334, 19, 70, "Output",ExpressionUUID->"f03ee251-da96-417d-98c4-5074cab4380a"],
Cell[143548, 3326, 1336, 19, 70, "Output",ExpressionUUID->"f035a491-24c0-4c28-8e65-21e24baad8d2"],
Cell[144887, 3347, 1334, 19, 70, "Output",ExpressionUUID->"616f96ed-88e1-41dc-be73-7bfc854eb3b7"],
Cell[146224, 3368, 1336, 19, 70, "Output",ExpressionUUID->"f601e525-dd92-4970-b150-0e456247979f"],
Cell[147563, 3389, 1336, 19, 70, "Output",ExpressionUUID->"f4f1520f-2d0e-4d40-a50b-4ce38ef111d8"],
Cell[148902, 3410, 1336, 19, 70, "Output",ExpressionUUID->"0d7a8d18-0c61-44b0-9463-ed5c679bf974"]
}, Open  ]]
}, Closed]],
Cell[150265, 3433, 1148, 18, 43, "Text",ExpressionUUID->"1d0e5ec9-ba1f-4908-b40b-9a2910293b55"]
}, Open  ]],
Cell[CellGroupData[{
Cell[151450, 3456, 1179, 18, 53, "Subsubsection",ExpressionUUID->"d7931742-65ff-45df-80d0-7839aca45925"],
Cell[CellGroupData[{
Cell[152654, 3478, 2844, 55, 36, "Input",ExpressionUUID->"e9209b3e-7c1f-4ebd-af36-e865590d5af1"],
Cell[155501, 3535, 620, 9, 70, "Output",ExpressionUUID->"ee685e65-ab33-4163-bed0-89680ad0c5c2"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[156182, 3551, 976, 18, 50, "Subsection",ExpressionUUID->"07e12d27-0dde-4d90-8f64-c6a961f230fd"],
Cell[157161, 3571, 1066, 18, 82, "Text",ExpressionUUID->"de2d3abe-0c47-4e74-ab8b-e35d2f3663b0"],
Cell[CellGroupData[{
Cell[158252, 3593, 237, 4, 34, "Subsubsection",ExpressionUUID->"88d8920c-9eeb-4c4a-bc3a-9ebcd7c21cbb"],
Cell[158492, 3599, 1067, 23, 50, "Input",ExpressionUUID->"fcf4ff0f-c4a4-4f13-975a-92a6df4b91c6"],
Cell[CellGroupData[{
Cell[159584, 3626, 510, 10, 25, "Item",ExpressionUUID->"8289e1ac-9997-4950-ad6b-c2b530fb5b81"],
Cell[CellGroupData[{
Cell[160119, 3640, 1456, 32, 21, "Input",ExpressionUUID->"05bff862-26a7-4d11-bcc3-9a48d6634c6b"],
Cell[161578, 3674, 1437, 34, 70, "Output",ExpressionUUID->"9429490f-a69d-4ece-8211-467cf764591e"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[163064, 3714, 543, 9, 25, "Item",ExpressionUUID->"2a736abd-e90e-44f6-b52c-8e20b06eab6f"],
Cell[CellGroupData[{
Cell[163632, 3727, 559, 12, 21, "Input",ExpressionUUID->"1f77570d-da27-49f4-b450-4b451f85cb49"],
Cell[164194, 3741, 4135, 73, 70, "Output",ExpressionUUID->"a5a5a3b4-51da-4ae9-8f18-baa4fa84ce2d"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[168390, 3821, 253, 6, 48, "Subsubsection",ExpressionUUID->"00ec375d-2422-451e-a73d-6d37cd1e45ca"],
Cell[168646, 3829, 364, 8, 63, "Text",ExpressionUUID->"f73e1c57-b7e3-4822-912d-979d3f851938"],
Cell[CellGroupData[{
Cell[169035, 3841, 1662, 41, 50, "Input",ExpressionUUID->"fd21c092-f99d-4ac6-a6fc-7c0451f2e1f6"],
Cell[170700, 3884, 721, 11, 70, "Output",ExpressionUUID->"3545e5d0-1688-4c68-9790-f14d8155d85d"]
}, Open  ]],
Cell[171436, 3898, 197, 3, 27, "Text",ExpressionUUID->"cf071b19-3546-4677-934e-dc752e47279f"],
Cell[CellGroupData[{
Cell[171658, 3905, 596, 13, 21, "Input",ExpressionUUID->"49c1fb18-7376-4275-bcbf-921941fe11df"],
Cell[172257, 3920, 2739, 57, 70, "Output",ExpressionUUID->"1ca73730-1191-4e18-a42c-de53cd401c67"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[175045, 3983, 598, 13, 48, "Subsubsection",ExpressionUUID->"4c6cfb47-c642-41ce-9e91-0147daf1e696"],
Cell[175646, 3998, 2551, 66, 64, "Input",ExpressionUUID->"d30da416-5e59-4422-975d-ba8e3fac867e"],
Cell[178200, 4066, 2148, 51, 64, "Input",ExpressionUUID->"3db61cb7-7c64-4615-b716-d657bf8dc686"],
Cell[180351, 4119, 3303, 75, 193, "Input",ExpressionUUID->"21729fd1-292c-4630-bb40-3a1c0c841f42"],
Cell[183657, 4196, 2051, 40, 36, "Input",ExpressionUUID->"b29a9626-6147-4602-acd4-709c18ebe158"],
Cell[185711, 4238, 915, 19, 45, "Text",ExpressionUUID->"dbe3d660-eedc-423b-a18f-cb6843f3b2cb"],
Cell[186629, 4259, 449, 13, 21, "Input",ExpressionUUID->"5226d951-8878-4071-bd39-47e663227e97"],
Cell[187081, 4274, 1585, 36, 50, "Input",ExpressionUUID->"bb42ae56-1d9b-450d-80c9-d99e66cfaf9c"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[188715, 4316, 157, 3, 41, "Subsection",ExpressionUUID->"4841dc4c-9c56-401c-a8ac-412d49f7fd90"],
Cell[CellGroupData[{
Cell[188897, 4323, 17781, 349, 692, "Input",ExpressionUUID->"c512e144-3e2f-438d-8f17-f184e444b24d"],
Cell[206681, 4674, 202190, 3314, 343, "Output",ExpressionUUID->"57c17075-b72d-4f6c-afac-c2d44699a73b"]
}, Open  ]],
Cell[408886, 7991, 1763, 27, 191, "Text",ExpressionUUID->"ba02451b-1f5e-4916-95f8-27414a9b488d"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

